/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.io;

import iso.mpeg.streamezzo.laser.*;
import iso.mpeg.streamezzo.sdl.*;
import iso.mpeg.streamezzo.util.Utils;

import java.io.IOException;
import java.util.Vector;

public class Decoder extends PreDecoder {

    public void decode(Xa self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ a");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("target")) { self.target = SDLClasses.class_byteAlignedString(br, "target");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv a");
    }

    public Object elements(BitReaderD br, String msg) throws IOException {
        Object child = null;
        int ch4;
        ch4 = br.readBits(6, "ch4");
        switch (ch4) {
            case 0:
                child = new Xa();
                decode((Xa) child, br);
                break;
            case 1:
                child = new Xanimate();
                decode((Xanimate) child, br);
                break;
            case 2:
                child = new XanimateColor();
                decode((Xanimate) child, br);
                break;
            case 3:
                child = new XanimateMotion();
                decode((XanimateMotion) child, br);
                break;
            case 4:
                child = new XanimateTransform();
                decode((XanimateTransform) child, br);
                break;
            case 5:
                child = new Xaudio();
                decode((Xaudio) child, br);
                break;
            case 6:
                child = new Xcircle();
                decode((Xcircle) child, br);
                break;
            case 7:
                child = new Xdefs();
                decode((Xdefs) child, br);
                break;
            case 8:
                child = new Xdesc();
                decode((Xdesc) child, br);
                break;
            case 9:
                child = new Xellipse();
                decode((Xellipse) child, br);
                break;
            case 10:
                child = new XforeignObject();
                decode((XforeignObject) child, br);
                break;
            case 11:
                child = new Xg();
                decode((Xg) child, br);
                break;
            case 12:
                child = new Ximage();
                decode((Ximage) child, br);
                break;
            case 13:
                child = new Xline();
                decode((Xline) child, br);
                break;
            case 14:
                child = new XlinearGradient();
                decode((XlinearGradient) child, br);
                break;
            case 15:
                child = new Xmetadata();
                decode((Xmetadata) child, br);
                break;
            case 16:
                child = new Xmpath();
                decode((Xmpath) child, br);
                break;
            case 17:
                child = new Xpath();
                decode((Xpath) child, br);
                break;
            case 18:
                child = new Xpolygon();
                decode((Xpolygon) child, br);
                break;
            case 19:
                child = new Xpolyline();
                decode((Xpolyline) child, br);
                break;
            case 20:
                child = new XradialGradient();
                decode((XradialGradient) child, br);
                break;
            case 21:
                child = new Xrect();
                decode((Xrect) child, br);
                break;
            case 22:
                child = new Xsameg();
                decode((Xsameg) child, br);
                break;
            case 23:
                child = new Xsameline();
                decode((Xsameline) child, br);
                break;
            case 24:
                child = new Xsamepath();
                decode((Xsamepath) child, br);
                break;
            case 25:
                child = new Xsamepathfill();
                decode((Xsamepathfill) child, br);
                break;
            case 26:
                child = new Xsamepolygon();
                decode((Xsamepolygon) child, br);
                break;
            case 27:
                child = new Xsamepolygonfill();
                decode((Xsamepolygonfill) child, br);
                break;
            case 28:
                child = new Xsamepolygonstroke();
                decode((Xsamepolygonstroke) child, br);
                break;
            case 29:
                child = new Xsamepolyline();
                decode((Xsamepolyline) child, br);
                break;
            case 30:
                child = new Xsamepolylinefill();
                decode((Xsamepolylinefill) child, br);
                break;
            case 31:
                child = new Xsamepolylinestroke();
                decode((Xsamepolylinestroke) child, br);
                break;
            case 32:
                child = new Xsamerect();
                decode((Xsamerect) child, br);
                break;
            case 33:
                child = new Xsamerectfill();
                decode((Xsamerectfill) child, br);
                break;
            case 34:
                child = new Xsametext();
                decode((Xsametext) child, br);
                break;
            case 35:
                child = new Xsametextfill();
                decode((Xsametextfill) child, br);
                break;
            case 36:
                child = new Xsameuse();
                decode((Xsameuse) child, br);
                break;
            case 37:
                child = new Xscript();
                decode((Xscript) child, br);
                break;
            case 38:
                child = new Xset();
                decode((Xset) child, br);
                break;
            case 39:
                child = new Xstop();
                decode((Xstop) child, br);
                break;
            case 40:
                child = new Xswitch();
                decode((Xswitch) child, br);
                break;
            case 41:
                child = new Xtext();
                decode((Xtext) child, br);
                break;
            case 42:
                child = new Xtitle();
                decode((Xtitle) child, br);
                break;
            case 43:
                child = new Xtspan();
                decode((Xtspan) child, br);
                break;
            case 44:
                child = new Xuse();
                decode((Xuse) child, br);
                break;
            case 45:
                child = new Xvideo();
                decode((Xvideo) child, br);
                break;
            case 46:
                child = new Xlistener();
                decode((Xlistener) child, br);
                break;
            case 47:
                child = new Xconditional();
                decode((Xconditional) child, br);
                break;
            case 48:
                child = new XcursorManager();
                decode((XcursorManager) child, br);
                break;
            case 49:
                child = class_anyElement_elements(br, "any");
                break;
            case 50:
                child = new XprivateElementContainer();
                decode((XprivateElementContainer) child, br);
                break;
            case 51:
                child = new XrectClip();
                decode((XrectClip) child, br);
                break;
            case 52:
                child = new Xselector();
                decode((Xselector) child, br);
                break;
            case 53:
                child = new XsimpleLayout();
                decode((XsimpleLayout) child, br);
                break;
            case 54:
                child = new XtextContent(SDLClasses.class_byteAlignedString(br, "textContent"));
                break;
            default:
                throw new RuntimeException("unknown element code:"+ch4);
        }
        return child;
    }

    public void decode(Xanimate self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ animate");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("attributeName")) { self.attributeName = SDLClasses.class_AttributeName(br, "attributeName");}
        if (br.readBit("accumulate")) { self.accumulate = br.readBit("accumulate");}
        if (br.readBit("additive")) { self.additive = br.readBit("additive");}
        if (br.readBit("by")) { self.by = SDLClasses.class_AnimatedValue(br, "by");}
        if (br.readBit("calcMode")) { self.calcMode = br.readBits(2, "calcMode");}
        if (br.readBit("from")) { self.from = SDLClasses.class_AnimatedValue(br, "from");}
        if (br.readBit("keySplines")) { self.keySplines = SDLClasses.class_fraction12List(br, "keySplines");}
        if (br.readBit("keyTimes")) { self.keyTimes = SDLClasses.class_fraction12List(br, "keyTimes");}
        if (br.readBit("values")) { self.values = SDLClasses.class_AnimatedValues(br, "values");}
        if (br.readBit("attributeType")) { self.attributeType = br.readBits(2, "attributeType");}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        if (br.readBit("dur")) { self.dur = SDLClasses.class_time(br, "dur", true);}
        if (br.readBit("fill")) { self.fill = br.readBits(1, "fill");}
        if (br.readBit("repeatCount")) { self.repeatCount = SDLClasses.repeatCountType(br, "repeatCount");}
        if (br.readBit("repeatDur")) { self.repeatDur = SDLClasses.repeatDurType(br, "repeatDur");}
        if (br.readBit("restart")) { self.restart = br.readBits(2, "restart");}
        if (br.readBit("to")) { self.to = SDLClasses.class_AnimatedValue(br, "to");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        self.enabled = br.readBit("enabled");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv animate");
    }

    public void decode(XanimateMotion self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ animateMotion");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("accumulate")) { self.accumulate = br.readBit("accumulate");}
        if (br.readBit("additive")) { self.additive = br.readBit("additive");}
        if (br.readBit("by")) { self.by = SDLClasses.class_AnimatedValue(br, "by");}
        if (br.readBit("calcMode")) { self.calcMode = br.readBits(2, "calcMode");}
        if (br.readBit("from")) { self.from = SDLClasses.class_AnimatedValue(br, "from");}
        if (br.readBit("keySplines")) { self.keySplines = SDLClasses.class_fraction12List(br, "keySplines");}
        if (br.readBit("keyTimes")) { self.keyTimes = SDLClasses.class_fraction12List(br, "keyTimes");}
        if (br.readBit("values")) { self.values = SDLClasses.class_AnimatedValues(br, "values");}
        if (br.readBit("attributeType")) { self.attributeType = br.readBits(2, "attributeType");}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        if (br.readBit("dur")) { self.dur = SDLClasses.class_time(br, "dur", true);}
        if (br.readBit("fill")) { self.fill = br.readBits(1, "fill");}
        if (br.readBit("repeatCount")) { self.repeatCount = SDLClasses.repeatCountType(br, "repeatCount");}
        if (br.readBit("repeatDur")) { self.repeatDur = SDLClasses.repeatDurType(br, "repeatDur");}
        if (br.readBit("restart")) { self.restart = br.readBits(2, "restart");}
        if (br.readBit("to")) { self.to = SDLClasses.class_AnimatedValue(br, "to");}
        if (br.readBit("keyPoints")) { self.keyPoints = SDLClasses.class_floatList(br, "keyPoints");}
        if (br.readBit("path")) { self.path = SDLClasses.class_path(br, "path");}
        if (br.readBit("rotate")) { self.rotate = SDLClasses.rotateType(br, "rotate");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        self.enabled = br.readBit("enabled");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv animateMotion");
    }

    public void decode(XanimateTransform self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ animateTransform");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("attributeName")) { self.attributeName = SDLClasses.class_AttributeName(br, "attributeName");}
        self.type = br.readBits(3, "rotscatra");
        if (br.readBit("accumulate")) { self.accumulate = br.readBit("accumulate");}
        if (br.readBit("additive")) { self.additive = br.readBit("additive");}
        if (br.readBit("by")) { self.by = SDLClasses.class_AnimatedValue(br, "by");}
        if (br.readBit("calcMode")) { self.calcMode = br.readBits(2, "calcMode");}
        if (br.readBit("from")) { self.from = SDLClasses.class_AnimatedValue(br, "from");}
        if (br.readBit("keySplines")) { self.keySplines = SDLClasses.class_fraction12List(br, "keySplines");}
        if (br.readBit("keyTimes")) { self.keyTimes = SDLClasses.class_fraction12List(br, "keyTimes");}
        if (br.readBit("values")) { self.values = SDLClasses.class_AnimatedValues(br, "values");}
        if (br.readBit("attributeType")) { self.attributeType = br.readBits(2, "attributeType");}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        if (br.readBit("dur")) { self.dur = SDLClasses.class_time(br, "dur", true);}
        if (br.readBit("fill")) { self.fill = br.readBits(1, "fill");}
        if (br.readBit("repeatCount")) { self.repeatCount = SDLClasses.repeatCountType(br, "repeatCount");}
        if (br.readBit("repeatDur")) { self.repeatDur = SDLClasses.repeatDurType(br, "repeatDur");}
        if (br.readBit("restart")) { self.restart = br.readBits(2, "restart");}
        if (br.readBit("to")) { self.to = SDLClasses.class_AnimatedValue(br, "to");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        self.enabled = br.readBit("enabled");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv animateTransform");
    }

    public void decode(Xaudio self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ audio");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        if (br.readBit("dur")) { self.dur = SDLClasses.class_time(br, "dur", true);}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("repeatCount")) { self.repeatCount = SDLClasses.repeatCountType(br, "repeatCount");}
        if (br.readBit("repeatDur")) { self.repeatDur = SDLClasses.repeatDurType(br, "repeatDur");}
        if (br.readBit("restart")) { self.restart = br.readBits(2, "restart");}
        if (br.readBit("syncBehavior")) { self.syncBehavior = br.readBits(2, "syncBehavior");}
        if (br.readBit("syncTolerance")) { self.syncTolerance = SDLClasses.syncTolerance(br, "syncTolerance");}
        if (br.readBit("type")) { self.type = SDLClasses.class_byteAlignedString(br, "type");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        if (br.readBit("clipBegin")) { self.clipBegin = SDLClasses.class_time(br, "clipBegin", true);}
        if (br.readBit("clipEnd")) { self.clipEnd = SDLClasses.class_time(br, "clipEnd", true);}
        if (br.readBit("syncReference")) { self.syncReference = SDLClasses.class_anyURI(br, "syncReference");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv audio");
    }

    public void decode(Xcircle self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ circle");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        if (br.readBit("cx")) { self.cx = SDLClasses.class_coordinate(br, "cx");}
        if (br.readBit("cy")) { self.cy = SDLClasses.class_coordinate(br, "cy");}
        self.r = SDLClasses.class_coordinate(br, "r");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv circle");
    }

    public void decode(XcursorManager self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ cursor");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("x")) { self.x = SDLClasses.class_coordinate(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.class_coordinate(br, "y");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv cursor");
    }

    public void decode(Xdefs self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ defs");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv defs");
    }

    public void decode(Xdesc_metaData_title self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ desc_metaData_title");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv desc_metaData_title");
    }

    public void decode(Xellipse self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ ellipse");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        if (br.readBit("cx")) { self.cx = SDLClasses.class_coordinate(br, "cx");}
        if (br.readBit("cy")) { self.cy = SDLClasses.class_coordinate(br, "cy");}
        self.rx = SDLClasses.class_coordinate(br, "rx");
        self.ry = SDLClasses.class_coordinate(br, "ry");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv ellipse");
    }

    public void decode(XforeignObject self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ foreignObject");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        self.height = SDLClasses.class_coordinate(br, "height");
        self.width = SDLClasses.class_coordinate(br, "width");
        if (br.readBit("x")) { self.x = SDLClasses.class_coordinate(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.class_coordinate(br, "y");}
        SDLClasses.class_anyAttribute(br, "anyAttribute", self);
        if (br.readBit("loop")) {
            int occ0 = br.readVlcInt(5, "occ0");
            for (int i = 0; i < occ0; i++) {
                XprivateElementContainer pec = new XprivateElementContainer();
                self.addChild(pec);
                decode(pec, br);
            }
        }
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv foreignObject");
    }

    public void decode(Xg self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ g");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv g");
    }

    public void decode(XsimpleLayout self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ simpleLayout");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        if (br.readBit("delta")) { self.delta = SDLClasses.pointType(br, "delta");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv simpleLayout");
    }

    public void decode(Xselector self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ selector");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("choice")) { self.choice = SDLClasses.choiceType(br, "choice");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv g");
    }

    public void decode(XrectClip self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ rectClip");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("size")) { self.size = SDLClasses.pointType(br, "size");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv rectClip");
    }

    public void decode(Ximage self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ image");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("height")) { self.height = SDLClasses.class_coordinate(br, "height");}
        if (br.readBit("opacity")) { self.opacity = SDLClasses.class_0to1float(br, "opacity");}
        if (br.readBit("preserveAspectRatio"))
        { self.preserveAspectRatio = SDLClasses.class_preserveAspectRatio(br, "preserveAspectRatio");}
        if (br.readBit("type")) { self.type = SDLClasses.class_byteAlignedString(br, "type");}
        if (br.readBit("width")) { self.width = SDLClasses.class_coordinate(br, "width");}
        if (br.readBit("x")) { self.x = SDLClasses.class_coordinate(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.class_coordinate(br, "y");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        if (br.readBit("transformBehavior")) { self.transformBehavior = br.readBits(4, "transformBehavior");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv image");
    }

    public void decode(Xline self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ line");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        if (br.readBit("x1")) { self.x1 = SDLClasses.class_coordinate(br, "x1");}
        self.x2 = SDLClasses.class_coordinate(br, "x2");
        if (br.readBit("y1")) { self.y1 = SDLClasses.class_coordinate(br, "y1");}
        self.y2 = SDLClasses.class_coordinate(br, "y2");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv line");
    }

    public void decode(XlinearGradient self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ linearGradient");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        if (br.readBit("gradientUnits")) { self.gradientUnits = br.readBits(1, "gradientUnits");}
        if (br.readBit("x1")) { self.x1 = SDLClasses.class_coordinate(br, "x1");}
        if (br.readBit("x2")) { self.x2 = SDLClasses.class_coordinate(br, "x2");}
        if (br.readBit("y1")) { self.y1 = SDLClasses.class_coordinate(br, "y1");}
        if (br.readBit("y2")) { self.y2 = SDLClasses.class_coordinate(br, "y2");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv linearGradient");
    }

    public void decode(Xmpath self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ mpath");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv mpath");
    }

    public void decode(Xpath self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ path");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.d = SDLClasses.class_path(br, "d");
        if (br.readBit("pathLength")) { self.pathLength = SDLClasses.class_fixed_16_8(br, "pathLength");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv path");
    }

    public void decode(Xpolygon self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ polygon");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.points = SDLClasses.class_pointSequence(br, "points");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv polygon");
    }

    public void decode(XradialGradient self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ radialGradient");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        if (br.readBit("cx")) { self.cx = SDLClasses.class_coordinate(br, "cx");}
        if (br.readBit("cy")) { self.cy = SDLClasses.class_coordinate(br, "cy");}
        if (br.readBit("gradientUnits")) { self.gradientUnits = br.readBits(1, "gradientUnits");}
        if (br.readBit("r")) { self.r = SDLClasses.class_coordinate(br, "r");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv radialGradient");
    }

    public void decode(Xrect self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ rect");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.height = SDLClasses.class_coordinate(br, "height");
        if (br.readBit("rx")) { self.rx = SDLClasses.class_coordinate(br, "rx");}
        if (br.readBit("ry")) { self.ry = SDLClasses.class_coordinate(br, "ry");}
        self.width = SDLClasses.class_coordinate(br, "width");
        if (br.readBit("x")) { self.x = SDLClasses.class_coordinate(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.class_coordinate(br, "y");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv rect");
    }

    public void decode(Xsameg self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ sameg");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv sameg");
    }

    public void decode(Xsameline self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ sameline");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("x1")) { self.x1 = SDLClasses.class_coordinate(br, "x1");}
        self.x2 = SDLClasses.class_coordinate(br, "x2");
        if (br.readBit("y1")) { self.y1 = SDLClasses.class_coordinate(br, "y1");}
        self.y2 = SDLClasses.class_coordinate(br, "y2");
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv sameline");
    }

    public void decode(Xsamepath self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ samepath");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        self.d = SDLClasses.class_path(br, "d");
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv samepath");
    }

    public void decode(Xsamepathfill self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ samepathfill");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        self.d = SDLClasses.class_path(br, "d");
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv samepathfill");
    }

    public void decode(Xsamepolygon self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ samepolygon");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        self.points = SDLClasses.class_pointSequence(br, "points");
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv samepolygon");
    }

    public void decode(Xsamepolygonfill self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ samepolygonfill");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        self.points = SDLClasses.class_pointSequence(br, "points");
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv samepolygonfill");
    }

    public void decode(Xsamepolygonstroke self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ samepolygonstroke");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.points = SDLClasses.class_pointSequence(br, "points");
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv samepolygonstroke");
    }

    public void decode(Xsamerect self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ samerect");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        self.height = SDLClasses.class_coordinate(br, "height");
        self.width = SDLClasses.class_coordinate(br, "width");
        if (br.readBit("x")) { self.x = SDLClasses.class_coordinate(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.class_coordinate(br, "y");}
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv samerect");
    }

    public void decode(Xsamerectfill self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ samerectfill");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        self.height = SDLClasses.class_coordinate(br, "height");
        self.width = SDLClasses.class_coordinate(br, "width");
        if (br.readBit("x")) { self.x = SDLClasses.class_coordinate(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.class_coordinate(br, "y");}
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv samerectfill");
    }

    public void decode(Xsametext self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ sametext");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("x")) { self.x = SDLClasses.coordinateListType(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.coordinateListType(br, "y");}
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv sametext");
    }

    public void decode(Xsametextfill self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ sametextfill");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("x")) { self.x = SDLClasses.coordinateListType(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.coordinateListType(br, "y");}
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv sametextfill");
    }

    public void decode(Xsameuse self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ sameuse");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        decodeSameContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv sameuse");
    }

    public void decode(Xscript self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ script");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("type")) { self.type = SDLClasses.class_scriptType(br, "type");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv script");
    }

    public void decode(Xconditional self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ conditional");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        self.enabled = br.readBit("enabled");
        SDLClasses.class_anyAttribute(br, "anyAttribute", self);
        br.readVlcInt(5, "len");
        br.flush("align");
        int occ0 = br.readVlcInt(5, "occ0");
        for (int i = 0; i < occ0; i++) {
            self.addChild(updates(br, "child"));
        }
        br.flush("align");
        self.privateAttributes = SDLClasses.privateAttributeContainer(br, "privateAttributeContainer");
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv conditional");
    }

    public Object updates(BitReaderD br, String msg) throws IOException {
        Object child = null;
        int ch4;
        ch4 = br.readBits(4, "ch4");
        switch (ch4) {
            case 0:
                child = new XAdd();
                decode((XAdd) child, br);
                break;
            case 1:
                child = new XClean();
                decode((XClean) child, br);
                break;
            case 2:
                child = new XDelete();
                decode((XDelete) child, br);
                break;
            case 3:
                child = new XInsert();
                decode((XInsert) child, br);
                break;
            case 4:
                child = new XNewScene();
                decode((XNewScene) child, br);
                break;
            case 5:
                child = new XRefreshScene();
                decode((XRefreshScene) child, br);
                break;
            case 6:
                child = new XReplace();
                decode((XReplace) child, br);
                break;
            case 7:
                child = new XRestore();
                decode((XRestore) child, br);
                break;
            case 8:
                child = new XSave();
                decode((XSave) child, br);
                break;
            case 9:
                child = new XSendEvent();
                decode((XSendEvent) child, br);
                break;
            case 10:
                child = class_anyElement_updates(br, "any");
                break;
            case 11:
                child = new XtextContent(SDLClasses.class_byteAlignedString(br, "textContent"));
                break;
            default:
                throw new RuntimeException("illegal update code:"+ch4);
        }
        return child;
    }

    public void decode(XAdd self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ Add");
        if (br.readBit("attributeName")) { self.attributeName = SDLClasses.class_AttributeName(br, "attributeName");}
        if (br.readBit("operandAttribute")) { self.operandAttribute = SDLClasses.class_AttributeName(br, "operandAttribute");}
        if (br.readBit("operandElementId")) { self.operandElementId = SDLClasses.class_IDREF(br, "operandElementId");}
        self.ref = SDLClasses.class_IDREF(br, "ref");
        if (br.readBit("value"))
        { self.value = SDLClasses.class_updateValue(SDLClasses.typeOf(self.attributeName, self.index), br, "value");}
        decodeSimpleUpdateContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv Add");
    }

    public void decode(XClean self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ Clean");
        self.groupID = SDLClasses.class_byteAlignedString(br, "groupID");
        decodeSimpleUpdateContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv Clean");
    }

    public void decode(XDelete self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ Delete");
        if (br.readBit("attributeName")) { self.attributeName = br.readBits(8, "attributeName");}
        if (br.readBit("index")) { self.index = br.readVlcInt(5, "index");}
        self.ref = SDLClasses.class_IDREF(br, "ref");
        decodeSimpleUpdateContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv Delete");
    }

    public void decode(XInsert self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ Insert");
        if (br.readBit("attributeName")) { self.attributeName = SDLClasses.class_AttributeName(br, "attributeName");}
        if (br.readBit("index")) { self.index = br.readVlcInt(5, "index");}
        self.ref = SDLClasses.class_IDREF(br, "ref");
        if (br.readBit("value"))
        { self.value = SDLClasses.class_updateValue(SDLClasses.typeOf(self.attributeName, self.index), br, "value");}
        decodeInsertContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv Insert");
    }

    public Object updatable_elements(BitReaderD br, String msg) throws IOException {
        Object child = null;
        int ch4;
        ch4 = br.readBits(1, "ch4");
        switch (ch4) {
            case 0:
                int ch6 = br.readBits(6, "ch6");
                switch (ch6) {
                    case 0:
                        child = new Xa();
                        decode((Xa) child, br);
                        break;
                    case 1:
                        child = new Xanimate();
                        decode((Xanimate) child, br);
                        break;
                    case 2:
                        child = new XanimateColor();
                        decode((XanimateColor) child, br);
                        break;
                    case 3:
                        child = new XanimateMotion();
                        decode((XanimateMotion) child, br);
                        break;
                    case 4:
                        child = new XanimateTransform();
                        decode((XanimateTransform) child, br);
                        break;
                    case 5:
                        child = new Xaudio();
                        decode((Xaudio) child, br);
                        break;
                    case 6:
                        child = new Xcircle();
                        decode((Xcircle) child, br);
                        break;
                    case 7:
                        child = new Xdefs();
                        decode((Xdefs) child, br);
                        break;
                    case 8:
                        child = new Xdesc();
                        decode((Xdesc) child, br);
                        break;
                    case 9:
                        child = new Xellipse();
                        decode((Xellipse) child, br);
                        break;
                    case 10:
                        child = new XforeignObject();
                        decode((XforeignObject) child, br);
                        break;
                    case 11:
                        child = new Xg();
                        decode((Xg) child, br);
                        break;
                    case 12:
                        child = new Ximage();
                        decode((Ximage) child, br);
                        break;
                    case 13:
                        child = new Xline();
                        decode((Xline) child, br);
                        break;
                    case 14:
                        child = new XlinearGradient();
                        decode((XlinearGradient) child, br);
                        break;
                    case 15:
                        child = new Xmetadata();
                        decode((Xmetadata) child, br);
                        break;
                    case 16:
                        child = new Xmpath();
                        decode((Xmpath) child, br);
                        break;
                    case 17:
                        child = new Xpath();
                        decode((Xpath) child, br);
                        break;
                    case 18:
                        child = new Xpolygon();
                        decode((Xpolygon) child, br);
                        break;
                    case 19:
                        child = new Xpolyline();
                        decode((Xpolyline) child, br);
                        break;
                    case 20:
                        child = new XradialGradient();
                        decode((XradialGradient) child, br);
                        break;
                    case 21:
                        child = new Xrect();
                        decode((Xrect) child, br);
                        break;
                    case 22:
                        child = new Xscript();
                        decode((Xscript) child, br);
                        break;
                    case 23:
                        child = new Xset();
                        decode((Xset) child, br);
                        break;
                    case 24:
                        child = new Xstop();
                        decode((Xstop) child, br);
                        break;
                    case 25:
                        child = new Xsvg();
                        decode((Xsvg) child, br);
                        break;
                    case 26:
                        child = new Xswitch();
                        decode((Xswitch) child, br);
                        break;
                    case 27:
                        child = new Xtext();
                        decode((Xtext) child, br);
                        break;
                    case 28:
                        child = new Xtitle();
                        decode((Xtitle) child, br);
                        break;
                    case 29:
                        child = new Xtspan();
                        decode((Xtspan) child, br);
                        break;
                    case 30:
                        child = new Xuse();
                        decode((Xuse) child, br);
                        break;
                    case 31:
                        child = new Xvideo();
                        decode((Xvideo) child, br);
                        break;
                    case 32:
                        child = new Xlistener();
                        decode((Xlistener) child, br);
                        break;
                }
                break;
            case 1:
                int ch61 = (byte) br.readBits(3, "ch61");
                switch (ch61) {
                    case 0:
                        child = new Xconditional();
                        decode((Xconditional) child, br);
                        break;
                    case 1:
                        child = new XcursorManager();
                        decode((XcursorManager) child, br);
                        break;
                    case 2:
                        child = null;
                        class_anyElement_elements(br, "any");
                        break;
                    case 3:
                        child = new XprivateElementContainer();
                        decode((XprivateElementContainer) child, br);
                        break;
                    case 4:
                        child = new XrectClip();
                        decode((XrectClip) child, br);
                        break;
                    case 5:
                        child = new Xselector();
                        decode((Xselector) child, br);
                        break;
                    case 6:
                        child = new XsimpleLayout();
                        decode((XsimpleLayout) child, br);
                        break;
                }
                break;
        }
        return child;
    }

    public void decode(Xset self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ set");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("attributeName")) { self.attributeName = SDLClasses.class_AttributeName(br, "attributeName");}
        if (br.readBit("attributeType")) { self.attributeType = br.readBits(2, "attributeType");}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        if (br.readBit("dur")) { self.dur = SDLClasses.class_time(br, "dur", true);}
        if (br.readBit("fill")) { self.fill = br.readBits(1, "fill");}
        if (br.readBit("repeatCount")) { self.repeatCount = SDLClasses.repeatCountType(br, "repeatCount");}
        if (br.readBit("repeatDur")) { self.repeatDur = SDLClasses.repeatDurType(br, "repeatDur");}
        if (br.readBit("restart")) { self.restart = br.readBits(2, "restart");}
        if (br.readBit("to")) { self.to = SDLClasses.class_AnimatedValue(br, "to");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        self.enabled = br.readBit("enabled");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv set");
    }

    public void decode(Xstop self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ stop");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.offset = SDLClasses.class_fixed_16_8(br, "offset");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv stop");
    }

    public void decode(Xsvg self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ svg");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        if (br.readBit("baseProfile")) { self.baseProfile = SDLClasses.class_byteAlignedString(br, "baseProfile");}
        if (br.readBit("contentScriptType"))
        { self.contentScriptType = SDLClasses.class_byteAlignedString(br, "contentScriptType");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        self.height = SDLClasses.class_valueWithUnits(br, "height");
        if (br.readBit("playbackOrder")) { self.playbackOrder = br.readBits(1, "playbackOrder");}
        if (br.readBit("preserveAspectRatio"))
        { self.preserveAspectRatio = SDLClasses.class_preserveAspectRatio(br, "preserveAspectRatio");}
        if (br.readBit("snapshotTime")) { self.snapshotTime = SDLClasses.class_time(br, "snapshotTime", true);}
        if (br.readBit("syncBehaviorDefault")) { self.syncBehaviorDefault = br.readBits(2, "syncBehaviorDefault");}
        if (br.readBit("syncToleranceDefault"))
        { self.syncToleranceDefault = SDLClasses.syncTolerance(br, "syncToleranceDefault");}
        if (br.readBit("timelineBegin")) { self.timelineBegin = br.readBits(1, "timelineBegin");}
        if (br.readBit("version")) { self.version = SDLClasses.class_byteAlignedString(br, "version");}
        if (br.readBit("viewBox")) { self.viewBox = SDLClasses.class_floatList(4, br, "viewBox");}
        self.width = SDLClasses.class_valueWithUnits(br, "width");
        if (br.readBit("zoomAndPan")) { self.zoomAndPan = br.readBits(1, "zoomAndPan");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv svg");
    }

    public void decode(Xswitch self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ switch");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv switch");
    }

    public void decode(Xtext self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ text");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.editable = br.readBit("editable");
        if (br.readBit("rotate")) { self.rotate = SDLClasses.class_floatList(br, "rotate");}
        if (br.readBit("x")) { self.x = SDLClasses.coordinateListType(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.coordinateListType(br, "y");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv text");
    }

    public void decode(Xtspan self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ tspan");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv tspan");
    }

    public void decode(Xuse self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ use");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("overflow")) { self.overflow = br.readBits(2, "overflow");}
        if (br.readBit("x")) { self.x = SDLClasses.class_coordinate(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.class_coordinate(br, "y");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv use");
    }

    public void decode(Xvideo self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ video");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        if (br.readBit("dur")) { self.dur = SDLClasses.class_time(br, "dur", true);}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("height")) { self.height = SDLClasses.class_coordinate(br, "height");}
        if (br.readBit("overlay")) { self.overlay = SDLClasses.class_overlay(br, "overlay");}
        if (br.readBit("preserveAspectRatio"))
        { self.preserveAspectRatio = SDLClasses.class_preserveAspectRatio(br, "preserveAspectRatio");}
        if (br.readBit("repeatCount")) { self.repeatCount = SDLClasses.repeatCountType(br, "repeatCount");}
        if (br.readBit("repeatDur")) { self.repeatDur = SDLClasses.repeatDurType(br, "repeatDur");}
        if (br.readBit("restart")) { self.restart = br.readBits(2, "restart");}
        if (br.readBit("syncBehavior")) { self.syncBehavior = br.readBits(2, "syncBehavior");}
        if (br.readBit("syncTolerance")) { self.syncTolerance = SDLClasses.syncTolerance(br, "syncTolerance");}
        if (br.readBit("transformBehavior")) { self.transformBehavior = br.readBits(4, "transformBehavior");}
        if (br.readBit("type")) { self.type = SDLClasses.class_byteAlignedString(br, "type");}
        if (br.readBit("width")) { self.width = SDLClasses.class_coordinate(br, "width");}
        if (br.readBit("x")) { self.x = SDLClasses.class_coordinate(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.class_coordinate(br, "y");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        if (br.readBit("clipBegin")) { self.clipBegin = SDLClasses.class_time(br, "clipBegin", true);}
        if (br.readBit("clipEnd")) { self.clipEnd = SDLClasses.class_time(br, "clipEnd", true);}
        if (br.readBit("syncReference")) { self.syncReference = SDLClasses.class_anyURI(br, "syncReference");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv video");
    }

    public void decode(Xlistener self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ listener");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("defaultAction")) { self.defaultAction = br.readBits(1, "defaultAction");}
        if (br.readBit("event")) { self.event = SDLClasses.class_event(br, "event");}
        if (br.readBit("handler")) { self.handler = SDLClasses.class_anyURI(br, "handler");}
        if (br.readBit("observer")) { self.observer = SDLClasses.class_IDREF(br, "observer");}
        self.phase = br.readBits(1, "phase");
        if (br.readBit("propagate")) { self.propagate = br.readBits(1, "propagate");}
        if (br.readBit("target")) { self.target = SDLClasses.class_IDREF(br, "target");}
        // if (br.readBit("delay")) { self.delay = SDLClasses.class_time(br, "delay", true);}
        // if (br.readBit("timeAttribute")) { self.timeAttribute = br.readBits(1, "timeAttribute");}
        self.enabled = br.readBit("enabled");
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv listener");
    }

    public void decode(XNewScene self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ NewScene");
        decodeSimpleUpdateContent(self, br);
        self.child = new Xsvg();
        decode((Xsvg) self.child, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv NewScene");
    }

    public void decode(XRefreshScene self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ RefreshScene");
        self.time = br.readVlcInt(5, "time");
        decodeSimpleUpdateContent(self, br);
        self.child = new Xsvg();
        decode((Xsvg) self.child, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv RefreshScene");
    }

    public void decode(XReplace self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ Replace");
        if (br.readBit("attributeName")) { self.attributeName = SDLClasses.class_AttributeName(br, "attributeName");}
        if (br.readBit("index")) { self.index = br.readVlcInt(5, "index");}
        if (br.readBit("operandAttribute")) { self.operandAttribute = br.readBits(8, "operandAttribute");}
        if (br.readBit("operandElementId")) { self.operandElementId = SDLClasses.class_IDREF(br, "operandElementId");}
        self.ref = SDLClasses.class_IDREF(br, "ref");
        if (br.readBit("value"))
        { self.value = SDLClasses.class_updateValue(SDLClasses.typeOf(self.attributeName, self.index), br, "value");}
        decodeReplaceContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv Replace");
    }

    public void decode(XRestore self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ Restore");
        self.groupID = SDLClasses.class_byteAlignedString(br, "groupID");
        decodeSimpleUpdateContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv Restore");
    }

    public void decode(XSave self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ Save");
        SDLClasses.class_ElementAttributeList(br, self);
        self.groupID = SDLClasses.class_byteAlignedString(br, "groupID");
        decodeSimpleUpdateContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv Save");
    }

    public void decode(XSendEvent self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ SendEvent");
        self.event = SDLClasses.class_event(br, "event");
        if (br.readBit("intvalue")) { self.intvalue = SDLClasses.class_signedInt(br, "intvalue");}
        if (br.readBit("pointvalue")) { self.pointvalue = SDLClasses.pointType(br, "pointvalue");}
        self.ref = SDLClasses.class_IDREF(br, "ref");
        if (br.readBit("stringvalue")) { self.stringvalue = SDLClasses.class_byteAlignedString(br, "stringvalue");}
        decodeSimpleUpdateContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv SendEvent");
    }

    public void decode(XLASeRHeader self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ LASeRHeader");
        self.profile = (byte) br.readBits(8, "profile");
        self.level = (byte) br.readBits(8, "level");
        self.reserved = br.readBits(3, "reserved");
        self.pointsCodec = br.readBits(2, "pointsCodec");
        self.pathComponents = (byte) br.readBits(4, "pathComponents");
        if (self.pathComponents == 15) self.pathComponents = -1;
        self.useFullRequestHost = br.readBit("useFullRequestHost");
        if (br.readBit("timeResolution")) {
            self.timeResolution = br.readBits(16, "timeResolution");
            SDLClasses.timeResolution = self.timeResolution;
        }
        self.colorComponentBits_minus_1 = br.readBits(4, "colorComponentBits_minus_1");
        self.resolution = br.readBits(4, "resolution");
        if (self.resolution > 7) self.resolution -= 16;
        SDLClasses.resolution = self.resolution;
        self.coordBits = br.readBits(5, "coordBits");
        SDLClasses.coordBits = self.coordBits;
        self.scaleBits_minus_coordBits = br.readBits(4, "scaleBits_minus_coordBits");
        SDLClasses.scaleBits = self.scaleBits_minus_coordBits + self.coordBits;
        self.newSceneIndicator = br.readBit("newSceneIndicator");
        br.readBits(3, "reserved");
        self.extensionIDBits = br.readBits(4, "extensionIDBits");
        boolean hasExtensionDescriptor;
        hasExtensionDescriptor = br.readBit("hasExtensionDescriptor");
        if (hasExtensionDescriptor) {
            int len;
            len = br.readVlcInt(5, "len");
            {
                self.extensionDescriptor = new byte[len];
                for (int i = 0; i < len; i++) {
                    self.extensionDescriptor[i] = (byte) br.readBits(8, "extensionDescriptor");
                }
            }
        }
        if (br.readBit("ext")) { SDLClasses.class_extension(br, "ext");}
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv LASeRHeader");
    }

    public Object class_anyElement_updates(BitReaderD br, String msg) throws IOException {
        int extensionID = br.readBits(SDLClasses.extensionIDBits, msg);
        int bitlen = br.readVlcInt(5, msg);
        if (extensionID != 0) {
            if (bitlen % 8 != 0) br.readBits(bitlen % 8, msg);
            if (bitlen >> 3 != 0) br.readByteArray(bitlen >> 3, msg);
        } else {
            Vector v = new Vector();
            // Deactivate and Activate
            br.readVlcInt(5, "must be 87");
            int len = br.readVlcInt(5, msg);
            for (int i = 0; i < len; i++) {
                br.readBits(2, msg);
                int ch2 = br.readBits(2, msg);
                switch(ch2) {
                  case 0:
                      XprivateElementContainer child = new XprivateElementContainer();
                      decode((XprivateElementContainer) child, br);
                      v.addElement(child);
                      break;
                  case 1:
                      XActivate a = new XActivate();
                      a.ref = SDLClasses.class_IDREF(br, "activate ref");
                      SDLClasses.class_anyAttribute(br, "anyAttribute", null);
                      v.addElement(a);
                      break;
                  case 2:
                      XDeactivate d = new XDeactivate();
                      d.ref = SDLClasses.class_IDREF(br, "activate ref");
                      SDLClasses.class_anyAttribute(br, "anyAttribute", null);
                      v.addElement(d);
                      break;
                  case 3:
                      XReleaseResource rr = new XReleaseResource();
                      rr.ref = SDLClasses.class_anyURI(br, "releaseResource ref");
                      SDLClasses.class_anyAttribute(br, "anyAttribute", null);
                      v.addElement(rr);
                      break;
                }
            }
            return v;
        }
        return null;
    }

    public Vector class_anyXMLElement(BitReaderD br, String msg) throws IOException {
        /*int len = */br.readVlcInt(5, msg);
        Vector children = new Vector();
        br.flush(msg);
        privateChildren(br, children);
        br.flush(msg);
        return children;
    }

    private XprivateElement privateElement(BitReaderD br) throws IOException {
        XprivateElement pe = new XprivateElement();
        int tagIndex = br.readBits(SDLClasses.tagIndexBits, "tagIndex");
        pe.name = SDLClasses.man.getTag(tagIndex);
        if (br.readBit("hasID")) {
            pe.id = SDLClasses.class_ID(br, "id");
        }
        int len = br.readVlcInt(5, "len");
        pe.privateAttributes = new privAttr[len];
        for (int i = 0; i < len; i++) {
            if (br.readBit("hasNS")) {
                pe.privateAttributes[i] = privateAttribute(br, tagIndex);
            } else {
                pe.privateAttributes[i] = privateAttribute(br, 0);
            }
        }
        if (br.readBit("hasChildren")) {
            pe.child = new Vector();
            privateChildren(br, pe.child);
        }
        return pe;
    }

    private privAttr privateAttribute(BitReaderD br, int tagIndex) throws IOException {
        int bits = Utils.nbBitsForPositiveInteger(SDLClasses.man.nbOfAttrs(tagIndex));
        privAttr pa = new privAttr();
        pa.set(SDLClasses.man.getAttr(tagIndex, br.readBits(bits, "attr idx")));
        pa.value = SDLClasses.class_byteAlignedString(br, "priv attr val");
        return pa;
    }

    private void privateChildren(BitReaderD br, Vector children) throws IOException {
        int len = br.readVlcInt(5, "privChildren");
        for (int i = 0; i < len; i++) {
            if (br.readBit("isPriv")) {
                if (br.readBit("isText")) {
                    children.addElement(new XtextContent(SDLClasses.class_byteAlignedString(br, "privChildren")));
                } else {
                    children.addElement(privateElement(br));
                }
            } else {
                children.addElement(updatable_elements(br, "privChildren"));
            }
        }
    }

    public void decode(XprivateElementContainer self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ privateElementContainer");
        int ch4;
        ch4 = br.readBits(4, "ch4");
        switch (ch4) {
            case 0:
                self.anyXML = class_anyXMLElement(br, "anyXML");
                break;
            case 1:
                SDLClasses.class_opaqueElement(br, self);
                break;
            case 2:
                class_anyElement_elements(br, "reserved");
                break;
            default:
                SDLClasses.class_extension(br, "ext");
                break;
        }
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv privateElementContainer");
    }

    public Object class_anyElement_elements(BitReaderD br, String msg) throws IOException {
        int extensionID = br.readBits(SDLClasses.extensionIDBits, "extensionID");
        int bitlen = br.readVlcInt(5, "bitSkipLen");
        if (extensionID != 2) {
            if (bitlen % 8 != 0) br.readBits(bitlen % 8, msg);
            if (bitlen >> 3 != 0) br.readByteArray(bitlen >> 3, msg);
        } else {
            br.readVlcInt(5, "must be 87");
            int len = br.readVlcInt(5, msg);
            Vector v = new Vector();
            for (int i = 0; i < len; i++) {
                // v2 elements
                int ch2 = br.readBits(2, "ch2");
                Object child = null;
                switch(ch2) {
                    case 0:
                        // svg
                        int ch5 = br.readBits(4, "ch5");
                        switch(ch5) {
                            case 0:
                                child = new Xanimation();
                                decode((Xanimation)child, br);
                                break;
                            case 1:
                                child = new Xdiscard();
                                decode((Xdiscard)child, br);
                                break;
                            case 2:
                                child = new Xfont();
                                decode((Xfont)child, br);
                                break;
                            case 3:
                                child = new XfontFace();
                                decode((XfontFace)child, br);
                                break;
                            case 4:
                                child = new XfontFaceSrc();
                                decode((XfontFaceSrc)child, br);
                                break;
                            case 5:
                                child = new XfontFaceUri();
                                decode((XfontFaceUri)child, br);
                                break;
                            case 6:
                                child = new Xglyph();
                                decode((Xglyph)child, br);
                                break;
                            case 7:
                                child = new Xhandler();
                                decode((Xhandler)child, br);
                                break;
                            case 8:
                                child = new Xhkern();
                                decode((Xhkern)child, br);
                                break;
                            case 9:
                                child = new XmissingGlyph();
                                decode((XmissingGlyph)child, br);
                                break;
                            case 10:
                                child = new Xprefetch();
                                decode((Xprefetch)child, br);
                                break;
                            case 11:
                                child = new XsolidColor();
                                decode((XsolidColor)child, br);
                                break;
                            case 12:
                                child = new XtBreak();
                                decode((XtBreak)child, br);
                                break;
                            case 13:
                                child = new XtextArea();
                                decode((XtextArea)child, br);
                                break;
                        }
                        break;
                    case 2:
                        // laser
                        int ch2b = br.readBits(2, "ch2");
                        switch (ch2b) {
                            case 0:
                                child = new XanimateScroll();
                                decode((XanimateScroll) child, br);
                                break;
                            case 1:
                                child = new XsetScroll();
                                decode((XsetScroll) child, br);
                                break;
                            case 2:
                                child = new XstreamSource();
                                decode((XstreamSource) child, br);
                                break;
                            case 3:
                                child = new XupdateSource();
                                decode((XupdateSource) child, br);
                                break;
                        }
                        break;
                }
                v.addElement(child);
            }
            return v;
        }
        return null;
    }

    public void decode(Xanimation self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ animation");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        if (br.readBit("dur")) { self.dur = SDLClasses.class_time(br, "dur", true);}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("fill")) { self.fill = br.readBits(1, "fill");}
        if (br.readBit("height")) { self.height = SDLClasses.class_coordinate(br, "height");}
        if (br.readBit("preserveAspectRatio"))
        { self.preserveAspectRatio = SDLClasses.class_preserveAspectRatio(br, "preserveAspectRatio");}
        if (br.readBit("repeatCount")) { self.repeatCount = SDLClasses.repeatCountType(br, "repeatCount");}
        if (br.readBit("repeatDur")) { self.repeatDur = SDLClasses.repeatDurType(br, "repeatDur");}
        if (br.readBit("restart")) { self.restart = br.readBits(2, "restart");}
        if (br.readBit("syncBehavior")) { self.syncBehavior = br.readBits(2, "syncBehavior");}
        if (br.readBit("syncTolerance")) { self.syncTolerance = SDLClasses.syncTolerance(br, "syncTolerance");}
        if (br.readBit("type")) { self.type = SDLClasses.class_byteAlignedString(br, "type");}
        if (br.readBit("width")) { self.width = SDLClasses.class_coordinate(br, "width");}
        if (br.readBit("x")) { self.x = SDLClasses.class_coordinate(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.class_coordinate(br, "y");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        if (br.readBit("clipBegin")) { self.clipBegin = SDLClasses.class_time(br, "clipBegin", true);}
        if (br.readBit("clipEnd")) { self.clipEnd = SDLClasses.class_time(br, "clipEnd", true);}
        if (br.readBit("syncReference")) { self.syncReference = SDLClasses.class_anyURI(br, "syncReference");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv animation");
    }

    public void decode(Xdiscard self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ discard");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv discard");
    }

    public void decode(Xfont self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ font");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("horizAdvX")) { self.horizAdvX = SDLClasses.class_fixed_16_8(br, "horizAdvX");}
        if (br.readBit("horizOriginX")) { self.horizOriginX = SDLClasses.class_fixed_16_8(br, "horizOriginX");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv font");
    }

    public void decode(XfontFace self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ fontFace");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("accentHeight")) { self.accentHeight = SDLClasses.class_fixed_16_8(br, "accentHeight");}
        if (br.readBit("alphabetic")) { self.alphabetic = SDLClasses.class_fixed_16_8(br, "alphabetic");}
        if (br.readBit("ascent")) { self.ascent = SDLClasses.class_fixed_16_8(br, "ascent");}
        if (br.readBit("bbox")) { self.bbox = SDLClasses.class_byteAlignedString(br, "bbox");}
        if (br.readBit("capHeight")) { self.capHeight = SDLClasses.class_fixed_16_8(br, "capHeight");}
        if (br.readBit("descent")) { self.descent = SDLClasses.class_fixed_16_8(br, "descent");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("fontFamily")) { self.fontFamily = SDLClasses.class_byteAlignedString(br, "fontFamily");}
        if (br.readBit("fontStretch")) { self.fontStretch = SDLClasses.class_byteAlignedString(br, "fontStretch");}
        if (br.readBit("fontStyle")) { self.fontStyle = SDLClasses.class_byteAlignedString(br, "fontStyle");}
        if (br.readBit("fontVariant")) { self.fontVariant = SDLClasses.class_byteAlignedString(br, "fontVariant");}
        if (br.readBit("fontWeight")) { self.fontWeight = SDLClasses.class_byteAlignedString(br, "fontWeight");}
        if (br.readBit("hanging")) { self.hanging = SDLClasses.class_fixed_16_8(br, "hanging");}
        if (br.readBit("ideographic")) { self.ideographic = SDLClasses.class_fixed_16_8(br, "ideographic");}
        if (br.readBit("mathematical")) { self.mathematical = SDLClasses.class_fixed_16_8(br, "mathematical");}
        if (br.readBit("overlinePosition")) { self.overlinePosition = SDLClasses.class_fixed_16_8(br, "overlinePosition");}
        if (br.readBit("overlineThickness")) { self.overlineThickness = SDLClasses.class_fixed_16_8(br, "overlineThickness");}
        if (br.readBit("panose1")) { self.panose1 = SDLClasses.class_byteAlignedString(br, "panose1");}
        if (br.readBit("slope")) { self.slope = SDLClasses.class_fixed_16_8(br, "slope");}
        if (br.readBit("stemh")) { self.stemh = SDLClasses.class_fixed_16_8(br, "stemh");}
        if (br.readBit("stemv")) { self.stemv = SDLClasses.class_fixed_16_8(br, "stemv");}
        if (br.readBit("strikethroughPosition")) { self.strikethroughPosition = SDLClasses.class_fixed_16_8(br, "strikethroughPosition");}
        if (br.readBit("strikethroughThickness")) { self.strikethroughThickness = SDLClasses.class_fixed_16_8(br, "strikethroughThickness");}
        if (br.readBit("underlinePosition")) { self.underlinePosition = SDLClasses.class_fixed_16_8(br, "underlinePosition");}
        if (br.readBit("underlineThickness")) { self.underlineThickness = SDLClasses.class_fixed_16_8(br, "underlineThickness");}
        if (br.readBit("unicodeRange")) { self.unicodeRange = SDLClasses.class_byteAlignedString(br, "unicodeRange");}
        if (br.readBit("unitsPerEm")) { self.unitsPerEm = SDLClasses.class_fixed_16_8(br, "unitsPerEm");}
        if (br.readBit("widths")) { self.widths = SDLClasses.class_byteAlignedString(br, "widths");}
        if (br.readBit("xHeight")) { self.xHeight = SDLClasses.class_coordinate(br, "xHeight");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv fontFace");
    }

    public void decode(XfontFaceSrc self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ fontFaceSrc");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv fontFaceSrc");
    }

    public void decode(XfontFaceUri self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ fontFaceUri");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv fontFaceUri");
    }

    public void decode(Xglyph self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ glyph");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("arabicForm")) { self.arabicForm = SDLClasses.class_byteAlignedString(br, "arabicForm");}
        if (br.readBit("d")) { self.d = SDLClasses.class_path(br, "d");}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("glyphName")) { self.glyphName = SDLClasses.class_byteAlignedString(br, "glyphName");}
        if (br.readBit("horizAdvX")) { self.horizAdvX = SDLClasses.class_fixed_16_8(br, "horizAdvX");}
        if (br.readBit("lang")) { self.lang = SDLClasses.class_byteAlignedString(br, "lang");}
        if (br.readBit("unicode")) { self.unicode = SDLClasses.class_byteAlignedString(br, "unicode");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv glyph");
    }

    public void decode(Xhandler self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ handler");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        self.externalResourcesRequired = br.readBit("externalResourcesRequired");
        if (br.readBit("type")) { self.type = SDLClasses.class_scriptType(br, "type");}
        if (br.readBit("event")) { self.event = SDLClasses.class_event(br, "event");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv handler");
    }

    public void decode(Xhkern self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ hkern");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("g1")) { self.g1 = SDLClasses.class_byteAlignedString(br, "g1");}
        if (br.readBit("g2")) { self.g2 = SDLClasses.class_byteAlignedString(br, "g2");}
        if (br.readBit("k")) { self.k = SDLClasses.class_fixed_16_8(br, "k");}
        if (br.readBit("u1")) { self.u1 = SDLClasses.class_byteAlignedString(br, "u1");}
        if (br.readBit("u2")) { self.u2 = SDLClasses.class_byteAlignedString(br, "u2");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv hkern");
    }

    public void decode(XmissingGlyph self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ missingGlyph");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("d")) { self.d = SDLClasses.class_path(br, "d");}
        if (br.readBit("horizAdvX")) { self.horizAdvX = SDLClasses.class_fixed_16_8(br, "horizAdvX");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv missingGlyph");
    }

    public void decode(Xprefetch self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ prefetch");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("bandwidth")) { self.bandwidth = br.readVlcInt(5, "bandwidth");}
        if (br.readBit("mediaCharacterEncoding")) { self.mediaCharacterEncoding = SDLClasses.class_byteAlignedString(br, "mediaCharacterEncoding");}
        if (br.readBit("mediaContentEncodings")) { self.mediaContentEncodings = SDLClasses.class_byteAlignedString(br, "mediaContentEncodings");}
        if (br.readBit("mediaSize")) { self.mediaSize = br.readVlcInt(5, "mediaSize");}
        if (br.readBit("mediaTime")) { self.mediaTime = br.readVlcInt(5, "mediaTime");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv prefetch");
    }

    public void decode(XsolidColor self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ solidColor");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv solidColor");
    }

    public void decode(XtBreak self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ tBreak");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv tBreak");
    }

    public void decode(XtextArea self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ textArea");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("fill")) { self.fill = SDLClasses.class_paint(br, "fill");}
        if (br.readBit("stroke")) { self.stroke = SDLClasses.class_paint(br, "stroke");}
        if (br.readBit("editable")) { self.editable = br.readBits(1, "editable");}
        if (br.readBit("height")) { self.height = SDLClasses.class_coordinate(br, "height");}
        if (br.readBit("width")) { self.width = SDLClasses.class_coordinate(br, "width");}
        if (br.readBit("x")) { self.x = SDLClasses.class_coordinate(br, "x");}
        if (br.readBit("y")) { self.y = SDLClasses.class_coordinate(br, "y");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv textArea");
    }

    public void decode(XanimateScroll self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ animateScroll");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        if (br.readBit("by")) { self.by = SDLClasses.pointType(br, "by");}
        if (br.readBit("delayAtEnd")) { self.delayAtEnd = SDLClasses.class_time(br, "delayAtEnd", true);}
        if (br.readBit("delayAtStart")) { self.delayAtStart = SDLClasses.class_time(br, "delayAtStart", true);}
        if (br.readBit("direction")) { self.direction = br.readBits(4, "delayAtEnd");}
        if (br.readBit("dur")) { self.dur = SDLClasses.class_time(br, "dur", true);}
        if (br.readBit("fill")) { self.fill = br.readBits(1, "fill");}
        if (br.readBit("from")) { self.from = SDLClasses.class_scrollStop(br, "from");}
        if (br.readBit("repeatCount")) { self.repeatCount = SDLClasses.repeatCountType(br, "repeatCount");}
        if (br.readBit("repeatDur")) { self.repeatDur = SDLClasses.repeatDurType(br, "repeatDur");}
        if (br.readBit("restart")) { self.restart = br.readBits(2, "restart");}
        if (br.readBit("speed")) { self.speed = SDLClasses.class_time(br, "speed", true);}
        if (br.readBit("to")) { self.to = SDLClasses.class_scrollStop(br, "to");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv animateScroll");
    }

    public void decode(XsetScroll self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ scrollStop");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        if (br.readBit("direction")) { self.direction = br.readBits(4, "delayAtEnd");}
        if (br.readBit("increment")) { self.increment = SDLClasses.class_scrollStop(br, "increment");}
        if (br.readBit("to")) { self.to = SDLClasses.class_scrollStop(br, "to");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv scrollStop");
    }

    public void decode(XstreamSource self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ streamSource");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("height")) { self.height = SDLClasses.class_coordinate(br, "height");}
        if (br.readBit("mode")) { self.mode = br.readBits(3, "mode"); }
        if (br.readBit("sourceIndex")) { self.sourceIndex = br.readVlcInt(5, "sourceIndex");}
        int len = br.readVlcInt(5, "nbOfSources");
        self.source = new URI[len];
        for (int i = 0; i < len; i++) {
            self.source[i] = SDLClasses.class_anyURI(br, "source");
        }
        if (br.readBit("width")) { self.width = SDLClasses.class_coordinate(br, "width");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv streamSource");
    }

    public void decode(XupdateSource self, BitReaderD br) throws IOException {
        br.traceFile.println("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ updateSource");
        if (br.readBit("id")) { self.id = SDLClasses.class_ID(br, "id");}
        if (br.readBit("rare")) { decodeRare(self, br);}
        if (br.readBit("begin")) { self.begin = SDLClasses.timesType(br, "begin");}
        if (br.readBit("dur")) { self.dur = SDLClasses.class_time(br, "dur", true);}
        if (br.readBit("repeatCount")) { self.repeatCount = SDLClasses.repeatCountType(br, "repeatCount");}
        if (br.readBit("repeatDur")) { self.repeatDur = SDLClasses.repeatDurType(br, "repeatDur");}
        if (br.readBit("restart")) { self.restart = br.readBits(2, "restart");}
        if (br.readBit("syncBehavior")) { self.syncBehavior = br.readBits(2, "syncBehavior");}
        if (br.readBit("syncTolerance")) { self.syncTolerance = SDLClasses.syncTolerance(br, "syncTolerance");}
        if (br.readBit("href")) { self.setHref(SDLClasses.class_anyURI(br, "href"));}
        if (br.readBit("clipBegin")) { self.clipBegin = SDLClasses.class_time(br, "clipBegin", true);}
        if (br.readBit("clipEnd")) { self.clipEnd = SDLClasses.class_time(br, "clipEnd", true);}
        if (br.readBit("flow")) { self.flow = br.readBits(2, "flow");}
        if (br.readBit("syncReference")) { self.syncReference = SDLClasses.class_anyURI(br, "syncReference");}
        if (br.readBit("security")) { self.security = br.readBits(2, "security");}
        decodeContent(self, br);
        br.traceFile.println("vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv updateSource");
    }

}
