/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.io;

import iso.mpeg.streamezzo.laser.*;
import iso.mpeg.streamezzo.sdl.*;
import java.io.IOException;
import java.io.ByteArrayOutputStream;

public abstract class Encoder {
    public abstract void encode(XtextContent self, BitWriter bw) throws IOException;
    public abstract void encodeContent(X self, BitWriter bw) throws IOException;
    public abstract void encodeFOContent(XforeignObject self, BitWriter bw) throws IOException;
    public abstract void encodeSameContent(X self, BitWriter bw) throws IOException;
    public abstract void encodeInsertContent(Uc self, BitWriter bw) throws IOException;
    public abstract void encodeReplaceContent(XReplace self, BitWriter bw) throws IOException;
    public abstract void encodeSimpleUpdateContent(U self, BitWriter bw) throws IOException;
    public abstract void encode(XprivateElementContainer self, BitWriter bw) throws IOException;
    public abstract void encode(XprivateElement self, BitWriter bw) throws IOException;
    public void encode(XextendUpdate self, BitWriter bw) throws IOException {}

    public void encode(Xa self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        bw.putBit(self.externalResourcesRequired);
        b = (self.target != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.target); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        encodeContent(self, bw);
    }

    public void elements(BitWriter bw, Object o) throws IOException {
        if (o.getClass() == Xa.class) {
            bw.putBits(6, 0);
            encode((Xa)o, bw);
            return;
        }
        if (o.getClass() == Xanimate.class) {
            bw.putBits(6, 1);
            encode((Xanimate)o, bw);
            return;
        }
        if (o.getClass() == XanimateColor.class) {
            bw.putBits(6, 2);
            encode((XanimateColor)o, bw);
            return;
        }
        if (o.getClass() == XanimateMotion.class) {
            bw.putBits(6, 3);
            encode((XanimateMotion)o, bw);
            return;
        }
        if (o.getClass() == XanimateTransform.class) {
            bw.putBits(6, 4);
            encode((XanimateTransform)o, bw);
            return;
        }
        if (o.getClass() == Xaudio.class) {
            bw.putBits(6, 5);
            encode((Xaudio)o, bw);
            return;
        }
        if (o.getClass() == Xcircle.class) {
            bw.putBits(6, 6);
            encode((Xcircle)o, bw);
            return;
        }
        if (o.getClass() == Xdefs.class) {
            bw.putBits(6, 7);
            encode((Xdefs)o, bw);
            return;
        }
        if (o.getClass() == Xdesc.class) {
            bw.putBits(6, 8);
            encode((Xdesc)o, bw);
            return;
        }
        if (o.getClass() == Xellipse.class) {
            bw.putBits(6, 9);
            encode((Xellipse)o, bw);
            return;
        }
        if (o.getClass() == XforeignObject.class) {
            bw.putBits(6, 10);
            encode((XforeignObject)o, bw);
            return;
        }
        if (o.getClass() == Xg.class) {
            bw.putBits(6, 11);
            encode((Xg)o, bw);
            return;
        }
        if (o.getClass() == Ximage.class) {
            bw.putBits(6, 12);
            encode((Ximage)o, bw);
            return;
        }
        if (o.getClass() == Xline.class) {
            bw.putBits(6, 13);
            encode((Xline)o, bw);
            return;
        }
        if (o.getClass() == XlinearGradient.class) {
            bw.putBits(6, 14);
            encode((XlinearGradient)o, bw);
            return;
        }
        if (o.getClass() == Xmetadata.class) {
            bw.putBits(6, 15);
            encode((Xmetadata)o, bw);
            return;
        }
        if (o.getClass() == Xmpath.class) {
            bw.putBits(6, 16);
            encode((Xmpath)o, bw);
            return;
        }
        if (o.getClass() == Xpath.class) {
            bw.putBits(6, 17);
            encode((Xpath)o, bw);
            return;
        }
        if (o.getClass() == Xpolygon.class) {
            bw.putBits(6, 18);
            encode((Xpolygon)o, bw);
            return;
        }
        if (o.getClass() == Xpolyline.class) {
            bw.putBits(6, 19);
            encode((Xpolyline)o, bw);
            return;
        }
        if (o.getClass() == XradialGradient.class) {
            bw.putBits(6, 20);
            encode((XradialGradient)o, bw);
            return;
        }
        if (o.getClass() == Xrect.class) {
            bw.putBits(6, 21);
            encode((Xrect)o, bw);
            return;
        }
        if (o.getClass() == Xsameg.class) {
            bw.putBits(6, 22);
            encode((Xsameg)o, bw);
            return;
        }
        if (o.getClass() == Xsameline.class) {
            bw.putBits(6, 23);
            encode((Xsameline)o, bw);
            return;
        }
        if (o.getClass() == Xsamepath.class) {
            bw.putBits(6, 24);
            encode((Xsamepath)o, bw);
            return;
        }
        if (o.getClass() == Xsamepathfill.class) {
            bw.putBits(6, 25);
            encode((Xsamepathfill)o, bw);
            return;
        }
        if (o.getClass() == Xsamepolygon.class) {
            bw.putBits(6, 26);
            encode((Xsamepolygon)o, bw);
            return;
        }
        if (o.getClass() == Xsamepolygonfill.class) {
            bw.putBits(6, 27);
            encode((Xsamepolygonfill)o, bw);
            return;
        }
        if (o.getClass() == Xsamepolygonstroke.class) {
            bw.putBits(6, 28);
            encode((Xsamepolygonstroke)o, bw);
            return;
        }
        if (o.getClass() == Xsamepolyline.class) {
            bw.putBits(6, 29);
            encode((Xsamepolyline)o, bw);
            return;
        }
        if (o.getClass() == Xsamepolylinefill.class) {
            bw.putBits(6, 30);
            encode((Xsamepolylinefill)o, bw);
            return;
        }
        if (o.getClass() == Xsamepolylinestroke.class) {
            bw.putBits(6, 31);
            encode((Xsamepolylinestroke)o, bw);
            return;
        }
        if (o.getClass() == Xsamerect.class) {
            bw.putBits(6, 32);
            encode((Xsamerect)o, bw);
            return;
        }
        if (o.getClass() == Xsamerectfill.class) {
            bw.putBits(6, 33);
            encode((Xsamerectfill)o, bw);
            return;
        }
        if (o.getClass() == Xsametext.class) {
            bw.putBits(6, 34);
            encode((Xsametext)o, bw);
            return;
        }
        if (o.getClass() == Xsametextfill.class) {
            bw.putBits(6, 35);
            encode((Xsametextfill)o, bw);
            return;
        }
        if (o.getClass() == Xsameuse.class) {
            bw.putBits(6, 36);
            encode((Xsameuse)o, bw);
            return;
        }
        if (o.getClass() == Xscript.class) {
            bw.putBits(6, 37);
            encode((Xscript)o, bw);
            return;
        }
        if (o.getClass() == Xset.class) {
            bw.putBits(6, 38);
            encode((Xset)o, bw);
            return;
        }
        if (o.getClass() == Xstop.class) {
            bw.putBits(6, 39);
            encode((Xstop)o, bw);
            return;
        }
        if (o.getClass() == Xswitch.class) {
            bw.putBits(6, 40);
            encode((Xswitch)o, bw);
            return;
        }
        if (o.getClass() == Xtext.class) {
            bw.putBits(6, 41);
            encode((Xtext)o, bw);
            return;
        }
        if (o.getClass() == Xtitle.class) {
            bw.putBits(6, 42);
            encode((Xtitle)o, bw);
            return;
        }
        if (o.getClass() == Xtspan.class) {
            bw.putBits(6, 43);
            encode((Xtspan)o, bw);
            return;
        }
        if (o.getClass() == Xuse.class) {
            bw.putBits(6, 44);
            encode((Xuse)o, bw);
            return;
        }
        if (o.getClass() == Xvideo.class) {
            bw.putBits(6, 45);
            encode((Xvideo)o, bw);
            return;
        }
        if (o.getClass() == Xlistener.class) {
            bw.putBits(6, 46);
            encode((Xlistener)o, bw);
            return;
        }
        if (o.getClass() == XprivateElementContainer.class) {
            bw.putBits(6, 50);
            encode((XprivateElementContainer)o, bw);
            return;
        }
        if (o.getClass() == XprivateElement.class) {
            bw.putBits(6, 50);
            encode((XprivateElement)o, bw);
            return;
        }
        if (o.getClass() == XtextContent.class) {
            bw.putBits(6, 54);
            encode((XtextContent)o, bw);
            return;
        }
        if (o.getClass() == Xconditional.class) {
            bw.putBits(6, 47);
            encode((Xconditional)o, bw);
            return;
        }
        if (o.getClass() == XcursorManager.class) {
            bw.putBits(6, 48);
            encode((XcursorManager)o, bw);
            return;
        }
        if (o.getClass() == XrectClip.class) {
            bw.putBits(6, 51);
            encode((XrectClip)o, bw);
            return;
        }
        if (o.getClass() == Xselector.class) {
            bw.putBits(6, 52);
            encode((Xselector)o, bw);
            return;
        }
        if (o.getClass() == XsimpleLayout.class) {
            bw.putBits(6, 53);
            encode((XsimpleLayout)o, bw);
            return;
        }
        if (o.getClass() == Xanimation.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for animation, estimate len < 4096 bits, so additional offset 15 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 7 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 0);
            encode((Xanimation)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 3);
            return;
        }
        if (o.getClass() == Xdiscard.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for discard, estimate len < 256 bits, so additional offset 10 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 7 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 1);
            encode((Xdiscard)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 3);
            return;
        }
        if (o.getClass() == Xfont.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for font, estimate len < 65536 bits, so additional offset 25 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 1 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 2);
            encode((Xfont)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 5);
            return;
        }
        if (o.getClass() == XfontFace.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for fontFace, estimate len < 256 bits, so additional offset 10 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 7 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 3);
            encode((XfontFace)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 3);
            return;
        }
        if (o.getClass() == XfontFaceSrc.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for fontFaceSrc, estimate len < 256 bits, so additional offset 10 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 2 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 4);
            encode((XfontFaceSrc)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 2);
            return;
        }
        if (o.getClass() == XfontFaceUri.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for fontFaceUri, estimate len < 4096 bits, so additional offset 15 bits
           BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 7 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 5);
            encode((XfontFaceUri)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 3);
            return;
        }
        if (o.getClass() == Xglyph.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for glyph, estimate len < 65536 bits, so additional offset 25 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 1 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 6);
            encode((Xglyph)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 5);
            return;
        }
        if (o.getClass() == Xhandler.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for handler, estimate len < 65536 bits, so additional offset 25 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 1 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 7);
            encode((Xhandler)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 5);
            return;
        }
        if (o.getClass() == Xhkern.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for hkern, estimate len < 256 bits, so additional offset 10 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 2 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 8);
            encode((Xhkern)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 2);
            return;
        }
        if (o.getClass() == XmissingGlyph.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for font, estimate len < 65536 bits, so additional offset 25 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 1 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 9);
            encode((XmissingGlyph)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 5);
            return;
        }
        if (o.getClass() == Xprefetch.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for prefetch, estimate len < 4096 bits, so additional offset 15 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 7 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 10);
            encode((Xprefetch)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 3);
            return;
        }
        if (o.getClass() == XsolidColor.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for solidColor, estimate len < 256 bits, so additional offset 10 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 2 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 11);
            encode((XsolidColor)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 2);
            return;
        }
        if (o.getClass() == XtBreak.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for tBreak, estimate len < 16 bits, so additional offset 5 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 2 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 12);
            encode((XtBreak)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 2);
            return;
        }
        if (o.getClass() == XtextArea.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for textArea, estimate len < 65536 bits, so additional offset 25 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 1 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 0);
            bw1.putBits(4, 13);
            encode((XtextArea)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 5);
            return;
        }
        if (o.getClass() == XanimateScroll.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for animateScroll, estimate len < 256 bits, so additional offset 10 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 7 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 2);
            bw1.putBits(2, 0);
            encode((XanimateScroll)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 3);
            return;
        }
        if (o.getClass() == XsetScroll.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for scrollStop, estimate len < 256 bits, so additional offset 10 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 7 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 2);
            bw1.putBits(2, 1);
            encode((XsetScroll)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 3);
            return;
        }
        if (o.getClass() == XstreamSource.class) {
            bw.putBits(6, 49);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for streamSource, estimate len < 65536 bits, so additional offset 25 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 1 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 2);
            bw1.putBits(2, 2);
            encode((XstreamSource)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 5);
            return;
        }
        throw new RuntimeException("trying to encode as element a "+o);
    }

    public void encode(Xanimate self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.attributeName != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AttributeName(bw, self.attributeName); }
        b = (self.accumulate != false);
        bw.putBit(b);
        if (b) { bw.putBit(self.accumulate); }
        b = (self.additive != false);
        bw.putBit(b);
        if (b) { bw.putBit(self.additive); }
        b = (self.by != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValue(bw, self.by); }
        b = (self.calcMode !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.calcMode); }
        b = (self.from != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValue(bw, self.from); }
        b = (self.keySplines != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_fraction12List(bw, self.keySplines); }
        b = (self.keyTimes != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_fraction12List(bw, self.keyTimes); }
        b = (self.values != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValues(bw, self.values); }
        b = self.attributeType != Integer.MIN_VALUE;
        bw.putBit(b);
        if (b) { bw.putBits(2, self.attributeType); }
        b = (self.begin != null);
        bw.putBit(b);
        if (b) { SDLClasses.timesType(bw, self.begin); }
        b = (self.dur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.dur, true); }
        b = (self.fill !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.fill); }
        b = (self.repeatCount != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatCountType(bw, self.repeatCount); }
        b = (self.repeatDur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatDurType(bw, self.repeatDur); }
        b = (self.restart !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.restart); }
        b = (self.to != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValue(bw, self.to); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        bw.putBit(self.enabled);
        encodeContent(self, bw);
    }

    public void encode(XanimateMotion self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.accumulate != false);
        bw.putBit(b);
        if (b) { bw.putBit(self.accumulate); }
        b = (self.additive != false);
        bw.putBit(b);
        if (b) { bw.putBit(self.additive); }
        b = (self.by != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValue(bw, self.by); }
        b = (self.calcMode !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.calcMode); }
        b = (self.from != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValue(bw, self.from); }
        b = (self.keySplines != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_fraction12List(bw, self.keySplines); }
        b = (self.keyTimes != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_fraction12List(bw, self.keyTimes); }
        b = (self.values != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValues(bw, self.values); }
        b = self.attributeType != Integer.MIN_VALUE;
        bw.putBit(b);
        if (b) { bw.putBits(2, self.attributeType); }
        b = (self.begin != null);
        bw.putBit(b);
        if (b) { SDLClasses.timesType(bw, self.begin); }
        b = (self.dur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.dur, true); }
        b = (self.fill !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.fill); }
        b = (self.repeatCount != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatCountType(bw, self.repeatCount); }
        b = (self.repeatDur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatDurType(bw, self.repeatDur); }
        b = (self.restart !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.restart); }
        b = (self.to != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValue(bw, self.to); }
        b = (self.keyPoints != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_floatList(bw, self.keyPoints); }
        b = (self.path != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_path(bw, self.path); }
        b = (self.rotate != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.rotateType(bw, self.rotate); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        bw.putBit(self.enabled);
        encodeContent(self, bw);
    }

    public void encode(XanimateTransform self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.attributeName != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AttributeName(bw, self.attributeName); }
        bw.putBits(3, self.type);
        b = (self.accumulate != false);
        bw.putBit(b);
        if (b) { bw.putBit(self.accumulate); }
        b = (self.additive != false);
        bw.putBit(b);
        if (b) { bw.putBit(self.additive); }
        b = (self.by != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValue(bw, self.by); }
        b = (self.calcMode !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.calcMode); }
        b = (self.from != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValue(bw, self.from); }
        b = (self.keySplines != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_fraction12List(bw, self.keySplines); }
        b = (self.keyTimes != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_fraction12List(bw, self.keyTimes); }
        b = (self.values != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValues(bw, self.values); }
        b = self.attributeType != Integer.MIN_VALUE;
        bw.putBit(b);
        if (b) { bw.putBits(2, self.attributeType); }
        b = (self.begin != null);
        bw.putBit(b);
        if (b) { SDLClasses.timesType(bw, self.begin); }
        b = (self.dur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.dur, true); }
        b = (self.fill !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.fill); }
        b = (self.repeatCount != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatCountType(bw, self.repeatCount); }
        b = (self.repeatDur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatDurType(bw, self.repeatDur); }
        b = (self.restart !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.restart); }
        b = (self.to != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValue(bw, self.to); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        bw.putBit(self.enabled);
        encodeContent(self, bw);
    }

    public void encode(Xaudio self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.begin != null);
        bw.putBit(b);
        if (b) { SDLClasses.timesType(bw, self.begin); }
        b = (self.dur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.dur, true); }
        bw.putBit(self.externalResourcesRequired);
        b = (self.repeatCount != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatCountType(bw, self.repeatCount); }
        b = (self.repeatDur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatDurType(bw, self.repeatDur); }
        b = (self.restart !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.restart); }
        b = (self.syncBehavior !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.syncBehavior); }
        b = (self.syncTolerance != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.syncTolerance(bw, self.syncTolerance); }
        b = (self.type != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.type); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        b = (self.clipBegin != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.clipBegin, true); }
        b = (self.clipEnd != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.clipEnd, true); }
        b = (self.syncReference != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.syncReference); }
        encodeContent(self, bw);
    }

    public void encode(Xcircle self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        b = (self.cx != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.cx); }
        b = (self.cy != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.cy); }
        SDLClasses.class_coordinate(bw, self.r);
        encodeContent(self, bw);
    }

    public void encode(XcursorManager self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.x != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x); }
        b = (self.y != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        encodeContent(self, bw);
    }

    public void encode(Xdefs self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        encodeContent(self, bw);
    }

    public void encode(Xdesc_metaData_title self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        encodeContent(self, bw);
    }

    public void encode(Xellipse self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        b = (self.cx != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.cx); }
        b = (self.cy != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.cy); }
        SDLClasses.class_coordinate(bw, self.rx);
        SDLClasses.class_coordinate(bw, self.ry);
        encodeContent(self, bw);
    }

    public void encode(XforeignObject self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        bw.putBit(self.externalResourcesRequired);
        SDLClasses.class_coordinate(bw, self.height);
        SDLClasses.class_coordinate(bw, self.width);
        b = (self.x != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x); }
        b = (self.y != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y); }
        encodeFOContent(self, bw);
    }

    public void encode(Xg self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        bw.putBit(self.externalResourcesRequired);
        encodeContent(self, bw);
    }

    public void encode(XsimpleLayout self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        b = (self.delta != null);
        bw.putBit(b);
        if (b) { SDLClasses.pointType(bw, self.delta); }
        bw.putBit(self.externalResourcesRequired);
        encodeContent(self, bw);
    }

    public void encode(Xselector self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        bw.putBit(self.externalResourcesRequired);
        b = (self.choice != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.choiceType(bw, self.choice); }
        encodeContent(self, bw);
    }

    public void encode(XrectClip self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        bw.putBit(self.externalResourcesRequired);
        b = (self.size != null);
        bw.putBit(b);
        if (b) { SDLClasses.pointType(bw, self.size); }
        encodeContent(self, bw);
    }

    public void encode(Ximage self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        bw.putBit(self.externalResourcesRequired);
        b = (self.height != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.height); }
        b = (self.opacity != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_0to1float(bw, self.opacity); }
        b = (self.preserveAspectRatio != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_preserveAspectRatio(bw, self.preserveAspectRatio); }
        b = (self.type != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.type); }
        b = (self.width != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.width); }
        b = (self.x != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x); }
        b = (self.y != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        b = (self.transformBehavior !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(4, self.transformBehavior); }
        encodeContent(self, bw);
    }

    public void encode(Xline self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        b = (self.x1 != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x1); }
        SDLClasses.class_coordinate(bw, self.x2);
        b = (self.y1 != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y1); }
        SDLClasses.class_coordinate(bw, self.y2);
        encodeContent(self, bw);
    }

    public void encode(XlinearGradient self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        b = (self.gradientUnits !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.gradientUnits); }
        b = (self.x1 != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x1); }
        b = (self.x2 != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x2); }
        b = (self.y1 != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y1); }
        b = (self.y2 != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y2); }
        encodeContent(self, bw);
    }

    public void encode(Xmpath self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        encodeContent(self, bw);
    }

    public void encode(Xpath self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { 
            SDLClasses.class_ID(bw, self.id);
        }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { 
            SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { 
            SDLClasses.class_paint(bw, self.stroke); }
        SDLClasses.class_path(bw, self.d);
        b = (self.pathLength != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { 
            SDLClasses.class_fixed_16_8(bw, self.pathLength); }
        encodeContent(self, bw);
    }

    public void encode(Xpolygon self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        SDLClasses.class_pointSequence(bw, self.points);
        encodeContent(self, bw);
    }

    public void encode(XradialGradient self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        b = (self.cx != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.cx); }
        b = (self.cy != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.cy); }
        b = (self.gradientUnits !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.gradientUnits); }
        b = (self.r != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.r); }
        encodeContent(self, bw);
    }

    public void encode(Xrect self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        SDLClasses.class_coordinate(bw, self.height);
        b = (self.rx != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.rx); }
        b = (self.ry != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.ry); }
        SDLClasses.class_coordinate(bw, self.width);
        b = (self.x != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x); }
        b = (self.y != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y); }
        encodeContent(self, bw);
    }

    public void encode(Xsameg self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        encodeSameContent(self, bw);
    }

    public void encode(Xsameline self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        b = (self.x1 != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x1); }
        SDLClasses.class_coordinate(bw, self.x2);
        b = (self.y1 != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y1); }
        SDLClasses.class_coordinate(bw, self.y2);
        encodeSameContent(self, bw);
    }

    public void encode(Xsamepath self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_path(bw, self.d);
        encodeSameContent(self, bw);
    }

    public void encode(Xsamepathfill self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        SDLClasses.class_path(bw, self.d);
        encodeSameContent(self, bw);
    }

    public void encode(Xsamepolygon self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_pointSequence(bw, self.points);
        encodeSameContent(self, bw);
    }

    public void encode(Xsamepolygonfill self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        SDLClasses.class_pointSequence(bw, self.points);
        encodeSameContent(self, bw);
    }

    public void encode(Xsamepolygonstroke self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        SDLClasses.class_pointSequence(bw, self.points);
        encodeSameContent(self, bw);
    }

    public void encode(Xsamerect self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_coordinate(bw, self.height);
        SDLClasses.class_coordinate(bw, self.width);
        b = (self.x != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x); }
        b = (self.y != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y); }
        encodeSameContent(self, bw);
    }

    public void encode(Xsamerectfill self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        SDLClasses.class_coordinate(bw, self.height);
        SDLClasses.class_coordinate(bw, self.width);
        b = (self.x != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x); }
        b = (self.y != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y); }
        encodeSameContent(self, bw);
    }

    public void encode(Xsametext self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        b = (self.x != null);
        bw.putBit(b);
        if (b) { SDLClasses.coordinateListType(bw, self.x); }
        b = (self.y != null);
        bw.putBit(b);
        if (b) { SDLClasses.coordinateListType(bw, self.y); }
        encodeSameContent(self, bw);
    }

    public void encode(Xsametextfill self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.x != null);
        bw.putBit(b);
        if (b) { SDLClasses.coordinateListType(bw, self.x); }
        b = (self.y != null);
        bw.putBit(b);
        if (b) { SDLClasses.coordinateListType(bw, self.y); }
        encodeSameContent(self, bw);
    }

    public void encode(Xsameuse self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        encodeSameContent(self, bw);
    }

    public void encode(Xscript self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        bw.putBit(self.externalResourcesRequired);
        b = (self.type != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_script(bw, self.type); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        encodeContent(self, bw);
    }

    public void encode(Xconditional self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.begin != null);
        bw.putBit(b);
        if (b) { SDLClasses.timesType(bw, self.begin); }
        bw.putBit(self.externalResourcesRequired);
        bw.putBit(self.enabled);
        bw.putBit(false);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        BitWriter bw1 = new BitWriter(baos);
        bw1.putVlcInt(5, self.child.size());
        for (int t = 0; t < self.child.size(); t++) {
            updates(bw1, self.child.elementAt(t));
        }
        bw1.flush();
        bw.putVlcInt(5, baos.size());
        bw.flush();
        bw.putByteArray(baos.toByteArray());
        b = self.privateAttributes != null;
        bw.putBit(b);
        if(b) {
            SDLClasses.privateAttributeContainer(bw, self.privateAttributes);
        }
    }

    public void updates(BitWriter bw, Object o) throws IOException {
        if (o.getClass() == XAdd.class) {
            bw.putBits(4, 0);
            encode((XAdd)o, bw);
            return;
        }
        if (o.getClass() == XClean.class) {
            bw.putBits(4, 1);
            encode((XClean)o, bw);
            return;
        }
        if (o.getClass() == XDelete.class) {
            bw.putBits(4, 2);
            encode((XDelete)o, bw);
            return;
        }
        if (o.getClass() == XInsert.class) {
            bw.putBits(4, 3);
            encode((XInsert)o, bw);
            return;
        }
        if (o.getClass() == XNewScene.class) {
            bw.putBits(4, 4);
            encode((XNewScene)o, bw);
            return;
        }
        if (o.getClass() == XRefreshScene.class) {
            bw.putBits(4, 5);
            encode((XRefreshScene)o, bw);
            return;
        }
        if (o.getClass() == XReplace.class) {
            bw.putBits(4, 6);
            encode((XReplace)o, bw);
            return;
        }
        if (o.getClass() == XRestore.class) {
            bw.putBits(4, 7);
            encode((XRestore)o, bw);
            return;
        }
        if (o.getClass() == XSave.class) {
            bw.putBits(4, 8);
            encode((XSave)o, bw);
            return;
        }
        if (o.getClass() == XSendEvent.class) {
            bw.putBits(4, 9);
            encode((XSendEvent)o, bw);
            return;
        }
        if (o.getClass() == XtextContent.class) {
            bw.putBits(4, 11);
            encode((XtextContent)o, bw);
            return;
        }
        if (o.getClass() == XActivate.class) {
            bw.putBits(4, 10);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for discard, estimate len < 256 bits, so additional offset 2 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 2 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 1);
            bw1.putBits(2, 1);
            encode((XActivate)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 2);
            return;
        }
        if (o.getClass() == XDeactivate.class) {
            bw.putBits(4, 10);
            bw.putBits(4, 2); // extensionID of v2
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // because of byte alignment within the bitarray, we have to estimate the length
            // of the bitarray and take it into account for the starting bitoffset
            // for discard, estimate len < 256 bits, so additional offset 2 bits
            BitWriter bw1 = new BitWriter(baos);
            int bitOffset = 2 + (bw.bitlength()%8);
            bw1.putBits(bitOffset, 0);
            bw1.putVlcInt(5, 87);
            bw1.putVlcInt(5, 1);
            bw1.putBits(2, 1);
            bw1.putBits(2, 2);
            encode((XDeactivate)o, bw1);
            int len = bw1.bitlength()-bitOffset;
            bw1.flush();
            bw.putBitArray(baos.toByteArray(), len, bitOffset, 2);
            return;
        }
    }

    public void encode(XAdd self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.attributeName != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AttributeName(bw, self.attributeName); }
        b = (self.operandAttribute != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AttributeName(bw, self.operandAttribute); }
        b = (self.operandElementId != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_IDREF(bw, self.operandElementId); }
        SDLClasses.class_IDREF(bw, self.ref);
        b = (self.value != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_updateValue(bw, self.value); }
        encodeSimpleUpdateContent(self, bw);
    }

    public void encode(XClean self, BitWriter bw) throws IOException {
        boolean b = false;
        SDLClasses.class_byteAlignedString(bw, self.groupID);
        encodeSimpleUpdateContent(self, bw);
    }

    public void encode(XActivate self, BitWriter bw) throws IOException {
        boolean b = false;
        SDLClasses.class_IDREF(bw, self.ref);
        encodeSimpleUpdateContent(self, bw);
    }

    public void encode(XDeactivate self, BitWriter bw) throws IOException {
        boolean b = false;
        SDLClasses.class_IDREF(bw, self.ref);
        encodeSimpleUpdateContent(self, bw);
    }

    public void encode(XDelete self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.attributeName !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(8, self.attributeName); }
        b = (self.index != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putVlcInt(5, self.index); }
        SDLClasses.class_IDREF(bw, self.ref);
        encodeSimpleUpdateContent(self, bw);
    }

    public void encode(XInsert self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.attributeName != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AttributeName(bw, self.attributeName); }
        b = (self.index != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putVlcInt(5, self.index); }
        SDLClasses.class_IDREF(bw, self.ref);
        b = (self.value != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_updateValue(bw, self.value); }
        encodeInsertContent(self, bw);
    }

    public void updatable_elements(BitWriter bw, Object o) throws IOException {
        if (o.getClass() == Xa.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 0);
            encode((Xa)o, bw);
            return;
        }
        if (o.getClass() == Xanimate.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 1);
            encode((Xanimate)o, bw);
            return;
        }
        if (o.getClass() == XanimateColor.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 2);
            encode((XanimateColor)o, bw);
            return;
        }
        if (o.getClass() == XanimateMotion.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 3);
            encode((XanimateMotion)o, bw);
            return;
        }
        if (o.getClass() == XanimateTransform.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 4);
            encode((XanimateTransform)o, bw);
            return;
        }
        if (o.getClass() == Xaudio.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 5);
            encode((Xaudio)o, bw);
            return;
        }
        if (o.getClass() == Xcircle.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 6);
            encode((Xcircle)o, bw);
            return;
        }
        if (o.getClass() == Xdefs.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 7);
            encode((Xdefs)o, bw);
            return;
        }
        if (o.getClass() == Xdesc.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 8);
            encode((Xdesc)o, bw);
            return;
        }
        if (o.getClass() == Xellipse.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 9);
            encode((Xellipse)o, bw);
            return;
        }
        if (o.getClass() == XforeignObject.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 10);
            encode((XforeignObject)o, bw);
            return;
        }
        if (o.getClass() == Xg.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 11);
            encode((Xg)o, bw);
            return;
        }
        if (o.getClass() == Ximage.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 12);
            encode((Ximage)o, bw);
            return;
        }
        if (o.getClass() == Xline.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 13);
            encode((Xline)o, bw);
            return;
        }
        if (o.getClass() == XlinearGradient.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 14);
            encode((XlinearGradient)o, bw);
            return;
        }
        if (o.getClass() == Xmetadata.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 15);
            encode((Xmetadata)o, bw);
            return;
        }
        if (o.getClass() == Xmpath.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 16);
            encode((Xmpath)o, bw);
            return;
        }
        if (o.getClass() == Xpath.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 17);
            encode((Xpath)o, bw);
            return;
        }
        if (o.getClass() == Xpolygon.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 18);
            encode((Xpolygon)o, bw);
            return;
        }
        if (o.getClass() == Xpolyline.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 19);
            encode((Xpolyline)o, bw);
            return;
        }
        if (o.getClass() == XradialGradient.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 20);
            encode((XradialGradient)o, bw);
            return;
        }
        if (o.getClass() == Xrect.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 21);
            encode((Xrect)o, bw);
            return;
        }
        if (o.getClass() == Xscript.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 22);
            encode((Xscript)o, bw);
            return;
        }
        if (o.getClass() == Xset.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 23);
            encode((Xset)o, bw);
            return;
        }
        if (o.getClass() == Xstop.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 24);
            encode((Xstop)o, bw);
            return;
        }
        if (o.getClass() == Xsvg.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 25);
            encode((Xsvg)o, bw);
            return;
        }
        if (o.getClass() == Xswitch.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 26);
            encode((Xswitch)o, bw);
            return;
        }
        if (o.getClass() == Xtext.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 27);
            encode((Xtext)o, bw);
            return;
        }
        if (o.getClass() == Xtitle.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 28);
            encode((Xtitle)o, bw);
            return;
        }
        if (o.getClass() == Xtspan.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 29);
            encode((Xtspan)o, bw);
            return;
        }
        if (o.getClass() == Xuse.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 30);
            encode((Xuse)o, bw);
            return;
        }
        if (o.getClass() == Xvideo.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 31);
            encode((Xvideo)o, bw);
            return;
        }
        if (o.getClass() == Xlistener.class) {
            bw.putBits(1, 0);
            bw.putBits(6, 32);
            encode((Xlistener)o, bw);
            return;
        }
        if (o.getClass() == Xconditional.class) {
            bw.putBits(1, 1);
            bw.putBits(3, 0);
            encode((Xconditional)o, bw);
            return;
        }
        if (o.getClass() == XcursorManager.class) {
            bw.putBits(1, 1);
            bw.putBits(3, 1);
            encode((XcursorManager)o, bw);
            return;
        }
        if (o.getClass() == XprivateElement.class) {
            bw.putBits(1, 1);
            bw.putBits(3, 3);
            SDLClasses.privateElement(bw, (XprivateElement)o, this);
            return;
        }
        if (o.getClass() == XrectClip.class) {
            bw.putBits(1, 1);
            bw.putBits(3, 4);
            encode((XrectClip)o, bw);
            return;
        }
        if (o.getClass() == Xselector.class) {
            bw.putBits(1, 1);
            bw.putBits(3, 5);
            encode((Xselector)o, bw);
            return;
        }
        if (o.getClass() == XsimpleLayout.class) {
            bw.putBits(1, 1);
            bw.putBits(3, 6);
            encode((XsimpleLayout)o, bw);
            return;
        }
        throw new RuntimeException("trying to encode as updatable element a "+o);
    }

    public void encode(Xset self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.attributeName != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AttributeName(bw, self.attributeName); }
        b = self.attributeType != Integer.MIN_VALUE;
        bw.putBit(b);
        if (b) { bw.putBits(2, self.attributeType); }
        b = (self.begin != null);
        bw.putBit(b);
        if (b) { SDLClasses.timesType(bw, self.begin); }
        b = (self.dur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.dur, true); }
        b = (self.fill !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.fill); }
        b = (self.repeatCount != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatCountType(bw, self.repeatCount); }
        b = (self.repeatDur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatDurType(bw, self.repeatDur); }
        b = (self.restart !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.restart); }
        b = (self.to != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AnimatedValue(bw, self.to); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        bw.putBit(self.enabled);
        encodeContent(self, bw);
    }

    public void encode(Xstop self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        SDLClasses.class_fixed_16_8(bw, self.offset);
        encodeContent(self, bw);
    }

    public void encode(Xsvg self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        b = (self.baseProfile != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.baseProfile); }
        b = (self.contentScriptType != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.contentScriptType); }
        bw.putBit(self.externalResourcesRequired);
        SDLClasses.class_valueWithUnits(bw, self.height);
        b = (self.playbackOrder !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.playbackOrder); }
        b = (self.preserveAspectRatio != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_preserveAspectRatio(bw, self.preserveAspectRatio); }
        b = (self.snapshotTime != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.snapshotTime, true); }
        b = (self.syncBehaviorDefault !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.syncBehaviorDefault); }
        b = (self.syncToleranceDefault != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.syncTolerance(bw, self.syncToleranceDefault); }
        b = (self.timelineBegin !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.timelineBegin); }
        b = (self.version != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.version); }
        b = (self.viewBox != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_floatListNoLength(bw, self.viewBox); }
        SDLClasses.class_valueWithUnits(bw, self.width);
        b = (self.zoomAndPan !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.zoomAndPan); }
        encodeContent(self, bw);
    }

    public void encode(Xswitch self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        bw.putBit(self.externalResourcesRequired);
        encodeContent(self, bw);
    }

    public void encode(Xtext self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        bw.putBit(self.editable);
        b = (self.rotate != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_floatList(bw, self.rotate); }
        b = (self.x != null);
        bw.putBit(b);
        if (b) { SDLClasses.coordinateListType(bw, self.x); }
        b = (self.y != null);
        bw.putBit(b);
        if (b) { SDLClasses.coordinateListType(bw, self.y); }
        encodeContent(self, bw);
    }

    public void encode(Xtspan self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        encodeContent(self, bw);
    }

    public void encode(Xuse self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        bw.putBit(self.externalResourcesRequired);
        b = (self.overflow !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.overflow); }
        b = (self.x != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x); }
        b = (self.y != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        encodeContent(self, bw);
    }

    public void encode(Xvideo self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.begin != null);
        bw.putBit(b);
        if (b) { SDLClasses.timesType(bw, self.begin); }
        b = (self.dur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.dur, true); }
        bw.putBit(self.externalResourcesRequired);
        b = (self.height != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.height); }
        b = (self.overlay != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_overlay(bw, self.overlay); }
        b = (self.preserveAspectRatio != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_preserveAspectRatio(bw, self.preserveAspectRatio); }
        b = (self.repeatCount != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatCountType(bw, self.repeatCount); }
        b = (self.repeatDur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatDurType(bw, self.repeatDur); }
        b = (self.restart !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.restart); }
        b = (self.syncBehavior !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.syncBehavior); }
        b = (self.syncTolerance != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.syncTolerance(bw, self.syncTolerance); }
        b = (self.transformBehavior !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(4, self.transformBehavior); }
        b = (self.type != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.type); }
        b = (self.width != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.width); }
        b = (self.x != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x); }
        b = (self.y != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        b = (self.clipBegin != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.clipBegin, true); }
        b = (self.clipEnd != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.clipEnd, true); }
        b = (self.syncReference != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.syncReference); }
        encodeContent(self, bw);
    }

    public void encode(Xlistener self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.defaultAction !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.defaultAction); }
        b = (self.event != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_event(bw, self.event); }
        b = (self.handler != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.handler); }
        b = (self.observer != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_IDREF(bw, self.observer); }
        bw.putBits(1, self.phase);
        b = (self.propagate !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.propagate); }
        b = (self.target != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_IDREF(bw, self.target); }
        bw.putBit(self.enabled);
        /*b = (self.delay != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.delay, true); }
        b = (self.timeAttribute !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.timeAttribute); }*/
        encodeContent(self, bw);
    }

    public void encode(XNewScene self, BitWriter bw) throws IOException {
        boolean b = false;
        encodeSimpleUpdateContent(self, bw);
        encode((Xsvg)self.child, bw);
    }

    public void encode(XRefreshScene self, BitWriter bw) throws IOException {
        boolean b = false;
        bw.putVlcInt(5, self.time);
        encodeSimpleUpdateContent(self, bw);
        encode((Xsvg)self.child, bw);
    }

    public void encode(XReplace self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.attributeName != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_AttributeName(bw, self.attributeName); }
        b = (self.index != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putVlcInt(5, self.index); }
        b = (self.operandAttribute !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(8, self.operandAttribute); }
        b = (self.operandElementId != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_IDREF(bw, self.operandElementId); }
        SDLClasses.class_IDREF(bw, self.ref);
        b = (self.value != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_updateValue(bw, self.value); }
        encodeReplaceContent(self, bw);
    }

    public void encode(XRestore self, BitWriter bw) throws IOException {
        boolean b = false;
        SDLClasses.class_byteAlignedString(bw, self.groupID);
        encodeSimpleUpdateContent(self, bw);
    }

    public void encode(XSave self, BitWriter bw) throws IOException {
        boolean b = false;
        SDLClasses.class_ElementAttributeList(bw, self);
        SDLClasses.class_byteAlignedString(bw, self.groupID);
        encodeSimpleUpdateContent(self, bw);
    }

    public void encode(XSendEvent self, BitWriter bw) throws IOException {
        boolean b = false;
        SDLClasses.class_event(bw, self.event);
        b = (self.intvalue != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_signedInt(bw, self.intvalue); }
        b = (self.pointvalue != null);
        bw.putBit(b);
        if (b) { SDLClasses.pointType(bw, self.pointvalue); }
        SDLClasses.class_IDREF(bw, self.ref);
        b = (self.stringvalue != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.stringvalue); }
        encodeSimpleUpdateContent(self, bw);
    }

    public void encode(XLASeRHeader self, BitWriter bw) throws IOException {
        boolean b = false;
        bw.putBits(8, self.profile);
        bw.putBits(8, self.level);
        bw.putBits(3, self.reserved);
        bw.putBits(2, self.pointsCodec);
        bw.putBits(4, self.pathComponents);
        bw.putBit(self.useFullRequestHost);
        b = (self.timeResolution != -1);
        bw.putBit(b);
        if (b) {
            bw.putBits(16, self.timeResolution);
            SDLClasses.timeResolution = self.timeResolution;
        }
        bw.putBits(4, self.colorComponentBits_minus_1);
        bw.putBits(4, self.resolution);
        bw.putBits(5, self.coordBits);
        bw.putBits(4, self.scaleBits_minus_coordBits);
        bw.putBit(self.newSceneIndicator);
        bw.putBits(3, 0);
        bw.putBits(4, self.extensionIDBits);
        boolean hasExtensionDescriptor = (self.extensionDescriptor != null);
        bw.putBit(hasExtensionDescriptor);
        if (hasExtensionDescriptor) {
            bw.putVlcInt(5, self.extensionDescriptor.length);
            for (int i = 0; i < self.extensionDescriptor.length; i++) {
                bw.putBits(8, self.extensionDescriptor[i]);
            }
        }
        bw.putBit(false);
    }

    public void encode(Xanimation self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.begin != null);
        bw.putBit(b);
        if (b) { SDLClasses.timesType(bw, self.begin); }
        b = (self.dur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.dur, true); }
        bw.putBit(self.externalResourcesRequired);
        b = (self.fill !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.fill); }
        b = (self.height != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.height); }
        b = (self.preserveAspectRatio != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_preserveAspectRatio(bw, self.preserveAspectRatio); }
        b = (self.repeatCount != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatCountType(bw, self.repeatCount); }
        b = (self.repeatDur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatDurType(bw, self.repeatDur); }
        b = (self.restart !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.restart); }
        b = (self.syncBehavior !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.syncBehavior); }
        b = (self.syncTolerance != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.syncTolerance(bw, self.syncTolerance); }
        b = (self.type != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.type); }
        b = (self.width != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.width); }
        b = (self.x != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x); }
        b = (self.y != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        b = (self.clipBegin != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.clipBegin, true); }
        b = (self.clipEnd != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.clipEnd, true); }
        b = (self.syncReference != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.syncReference); }
        encodeContent(self, bw);
    }

    public void encode(Xdiscard self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.begin != null);
        bw.putBit(b);
        if (b) { SDLClasses.timesType(bw, self.begin); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        encodeContent(self, bw);
    }

    public void encode(Xfont self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { 
            SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        bw.putBit(self.externalResourcesRequired);
        b = (self.horizAdvX != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { 
            SDLClasses.class_fixed_16_8(bw, self.horizAdvX); }
        b = (self.horizOriginX != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { 
            SDLClasses.class_fixed_16_8(bw, self.horizOriginX); }
        encodeContent(self, bw);
    }

    public void encode(XfontFace self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.accentHeight != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.accentHeight); }
        b = (self.alphabetic != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.alphabetic); }
        b = (self.ascent != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.ascent); }
        b = (self.bbox != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.bbox); }
        b = (self.capHeight != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.capHeight); }
        b = (self.descent != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.descent); }
        bw.putBit(self.externalResourcesRequired);
        b = (self.fontFamily != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.fontFamily); }
        b = (self.fontStretch != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.fontStretch); }
        b = (self.fontStyle != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.fontStyle); }
        b = (self.fontVariant != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.fontVariant); }
        b = (self.fontWeight != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.fontWeight); }
        b = (self.hanging != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.hanging); }
        b = (self.ideographic != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.ideographic); }
        b = (self.mathematical != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.mathematical); }
        b = (self.overlinePosition != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.overlinePosition); }
        b = (self.overlineThickness != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.overlineThickness); }
        b = (self.panose1 != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.panose1); }
        b = (self.slope != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.slope); }
        b = (self.stemh != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.stemh); }
        b = (self.stemv != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.stemv); }
        b = (self.strikethroughPosition != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.strikethroughPosition); }
        b = (self.strikethroughThickness != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.strikethroughThickness); }
        b = (self.underlinePosition != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.underlinePosition); }
        b = (self.underlineThickness != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.underlineThickness); }
        b = (self.unicodeRange != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.unicodeRange); }
        b = (self.unitsPerEm != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.unitsPerEm); }
        b = (self.widths != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.widths); }
        b = (self.xHeight != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.xHeight); }
        encodeContent(self, bw);
    }

    public void encode(XfontFaceSrc self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        encodeContent(self, bw);
    }

    public void encode(XfontFaceUri self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        bw.putBit(self.externalResourcesRequired);
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        encodeContent(self, bw);
    }

    public void encode(Xglyph self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.arabicForm != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.arabicForm); }
        b = (self.d != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_path(bw, self.d); }
        bw.putBit(self.externalResourcesRequired);
        b = (self.glyphName != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.glyphName); }
        b = (self.horizAdvX != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.horizAdvX); }
        b = (self.lang != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.lang); }
        b = (self.unicode != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.unicode); }
        encodeContent(self, bw);
    }

    public void encode(Xhandler self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        bw.putBit(self.externalResourcesRequired);
        b = (self.type != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_script(bw, self.type); }
        b = (self.event != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_event(bw, self.event); }
        encodeContent(self, bw);
    }

    public void encode(Xhkern self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.g1 != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.g1); }
        b = (self.g2 != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.g2); }
        b = (self.k != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.k); }
        b = (self.u1 != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.u1); }
        b = (self.u2 != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.u2); }
        encodeContent(self, bw);
    }

    public void encode(XmissingGlyph self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.d != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_path(bw, self.d); }
        b = (self.horizAdvX != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_fixed_16_8(bw, self.horizAdvX); }
        encodeContent(self, bw);
    }

    public void encode(Xprefetch self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.bandwidth != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putVlcInt(5, self.bandwidth); }
        b = (self.mediaCharacterEncoding != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.mediaCharacterEncoding); }
        b = (self.mediaContentEncodings != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_byteAlignedString(bw, self.mediaContentEncodings); }
        b = (self.mediaSize != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putVlcInt(5, self.mediaSize); }
        b = (self.mediaTime != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putVlcInt(5, self.mediaTime); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        encodeContent(self, bw);
    }

    public void encode(XsolidColor self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        encodeContent(self, bw);
    }

    public void encode(XtBreak self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        encodeContent(self, bw);
    }

    public void encode(XtextArea self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.fill != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.fill); }
        b = (self.stroke != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_paint(bw, self.stroke); }
        b = (self.editable != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBit(self.editable != 0); }
        b = (self.height != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.height); }
        b = (self.width != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.width); }
        b = (self.x != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.x); }
        b = (self.y != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.y); }
        encodeContent(self, bw);
    }

    public void encode(XanimateScroll self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.begin != null);
        bw.putBit(b);
        if (b) { SDLClasses.timesType(bw, self.begin); }
        b = (self.by != null);
        bw.putBit(b);
        if (b) { SDLClasses.pointType(bw, self.by); }
        b = (self.delayAtEnd != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.delayAtEnd, true); }
        b = (self.delayAtStart != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.delayAtStart, true); }
        b = (self.direction != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(4, self.direction); }
        b = (self.dur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.dur, true); }
        b = (self.fill !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(1, self.fill); }
        b = (self.from !=  null);
        bw.putBit(b);
        if (b) { SDLClasses.class_scrollStop(bw, self.from); }
        b = (self.repeatCount != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatCountType(bw, self.repeatCount); }
        b = (self.repeatDur != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.repeatDurType(bw, self.repeatDur); }
        b = (self.restart !=  Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.restart); }
        b = (self.speed != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_time(bw, self.speed, true); }
        b = (self.to !=  null);
        bw.putBit(b);
        if (b) { SDLClasses.class_scrollStop(bw, self.to); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        encodeContent(self, bw);
    }

    public void encode(XsetScroll self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.begin != null);
        bw.putBit(b);
        if (b) { SDLClasses.timesType(bw, self.begin); }
        b = (self.direction != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(4, self.direction); }
        b = (self.increment != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_scrollStop(bw, self.increment); }
        b = (self.to !=  null);
        bw.putBit(b);
        if (b) { SDLClasses.class_scrollStop(bw, self.to); }
        b = (self.href != null);
        bw.putBit(b);
        if (b) { SDLClasses.class_anyURI(bw, self.href); }
        encodeContent(self, bw);
    }

    public void encode(XstreamSource self, BitWriter bw) throws IOException {
        boolean b = false;
        b = (self.id != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_ID(bw, self.id); }
        SDLClasses.class_rare(bw, self);
        b = (self.height != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.height); }
        b = (self.mode != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(3, self.mode); }
        b = (self.sourceIndex != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putVlcInt(5, self.sourceIndex); }
        bw.putVlcInt(5, self.source.length);
        for (int i = 0; i < self.source.length; i++) {
            SDLClasses.class_anyURI(bw, self.source[i]);
        }
        b = (self.width != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.class_coordinate(bw, self.width); }
        encodeContent(self, bw);
    }
}
