/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.io;

import iso.mpeg.streamezzo.laser.*;
import iso.mpeg.streamezzo.saf.*;
import iso.mpeg.streamezzo.sdl.BitWriter;
import iso.mpeg.streamezzo.sdl.SDLClasses;
import iso.mpeg.streamezzo.sdl.RandomAccessMemoryStream;
import iso.mpeg.streamezzo.util.*;
import iso.mpeg.streamezzo.resource.Resource;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Vector;

public class EncoderImpl extends Encoder {
    public final static boolean debug = false;
    private Scene scene = null;
    private Manager man = null;
    public static int laserBytes = 0;
    public static int mediaBytes = 0;
    public RandomAccessMemoryStream rams = null;

    public EncoderImpl(Scene s) {
        scene = s;
        man = new Manager();
    }

    public EncoderImpl(Scene s, boolean f) {
        scene = s;
        man = s.man;
    }

    public void encodeContent(X self, BitWriter bw) throws IOException {
        bw.putBit(false);
        boolean b = self.privateAttributes != null;
        bw.putBit(b);
        if(b) {
            SDLClasses.privateAttributeContainer(bw, self.privateAttributes);
        }
        b = (self.child != null && self.child.size() > 0);
        bw.putBit(b);
        if(b) {
            bw.putVlcInt(5, self.child.size());
            for(int t=0;t<self.child.size();t++) {
                elements(bw, self.child.elementAt(t));
            }
        }
    }

    public void encodeFOContent(XforeignObject self, BitWriter bw) throws IOException {
        bw.putBit(false);
        boolean b = (self.child != null && self.child.size() > 0);
        bw.putBit(b);
        if(b) {
            bw.putVlcInt(5, self.child.size());
            for(int t=0;t<self.child.size();t++) {
                SDLClasses.privateElement(bw, (XprivateElement)self.child.elementAt(t), this);
            }
        }
    }

    public void encodeSameContent(X self, BitWriter bw) throws IOException {
        boolean b = (self.child != null && self.child.size() > 0);
        bw.putBit(b);
        if(b) {
            bw.putVlcInt(5, self.child.size());
            for(int t=0;t<self.child.size();t++) {
                elements(bw, self.child.elementAt(t));
            }
        }
    }

    public void encodeInsertContent(Uc self, BitWriter bw) throws IOException {
        bw.putBit(false);
        boolean b = (self.child != null);
        bw.putBit(b);
        if(b) {
            updatable_elements(bw, self.child);
        }
    }

    public void encodeReplaceContent(XReplace self, BitWriter bw) throws IOException {
        bw.putBit(false);
        boolean b = (self.child != null);
        bw.putBit(b);
        if(b) {
            bw.putVlcInt(5, self.child.size());
            for(int t=0;t<self.child.size();t++) {
                updatable_elements(bw, self.child.elementAt(t));
            }
        }
    }

    public void encodeSimpleUpdateContent(U self, BitWriter bw) throws IOException {
        bw.putBit(false);
    }

    public void encode(XprivateElementContainer self, BitWriter bw) throws IOException {
        if (self.anyXML != null) {
            bw.putBits(4, 0);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            BitWriter bw1 = new BitWriter(baos);
            SDLClasses.privateChildren(bw1, self.anyXML, this);
            bw1.flush();
            bw.putVlcInt(5, bw1.length());
            bw.flush();
            bw.putByteArray(baos.toByteArray());
        } else if (self.opaque != null) {
            bw.putVlcInt(5, self.opaque.length);
            bw.putBits(SDLClasses.privateDataIdBits, self.privateDataIdentifier);
            bw.putByteArray(self.opaque);
        }
    }

    private static Vector vv = new Vector();

    public void encode(XprivateElement self, BitWriter bw) throws IOException {
        bw.putBits(4, 0);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        BitWriter bw1 = new BitWriter(baos);
        vv.addElement(self);
        SDLClasses.privateChildren(bw1, vv, this);
        vv.removeAllElements();
        bw1.flush();
        bw.putVlcInt(5, bw1.length());
        bw.flush();
        bw.putByteArray(baos.toByteArray());
    }

    /**
     * the packet sequence number
     */
    private int packetSeqNum = 0;

    public void encodeAnySaf(Object o, BitWriter bw) throws IOException {
        if (o instanceof SceneHeader) {
            encode((SceneHeader)o, bw);
            return;
        }
        if (o instanceof ImageHeader) {
            encode((ImageHeader)o, bw);
            return;
        }
        if (o instanceof MediaHeader) {
            encode((MediaHeader)o, bw);
            return;
        }
        if (o instanceof MediaUnit) {
            encode((MediaUnit)o, bw);
            return;
        }
        if (o instanceof SceneUnit) {
            encode((SceneUnit)o, bw);
            return;
        }
        if (o instanceof ImageUnit) {
            encode((ImageUnit)o, bw);
            return;
        }
        if (o instanceof EndOfSAFSession) {
            encode((EndOfSAFSession)o, bw);
            return;
        }
        if (o instanceof EndOfStream) {
            encode((EndOfStream)o, bw);
            return;
        }
        if (o instanceof RemoteStreamHeader) {
            encode((RemoteStreamHeader)o, bw);
            return;
        }
        throw new RuntimeException("encodeAnySAF:"+o);
    }

    public void encode(BitWriter bw) throws IOException {
        SDLClasses.currentScene = scene;
        SDLClasses.man = man;
        for (int i = 0; i < scene.size(); i++) {
            encodeAnySaf(scene.elementAt(i), bw);
        }
    }

    public void encodeAny(BitWriter bw) throws IOException {
        SDLClasses.currentScene = scene;
        SDLClasses.man = man;
        SDLClasses.class_initialisation(bw, null);
        for (int i = 0; i < scene.size(); i++) {
            XprivateElement pe = (XprivateElement)(scene.elementAt(i));
            encode(pe, bw);
        }
    }

    public void encode(SceneHeader self, BitWriter bw) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        BitWriter bw1 = new BitWriter(baos);
        bw1.putBits(8, MPEG4Constants.LASeROTI);
        bw1.putBits(8, MPEG4Constants.SceneDescriptionStreamType);
        bw1.putBits(24, self.timeStampResolution);
        bw1.putBits(16, self.bufferSize);
        if (self.lh != null) encode(self.lh, bw1);
        bw1.flush();
        if (self.lh != null) {
            SDLClasses.colorComponentBits = 1 + self.lh.colorComponentBits_minus_1;
            SDLClasses.scaleBits = self.lh.scaleBits_minus_coordBits;
            SDLClasses.coordBits = self.lh.coordBits;
            SDLClasses.resolution = self.lh.resolution;
        }
        SAF_SL_PacketHeader(false, self.time, bw1.length(), bw);
        safAUHeader(self, scene.idMan.getOrCreateStreamID(self.streamID), bw);
        bw.putByteArray(baos.toByteArray());
        laserBytes += baos.size();

    }

    public void encode(ImageHeader au, BitWriter pw) throws IOException {
        SAF_SL_PacketHeader(true, au.getTime(), 7 + (au.length > 0 ? au.length : (au.resource == null ? 0 : au.resource.getSize())), pw);
        safAUHeader(au, scene.idMan.getOrCreateStreamID(au.getStreamID()), pw);
        pw.putBits(8, au.objectTypeIndication);
        pw.putBits(8, au.streamType);
        pw.putBits(24, au.timeStampResolution);
        pw.putBits(16, au.bufferSize);
        if (au.resource != null) {
            Resource.move(au.resource, pw, au.startOffset, au.length);
            mediaBytes += au.resource.getSize();
        }
    }

    public void encode(MediaHeader au, BitWriter pw) throws IOException {
        SAF_SL_PacketHeader(true, au.getTime(), 7 + (au.length > 0 ? au.length : (au.resource == null ? 0 : au.resource.getSize())), pw);
        safAUHeader(au, scene.idMan.getOrCreateStreamID(au.getStreamID()), pw);
        pw.putBits(8, au.objectTypeIndication);
        pw.putBits(8, au.streamType);
        pw.putBits(24, au.timeStampResolution);
        pw.putBits(16, au.bufferSize);
        if (au.resource != null) {
            Resource.move(au.resource, pw, au.startOffset, au.length);
            mediaBytes += au.resource.getSize();
        }
    }

    public void encode(RemoteStreamHeader au, BitWriter pw) throws IOException {
        int len = pw.getUTFStringLength(au.source);
        SAF_SL_PacketHeader(true, au.getTime(), 7 + len +
            (au.length > 0 ? au.length : (au.resource == null ? 0 : au.resource.getSize())), pw);
        safAUHeader(au, scene.idMan.getOrCreateStreamID(au.getStreamID()), pw);
        pw.putBits(8, au.objectTypeIndication);
        pw.putBits(8, au.streamType);
        pw.putBits(24, au.timeStampResolution);
        pw.putBits(16, au.bufferSize);
        pw.putUTFString(au.source);
        if (au.resource != null) {
            Resource.move(au.resource, pw, au.startOffset, au.length);
            mediaBytes += au.resource.getSize();
        }
    }

    public void encode(ImageUnit au, BitWriter pw) throws IOException {
        SAF_SL_PacketHeader(true, 0, au.resource.getSize(), pw);
        safAUHeader(au, scene.idMan.getOrCreateStreamID(au.streamID), pw);
        Resource.move(au.resource, pw, 0, 0);
        mediaBytes += au.resource.getSize();
    }

    public void encode(MediaUnit au, BitWriter pw) throws IOException {
        SAF_SL_PacketHeader(au.isRAP, 0, au.resource.getSize(), pw);
        safAUHeader(au, scene.idMan.getOrCreateStreamID(au.streamID), pw);
        Resource.move(au.resource, pw, 0, 0);
        mediaBytes += au.resource.getSize();
    }

    public void encode(EndOfStream au, BitWriter pw) throws IOException {
        SAF_SL_PacketHeader(false, 0, 0, pw);
        safAUHeader(au, scene.idMan.getOrCreateStreamID(au.streamID), pw);
    }

    public void encode(SceneUnit self, BitWriter bw) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        BitWriter bw1 = new BitWriter(baos);
        bw1.trace = bw.trace;
        PerUnit pu = new PerUnit(man);
        try { self.visit(pu, null); } catch(Exception e) { e.printStackTrace(); }
        // LASeRUnitHeader
        bw1.putBit(self.resetEncodingContext);
        bw1.putBit(false); // no extension
        // init
        SDLClasses.class_initialisation(bw1, self.stringIDTable);
        self.isRAP = false;
        // update list
        bw1.putVlcInt(5, self.updates.size()-1);
        for (int i = 0; i < self.updates.size(); i++) {
            U u = (U)self.updates.elementAt(i);
            if (u instanceof XNewScene || u instanceof XRefreshScene) self.isRAP = true;
            updates(bw1, u);
        }
        // extension
        bw1.putBit(false); // no extension
        bw1.flush();
        SAF_SL_PacketHeader(self.isRAP, self.time, bw1.length(), bw);
        safAUHeader(self, 0, bw);
        bw.putByteArray(baos.toByteArray());
        laserBytes += baos.size();

    }

    public void encode(EndOfSAFSession self, BitWriter bw) throws IOException {
        SAF_SL_PacketHeader(false, self.time, 0, bw);
        safAUHeader(self, -1, bw);
    }

    private void SAF_SL_PacketHeader(boolean rap, int time, int len, BitWriter bw) throws IOException {
        bw.putBit(rap);
        bw.putBits(15, packetSeqNum++);
        bw.putBits(2, 1); //DTSflag = 0, CTSflag=1
        bw.putBits(30, time);
        SDLClasses.currentTimeInTics = time;
        bw.putBits(16, len + 2); //+2 = take the SAF AU header into account
    }

    private void safAUHeader(Object o, int streamID, BitWriter bw) throws IOException {
        int type = 0;
        if (o instanceof SceneHeader) {
            if (debug) System.out.print("sceneHeader ");
            type = 0x01;
        } else if (o instanceof EndOfStream) {
            if (debug) System.out.print("EOS ");
            type = 0x03;
        } else if (o instanceof EndOfSAFSession) {
            if (debug) System.out.println("endSession");
            type = 0x05;
        } else if (o instanceof MediaHeader) {
            if (debug) System.out.print("mediaHeader ");
            type = (((MediaHeader) o).isPermanent ? 0x02 : 0x01);
        } else if (o instanceof ImageHeader) {
            if (debug) System.out.print("imageHeader ");
            type = (((ImageHeader) o).isPermanent ? 0x02 : 0x01);
        } else if (o instanceof FontHeader) {
            if (debug) System.out.print("fontHeader ");
            type = (((MediaHeader) o).isPermanent ? 0x02 : 0x01);
        } else if (o instanceof SceneUnit) {
            if (debug) System.out.print("sceneUnit ");
            type = 0x04;
        } else if (o instanceof MediaUnit) {
            if (debug) System.out.print("mediaUnit ");
            type = 0x04;
        } else if (o instanceof ImageUnit) {
            if (debug) System.out.print("imageUnit ");
            type = 0x04;
        } else if (o instanceof FontUnit) {
            if (debug) System.out.print("fontUnit ");
            type = 0x04;
        } else if (o instanceof CacheObject) {
            if (debug) System.out.print("cacheObject ");
            type = 0x06;
        } else if (o instanceof RemoteStreamHeader) {
            if (debug) System.out.print("remoteHeader ");
            type = 0x07;
        }
        bw.putBits(4, type);
        bw.putBits(12, streamID);
    }

    public void encode(XtextContent self, BitWriter bw) throws IOException {
        SDLClasses.class_byteAlignedString(bw, self.getContent());
    }

    public void encodeFocus(X self, BitWriter bw) throws IOException {
        boolean b = (self.navRight != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.focusType(bw, self.navRight); }
        b = (self.navUp != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.focusType(bw, self.navUp); }
        b = (self.navUpRight != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.focusType(bw, self.navUpRight); }
        b = (self.navUpLeft != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.focusType(bw, self.navUpLeft); }
        b = (self.navDown != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.focusType(bw, self.navDown); }
        b = (self.navDownRight != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.focusType(bw, self.navDownRight); }
        b = (self.navDownLeft != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.focusType(bw, self.navDownLeft); }
        b = (self.navLeft != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { SDLClasses.focusType(bw, self.navLeft); }
        b = (self.focusable != Integer.MIN_VALUE);
        bw.putBit(b);
        if (b) { bw.putBits(2, self.focusable); }
    }

    public int generate(OutputStream os) throws Exception {
        rams = new RandomAccessMemoryStream();
        BitWriter bw = new BitWriter(rams);
        encode(bw);
        rams.writeTo(os);
        os.flush();
        return rams.size();
    }

    public int generateAny(OutputStream os) throws Exception {
        rams = new RandomAccessMemoryStream();
        BitWriter bw = new BitWriter(rams);
        encodeAny(bw);
        rams.writeTo(os);
        os.flush();
        return rams.size();
    }
}
