/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.io;

import iso.mpeg.streamezzo.sdl.Attribute;

public final class IOUtils {
    public static int animatable(String s) {
        return updatable(s);
    }

    public static String updatableStrings[] = new String[]{
            "a.target",
            "accumulate",
            "additive",
            "audio-level",
            "bandwidth",
            "begin",
            "calcMode",
            "children",
            "choice",
            "clipBegin",
            "clipEnd",
            "color",
            "color-rendering",
            "cx",
            "cy",
            "d",
            "delta",
            "display",
            "display-align",
            "dur",
            "editable",
            "enabled",
            "end",
            "event",
            "externalResourcesRequired",
            "fill",
            "fill-opacity",
            "fill-rule",
            "focusable",
            "font-family",
            "font-size",
            "font-style",
            "font-variant",
            "font-weight",
            "fullscreen",
            "gradientUnits",
            "handler",
            "height",
            "image-rendering",
            "keyPoints",
            "keySplines",
            "keyTimes",
            "line-increment",
            "listener.target",
            "mediaCharacterEncoding",
            "mediaContentEncodings",
            "mediaSize",
            "mediaTime",
            "nav-down",
            "nav-down-left",
            "nav-down-right",
            "nav-left",
            "nav-next",
            "nav-prev",
            "nav-right",
            "nav-up",
            "nav-up-left",
            "nav-up-right",
            "observer",
            "offset",
            "opacity",
            "overflow",
            "overlay",
            "path",
            "pathLength",
            "pointer-events",
            "points",
            "preserveAspectRatio",
            "r",
            "repeatCount",
            "repeatDur",
            "requiredExtensions",
            "requiredFeatures",
            "requiredFormats",
            "restart",
            "rotate",
            "rotation",
            "rx",
            "ry",
            "scale",
            "shape-rendering",
            "size",
            "solid-color",
            "solid-opacity",
            "stop-color",
            "stop-opacity",
            "stroke",
            "stroke-dasharray",
            "stroke-dashoffset",
            "stroke-linecap",
            "stroke-linejoin",
            "stroke-miterlimit",
            "stroke-opacity",
            "stroke-width",
            "svg.height",
            "svg.width",
            "syncBehavior",
            "syncBehaviorDefault",
            "syncReference",
            "syncTolerance",
            "syncToleranceDefault",
            "systemLanguage",
            "text-align",
            "text-anchor",
            "text-decoration",
            "text-display",
            "text-rendering",
            "textContent",
            "transform",
            "transformBehavior",
            "translation",
            "vector-effect",
            "viewBox",
            "viewport-fill",
            "viewport-fill-opacity",
            "visibility",
            "width",
            "x",
            "x1",
            "x2",
            "xlink:actuate",
            "xlink:arcrole",
            "xlink:href",
            "xlink:role",
            "xlink:show",
            "xlink:title",
            "xlink:type",
            "xml:base",
            "xml:lang",
            "y",
            "y1",
            "y2",
            "zoomAndPan"};

    public static int updatable(String s) {
        for (int i = 0; i < updatableStrings.length; i++) {
            if (updatableStrings[i].equals(s)) return i;
        }
        return Integer.MIN_VALUE;
    }

    public final static String featureNames[] = {
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Animation",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Audio",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#ComposedVideo",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#ConditionalProcessing",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#ConditionalProcessingAttribute",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#CoreAttribute",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Discard",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Extensibility",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#ExternalResourcesRequired",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Font",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Gradient",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#GraphicsAttribute",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Handler",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Hyperlinking",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Image",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Listener",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#MediaAttribute",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#NavigationAttribute",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#OpacityAttribute",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#PaintAttribute",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Prefetch",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#SVG-all",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#SVG-animated",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#SVG-static",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#SVG-static-DOM",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Scripting",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Shape",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#SolidColor",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Structure",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Text",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#TextFlow",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#TimedAnimation",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#TransformedVideo",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#Video",
      "http://www.w3.org/Graphics/SVG/feature/1.2/#XlinkAttribute",
      "http://www.w3.org/TR/SVG11/feature#Animation",
      "http://www.w3.org/TR/SVG11/feature#AnimationEventsAttribute",
      "http://www.w3.org/TR/SVG11/feature#BasicClip",
      "http://www.w3.org/TR/SVG11/feature#BasicFilter",
      "http://www.w3.org/TR/SVG11/feature#BasicFont",
      "http://www.w3.org/TR/SVG11/feature#BasicGraphicsAttribute",
      "http://www.w3.org/TR/SVG11/feature#BasicPaintAttribute",
      "http://www.w3.org/TR/SVG11/feature#BasicStructure",
      "http://www.w3.org/TR/SVG11/feature#BasicText",
      "http://www.w3.org/TR/SVG11/feature#Clip",
      "http://www.w3.org/TR/SVG11/feature#ColorProfile",
      "http://www.w3.org/TR/SVG11/feature#ConditionalProcessing",
      "http://www.w3.org/TR/SVG11/feature#ContainerAttribute",
      "http://www.w3.org/TR/SVG11/feature#CoreAttribute",
      "http://www.w3.org/TR/SVG11/feature#Cursor",
      "http://www.w3.org/TR/SVG11/feature#DocumentEventsAttribute",
      "http://www.w3.org/TR/SVG11/feature#Extensibility",
      "http://www.w3.org/TR/SVG11/feature#ExternalResourcesRequired",
      "http://www.w3.org/TR/SVG11/feature#Filter",
      "http://www.w3.org/TR/SVG11/feature#Font",
      "http://www.w3.org/TR/SVG11/feature#Gradient",
      "http://www.w3.org/TR/SVG11/feature#GraphicalEventsAttribute",
      "http://www.w3.org/TR/SVG11/feature#GraphicsAttribute",
      "http://www.w3.org/TR/SVG11/feature#Hyperlinking",
      "http://www.w3.org/TR/SVG11/feature#Image",
      "http://www.w3.org/TR/SVG11/feature#Marker",
      "http://www.w3.org/TR/SVG11/feature#Mask",
      "http://www.w3.org/TR/SVG11/feature#OpacityAttribute",
      "http://www.w3.org/TR/SVG11/feature#PaintAttribute",
      "http://www.w3.org/TR/SVG11/feature#Pattern",
      "http://www.w3.org/TR/SVG11/feature#SVG",
      "http://www.w3.org/TR/SVG11/feature#SVG-animation",
      "http://www.w3.org/TR/SVG11/feature#SVG-dynamic",
      "http://www.w3.org/TR/SVG11/feature#SVG-static",
      "http://www.w3.org/TR/SVG11/feature#SVGDOM",
      "http://www.w3.org/TR/SVG11/feature#SVGDOM-animation",
      "http://www.w3.org/TR/SVG11/feature#SVGDOM-dynamic",
      "http://www.w3.org/TR/SVG11/feature#SVGDOM-static",
      "http://www.w3.org/TR/SVG11/feature#Script",
      "http://www.w3.org/TR/SVG11/feature#Shape",
      "http://www.w3.org/TR/SVG11/feature#Structure",
      "http://www.w3.org/TR/SVG11/feature#Style",
      "http://www.w3.org/TR/SVG11/feature#Text",
      "http://www.w3.org/TR/SVG11/feature#View",
      "http://www.w3.org/TR/SVG11/feature#ViewportAttribute",
      "http://www.w3.org/TR/SVG11/feature#XlinkAttribute",
      "http://www.w3.org/TR/SVGMobile/Basic/feature#SVGBasicDomCore",
      "http://www.w3.org/TR/SVGMobile/Basic/feature#SVGBasicDomExtended",
      "http://www.w3.org/TR/SVGMobile/Basic/feature#all",
      "http://www.w3.org/TR/SVGMobile/Basic/feature#base",
      "http://www.w3.org/TR/SVGMobile/Basic/feature#css",
      "http://www.w3.org/TR/SVGMobile/Basic/feature#interactivity",
      "http://www.w3.org/TR/SVGMobile/Tiny/feature#all",
      "http://www.w3.org/TR/SVGMobile/Tiny/feature#base",
      "http://www.w3.org/TR/SVGMobile/Tiny/feature#interactivity",
      "org.w3c.dom.svg{90} org.w3c.dom.svg.all",
      "org.w3c.dom.svg.animation",
      "org.w3c.dom.svg.dynamic",
      "org.w3c.dom.svg.static",
      "org.w3c.svg",
      "org.w3c.svg.all",
      "org.w3c.svg.animation",
      "org.w3c.svg.dynamic",
      "org.w3c.svg.static"
    };
}
