/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.io;

import iso.mpeg.streamezzo.laser.*;
import iso.mpeg.streamezzo.saf.*;
import iso.mpeg.streamezzo.sdl.*;
import iso.mpeg.streamezzo.util.Scene;
import iso.mpeg.streamezzo.util.MPEG4Constants;
import iso.mpeg.streamezzo.util.Base64;

import java.io.PrintWriter;
import java.io.DataInputStream;
import java.util.Hashtable;

public abstract class PreXMLWriter {
    public Hashtable classToWriter = new Hashtable();
    public Scene scene = null;
    private boolean nameSpacesSaved = false;

    public void write(PrintWriter pw, Scene scene) {
        this.scene = scene;
        pw.println("<?xml version='1.0' encoding='UTF-8'?>");
        writeAny(pw, scene);
    }

    public void writeNoSAF(PrintWriter pw, Scene scene) {
        this.scene = scene;
        pw.println("<?xml version='1.0' encoding='UTF-8'?>");
        for (int i = 0; i < scene.size(); i++) {
            writeAny(pw, scene.elementAt(i));
        }
    }

    public void writeAny(PrintWriter pw, Object o) {
        LASeRXMLWriter writer = null;
        if (o != null) {
            writer = (LASeRXMLWriter) classToWriter.get(o.getClass());
        }
        if (writer != null) {
            writer.write(pw, o);
        } else {
            //throw new RuntimeException("Unrecognized object " + o);
            pw.println("<<<" + o + ">>>");
        }
    }

    public void print(PrintWriter pw, int i) {
        pw.print(i);
    }

    public void print(PrintWriter pw, ScrollStop i) {
        if (i.value != null) {
            print(pw, null, i.value);
        } else {
            printID(pw, i.idRef);
        }
    }

    public void printProperties(XpropElem xo, PrintWriter pw) {
        if (xo.audio_level != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("audio-level");
            pw.print("='");
            printFloat(pw, xo.audio_level);
            pw.print("'");
        }
        if (xo.classe != null) {
            pw.print(" ");
            pw.print("class");
            pw.print("='");
            print(pw, xo.classe);
            pw.print("'");
        }
        if (xo.color != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("color");
            pw.print("='");
            printColor(pw, xo.color);
            pw.print("'");
        }
        if (xo.color_rendering != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("color-rendering");
            pw.print("='");
            XMLWriterUtil.printtyp_color_renderingType(pw, xo.color_rendering);
            pw.print("'");
        }
        if (xo.display != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("display");
            pw.print("='");
            XMLWriterUtil.printtyp_displayType(pw, xo.display);
            pw.print("'");
        }
        if (xo.display_align != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("display-align");
            pw.print("='");
            XMLWriterUtil.printtyp_displayAlignType(pw, xo.display_align);
            pw.print("'");
        }
        if (xo.fill != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("fill");
            pw.print("='");
            printColor(pw, xo.fill);
            pw.print("'");
        }
        if (xo.fill_opacity != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("fill-opacity");
            pw.print("='");
            printFloat(pw, xo.fill_opacity);
            pw.print("'");
        }
        if (xo.fill_rule != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("fill-rule");
            pw.print("='");
            XMLWriterUtil.printtyp_fill_ruleType(pw, xo.fill_rule);
            pw.print("'");
        }
        if (xo.focusable != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("focusable");
            pw.print("='");
            pw.print((xo.focusable == 1?"true":"false"));
            pw.print("'");
        }
        if (xo.font_family != null) {
            pw.print(" ");
            pw.print("font-family");
            pw.print("='");
            print(pw, xo.font_family);
            pw.print("'");
        }
        if (xo.font_size != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("font-size");
            pw.print("='");
            printFloat(pw, xo.font_size);
            pw.print("'");
        }
        if (xo.font_style != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("font-style");
            pw.print("='");
            XMLWriterUtil.printtyp_font_styleType(pw, xo.font_style);
            pw.print("'");
        }
        if (xo.text_decoration != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("text-decoration");
            pw.print("='");
            XMLWriterUtil.printtyp_text_decorationType(pw, xo.font_style);
            pw.print("'");
        }
        if (xo.font_variant != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("font-variant");
            pw.print("='");
            XMLWriterUtil.printtyp_font_variantType(pw, xo.font_variant);
            pw.print("'");
        }
        if (xo.font_weight != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("font-weight");
            pw.print("='");
            XMLWriterUtil.printtyp_font_weightType(pw, xo.font_weight);
            pw.print("'");
        }
        if (xo.image_rendering != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("image-rendering");
            pw.print("='");
            XMLWriterUtil.printtyp_image_renderingType(pw, xo.image_rendering);
            pw.print("'");
        }
        // MGR : nav-* attributes can be a local IDREF or a string equals to "auto" or "self" (focus)
        if (xo.navNext != Integer.MIN_VALUE) {
            pw.print(" nav-next='");
            if (xo.navNext > (Integer.MAX_VALUE - 2)) {
                String value =  (xo.navNext == Integer.MAX_VALUE - 1)? "self" : "auto";
                print(pw, value);
            } else {
                pw.print("#");
                printID(pw, xo.navNext);
            }
            pw.print("'");
        }
        if (xo.navPrev != Integer.MIN_VALUE) {
            pw.print(" nav-prev='");
            if (xo.navPrev > (Integer.MAX_VALUE - 2)) {
                String value =  (xo.navPrev == Integer.MAX_VALUE - 1)? "self" : "auto";
                print(pw, value);
            } else {
                pw.print("#");
                printID(pw, xo.navPrev);
            }
            pw.print("'");
        }
        if (xo.navUp != Integer.MIN_VALUE) {
            pw.print(" nav-up='");
            if (xo.navUp > (Integer.MAX_VALUE - 2)) {
                String value =  (xo.navUp == Integer.MAX_VALUE - 1)? "self" : "auto";
                print(pw, value);
            } else {
                pw.print("#");
                printID(pw, xo.navUp);
            }
            pw.print("'");
        }
        if (xo.navDown != Integer.MIN_VALUE) {
            pw.print(" nav-down='");
            if (xo.navDown > (Integer.MAX_VALUE - 2)) {
                String value =  (xo.navDown == Integer.MAX_VALUE - 1)? "self" : "auto";
                print(pw, value);
            } else {
                pw.print("#");
                printID(pw, xo.navDown);
            }
            pw.print("'");
        }
        if (xo.navUpRight != Integer.MIN_VALUE) {
            pw.print(" nav-up-right='");
            if (xo.navUpRight > (Integer.MAX_VALUE - 2)) {
                String value =  (xo.navUpRight == Integer.MAX_VALUE - 1)? "self" : "auto";
                print(pw, value);
            } else {
                pw.print("#");
                printID(pw, xo.navUpRight);
            }
            pw.print("'");
        }
        if (xo.navDownRight != Integer.MIN_VALUE) {
            pw.print(" nav-down-right='");
            if (xo.navDownRight > (Integer.MAX_VALUE - 2)) {
                String value =  (xo.navDownRight == Integer.MAX_VALUE - 1)? "self" : "auto";
                print(pw, value);
            } else {
                pw.print("#");
                printID(pw, xo.navDownRight);
            }
            pw.print("'");
        }
        if (xo.navUpLeft != Integer.MIN_VALUE) {
            pw.print(" nav-up-left='");
            if (xo.navUpLeft > (Integer.MAX_VALUE - 2)) {
                String value =  (xo.navUpLeft == Integer.MAX_VALUE - 1)? "self" : "auto";
                print(pw, value);
            } else {
                pw.print("#");
                printID(pw, xo.navUpLeft);
            }
            pw.print("'");
        }
        if (xo.navDownLeft != Integer.MIN_VALUE) {
            pw.print(" nav-down-left='");
            if (xo.navDownLeft > (Integer.MAX_VALUE - 2)) {
                String value =  (xo.navDownLeft == Integer.MAX_VALUE - 1)? "self" : "auto";
                print(pw, value);
            } else {
                pw.print("#");
                printID(pw, xo.navDownLeft);
            }
            pw.print("'");
        }
        if (xo.navRight != Integer.MIN_VALUE) {
            pw.print(" nav-right='");
            if (xo.navRight > (Integer.MAX_VALUE - 2)) {
                String value =  (xo.navRight == Integer.MAX_VALUE - 1)? "self" : "auto";
                print(pw, value);
            } else {
                pw.print("#");
                printID(pw, xo.navRight);
            }
            pw.print("'");
        }
        if (xo.navLeft != Integer.MIN_VALUE) {
            pw.print(" nav-left='");
            if (xo.navLeft > (Integer.MAX_VALUE - 2)) {
                String value =  (xo.navLeft == Integer.MAX_VALUE - 1)? "self" : "auto";
                print(pw, value);
            } else {
                pw.print("#");
                printID(pw, xo.navLeft);
            }
            pw.print("'");
        }
        if (xo.line_increment != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("line-increment");
            pw.print("='");
            if (xo.line_increment > 1) {
                printFloat(pw, xo.line_increment);
            } else {
                XMLWriterUtil.printtyp_line_increment(pw, xo.line_increment);
            }
            pw.print("'");
        }
        if (xo.pointer_events != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("pointer-events");
            pw.print("='");
            XMLWriterUtil.printtyp_pointer_eventsType(pw, xo.pointer_events);
            pw.print("'");
        }
        if (xo.shape_rendering != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("shape-rendering");
            pw.print("='");
            XMLWriterUtil.printtyp_shape_renderingType(pw, xo.shape_rendering);
            pw.print("'");
        }
        if (xo.solid_color != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("solid-color");
            pw.print("='");
            printColor(pw, xo.solid_color);
            pw.print("'");
        }
        if (xo.solid_opacity != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("solid-opacity");
            pw.print("='");
            printFloat(pw, xo.solid_opacity);
            pw.print("'");
        }
        if (xo.stop_color != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("stop-color");
            pw.print("='");
            printColor(pw, xo.stop_color);
            pw.print("'");
        }
        if (xo.stop_opacity != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("stop-opacity");
            pw.print("='");
            printFloat(pw, xo.stop_opacity);
            pw.print("'");
        }
        if (xo.stroke != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("stroke");
            pw.print("='");
            printColor(pw, xo.stroke);
            pw.print("'");
        }
        if (xo.stroke_dasharray != null) {
            pw.print(" ");
            pw.print("stroke-dasharray");
            pw.print("='");
            print(pw, xo.stroke_dasharray);
            pw.print("'");
        }
        if (xo.stroke_dashoffset != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("stroke-dashoffset");
            pw.print("='");
            printFloat(pw, xo.stroke_dashoffset);
            pw.print("'");
        }
        if (xo.stroke_linecap != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("stroke-linecap");
            pw.print("='");
            XMLWriterUtil.printtyp_stroke_linecapType(pw, xo.stroke_linecap);
            pw.print("'");
        }
        if (xo.stroke_linejoin != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("stroke-linejoin");
            pw.print("='");
            XMLWriterUtil.printtyp_stroke_linejoinType(pw, xo.stroke_linejoin);
            pw.print("'");
        }
        if (xo.stroke_miterlimit != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("stroke-miterlimit");
            pw.print("='");
            printFloat(pw, xo.stroke_miterlimit);
            pw.print("'");
        }
        if (xo.stroke_opacity != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("stroke-opacity");
            pw.print("='");
            printFloat(pw, xo.stroke_opacity);
            pw.print("'");
        }
        if (xo.stroke_width != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("stroke-width");
            pw.print("='");
            printFloat(pw, xo.stroke_width);
            pw.print("'");
        }
        if (xo.text_anchor != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("text-anchor");
            pw.print("='");
            XMLWriterUtil.printtyp_textAnchorType(pw, xo.text_anchor);
            pw.print("'");
        }
        if (xo.text_rendering != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("text-rendering");
            pw.print("='");
            XMLWriterUtil.printtyp_text_renderingType(pw, xo.text_rendering);
            pw.print("'");
        }
        if (xo.vector_effect != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("vector-effect");
            pw.print("='");
            XMLWriterUtil.printtyp_vector_effectType(pw, xo.visibility);
            pw.print("'");
        }
        if (xo.viewport_fill != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("viewport-fill");
            pw.print("='");
            printColor(pw, xo.viewport_fill);
            pw.print("'");
        }
        if (xo.viewport_fill_opacity != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("viewport-fill-opacity");
            pw.print("='");
            printFloat(pw, xo.viewport_fill_opacity);
            pw.print("'");
        }
        if (xo.visibility != Integer.MIN_VALUE) {
            pw.print(" ");
            pw.print("visibility");
            pw.print("='");
            XMLWriterUtil.printtyp_visibilityType(pw, xo.visibility);
            pw.print("'");
        }
    }
    
    public void printID(PrintWriter pw, int i) {
        if (scene.idMan != null) {
            String s = scene.idMan.getID(i);
            if (s != null) pw.print(s);
            else pw.print(i);
            return;
        }
        pw.print(i);
    }

    public void printStreamID(PrintWriter pw, int i) {
        pw.print("stream:");
        if (scene.idMan != null) {
            String s = scene.idMan.getStreamID(i);
            if (s != null) pw.print(s);
            else pw.print(i);
            return;
        }
        pw.print(i);
    }

    public void printURIs(PrintWriter pw, URI u[]) {
        for (int i = 0; i < u.length; i++) {
            if (i != 0) pw.print(" ");
            print(pw, u[i]);
        }
    }

    public void print(PrintWriter pw, URI u) {
        if (u.uri != null) {
            pw.print(u.uri);
            return;
        }
        if (u.id != Integer.MIN_VALUE) {
            pw.print("#");
            if (scene.idMan != null) {
                String s = scene.idMan.getID(u.id);
                if (s != null) {
                    pw.print(s);
                    return;
                }
            }
            pw.print(u.id);
            return;
        }
        pw.print("stream:");
        if (scene.idMan != null) {
            String s = scene.idMan.getStreamID(u.streamID);
            if (s != null) {
                pw.print(s);
                return;
            }
        }
        pw.print(u.streamID);
        return;
    }

    public void printURI(PrintWriter pw, UpdateValue u) {
        if (u.string != null) {
            pw.print(u.string);
            return;
        }
        if (u.x != Integer.MIN_VALUE) {
            pw.print("#");
            if (scene.idMan != null) {
                String s = scene.idMan.getID(u.x);
                if (s != null) {
                    pw.print(s);
                    return;
                }
            }
            pw.print(u.x);
            return;
        }
        pw.print("stream:");
        if (scene.idMan != null) {
            String s = scene.idMan.getStreamID(u.y);
            if (s != null) {
                pw.print(s);
                return;
            }
        }
        pw.print(u.y);
        return;
    }

    public void printIDs(PrintWriter pw, int[] ids) {
        for (int i = 0; i < ids.length; i++) {
            if (i > 0) {
                pw.print(" ");
            }
            if (scene.idMan != null) {
                String s = scene.idMan.getID(ids[i]);
                if (s != null) {
                    pw.print(s);
                    continue;
                }
            }
            pw.print(ids[i]);
        }
    }

    public void print(PrintWriter pw, byte i) {
        pw.print(i);
    }

    public void print(PrintWriter pw, String i) {
        pw.print(i);
    }

    public void printFloat(PrintWriter pw, int i) {
        String s = Float.toString(((float) i) / 4096f);
        if (s.endsWith(".0")) s = s.substring(0, s.length() - 2);
        pw.print(s);
    }

    public void printTime(PrintWriter pw, int i) {
        if (i < 0) {
            pw.print("indefinite");
        } else {
            pw.print(i);
        }
    }

    public void printDur(PrintWriter pw, int i) {
        if (i == Integer.MIN_VALUE+1) {
            pw.print("auto");
        } else if (i == Integer.MIN_VALUE) {
            pw.print("indefinite");
        } else if (i == Integer.MAX_VALUE) {
            pw.print("media");
        } else if (i == Integer.MAX_VALUE-1) {
            pw.print("none");
         } else {
            printFloat(pw, i);
        }
    }

    public void printTimes(PrintWriter pw, SmilTime[] i) {
        for (int j = 0; j < i.length; j++) {
            if (j != 0) pw.print(";");
            i[j].print(pw, scene.idMan);
        }
    }

    public void printColor(PrintWriter pw, int col) {
        if (col == -1) {
            pw.print("currentColor");
        } else if (col == -2) {
            pw.print("none");
        } else if (col == -3) {
            pw.print("inherit");
        } else if (col <= -4) {
            pw.print("url(");
            print(pw, scene.idMan.getPaintURI(-4 - col));
            pw.print(")");
        } else {
            pw.print("rgb(");
            pw.print(0xFF & (col >> 16));
            pw.print(",");
            pw.print(0xFF & (col >> 8));
            pw.print(",");
            pw.print(0xFF & col);
            pw.print(")");
        }
    }

    public void printUnits(PrintWriter pw, int un[]) {
        String s = Float.toString(((float) un[0]) / 256f);
        if (s.endsWith(".0")) s = s.substring(0, s.length() - 2);
        pw.print(s);
        switch (un[1]) {   // 1 in, 2 cm, 3 mm, 4 pt, 5 pc, 6 %
            case 0:
                break;
            case 1:
                pw.print("in");
                break;
            case 2:
                pw.print("cm");
                break;
            case 3:
                pw.print("mm");
                break;
            case 4:
                pw.print("pt");
                break;
            case 5:
                pw.print("pc");
                break;
            case 6:
                pw.print("%");
                break;
        }
    }

    public static final int UPDATE_choice = IOUtils.updatable("choice");
    public static final int UPDATE_focusEast = IOUtils.updatable("navRight");
    public static final int UPDATE_focusWest = IOUtils.updatable("navLeft");

    public void print(PrintWriter pw, Attribute attributeName, Point i) {
        if (i instanceof UpdateValue) {
            UpdateValue uv = (UpdateValue) i;
            if (uv.escapeEnum != -1) {
                if (attributeName.isLaser && attributeName.attributeIndex == UPDATE_choice) {     // lineIncrement and animateMotion.rotate should also be here
                    printtyp_choiceType(pw, uv.escapeEnum - 4);
                } else if (attributeName.isLaser && attributeName.attributeIndex <= UPDATE_focusEast && attributeName.attributeIndex <= UPDATE_focusWest)
                {     // lineIncrement and animateMotion.rotate should also be here
                    if (uv.escapeEnum == 0) pw.print("auto");
                    else pw.print("self");
                } else {
                    pw.print("inherit");
                }
                return;
            }
            switch (uv.type) {
                case SDLClasses.TYPE_URI:
                    printURI(pw, uv);
                    break;
                case SDLClasses.TYPE_boolean:
                    pw.print(uv.x == 1);
                    break;
                case SDLClasses.TYPE_enum:
                    XMLWriterUtil.printEnumUpdate(pw, uv.x, attributeName.attributeIndex);
                    break;
                case SDLClasses.TYPE_ID:
                    printID(pw, uv.x);
                    break;
                case SDLClasses.TYPE_index:
                    pw.print(uv.x);
                    break;
                case SDLClasses.TYPE_color:
                    printColor(pw, uv.x);
                    break;
                case SDLClasses.TYPE_fraction:
                    printFloat(pw, uv.x);
                    //pw.print(((float)uv.x)/255f);
                    break;
                case SDLClasses.TYPE_float:
                    printFloat(pw, uv.x);
                    break;
                case SDLClasses.TYPE_time:
                    printDur(pw, uv.x);
                    break;
                case SDLClasses.TYPE_point:
                    printFloat(pw, uv.x);
                    pw.print(" ");
                    printFloat(pw, uv.y);
                    break;
                case SDLClasses.TYPE_matrix:
                    print(pw, uv.matrix);
                    break;
                case SDLClasses.TYPE_string:
                    print(pw, uv.string);
                    break;
                case SDLClasses.TYPE_points:
                    print(pw, uv.list);
                    break;
                case SDLClasses.TYPE_path:
                    print(pw, uv.path);
                    break;
                case SDLClasses.TYPE_ints:
                    for (int ii = 0; ii < uv.list.length; ii++) {
                        if (ii > 0) pw.print(" ");
                        pw.print(uv.list[ii]);
                    }
                    break;
                case SDLClasses.TYPE_floats: // floatValues
                    for (int ii = 0; ii < uv.list.length; ii++) {
                        if (ii > 0) pw.print(" ");
                        printFloat(pw, uv.list[ii]);
                    }
                    break;
                case SDLClasses.TYPE_smil_times: // timeValues
                    for (int ii = 0; ii < uv.times.length; ii++) {
                        if (ii > 0) pw.print(" ");
                        uv.times[ii].print(pw, scene.idMan);
                    }
                case SDLClasses.TYPE_unit: // unitValue
                    printUnits(pw, uv.list);
                    break;
                case SDLClasses.TYPE_event:
                    uv.event.print(pw);
                    break;
                case SDLClasses.ITYPE_point:
                case SDLClasses.TYPE_scale:
                    printFloat(pw, uv.x);
                    pw.print(" ");
                    printFloat(pw, uv.y);
                    break;
                case SDLClasses.ITYPE_int:
                    pw.print(uv.x);
                    break;
                case SDLClasses.ITYPE_float:
                case SDLClasses.TYPE_coordinate:
                    printFloat(pw, uv.x);
                    break;
                case SDLClasses.ITYPE_keytime:
                    printTime(pw, uv.x);
                    break;
                case SDLClasses.ITYPE_0to1:
                    printFloat(pw, uv.x);
                    break;
                default:
                    // error
                    throw new RuntimeException("unknown replace type:" + uv.type);
            }
        } else {
            printFloat(pw, i.x);
            pw.print(" ");
            printFloat(pw, i.y);
        }
    }

    public void print(PrintWriter pw, Matrix m) {
        pw.print("matrix(");
        printFloat(pw, m.xx);
        pw.print(",");
        printFloat(pw, m.yx);
        pw.print(",");
        printFloat(pw, m.xy);
        pw.print(",");
        printFloat(pw, m.yy);
        pw.print(",");
        printFloat(pw, m.xz);
        pw.print(",");
        printFloat(pw, m.yz);
        pw.print(")");
    }

    public void print(PrintWriter pw, byte a[]) {
        for (int i = 0; i < a.length; i++) {
            if (i > 0) pw.print(" ");
            pw.print(a[i]);
        }
    }

    public void printAttributeNameList(PrintWriter pw, Attribute a[]) {
        for (int i = 0; i < a.length; i++) {
            if (i > 0) pw.print(" ");
            XMLWriterUtil.printAttributeName(pw, a[i]);
        }
    }

    public void print(PrintWriter pw, int a[]) {
        for (int i = 0; i < a.length; i++) {
            if (i > 0) pw.print(" ");
            printFloat(pw, a[i]);
        }
    }

    public void printInts(PrintWriter pw, int a[]) {
        for (int i = 0; i < a.length; i++) {
            if (i > 0) pw.print(" ");
            print(pw, a[i]);
        }
    }

    public void print(PrintWriter pw, Attribute attributeName, AnimValue av) {
        if (av.escapeEnum != -1) {
            XMLWriterUtil.printEscapeEnumAnim(pw, av.escapeEnum, attributeName.attributeIndex);
            return;
        }
        switch (av.type) {
            case SDLClasses.ANIMTYPE_path:
                print(pw, av.path);
                break;
            case SDLClasses.ANIMTYPE_pointSeq:
                for (int ii = 0; ii < av.pointSeq.length; ii++) {
                    if (ii > 0) pw.print(" ");
                    printFloat(pw, av.pointSeq[ii]);
                }
                break;
            case SDLClasses.ANIMTYPE_fraction:
                pw.print(((float) av.fractionOrFloatOrColor) / 4096f);
                break;
            case SDLClasses.ANIMTYPE_color:
                printColor(pw, av.fractionOrFloatOrColor);
                break;
            case SDLClasses.ANIMTYPE_float:
                printFloat(pw, av.fractionOrFloatOrColor);
                break;
            case SDLClasses.ANIMTYPE_enum:
                XMLWriterUtil.printEnumAnim(pw, av.fractionOrFloatOrColor, attributeName.attributeIndex);
                break;
            case SDLClasses.ANIMTYPE_id:
                printID(pw, av.fractionOrFloatOrColor);
                break;
            case SDLClasses.ANIMTYPE_font:
            case SDLClasses.ANIMTYPE_string:
                pw.print(av.string);
                break;
            case SDLClasses.ANIMTYPE_floats:
                print(pw, av.tab);
                break;
            case SDLClasses.ANIMTYPE_uri:
                print(pw, av.uri);
                break;
                /*case SDLClasses.ANIMTYPE_ints:
                printInts(pw, av.tab);
                break;*/
            case SDLClasses.ANIMTYPE_point:
                print(pw, av.tab);
                break;
        }
    }

    public void print(PrintWriter pw, Attribute attributeName, AnimValue av[]) {
        for (int i = 0; i < av.length; i++) {
            if (i > 0) pw.print(";");
            print(pw, attributeName, av[i]);
        }
    }

    public void print(PrintWriter pw, Path path) {
        int index = 0, startx, starty;
        pw.print("M ");
        startx = path.pointSeq[index++];
        printFloat(pw, startx);
        pw.print(" ");
        starty = path.pointSeq[index++];
        printFloat(pw, starty);
        for (int i = 0; i < path.types.length; i++) {
            boolean relative = false, skipX = false, skipY = false;
            switch (path.types[i]) {
                case Xpath.SEGMENT_C:
                    pw.print(" C");
                    break;
                case Xpath.SEGMENT_c:
                    pw.print(" c");
                    relative = true;
                    break;
                case Xpath.SEGMENT_S:
                    pw.print(" S");
                    break;
                case Xpath.SEGMENT_s:
                    pw.print(" s");
                    relative = true;
                    break;
                case Xpath.SEGMENT_Q:
                    pw.print(" Q");
                    break;
                case Xpath.SEGMENT_q:
                    pw.print(" q");
                    relative = true;
                    break;
                case Xpath.SEGMENT_M:
                    pw.print(" M");
                    break;
                case Xpath.SEGMENT_m:
                    pw.print(" m");
                    relative = true;
                    break;
                case Xpath.SEGMENT_L:
                    pw.print(" L");
                    break;
                case Xpath.SEGMENT_l:
                    pw.print(" l");
                    relative = true;
                    break;
                case Xpath.SEGMENT_T:
                    pw.print(" T");
                    break;
                case Xpath.SEGMENT_t:
                    pw.print(" t");
                    relative = true;
                    break;
                case Xpath.SEGMENT_H:
                    pw.print(" H");
                    skipY = true;
                    break;
                case Xpath.SEGMENT_h:
                    pw.print(" h");
                    skipY = true;
                    relative = true;
                    break;
                case Xpath.SEGMENT_V:
                    pw.print(" V");
                    skipX = true;
                    break;
                case Xpath.SEGMENT_v:
                    pw.print(" v");
                    relative = true;
                    skipX = true;
                    break;
                case Xpath.SEGMENT_Z:
                    pw.print(" Z");
                    break;
                case Xpath.SEGMENT_z:
                    pw.print(" z");
                    relative = true;
                    break;
            }
            int max = Xpath.nbOfPrintedCoords(path.types[i]);
            for (int k = 0; k < max; k++) {
                if (relative) {
                    if (k == (max-2)) {
                        if (!skipX) {
                            pw.print(" ");
                            printFloat(pw, path.pointSeq[index] - startx);
                        }
                        startx = path.pointSeq[index];
                    } else if (k == (max - 1)) {
                        if (!skipY) {
                            pw.print(" ");
                            printFloat(pw, path.pointSeq[index] - starty);
                        }
                        starty = path.pointSeq[index];
                    } else if (k % 2 == 1 && !skipY) {
                        pw.print(" ");
                        printFloat(pw, path.pointSeq[index] - starty);
                    } else if (k % 2 == 0 && !skipX) {
                        pw.print(" ");
                        printFloat(pw, path.pointSeq[index] - startx);
                    }
                } else {
                    if (k == (max-2)) {
                        if (index >= path.pointSeq.length) continue;
                        startx = path.pointSeq[index];
                        if (!skipX) {
                            pw.print(" ");
                            printFloat(pw, startx);
                        }
                    } else if (k == (max - 1)) {
                        if (index >= path.pointSeq.length) continue;
                        starty = path.pointSeq[index];
                        if (!skipY) {
                            pw.print(" ");
                            printFloat(pw, starty);
                        }
                    } else {
                        pw.print(" ");
                        printFloat(pw, path.pointSeq[index]);
                    }
                }
                index++;
            }
        }
    }

// version erohee
//    public void print(PrintWriter pw, Path path) {
//        int index = 0, startx, starty;
//        pw.print("M ");
//        startx = path.pointSeq[index++];
//        printFloat(pw, startx);
//        pw.print(" ");
//        starty = path.pointSeq[index++];
//        printFloat(pw, starty);
//        for (int i = 0; i < path.types.length; i++) {
//            boolean relative = false, skipX = false, skipY = false;
//            switch (path.types[i]) {
//                case Xpath.SEGMENT_C:
//                    pw.print(" C");
//                    break;
//                case Xpath.SEGMENT_c:
//                    pw.print(" c");
//                    relative = true;
//                    break;
//                case Xpath.SEGMENT_S:
//                    pw.print(" S");
//                    break;
//                case Xpath.SEGMENT_s:
//                    pw.print(" s");
//                    relative = true;
//                    break;
//                case Xpath.SEGMENT_Q:
//                    pw.print(" Q");
//                    break;
//                case Xpath.SEGMENT_q:
//                    pw.print(" q");
//                    relative = true;
//                    break;
//                case Xpath.SEGMENT_M:
//                    pw.print(" M");
//                    break;
//                case Xpath.SEGMENT_m:
//                    pw.print(" m");
//                    relative = true;
//                    break;
//                case Xpath.SEGMENT_L:
//                    pw.print(" L");
//                    break;
//                case Xpath.SEGMENT_l:
//                    pw.print(" l");
//                    relative = true;
//                    break;
//                case Xpath.SEGMENT_T:
//                    pw.print(" T");
//                    break;
//                case Xpath.SEGMENT_t:
//                    pw.print(" t");
//                    relative = true;
//                    break;
//                case Xpath.SEGMENT_H:
//                    pw.print(" H");
//                    skipY = true;
//                    break;
//                case Xpath.SEGMENT_h:
//                    pw.print(" h");
//                    skipY = true;
//                    relative = true;
//                    break;
//                case Xpath.SEGMENT_V:
//                    pw.print(" V");
//                    skipX = true;
//                    break;
//                case Xpath.SEGMENT_v:
//                    pw.print(" v");
//                    relative = true;
//                    skipX = true;
//                    break;
//                case Xpath.SEGMENT_Z:
//                    pw.print(" Z");
//                    break;
//                case Xpath.SEGMENT_z:
//                    pw.print(" z");
//                    relative = true;
//                    break;
//            }
//            int max = Xpath.nbOfPrintedCoords(path.types[i]);
//            for (int k = 0; k < max; k++) {
//                /*if (relative) {
//                    if (k == (max-2)) {
//                        if (!skipX) {
//                            pw.print(" ");
//                            printFloat(pw, path.pointSeq[index] - startx);
//                        }
//                        startx = path.pointSeq[index];
//                    } else if (k == (max - 1)) {
//                        if (!skipY) {
//                            pw.print(" ");
//                            printFloat(pw, path.pointSeq[index] - starty);
//                        }
//                        starty = path.pointSeq[index];
//                    } else if (k % 2 == 1 && !skipY) {
//                        pw.print(" ");
//                        printFloat(pw, path.pointSeq[index] - starty);
//                    } else if (k % 2 == 0 && !skipX) {
//                        pw.print(" ");
//                        printFloat(pw, path.pointSeq[index] - startx);
//                    }
//                } else */{
//                    if (k == (max-2)) {
//                        if (index >= path.pointSeq.length) continue;
//                        startx = path.pointSeq[index];
//                        if (!skipX) {
//                            pw.print(" ");
//                            printFloat(pw, startx);
//                        }
//                    } else if (k == (max - 1)) {
//                        if (index >= path.pointSeq.length) continue;
//                        starty = path.pointSeq[index];
//                        if (!skipY) {
//                            pw.print(" ");
//                            printFloat(pw, starty);
//                        }
//                    } else {
//                        pw.print(" ");
//                        printFloat(pw, path.pointSeq[index]);
//                    }
//                }
//                index++;
//            }
//        }
//    }

    public void print(PrintWriter pw, Xsvg o) {
        ((LASeRXMLWriter) classToWriter.get(Xsvg.class)).write(pw, o);
    }

    public void printtyp_choiceType(PrintWriter pw, int i) {
        switch (i) {
            case -1 :
                pw.print("none");
                break;
            case -2 :
                pw.print("all");
                break;
            default:
                pw.print(i);
        }
    }

    {
        classToWriter.put(XtextContent.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XtextContent xo = (XtextContent) o;
                printWithEntities(pw, xo.getContent());
            }
        });
    }

    {
        classToWriter.put(String.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                printWithEntities(pw, (String) o);
            }
        });
    }

    public void printWithEntities(PrintWriter pw, String s) {
        for (int i = 0; i < s.length(); i++) {
            char c = s.charAt(i);
            if (c == '&') {
                pw.print("&amp;");
            } else if (c == '\'') {
                pw.print("&apos;");
            } else if (c == '"') {
                pw.print("&quot;");
            } else if (c == '<') {
                pw.print("&lt;");
            } else if (c == '>') {
                pw.print("&gt;");
            } else {
                pw.print(c);
            }
        }
    }

    {
        classToWriter.put(Scene.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Scene xo = (Scene) o;
                pw.print("<saf:SAFSession xmlns:saf='urn:mpeg:mpeg4:SAF:2005' xmlns:lsr='urn:mpeg:mpeg4:LASeR:2005' xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink' xmlns:smil='http://www.w3.org/2005/SMIL21' xmlns:ev='http://www.w3.org/2001/xml-events'");
                for (int i = 0; i < NameSpace.nameSpaces.size(); i++) {
                    NameSpace n = (NameSpace) NameSpace.nameSpaces.elementAt(i);
                    pw.print(" xmlns:");
                    pw.print(n.prefix);
                    pw.print("='");
                    pw.print(n.uri);
                    pw.print("'");
                }
                nameSpacesSaved = true;
                pw.println(">");
                if (xo.size() > 0) {
                    for (int i = 0; i < xo.size(); i++) {
                        writeAny(pw, xo.elementAt(i));
                    }
                }
                pw.println("</saf:SAFSession>");
            }
        });
    }

    {
        classToWriter.put(SceneHeader.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                SceneHeader xo = (SceneHeader) o;
                pw.print("<saf:sceneHeader");
                if (xo.time > 0) {
                    pw.print(" time='");
                    printTime(pw, xo.time);
                    pw.print("'");
                }
                if (xo.objectTypeIndication != -1) {
                    pw.print(" objectTypeIndication='");
                    pw.print(xo.objectTypeIndication);
                    pw.print("'");
                }
                if (xo.streamID != null) {
                    pw.print(" streamID='");
                    print(pw, xo.streamID);
                    pw.print("'");
                }
                if (xo.streamType != MPEG4Constants.VisualStreamType && xo.streamType != -1) {
                    pw.print(" streamType='");
                    pw.print(xo.streamType);
                    pw.print("'");
                }
                if (xo.bufferSize > 0) {
                    pw.print(" bufferSize='");
                    pw.print(xo.bufferSize);
                    pw.print("'");
                }
                if (xo.isPermanent) {
                    pw.print(" permanent='true'");
                }
                if (xo.mimeType != null) {
                    pw.print(" mimeType='");
                    pw.print(xo.mimeType);
                    pw.print("'");
                }
                if (xo.lh != null) {
                    pw.println(">");
                    writeAny(pw, xo.lh);
                    pw.println("</saf:sceneHeader>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(ImageHeader.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                ImageHeader xo = (ImageHeader) o;
                pw.print("<saf:imageHeader");
                if (xo.time > 0) {
                    pw.print(" time='");
                    printTime(pw, xo.time);
                    pw.print("'");
                }
                if (xo.objectTypeIndication != -1) {
                    pw.print(" objectTypeIndication='");
                    pw.print(xo.objectTypeIndication);
                    pw.print("'");
                }
                if (xo.streamID != null) {
                    pw.print(" streamID='");
                    print(pw, xo.streamID);
                    pw.print("'");
                }
                if (xo.streamType != MPEG4Constants.VisualStreamType && xo.streamType != -1) {
                    pw.print(" streamType='");
                    pw.print(xo.streamType);
                    pw.print("'");
                }
                if (xo.bufferSize > 0) {
                    pw.print(" bufferSize='");
                    pw.print(xo.bufferSize);
                    pw.print("'");
                }
                if (xo.isPermanent) {
                    pw.print(" permanent='true'");
                }
                if (xo.mimeType != null) {
                    pw.print(" mimeType='");
                    pw.print(xo.mimeType);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }

    {
        classToWriter.put(MediaHeader.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                MediaHeader xo = (MediaHeader) o;
                pw.print("<saf:mediaHeader");
                if (xo.time > 0) {
                    pw.print(" time='");
                    printTime(pw, xo.time);
                    pw.print("'");
                }
                if (xo.objectTypeIndication != -1) {
                    pw.print(" objectTypeIndication='");
                    pw.print(xo.objectTypeIndication);
                    pw.print("'");
                }
                if (xo.streamID != null) {
                    pw.print(" streamID='");
                    print(pw, xo.streamID);
                    pw.print("'");
                }
                if (xo.streamType != -1) {
                    pw.print(" streamType='");
                    pw.print(xo.streamType);
                    pw.print("'");
                }
                if (xo.bufferSize > 0) {
                    pw.print(" bufferSize='");
                    pw.print(xo.bufferSize);
                    pw.print("'");
                }
                if (xo.isPermanent) {
                    pw.print(" permanent='true'");
                }
                if (xo.mimeType != null) {
                    pw.print(" mimeType='");
                    pw.print(xo.mimeType);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }

    {
        classToWriter.put(EndOfSAFSession.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                pw.println("<saf:endOfSAFSession/>");
            }
        });
    }

    {
        classToWriter.put(EndOfStream.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                pw.print("<saf:endOfStream ref='");
                pw.print(((EndOfStream) o).streamID);
                pw.println("'/>");
            }
        });
    }

    {
        classToWriter.put(SceneUnit.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                SceneUnit xo = (SceneUnit) o;
                pw.print("<saf:sceneUnit");
                if (xo.time > 0) {
                    pw.print(" time='");
                    printTime(pw, xo.time);
                    pw.print("'");
                }
                if (xo.resetEncodingContext != false) {
                    pw.print(" ");
                    pw.print("resetEncodingContext");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.resetEncodingContext);
                    pw.print("'");
                }
                if (xo.streamID != null) {
                    pw.print(" ref='");
                    print(pw, xo.streamID);
                    pw.print("'");
                }
                pw.println(">");
                if (xo.stringIDTable != null && xo.stringIDTable.strings != null) {
                    pw.print("<lsr:StringIDTable strings='");
                    for (int i = 0; i < xo.stringIDTable.strings.size(); i++) {
                        if (i > 0) pw.print(" ");
                        pw.print((String)xo.stringIDTable.strings.elementAt(i));
                    }
                    pw.print("'/>");
                }
                if (xo.updates.size() > 0) {
                    for (int i = 0; i < xo.updates.size(); i++) {
                        writeAny(pw, xo.updates.elementAt(i));
                    }
                }
                pw.println("</saf:sceneUnit>");
            }
        });
    }

    {
        classToWriter.put(ImageUnit.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                ImageUnit xo = (ImageUnit) o;
                pw.print("<saf:imageUnit ref='");
                pw.print(xo.streamID);
                int header = -1;
                try {
                    header = new DataInputStream(xo.resource.getInputStream()).readShort();
                } catch (Exception e) {}
                if ((header & 0xFFFF) == 0xFFD8) {
                    pw.print("' source='data:image/jpeg;base64,");
                } else {
                    pw.print("' source='data:image/png;base64,");
                }
                pw.print(Base64.encodeBytes(xo.resource.getBytes()));
                pw.println("'/>");
            }
        });
    }

    {
        classToWriter.put(MediaUnit.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                MediaUnit xo = (MediaUnit) o;
                pw.print("<saf:mediaUnit ref='");
                pw.print(xo.streamID);
                pw.print("' source='data:;base64,");
                if (xo.resource != null) pw.print(Base64.encodeBytes(xo.resource.getBytes()));
                pw.println("'/>");
            }
        });
    }

    {
        classToWriter.put(FontUnit.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                FontUnit xo = (FontUnit) o;
                pw.print("<saf:fontUnit ref='");
                pw.print(xo.streamID);
                pw.print("' source='data:;base64,");
                pw.print(Base64.encodeBytes(xo.resource.getBytes()));
                pw.println("'/>");
            }
        });
    }

    {
        classToWriter.put(CacheObject.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                //TODO
            }
        });
    }

    {
        classToWriter.put(FontHeader.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                //TODO
            }
        });
    }

    {
        classToWriter.put(RemoteStreamHeader.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                RemoteStreamHeader xo = (RemoteStreamHeader) o;
                pw.print("<saf:RemoteStreamHeader");
                if (xo.time > 0) {
                    pw.print(" time='");
                    printTime(pw, xo.time);
                    pw.print("'");
                }
                if (xo.objectTypeIndication != -1) {
                    pw.print(" objectTypeIndication='");
                    pw.print(xo.objectTypeIndication);
                    pw.print("'");
                }
                if (xo.streamID != null) {
                    pw.print(" streamID='");
                    print(pw, xo.streamID);
                    pw.print("'");
                }
                if (xo.streamType != -1) {
                    pw.print(" streamType='");
                    pw.print(xo.streamType);
                    pw.print("'");
                }
                if (xo.bufferSize > 0) {
                    pw.print(" bufferSize='");
                    pw.print(xo.bufferSize);
                    pw.print("'");
                }
                if (xo.source != null) {
                    pw.print(" source='");
                    pw.print(xo.source);
                    pw.print("'");
                }
                if (xo.isPermanent) {
                    pw.print(" permanent='true'");
                }
                if (xo.mimeType != null) {
                    pw.print(" mimeType='");
                    pw.print(xo.mimeType);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }

    public void printRepeat(PrintWriter pw, int i) {
        if (i == Integer.MAX_VALUE) pw.print("indefinite");
        else printFloat(pw, i);
    }

    public void printRepeatCount(PrintWriter pw, int i) {
        if (i == Integer.MAX_VALUE) pw.print("indefinite");
        else printFloat(pw, i);
    }

    /*public void printKeyCodeList(PrintWriter pw, byte a[]) {
        for (int i = 0; i < a.length; i++) {
            if (i > 0) pw.print(" ");
            XMLWriterUtil.printKeyCode(pw, a[i]);
        }
    }*/

    {
        classToWriter.put(XprivateElementContainer.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XprivateElementContainer pe = (XprivateElementContainer) o;
                if (pe.anyXML != null) {
                    for (int i = 0; i < pe.anyXML.size(); i++) {
                        writeAny(pw, pe.anyXML.elementAt(i));
                    }
                }
            }
        });
    }

    {
        classToWriter.put(XprivateElement.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XprivateElement pe = (XprivateElement) o;
                pw.print("<");
                if (pe.name.ns != null) {
                    pw.print(pe.name.ns.prefix);
                    pw.print(":");
                }
                pw.print(pe.name.name);
                if (pe.id != Integer.MIN_VALUE) {
                    pw.print(" id='");
                    printID(pw, pe.id);
                    pw.print("'");
                }
                if (pe.privateAttributes != null) {
                    for (int i = 0; i < pe.privateAttributes.length; i++) {
                        pw.print(" ");
                        if (pe.privateAttributes[i].ns != null) {
                            pw.print(pe.privateAttributes[i].ns.prefix);
                            pw.print(":");
                        }
                        pw.print(pe.privateAttributes[i].name);
                        pw.print("='");
                        pw.print(pe.privateAttributes[i].value);
                        pw.print("'");
                    }
                }
                if (!nameSpacesSaved) {
                    for (int i = 0; i < NameSpace.nameSpaces.size(); i++) {
                        NameSpace n = (NameSpace) NameSpace.nameSpaces.elementAt(i);
                        pw.print(" xmlns:");
                        pw.print(n.prefix);
                        pw.print("='");
                        pw.print(n.uri);
                        pw.print("'");
                    }
                    nameSpacesSaved = true;
                }
                if (pe.child != null && pe.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < pe.child.size(); i++) {
                        writeAny(pw, pe.child.elementAt(i));
                    }
                    pw.print("</");
                    if (pe.name.ns != null) {
                        pw.print(pe.name.ns.prefix);
                        pw.print(":");
                    }
                    pw.print(pe.name.name);
                    pw.println(">");
                } else {
                    pw.println("/>");
                }
            }
        });
    }


}
