/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.io;

import iso.mpeg.streamezzo.laser.*;
import iso.mpeg.streamezzo.resource.Resource;
import iso.mpeg.streamezzo.saf.*;
import iso.mpeg.streamezzo.sdl.*;
import iso.mpeg.streamezzo.util.*;
import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.io.EOFException;
import java.io.Reader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Stack;
import java.util.Vector;

public abstract class XMLParser extends DefaultHandler {
    public static final boolean debug = false;
    Scene scene = null;
    Parent root = null;
    Hashtable XMLElements = new Hashtable();
    Hashtable SVGElements = new Hashtable();
    Hashtable LASeRElements = new Hashtable();
    Hashtable XMLEventsElements = new Hashtable();
    Hashtable SAFElements = new Hashtable();

    {
        XMLElements.put("http://www.w3.org/2000/svg", SVGElements);
        XMLElements.put("urn:mpeg:mpeg4:LASeR:2005", LASeRElements);
        XMLElements.put("urn:mpeg:mpeg4:SAF:2005", SAFElements);
        XMLElements.put("http://www.w3.org/2001/xml-events", XMLEventsElements);
    }

    public void parseProperties(XpropElem o, XMLParser xr, Attributes atts) {
        String s = atts.getValue("audio-level");
        if (s != null) {
            o.audio_level = xr.readFloat(s);
        }
        s = atts.getValue("class");
        if (s != null) {
            o.classe = s;
        }
        s = atts.getValue("color");
        if (s != null) {
            o.color = xr.readColor(s);
        }
        s = atts.getValue("color-rendering");
        if (s != null) {
            o.color_rendering = ((XMLParserImpl) xr).readtyp_color_renderingType(s);
        }
        s = atts.getValue("display");
        if (s != null) {
            o.display = ((XMLParserImpl) xr).readtyp_displayType(s);
        }
        s = atts.getValue("display-align");
        if (s != null) {
            o.display_align = ((XMLParserImpl) xr).readtyp_displayAlignType(s);
        }
        s = atts.getValue("fill");
        if (s != null) {
            o.fill = xr.readColor(s);
        }
        s = atts.getValue("fill-opacity");
        if (s != null) {
            o.fill_opacity = xr.readFloat(s);
        }
        s = atts.getValue("fill-rule");
        if (s != null) {
            o.fill_rule = ((XMLParserImpl) xr).readtyp_fill_ruleType(s);
        }
        s = atts.getValue("focusable");
        if (s != null) {
            o.focusable = (((XMLParserImpl)xr).readBoolean(s) ? 1 : 0);
        }
        s = atts.getValue("font-family");
        if (s != null) {
            o.font_family = xr.readFontFamily(s);
        }
        s = atts.getValue("font-size");
        if (s != null) {
            o.font_size = xr.readFloat(s);
        }
        s = atts.getValue("font-style");
        if (s != null) {
            o.font_style = ((XMLParserImpl) xr).readtyp_font_styleType(s);
        }
        s = atts.getValue("text-decoration");
        if (s != null) {
            o.text_decoration = ((XMLParserImpl) xr).readtyp_text_decorationType(s);
        }
        s = atts.getValue("font-variant");
        if (s != null) {
            o.font_variant = ((XMLParserImpl) xr).readtyp_font_variantType(s);
        }
        s = atts.getValue("font-weight");
        if (s != null) {
            o.font_weight = ((XMLParserImpl) xr).readtyp_font_weightType(s);
        }
        s = atts.getValue("image-rendering");
        if (s != null) {
            o.image_rendering = ((XMLParserImpl) xr).readtyp_image_renderingType(s);
        }
        s = atts.getValue("line-increment");
        if (s != null) {
            o.line_increment = ((XMLParserImpl) xr).readtyp_lineIncrement(s);
        }
        s = atts.getValue("pointer-events");
        if (s != null) {
            o.pointer_events = ((XMLParserImpl) xr).readtyp_pointer_eventsType(s);
        }
        s = atts.getValue("shape-rendering");
        if (s != null) {
            o.shape_rendering = ((XMLParserImpl) xr).readtyp_shape_renderingType(s);
        }
        s = atts.getValue("solid-color");
        if (s != null) {
            o.solid_color = xr.readColor(s);
        }
        s = atts.getValue("solid-opacity");
        if (s != null) {
            o.solid_opacity = xr.readFloat(s);
        }
        s = atts.getValue("stop-color");
        if (s != null) {
            o.stop_color = xr.readColor(s);
        }
        s = atts.getValue("stop-opacity");
        if (s != null) {
            o.stop_opacity = xr.readFloat(s);
        }
        s = atts.getValue("stroke");
        if (s != null) {
            o.stroke = xr.readColor(s);
        }
        s = atts.getValue("stroke-dasharray");
        if (s != null) {
            o.stroke_dasharray = xr.readFloatList(s);
        }
        s = atts.getValue("stroke-dashoffset");
        if (s != null) {
            o.stroke_dashoffset = xr.readFloat(s);
        }
        s = atts.getValue("stroke-linecap");
        if (s != null) {
            o.stroke_linecap = ((XMLParserImpl) xr).readtyp_stroke_linecapType(s);
        }
        s = atts.getValue("stroke-linejoin");
        if (s != null) {
            o.stroke_linejoin = ((XMLParserImpl) xr).readtyp_stroke_linejoinType(s);
        }
        s = atts.getValue("stroke-miterlimit");
        if (s != null) {
            o.stroke_miterlimit = xr.readFloat(s);
        }
        s = atts.getValue("stroke-opacity");
        if (s != null) {
            o.stroke_opacity = xr.readFloat(s);
        }
        s = atts.getValue("stroke-width");
        if (s != null) {
            o.stroke_width = xr.readFloat(s);
        }
        s = atts.getValue("text-anchor");
        if (s != null) {
            o.text_anchor = ((XMLParserImpl) xr).readtyp_textAnchorType(s);
        }
        s = atts.getValue("text-rendering");
        if (s != null) {
            o.text_rendering = ((XMLParserImpl) xr).readtyp_text_renderingType(s);
        }
        s = atts.getValue("vector-effect");
        if (s != null) {
            o.vector_effect = ((XMLParserImpl) xr).readtyp_vector_effectType(s);
        }
        s = atts.getValue("visibility");
        if (s != null) {
            o.visibility = ((XMLParserImpl) xr).readtyp_visibilityType(s);
        }
        // MGR : nav-* attributes can be a local IDREF or a string equals to "auto" or "self" (focus)
        s = atts.getValue("nav-prev");
        if (s != null) {
            //o.navPrev = ((XMLParserImpl)xr).readID(s);
            o.navPrev = readNav(s);
        }
        s = atts.getValue("nav-next");
        if (s != null) {
            //o.navNext = ((XMLParserImpl)xr).readID(s);
            o.navNext = readNav(s);
        }
        s = atts.getValue("nav-up");
        if (s != null) {
            //o.navUp = ((XMLParserImpl)xr).readID(s);
            o.navUp = readNav(s);
        }
        s = atts.getValue("nav-down");
        if (s != null) {
            //o.navDown = ((XMLParserImpl)xr).readID(s);
            o.navDown = readNav(s);
        }
        s = atts.getValue("nav-up-left");
        if (s != null) {
            //o.navUpLeft = ((XMLParserImpl)xr).readID(s);
            o.navUpLeft = readNav(s);
        }
        s = atts.getValue("nav-down-left");
        if (s != null) {
            //o.navDownLeft = ((XMLParserImpl)xr).readID(s);
            o.navDownLeft = readNav(s);
        }
        s = atts.getValue("nav-up-right");
        if (s != null) {
            //o.navUpRight = ((XMLParserImpl)xr).readID(s);
            o.navUpRight = readNav(s);
        }
        s = atts.getValue("nav-down-right");
        if (s != null) {
            //o.navDownRight = ((XMLParserImpl)xr).readID(s);
            o.navDownRight = readNav(s);
        }
        s = atts.getValue("nav-left");
        if (s != null) {
            //o.navLeft = ((XMLParserImpl)xr).readID(s);
            o.navLeft = readNav(s);
        }
        s = atts.getValue("nav-right");
        if (s != null) {
            //o.navRight = ((XMLParserImpl)xr).readID(s);
            o.navRight = readNav(s);
        }
        s = atts.getValue("viewport-fill");
        if (s != null) {
            o.viewport_fill = xr.readColor(s);
        }
        s = atts.getValue("viewport-fill-opacity");
        if (s != null) {
            o.viewport_fill_opacity = xr.readFloat(s);
        }
        s = atts.getValue("focusHighlight");
        if (s != null) {
            o.focusHighlight = xr.readEnum(s, "focusHighlight");
        }
    }

    public void parse(Reader reader, Scene scene) throws IOException, SAXException {
        this.scene = scene;
        XMLReader xr = new SAXParser();
        xr.setContentHandler(this);
        xr.parse(new InputSource(reader));
        SDLClasses.currentScene = scene;
    }

    public void parseAny(Reader reader, Scene scene) throws IOException, SAXException {
        this.scene = scene;
        root = scene;
        XMLReader xr = new SAXParser();
        xr.setContentHandler(this);
        xr.parse(new InputSource(reader));
        SDLClasses.currentScene = scene;
    }

    public Resource getResource(String s) {
        return Resource.create(s, scene.dir);
    }

    private Stack nodeStack = new Stack();

    public void startDocument() {
    }

    public void endDocument() {
    }

    public void startElement(String uri, String name, String qName, Attributes atts) {
        if (debug) {
            System.out.println("start " + name);
        }
        Object o = XMLElements.get(uri);
        if (o instanceof Hashtable) o = ((Hashtable)o).get(name);
        if (o instanceof LASeRXMLReader) {
            Object res = ((LASeRXMLReader) o).read(this, atts);
            if (root != null) root.addChild(res);
            nodeStack.push(root);
            root = (Parent) res;
        } else {
            //System.out.println("reading private "+uri+":"+name);
            Object res = read(this, atts, uri, name, qName.substring(0, Math.max(0,qName.length() - (name.length() + 1))));
            if (root != null) root.addChild(res);
            nodeStack.push(root);
            root = (Parent) res;
        }
    }

    public void endElement(String uri, String name, String qName) {
        if (debug) {
            System.out.println("end " + name);
        }
        root = (Parent)nodeStack.pop();
        if (debug) {
            System.out.println("back to:" + root);
        }
    }

    public void characters(char ch[], int start, int length) {
        if (textOk() || root instanceof XprivateElement) {
            //System.out.println("Taking: "+new String(ch, start, length));
            String n = new String(ch, start, length);
            //n = n.trim();
            if (n.length() > 0) root.addChild(n);
        } else {
            //System.out.println("Leaving out: "+new String(ch, start, length));
        }
    }

    private boolean textOk() {
        return (root instanceof Xtspan) ||
                (root instanceof Xtext) ||
                (root instanceof XtextArea) ||
                (root instanceof Xscript && "application/ecmaScript".equalsIgnoreCase(((Xscript)root).type)) ||
                (root instanceof Xdesc) ||
                (root instanceof Xtitle) ||
                (root instanceof XReplace) ||
                (root instanceof Xmetadata);
    }

    public boolean readBoolean(String s) { return "true".equalsIgnoreCase(s); }

    public int readInt(String s) { return Integer.parseInt(s); }

    public int readIntExcept(String s, String exc) {
        if (exc.equals(s)) return -1;
        return Integer.parseInt(s);
    }

    public int readRepeatDur(String s, String exc) {
        if (exc.equals(s)) return Integer.MAX_VALUE;
        return readFloat(s);
    }

    public SmilTime[] readBeginEnd(String s) {
        if ("indefinite".equals(s)) return new SmilTime[] {new SmilTime()};
        if (s.indexOf(";") >= 0) {
            Vector v = Utils.strings(s);
            SmilTime st[] = new SmilTime[v.size()];
            for (int i = 0; i < v.size(); i++) {
                String s1 = (String)v.elementAt(i);
                st[i] = new SmilTime().read(s1, scene.idMan);
            }
            return st;
        } else {
            return new SmilTime[] {new SmilTime().read(s, scene.idMan)};
        }
    }

    public int readNav(String s) {
        int value = 0;
        if (s.startsWith("#")) value = scene.idMan.getOrCreateNodeID(s.substring(1));
        if (s.equals("auto"))  value = Integer.MAX_VALUE;
        if (s.equals("self"))  value = Integer.MAX_VALUE - 1;
        return value;
    }

    public int readID(String s) {
        if (s.startsWith("#")) return scene.idMan.getOrCreateNodeID(s.substring(1));
        return scene.idMan.getOrCreateNodeID(s);
    }

    public URI readURI(String s) {
        URI u = new URI();
        if (s.startsWith("#")) {
            u.id = scene.idMan.getOrCreateNodeID(s.substring(1));
            return u;
        }
        if (s.startsWith("stream:")) {
            u.streamID = scene.idMan.getOrCreateStreamID(s.substring(7));
            return u;
        }
        u.uri = s;
        return u;
    }

    public URI[] readURIs(String s) {
        Vector v = Utils.strings(s, ' ');
        URI u[] = new URI[v.size()];
        for (int i = 0; i < v.size(); i++) u[i] = readURI((String)v.elementAt(i));
        return u;
    }

    public void readURI(String s, UpdateValue uv) {
        if (s.startsWith("#")) {
            uv.x = scene.idMan.getOrCreateNodeID(s.substring(1));
            return;
        }
        if (s.startsWith("stream:")) {
            uv.y = scene.idMan.getOrCreateStreamID(s.substring(7));
            return;
        }
        uv.string = s;
        return;
    }

    public int readFloat(String s) {
        s = s.trim();
        if (s.endsWith("px")) s = s.substring(0, s.length()-2);
        return (int) (Float.parseFloat(s) * 4096);
    }

    public int readDur(String s) {
        if ("indefinite".equals(s)) return Integer.MIN_VALUE;
        if ("none".equals(s)) return Integer.MAX_VALUE-1;
        if ("media".equals(s)) return Integer.MAX_VALUE;
        if ("auto".equals(s)) return Integer.MIN_VALUE+1;
        float factor = 1f;
        if (s.endsWith("ms")) {
            s = s.substring(0, s.length()-2);
            factor = 0.001f;
        }
        if (s.endsWith("s")) {
            s = s.substring(0, s.length()-1);
        }
        return (int) (Float.parseFloat(s) * factor * 4096);
    }

    public int readClip(String s) {
        if ("indefinite".equals(s)) return Integer.MIN_VALUE;
        if ("media".equals(s)) return Integer.MAX_VALUE;
        if ("none".equals(s)) return Integer.MAX_VALUE-1;
        if ("auto".equals(s)) return Integer.MIN_VALUE+1;
        float factor = 1f;
        if (s.endsWith("ms")) {
            s = s.substring(0, s.length()-2);
            factor = 0.001f;
        }
        if (s.endsWith("s")) {
            s = s.substring(0, s.length()-1);
        }
        return (int) (Float.parseFloat(s) * factor * 4096);
    }

    public int readFloatExcept(String s, String exc, String exc2) {
        if (exc.equals(s)) return Integer.MAX_VALUE;
        if (exc2.equals(s)) return Integer.MAX_VALUE-1;
        return (int) (Float.parseFloat(s) * 4096);
    }

    public int readFloatExcept(String s, String exc) {
        if (exc.equals(s)) return Integer.MAX_VALUE;
        return (int) (Float.parseFloat(s) * 4096);
    }

    public int readRepeatCount(String s, String exc) {
        if (exc.equals(s)) return Integer.MAX_VALUE;
        return (int) (Float.parseFloat(s) * 4096);
    }

    public int readColor(String s) {
        int col = 0;
        s = s.trim();
        if (s.equals("inherit")) { return -3; }
        if (s.equals("none")) { return -2; }
        if (s.equals("currentColor")) { return -1; }
        if (s.startsWith("url(") && s.endsWith(")")) {
            s = s.substring(4, s.length()-1);
            return -4-scene.idMan.registerPaintURI(readURI(s));
        } else if (s.startsWith("#") && s.length() == 7) {
            s = s.substring(1);
            col += Integer.parseInt(s.substring(0, 2), 16) << 16;
            col += Integer.parseInt(s.substring(2, 4), 16) << 8;
            col += Integer.parseInt(s.substring(4), 16);
            scene.man.registerColor(col);
            return col;
        } else if (s.startsWith("#") && s.length() == 4) {
            s = s.substring(1);
            col += Integer.parseInt(s.substring(0, 1), 16) << 20;
            col += Integer.parseInt(s.substring(1, 2), 16) << 12;
            col += Integer.parseInt(s.substring(2), 16) << 4;
            scene.man.registerColor(col);
            return col;
        } else if (s.startsWith("rgb(")) {
            FloatAndStringListReader fr = new FloatAndStringListReader(s.substring(4));
            try {
                col += ((int) fr.nextFloat()) << 16;
                col += ((int) fr.nextFloat()) << 8;
                col += ((int) fr.nextFloat());
            } catch (Exception e) {
            }
            scene.man.registerColor(col);
            return col;
        } else {
            FloatAndStringListReader fr = new FloatAndStringListReader(s);
            try {
                if (fr.nextIsString()) {
                    s = Utils.translateColors(s);
                    fr = new FloatAndStringListReader(s.substring(4));
                    try {
                        col += ((int) fr.nextFloat()) << 16;
                        col += ((int) fr.nextFloat()) << 8;
                        col += ((int) fr.nextFloat());
                    } catch (Exception e) {
                    }
                } else {
                    col += ((int) fr.nextFloat()) << 16;
                    col += ((int) fr.nextFloat()) << 8;
                    col += ((int) fr.nextFloat());
                }
            } catch (Exception e) {
            }
            scene.man.registerColor(col);
            return col;
        }
    }

    private static int tab[] = new int[1024];
    private static String tabS[] = new String[1024];

    public int[] readFloatList(String s) {
        FloatAndStringListReader fsr = new FloatAndStringListReader(s);
        int i = 0;
        try {
            while (true) {
                tab[i] = (int) (fsr.nextFloat() * 4096f);
                i++;
            }
        } catch (EOFException a) {}
        int res[] = new int[i];
        System.arraycopy(tab, 0, res, 0, i);
        return res;
    }

    public Point readPoint(String s) {
        FloatAndStringListReader fsr = new FloatAndStringListReader(s);
        Point p = new Point();
        try {
            p.x = (int) (fsr.nextFloat() * 4096f);
            p.y = (int) (fsr.nextFloat() * 4096f);
        } catch (EOFException a) {}
        return p;
    }

    public Matrix readMatrix(String s) {
        if (s.startsWith("translate(")) {
            s = s.substring(10);
            FloatAndStringListReader fsr = new FloatAndStringListReader(s);
            Matrix p = new Matrix();
            try {
                p.xz = (int) (fsr.nextFloat() * 4096f);
                p.xzValid = true;
                p.yz = (int) (fsr.nextFloat() * 4096f);
                p.yzValid = true;
            } catch (EOFException a) {}
            return p;
        }
        if (s.startsWith("scale(")) {
            s = s.substring(6);
            FloatAndStringListReader fsr = new FloatAndStringListReader(s);
            Matrix p = new Matrix();
            try {
                p.xx = (int) (fsr.nextFloat() * 4096f);
                p.xxValid = true;
                p.yy = (int) (fsr.nextFloat() * 4096f);
                p.yyValid = true;
            } catch (EOFException a) {}
            return p;
        }
        if (s.startsWith("matrix(")) s = s.substring(7);
        FloatAndStringListReader fsr = new FloatAndStringListReader(s);
        Matrix p = new Matrix();
        try {
            p.xx = (int) (fsr.nextFloat() * 4096f);
            p.xxValid = true;
            p.yx = (int) (fsr.nextFloat() * 4096f);
            p.yxValid = true;
            p.xy = (int) (fsr.nextFloat() * 4096f);
            p.xyValid = true;
            p.yy = (int) (fsr.nextFloat() * 4096f);
            p.yyValid = true;
            p.xz = (int) (fsr.nextFloat() * 4096f);
            p.xzValid = true;
            p.yz = (int) (fsr.nextFloat() * 4096f);
            p.yzValid = true;
        } catch (EOFException a) {}
        return p;
    }

    public byte[] readFeatureList(String s) {
        FloatAndStringListReader fsr = new FloatAndStringListReader(s);
        int i = 0;
        try {
            tabS[i++] = fsr.nextString();
        } catch (EOFException a) {}
        byte res[] = new byte[i];
        for (int k = 0; k < i; k++) {
            res[k] = (byte) readtyp_featureType(tabS[k]);
        }
        return res;
    }

    public abstract int readtyp_featureType(String s);

    public abstract int readtyp_AttributeNameType(String s);

    public abstract int readtyp_animatable(String s);

    public abstract int readEnum(String s, String aN);

    public int[] readIntList(String s) {
        IntAndStringListReader fsr = new IntAndStringListReader(s);
        int i = 0;
        try {
            while (true) {
                tab[i] = (int) (fsr.nextInt());
                i++;
            }
        } catch (EOFException a) {}
        int res[] = new int[i];
        System.arraycopy(tab, 0, res, 0, i);
        return res;
    }

    public AnimValue readAnimValue(String s, Object o, String attributeName) {
        AnimValue av = new AnimValue();
        if (o instanceof XanimateMotion || o instanceof XanimateTransform) {
            av.type = SDLClasses.animType[readtyp_animatable("transform")];
            attributeName = "transform";
        } else {
            try {
            av.type = SDLClasses.animType[readtyp_animatable(attributeName)];
            } catch(ArrayIndexOutOfBoundsException aioob) {
                System.out.println("error with attributeName:"+attributeName);
                throw aioob;
            }
        }
        return readAnimValue1(s, av, attributeName);
    }

    private AnimValue readAnimValue1(String s, AnimValue av, String attributeName) {
        if (s.equals("inherit") && av.type != SDLClasses.ANIMTYPE_enum) {
            av.escapeEnum = 0;
            return av;
        }
        if (attributeName != null && attributeName.equals("choice"))
        {
            if (s.equals("all")) { av.escapeEnum = 0; return av; }
            if (s.equals("clip")) { av.escapeEnum = 1; return av; }
            if (s.equals("delta")) { av.escapeEnum = 2; return av; }
            if (s.equals("none")) { av.escapeEnum = 3; return av; }
        }
        switch (av.type) {
            case SDLClasses.ANIMTYPE_path:
                av.path = readPath(s);
                break;
            case SDLClasses.ANIMTYPE_pointSeq:
                av.pointSeq = readFloatList(s);
                break;
            case SDLClasses.ANIMTYPE_fraction:
                av.fractionOrFloatOrColor = readFloat(s);
                break;
            case SDLClasses.ANIMTYPE_color:
                av.fractionOrFloatOrColor = readColor(s);
                break;
            case SDLClasses.ANIMTYPE_float:
                av.fractionOrFloatOrColor = readFloat(s);
                break;
            case SDLClasses.ANIMTYPE_enum:
                av.fractionOrFloatOrColor = readEnum(s, attributeName);
                break;
            case SDLClasses.ANIMTYPE_id:
                av.fractionOrFloatOrColor = readID(s);
                break;
            case SDLClasses.ANIMTYPE_font:
                av.string = readFontFamily(s);
                break;
            case SDLClasses.ANIMTYPE_string:
                av.string = s;
                break;
            case SDLClasses.ANIMTYPE_floats:
                av.tab = readFloatList(s);
                break;
            /*case SDLClasses.ANIMTYPE_ints:
                av.tab = readIntList(s);
                break;*/
            case SDLClasses.ANIMTYPE_uri:
                av.uri = readURI(s);
                break;
            case SDLClasses.ANIMTYPE_point:
                av.tab = readFloatList(s);
                break;
        }
        return av;
    }

    public Path readPath(String s) {
        Path path = new Path();
        Vector types = new Vector();
        Vector points = new Vector();
        try {
            FloatAndStringListReader fr = new FloatAndStringListReader(s);
            int type = -1;
            int xx = 0, yy = 0, startx = 0, starty = 0;
            while (true) {
                if (fr.nextIsString()) {
                    type = Xpath.typeFromString(fr.nextString());
                }
                types.addElement(new Integer(type));
                int nb = Xpath.nbOfPrintedCoords(type % 100);
                if (type == Xpath.SEGMENT_z || type == Xpath.SEGMENT_Z) {
                    //z or Z, do nothing
                } else if (type == Xpath.SEGMENT_H) { //H
                    // horiz absolute
                    startx = (int) (fr.nextFloat() * 4096f);
                    points.addElement(new Integer(startx));
                    points.addElement(new Integer(starty));
                } else if (type == Xpath.SEGMENT_h) { //h
                    //relative
                    startx = startx + (int) (fr.nextFloat() * 4096f);
                    points.addElement(new Integer(startx));
                    points.addElement(new Integer(starty));
                } else if (type == Xpath.SEGMENT_V) { //V
                    // vertical absolute
                    starty = (int) (fr.nextFloat() * 4096f);
                    points.addElement(new Integer(startx));
                    points.addElement(new Integer(starty));
                } else if (type == Xpath.SEGMENT_v) { //v
                    //relative
                    starty = starty + (int) (fr.nextFloat() * 4096);
                    points.addElement(new Integer(startx));
                    points.addElement(new Integer(starty));
                } else if (type < Xpath.SEGMENT_c) {
                    for (int u = 2; u < nb; u += 2) {
                        xx = (int) (fr.nextFloat() * 4096f);
                        yy = (int) (fr.nextFloat() * 4096f);
                        points.addElement(new Integer(xx));
                        points.addElement(new Integer(yy));
                    }
                    startx = (int) (fr.nextFloat() * 4096f);
                    starty = (int) (fr.nextFloat() * 4096f);
                    points.addElement(new Integer(startx));
                    points.addElement(new Integer(starty));
                } else {
                    //relative
                    for (int u = 2; u < nb; u += 2) {
                        xx = startx + (int) (fr.nextFloat() * 4096f);
                        yy = starty + (int) (fr.nextFloat() * 4096f);
                        points.addElement(new Integer(xx));
                        points.addElement(new Integer(yy));
                    }
                    startx = startx + (int) (fr.nextFloat() * 4096f);
                    starty = starty + (int) (fr.nextFloat() * 4096f);
                    points.addElement(new Integer(startx));
                    points.addElement(new Integer(starty));
                }
            }
        } catch (EOFException e) {
        }
        path.pointSeq = new int[points.size()];
        for (int i = 0; i < points.size(); i++) {
            path.pointSeq[i] = ((Integer) points.elementAt(i)).intValue();
            //System.out.print(path.pointSeq[i]>>12);
            //System.out.print(" ");
        }
        //System.out.println();
        if (types.size() > 1) {
            path.types = new byte[types.size()-1]; // initial M omitted
            for (int i = 0; i < path.types.length; i++) {
                path.types[i] = ((Integer) types.elementAt(i+1)).byteValue();
            }
        }
        return path;
    }

    public int[] readValueWithUnits(String s) {
        int u[] = new int[2];
        int i = s.indexOf("in");
        if (i > 0) {
            u[1] = 1;
            u[0] = ((int)(Float.parseFloat(s.substring(0, i))*256f));
            return u;
        }
        i = s.indexOf("cm");
        if (i > 0) {
            u[1] = 2;
            u[0] = ((int)(Float.parseFloat(s.substring(0, i))*256f));
            return u;
        }
        i = s.indexOf("mm");
        if (i > 0) {
            u[1] = 3;
            u[0] = ((int)(Float.parseFloat(s.substring(0, i))*256f));
            return u;
        }
        i = s.indexOf("pt");
        if (i > 0) {
            u[1] = 4;
            u[0] = ((int)(Float.parseFloat(s.substring(0, i))*256f));
            return u;
        }
        i = s.indexOf("pc");
        if (i > 0) {
            u[1] = 5;
            u[0] = ((int)(Float.parseFloat(s.substring(0, i))*256f));
            return u;
        }
        i = s.indexOf("px");
        if (i > 0) {
            u[1] = 0;
            u[0] = ((int)(Float.parseFloat(s.substring(0, i))*256f));
            return u;
        }
        i = s.indexOf("%");
        if (i > 0) {
            u[1] = 6;
            u[0] = ((int)(Float.parseFloat(s.substring(0, i))*256f));
            return u;
        }
        u[1] = 0;
        u[0] = ((int)(Float.parseFloat(s)*256f));
        return u;
    }

    private static int tempInts[] = null;

    public abstract int readtyp_choiceType(String s);

    private boolean dealWithMixedEnumTypes(String s, String attributeName, UpdateValue uv) {
        if (attributeName.equals("choice")) {
            int i = readtyp_choiceType(s);
            if (i != Integer.MIN_VALUE) {
                uv.escapeEnum = i;
                return true;
            }
        }
        return false;
    }

    public UpdateValue readUpdateValue(String s, String attributeName, String index) {
        UpdateValue uv = null;
        if (index == null) {
            uv = new UpdateValue(SDLClasses.typeOf(readAttributeName(attributeName), -1));
            if (uv.type != SDLClasses.TYPE_enum) {
                if (s.equals("inherit")) {
                    uv.escapeEnum = 0;
                    return uv;
                } else if (dealWithMixedEnumTypes(s, attributeName, uv)) {
                    return uv;
                }
            }
            switch (uv.type) {
                case SDLClasses.TYPE_URI:
                    readURI(s, uv);
                    return uv;
                case SDLClasses.TYPE_boolean:
                    uv.x = ("true".equals(s) ? 1 : 0);
                    return uv;
                case SDLClasses.TYPE_enum:
                    uv.x = readEnum(s, attributeName);
                    break;
                case SDLClasses.TYPE_index:
                    if (attributeName.equals("font-family")) {
                        uv.x = scene.man.registerFont(s);
                    } else {
                        uv.x = readInt(s);
                    }
                    break;
                case SDLClasses.TYPE_ID:
                    if (s.startsWith("#")) {
                        uv.x = scene.idMan.getOrCreateNodeID(s.substring(1));
                    } else {
                        throw new RuntimeException("in update, wrong syntax for ID: "+s);
                    }
                    break;
                case SDLClasses.TYPE_color:
                    uv.x = readColor(s);
                    break;
                case SDLClasses.TYPE_fraction:
                    uv.x = readFloat(s);
                    break;
                case SDLClasses.TYPE_float:
                    uv.x = readFloat(s);
                    break;
                case SDLClasses.TYPE_time:
                    uv.x = readDur(s);
                    break;
                case SDLClasses.TYPE_point:
                    tempInts = readFloatList(s);
                    uv.x = tempInts[0];
                    uv.y = tempInts[1];
                    break;
                case SDLClasses.TYPE_matrix:
                    tempInts = readFloatList(s.substring(7));
                    uv.matrix = new Matrix(tempInts);
                    break;
                case SDLClasses.TYPE_string:
                    uv.string = s;
                    break;
                case SDLClasses.TYPE_points:
                    uv.list = readFloatList(s);
                    break;
                case SDLClasses.TYPE_path:
                    uv.path = readPath(s);
                    break;
                case SDLClasses.TYPE_ints:
                    uv.list = readIntList(s);
                    break;
                case SDLClasses.TYPE_floats: // floatValues
                    uv.list = readFloatList(s);
                    break;
                case SDLClasses.TYPE_smil_times: // timeValues
                    uv.times = readBeginEnd(s);
                    break;
                case SDLClasses.TYPE_unit: // unitValue
                    uv.list = readValueWithUnits(s);
                    break;
                case SDLClasses.TYPE_event:
                    uv.event = new Event().read(s);
                    break;
                case SDLClasses.TYPE_scale:
                    tempInts = readFloatList(s);
                    uv.x = tempInts[0];
                    uv.y = tempInts[1];
                    break;
                case SDLClasses.TYPE_coordinate:
                    uv.x = readFloat(s);
                    break;
                default:
                    // error
                    throw new RuntimeException("unknown replace type:"+uv.type);
            }
        } else {
            uv = new UpdateValue(SDLClasses.typeOf(readAttributeName(attributeName), 1));
            switch (uv.type) {
                case SDLClasses.ITYPE_point:
                    tempInts = readFloatList(s);
                    uv.x = tempInts[0];
                    uv.y = tempInts[1];
                    break;
                case SDLClasses.ITYPE_int:
                    uv.x = readInt(s);
                    break;
                case SDLClasses.ITYPE_float:
                    uv.x = readFloat(s);
                    break;
                case SDLClasses.ITYPE_keytime:
                    uv.x = readIntExcept(s, "indefinite");
                    break;
                default:
                    // error
                    throw new RuntimeException("unknown replace type:"+uv.type);
            }
        }
        return uv;
    }

    public ScrollStop readScrollStopValue(String s) {
        ScrollStop ssv = new ScrollStop();
        s = s.trim();
        if (s.startsWith("#")) {
            // id
            ssv.idRef = readID(s.substring(1));
        } else {
            // point
            ssv.value = readPoint(s);
        }
        return ssv;
    }

    public AnimValue[] readAnimValues(String s, Object o, String attributeName) {
        int attrIndex;
        if (o instanceof XanimateMotion || o instanceof XanimateTransform) {
            attrIndex = readtyp_animatable("transform");
        } else {
            attrIndex = readtyp_animatable(attributeName);
        }
        Vector v = Utils.strings(s);
        int max = v.size();
        AnimValue tab[] = new AnimValue[max];
        for (int i = 0; i < max; i++) {
            AnimValue av = new AnimValue();
            av.type = SDLClasses.animType[attrIndex];
            tab[i] = readAnimValue1((String) v.elementAt(i), av, attributeName);
        }
        return tab;
    }

    public int readChoice(String s) {
        if ("none".equals(s))
            return -1;
        else if ("delta".equals(s))
            return -2;
        else if ("clip".equals(s))
            return -3;
        else if ("all".equals(s))
            return -4;
        else
            return readInt(s);
    }

    public Attribute[] readAttrNameList(String s) {
        Vector v = Utils.strings(s, ' ');
        Attribute a[] = new Attribute[v.size()];
        for (int i = 0; i < v.size(); i++) {
            a[i] = readAttributeName((String)v.elementAt(i));
        }
        return a;
    }

    public abstract int readtyp_keyCode(String s);

    /*public byte[] readKeyList(String s) {
        Vector v = Utils.strings(s, ' ');
        byte a[] = new byte[v.delta()];
        for (int i = 0; i < v.delta(); i++) {
            a[i] = (byte)readKeyCode((String)v.elementAt(i));
        }
        return a;
    }*/

    public int[] readIDList(String s) {
        Vector v = Utils.strings(s, ' ');
        int a[] = new int[v.size()];
        for (int i = 0; i < v.size(); i++) {
            a[i] = (byte)readID((String)v.elementAt(i));
        }
        return a;
    }

    public String readFontFamily(String s) {
        scene.man.registerFont(s);
        return s;
    }

    {
        SAFElements.put("SAFSession", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                return scene;
            }
        });
    }

    {
        LASeRElements.put("textContent", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                return new XtextContent();
            }
        });
    }

    {
        LASeRElements.put("codecInitialisations", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                return null;
            }
        });
    }

    {
        SAFElements.put("sceneUnit", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                SceneUnit o = new SceneUnit();
                String s = null;
                s = atts.getValue("streamID");
                if (s != null) {
                    o.streamID = s;
                } else {
                    o.streamID = "defaultMainSceneID";
                }
                s = atts.getValue("ref");
                if (s != null) {
                    o.streamID = s;
                }
                s = atts.getValue("rap");
                if (s != null) {
                    o.isRAP = readBoolean(s);
                }
                s = atts.getValue("time");
                if (s != null) {
                    o.time = readInt(s);   // TODO
                }
                s = atts.getValue("resetEncodingContext");
                if (s != null) {
                    o.resetEncodingContext = xr.readBoolean(s);
                }
                return o;
            }
        });
    }

    {
        SAFElements.put("mediaUnit", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                MediaUnit o = new MediaUnit();
                String s = null;
                s = atts.getValue("ref");
                if (s != null) {
                    o.streamID = s;
                }
                s = atts.getValue("startOffset");
                if (s != null) {
                    o.startOffset = readInt(s);
                }
                s = atts.getValue("length");
                if (s != null) {
                    o.length = readInt(s);
                }
                s = atts.getValue("time");
                if (s != null) {
                    o.time = readInt(s);      // TODO
                }
                s = atts.getValue("source");
                if (s != null) {
                    if (s.startsWith("data:;base64,")) {
                        o.resource = Resource.create(atts.getValue("ref") + o.time + ".media",
                                                     Base64.decode(s.substring(23)));
                    } else {
                        o.resource = getResource(s);
                    }
                }
                s = atts.getValue("rap");
                if (s != null) {
                    o.setRAP(readBoolean(s));
                }
                return o;
            }
        });
    }

    {
        SAFElements.put("fontUnit", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                FontUnit o = new FontUnit();
                String s = null;
                s = atts.getValue("ref");
                if (s != null) {
                    o.streamID = s;
                }
                s = atts.getValue("startOffset");
                if (s != null) {
                    o.startOffset = readInt(s);
                }
                s = atts.getValue("length");
                if (s != null) {
                    o.length = readInt(s);
                }
                s = atts.getValue("time");
                if (s != null) {
                    o.time = readInt(s);   // TODO
                }
                s = atts.getValue("source");
                if (s != null) {
                    if (s.startsWith("data:;base64,")) {
                        o.resource = Resource.create(atts.getValue("ref") + o.time + ".media",
                                                     Base64.decode(s.substring(23)));
                    } else {
                        o.resource = getResource(s);
                    }
                }
                s = atts.getValue("rap");
                if (s != null) {
                    o.setRAP(readBoolean(s));
                }
                return o;
            }
        });
    }

    {
        SAFElements.put("imageUnit", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                ImageUnit o = new ImageUnit();
                String s = null;
                s = atts.getValue("ref");
                if (s != null) {
                    o.streamID = s;
                }
                s = atts.getValue("source");
                if (s != null) {
                    if (s.startsWith("data:")) {
                        if (s.startsWith("data:image/jpeg;base64,")) {
                            o.resource = Resource.create(atts.getValue("ref") + ".jpg", Base64.decode(s.substring(23)));
                        } else if (s.startsWith("data:image/jpg;base64,")) {
                            o.resource = Resource.create(atts.getValue("ref") + ".jpg", Base64.decode(s.substring(22)));
                        } else if (s.startsWith("data:image/png;base64,")) {
                            o.resource = Resource.create(atts.getValue("ref") + ".png", Base64.decode(s.substring(22)));
                        } else
                            throw new RuntimeException("unknown data url");
                    } else {
                        o.resource = getResource(s);
                    }
                }
                return o;
            }
        });
    }

    {
        SAFElements.put("mediaHeader", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                MediaHeader o = new MediaHeader();
                String s = null;
                s = atts.getValue("streamID");
                if (s != null) {
                    o.streamID = s;
                }
                s = atts.getValue("streamType");
                if (s != null) {
                    o.streamType = (byte) readInt(s);
                }
                s = atts.getValue("objectTypeIndication");
                if (s != null) {
                    o.objectTypeIndication = (byte) readInt(s);
                }
                s = atts.getValue("source");
                if (s != null) {
                    o.resource = getResource(s);
                }
                return o;
            }
        });
    }

    {
        SAFElements.put("imageHeader", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                ImageHeader o = new ImageHeader();
                String s = null;
                s = atts.getValue("streamID");
                if (s != null) {
                    o.streamID = s;
                }
                s = atts.getValue("streamType");
                if (s != null) {
                    o.streamType = (byte) readInt(s);
                }
                s = atts.getValue("objectTypeIndication");
                if (s != null) {
                    o.objectTypeIndication = (byte) readInt(s);
                }
                s = atts.getValue("source");
                if (s != null) {
                    o.resource = getResource(s);
                }
                return o;
            }
        });
    }

    {
        SAFElements.put("fontHeader", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                FontHeader o = new FontHeader();
                String s = null;
                s = atts.getValue("streamID");
                if (s != null) {
                    o.streamID = s;
                }
                s = atts.getValue("streamType");
                if (s != null) {
                    o.streamType = (byte) readInt(s);
                }
                s = atts.getValue("objectTypeIndication");
                if (s != null) {
                    o.objectTypeIndication = (byte) readInt(s);
                }
                s = atts.getValue("source");
                if (s != null) {
                    o.resource = getResource(s);
                }
                return o;
            }
        });
    }

    {
        SAFElements.put("sceneHeader", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                SceneHeader o = new SceneHeader();
                String s = null;
                s = atts.getValue("streamID");
                if (s != null) {
                    o.streamID = s;
                } else {
                    o.streamID = "defaultMainSceneID";
                }
                scene.idMan.getOrCreateStreamID(o.streamID);
                s = atts.getValue("streamType");
                if (s != null) {
                    o.streamType = (byte) readInt(s);
                }
                s = atts.getValue("objectTypeIndication");
                if (s != null) {
                    o.objectTypeIndication = (byte) readInt(s);
                }
                return o;
            }
        });
    }

    {
        SAFElements.put("RemoteStreamHeader", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                RemoteStreamHeader o = new RemoteStreamHeader();
                String s = null;
                s = atts.getValue("streamType");
                if (s != null) {
                    o.streamType = (byte) readInt(s);
                }
                s = atts.getValue("objectTypeIndication");
                if (s != null) {
                    o.objectTypeIndication = (byte) readInt(s);
                }
                s = atts.getValue("source");
                if (s != null) {
                    o.source = s;
                }
                s = atts.getValue("streamID");
                if (s != null) {
                    o.streamID = s;
                }
                return o;
            }
        });
    }

    {
        SAFElements.put("endOfStream", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                EndOfStream o = new EndOfStream();
                String s = null;
                s = atts.getValue("ref");
                if (s != null) {
                    o.streamID = s;
                }
                return o;
            }
        });
    }

    {
        SAFElements.put("endOfSAFSession", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                EndOfSAFSession o = new EndOfSAFSession();
                return o;
            }
        });
    }

    {
        SAFElements.put("cacheObject", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                CacheObject o = new CacheObject();
                String s = null;
                s = atts.getValue("url");
                if (s != null) {
                    o.url = s;
                }
                s = atts.getValue("endTime");
                if (s != null) {
                    o.endTime = readInt(s);
                }
                s = atts.getValue("startOffset");
                if (s != null) {
                    o.startOffset = readInt(s);
                }
                s = atts.getValue("length");
                if (s != null) {
                    o.length = readInt(s);
                }
                s = atts.getValue("source");
                if (s != null) {
                    o.resource = getResource(s);
                }
                s = atts.getValue("permanent");
                if (s != null) {
                    o.isPermanent = readBoolean(s);
                }
                s = atts.getValue("replace");
                if (s != null) {
                    o.isReplace = readBoolean(s);
                }
                return o;
            }
        });
    }


    public Object read(XMLParser xr, Attributes atts, String uri, String tag, String prefix) {
        XprivateElement o = new XprivateElement();
        o.name.ns = NameSpace.get(uri, prefix);
        scene.man.registerNamespace(o.name.ns);
        o.name.name = tag;
        int max = atts.getLength();
        String s = atts.getValue("id");
        if (s != null) {
            o.id = scene.idMan.getOrCreateNodeID(s);
            o.privateAttributes = new privAttr[max-1];
        } else {
            o.privateAttributes = new privAttr[max];
        }
        for (int i = 0, index = 0; i < max; i++) {
            if (atts.getLocalName(i).equals("id")) continue;
            o.privateAttributes[index] = new privAttr();
            o.privateAttributes[index].name = atts.getLocalName(i);
            s = atts.getURI(i);
            String spref = atts.getQName(i);
            spref = spref.substring(0, spref.length() - o.privateAttributes[index].name.length());
            if (spref.endsWith(":")) spref = spref.substring(0, spref.length()-1);
            o.privateAttributes[index].ns = ((s == null || "".equals(s)) ? o.name.ns : NameSpace.get(s, spref));
            o.privateAttributes[index].value = atts.getValue(i);
            index++;
        }
        scene.man.registerTag(o.name, o.privateAttributes);
        return o;
    }

    public QName parseQName(String s) {
        QName n = new QName();
        int colon = s.indexOf(":");
        if (colon < 0) {
            n.name = s;
        } else {
            n.ns = scene.man.getNameSpace(s.substring(0, colon));
            n.name = s.substring(colon+1);
        }
        return n;
    }

    public Attribute readAttributeName(String s) {
        Attribute a = new Attribute();
        int i = readtyp_AttributeNameType(s);
        if (i == Integer.MIN_VALUE) {
            a.isLaser = false;
            int dot = s.indexOf(".");
            if (dot < 0) {
                a.attr = parseQName(s);
            } else {
                a.tag = parseQName(s.substring(0, dot));
                a.attr = parseQName(s.substring(dot+1));
            }
        } else {
            a.isLaser = true;
            a.attributeIndex = i;
        }
        return a;
    }
}
