/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.io;

import iso.mpeg.streamezzo.laser.*;
import iso.mpeg.streamezzo.sdl.SDLClasses;
import iso.mpeg.streamezzo.sdl.Event;
import iso.mpeg.streamezzo.sdl.URI;
import iso.mpeg.streamezzo.util.Utils;
import org.xml.sax.Attributes;

public class XMLParserImpl extends XMLParser {
    public int readEnum(String s, String attributeName) {
        if (attributeName.equals("zoomAndPan")) {
            return readtyp_zoomAndPanType(s);
        }
        if (attributeName.equals("encoding")) {
            return readtyp_encodingType(s);
        }
        if (attributeName.equals("pointsCodec")) {
            return readtyp_pointsCodecType(s);
        }
        if (attributeName.equals("profile")) {
            return readtyp_profileType(s);
        }
        if (attributeName.equals("color-rendering")) {
            return readtyp_color_renderingType(s);
        }
        if (attributeName.equals("display")) {
            return readtyp_displayType(s);
        }
        if (attributeName.equals("display-align")) {
            return readtyp_displayAlignType(s);
        }
        if (attributeName.equals("editable")) {
            return readtyp_editable(s);
        }
        if (attributeName.equals("fill-rule")) {
            return readtyp_fill_ruleType(s);
        }
        if (attributeName.equals("fill")) {
            return readtyp_fill(s);
        }
        if (attributeName.equals("font-style")) {
            return readtyp_font_styleType(s);
        }
        if (attributeName.equals("font-variant")) {
            return readtyp_font_variantType(s);
        }
        if (attributeName.equals("font-weight")) {
            return readtyp_font_weightType(s);
        }
        if (attributeName.equals("image-rendering")) {
            return readtyp_image_renderingType(s);
        }
        if (attributeName.equals("pointer-events")) {
            return readtyp_pointer_eventsType(s);
        }
        if (attributeName.equals("shape-rendering")) {
            return readtyp_shape_renderingType(s);
        }
        if (attributeName.equals("stroke-linecap")) {
            return readtyp_stroke_linecapType(s);
        }
        if (attributeName.equals("stroke-linejoin")) {
            return readtyp_stroke_linejoinType(s);
        }
        if (attributeName.equals("text-anchor")) {
            return readtyp_textAnchorType(s);
        }
        if (attributeName.equals("text-rendering")) {
            return readtyp_text_renderingType(s);
        }
        if (attributeName.equals("vector-effect")) {
            return readtyp_vector_effectType(s);
        }
        if (attributeName.equals("visibility")) {
            return readtyp_visibilityType(s);
        }
        if (attributeName.equals("xlink-actuate")) {
            return readactuate_anonymousType_1(s);
        }
        if (attributeName.equals("xlink-show")) {
            return readshow_anonymousType_3(s);
        }
        if (attributeName.equals("xlink-type")) {
            return readtype_anonymousType_2(s);
        }
        if (attributeName.equals("space")) {
            return readspace_anonymousType_4(s);
        }
        if (attributeName.equals("calcMode")) {
            return readtyp_calcModeType(s);
        }
        if (attributeName.equals("attributeName") || attributeName.equals("animatable")) {
            return readtyp_animatable(s);
        }
        if (attributeName.equals("fill") || attributeName.equals("animFill")) {
            return readtyp_animFillType(s);
        }
        if (attributeName.equals("restart")) {
            return readtyp_restartType(s);
        }
        if (attributeName.equals("type")) {
            return readtyp_rotscatra(s);
        }
        if (attributeName.equals("syncBehavior")) {
            return readtyp_syncBehaviorType(s);
        }
        if (attributeName.equals("syncBehaviorDefault")) {
            return readtyp_syncBehaviorDefaultType(s);
        }
        if (attributeName.equals("transformBehavior")) {
            return readtyp_transformBehaviorType(s);
        }
        if (attributeName.equals("gradient-units") || attributeName.equals("gradientUnits")) {
            return readtyp_gradientUnits(s);
        }
        if (attributeName.equals("attributeName") || attributeName.equals("operandAttribute") || attributeName.equals("AttributeName")) {
            return readtyp_AttributeNameType(s);
        }
        if (attributeName.equals("preserveAspectRatio")) {
            return readtyp_preserveAspectRatioType(s);
        }
        if (attributeName.equals("event")) {
            return readtyp_eventType(s);
        }
        /*if (attributeName.equals("timeAttribute")) {
            return readtyp_beginEndType(s);
        }*/
        if (attributeName.equals("overflow")) {
            return readtyp_overflowType(s);
        }
        if (attributeName.equals("additive")) {
            return readtyp_additiveType(s);
        }
        if (attributeName.equals("accumulate")) {
            return readtyp_accumulateType(s);
        }
        if (attributeName.equals("choice")) {
            return super.readInt(s);
        }
        if (attributeName.equals("direction")) {
            return read_direction(s);
        }
        if (attributeName.equals("focusHighlight")) {
            return read_focusHighlight(s);
        }
        throw new RuntimeException("program error in readEnum:"+attributeName);
    }

    public int read_focusHighlight(String s) {
        if ("auto".equals(s)) return 0;
        if ("none".equals(s)) return 1;
        return Integer.MIN_VALUE;
    }

    public int read_direction(String s) {
        if ("up".equals(s)) return 3;
        if ("down".equals(s)) return 0;
        if ("left".equals(s)) return 1;
        if ("right".equals(s)) return 2;
        return Integer.MIN_VALUE;
    }

    public int readtyp_encodingType(String s) {
        if ("SimpleEncoding".equals(s)) return 1;
        if ("BiMEncoding".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readModeType(String s) {
        if ("useOld".equals(s)) return 3;
        if ("replace".equals(s)) return 2;
        if ("playList".equals(s)) return 1;
        if ("keepOld".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_zoomAndPanType(String s) {
        if ("magnify".equals(s)) return 1;
        if ("disable".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    // MGR : add method to retrieve value of line-increment parameter
    public int readtyp_lineIncrement(String s) {
        if ("auto".equals(s)) return Integer.MAX_VALUE;
        if ("inherit".equals(s)) return (Integer.MAX_VALUE - 1);
        if (!(("inherit".equals(s)) || ("auto".equals(s)))) readFloat(s);
        return Integer.MIN_VALUE;
    }

    public int readtyp_additiveType(String s) {
        if ("sum".equals(s)) return 1;
        if ("replace".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_accumulateType(String s) {
        if ("sum".equals(s)) return 1;
        if ("none".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_pointsCodecType(String s) {
        if ("ExpGolombPointsCodec".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_profileType(String s) {
        if ("full".equals(s)) return 0;
        if ("mini".equals(s)) return 1;
        return Integer.MIN_VALUE;
    }

    public int readcolorType(String s) {
        if ("inherit".equals(s)) return 0;
        if ("none".equals(s)) return 1;
        return Integer.MIN_VALUE;
    }

    public int readtyp_color_renderingType(String s) {
        if ("inherit".equals(s)) return 1;
        if ("optimizeQuality".equals(s)) return 2;
        if ("optimizeSpeed".equals(s)) return 3;
        if ("auto".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_displayType(String s) {
        if ("none".equals(s)) return 7;
        if ("table-column-group".equals(s)) return 13;
        if ("inline".equals(s)) return 3;
        if ("table-header-group".equals(s)) return 15;
        if ("table-caption".equals(s)) return 10;
        if ("list-item".equals(s)) return 5;
        if ("inline-table".equals(s)) return 4;
        if ("marker".equals(s)) return 6;
        if ("block".equals(s)) return 0;
        if ("compact".equals(s)) return 1;
        if ("table-footer-group".equals(s)) return 14;
        if ("table-cell".equals(s)) return 11;
        if ("inherit".equals(s)) return 2;
        if ("table".equals(s)) return 9;
        if ("table-column".equals(s)) return 12;
        if ("run-in".equals(s)) return 8;
        if ("table-row-group".equals(s)) return 17;
        if ("table-row".equals(s)) return 16;
        return Integer.MIN_VALUE;
    }

    public int readtyp_displayAlignType(String s) {
        if ("inherit".equals(s)) return 4;
        if ("center".equals(s)) return 3;
        if ("before".equals(s)) return 2;
        if ("after".equals(s)) return 0;
        if ("auto".equals(s)) return 1;
        return Integer.MIN_VALUE;
    }

    public int readtyp_editable(String s) {
        if ("true".equals(s)) return 1;
        if ("false".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_fill_ruleType(String s) {
        if ("evenodd".equals(s)) return 0;
        if ("inherit".equals(s)) return 1;
        if ("nonzero".equals(s)) return 2;
        return Integer.MIN_VALUE;
    }

    public int readtyp_fill(String s) {
        if ("replace".equals(s)) return 1;
        if ("freeze".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_fontFamilyType(String s) {
        if ("inherit".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_font_styleType(String s) {
        if ("oblique".equals(s)) return 3;
        if ("inherit".equals(s)) return 0;
        if ("italic".equals(s)) return 1;
        if ("normal".equals(s)) return 2;
        return Integer.MIN_VALUE;
    }

    public int readtyp_text_decorationType(String s) {
        if ("LASeR_EMBOSS".equals(s)) return 0;
        if ("LASeR_ENGRAVE".equals(s)) return 1;
        if ("LASeR_LEFTDROPSHADOW".equals(s)) return 2;
        if ("LASeR_OUTLINE".equals(s)) return 3;
        if ("LASeR_RIGHTDROPSHADOW".equals(s)) return 4;
        if ("inherit".equals(s)) return 5;
        if ("none".equals(s)) return 6;
        if ("underline".equals(s)) return 7;
        return Integer.MIN_VALUE;
    }

    public int readtyp_font_variantType(String s) {
        if ("normal".equals(s)) return 1;
        if ("smallcaps".equals(s)) return 2;
        if ("inherit".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_font_weightType(String s) {
        if ("200".equals(s)) return 1;
        if ("700".equals(s)) return 6;
        if ("300".equals(s)) return 2;
        if ("800".equals(s)) return 7;
        if ("inherit".equals(s)) return 11;
        if ("400".equals(s)) return 3;
        if ("normal".equals(s)) return 13;
        if ("900".equals(s)) return 8;
        if ("bold".equals(s)) return 9;
        if ("500".equals(s)) return 4;
        if ("100".equals(s)) return 0;
        if ("bolder".equals(s)) return 10;
        if ("lighter".equals(s)) return 12;
        if ("600".equals(s)) return 5;
        return Integer.MIN_VALUE;
    }

    public int readtyp_image_renderingType(String s) {
        if ("inherit".equals(s)) return 1;
        if ("optimizeQuality".equals(s)) return 2;
        if ("optimizeSpeed".equals(s)) return 3;
        if ("auto".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_pointer_eventsType(String s) {
        if ("all".equals(s)) return 0;
        if ("boundingBox".equals(s)) return 1;
        if ("fill".equals(s)) return 2;
        if ("inherit".equals(s)) return 3;
        if ("none".equals(s)) return 4;
        if ("painted".equals(s)) return 5;
        if ("stroke".equals(s)) return 6;
        if ("visible".equals(s)) return 7;
        if ("visibleFill".equals(s)) return 8;
        if ("visiblePainted".equals(s)) return 9;
        if ("visibleStroke".equals(s)) return 10;
        return Integer.MIN_VALUE;
    }

    public int readtyp_shape_renderingType(String s) {
        if ("auto".equals(s)) return 0;
        if ("crispEdges".equals(s)) return 1;
        if ("geometricPrecision".equals(s)) return 2;
        if ("inherit".equals(s)) return 3;
        if ("optimizeSpeed".equals(s)) return 4;
        return Integer.MIN_VALUE;
    }

    public int readtyp_stroke_linecapType(String s) {
        if ("round".equals(s)) return 2;
        if ("butt".equals(s)) return 0;
        if ("inherit".equals(s)) return 1;
        if ("square".equals(s)) return 3;
        return Integer.MIN_VALUE;
    }

    public int readtyp_stroke_linejoinType(String s) {
        if ("round".equals(s)) return 3;
        if ("inherit".equals(s)) return 1;
        if ("bevel".equals(s)) return 0;
        if ("miter".equals(s)) return 2;
        return Integer.MIN_VALUE;
    }

    public int readtyp_textAnchorType(String s) {
        if ("end".equals(s)) return 0;
        if ("middle".equals(s)) return 2;
        if ("inherit".equals(s)) return 1;
        if ("start".equals(s)) return 3;
        return Integer.MIN_VALUE;
    }

    public int readtyp_text_renderingType(String s) {
        if ("geometricPrecision".equals(s)) return 1;
        if ("inherit".equals(s)) return 2;
        if ("optimizeLegibility".equals(s)) return 3;
        if ("optimizeSpeed".equals(s)) return 4;
        if ("auto".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_visibilityType(String s) {
        if ("inherit".equals(s)) return 1;
        if ("hidden".equals(s)) return 0;
        if ("visible".equals(s)) return 2;
        return Integer.MIN_VALUE;
    }

    public int readtyp_vector_effectType(String s) {
        if ("inherit".equals(s)) return 1;
        if ("none".equals(s)) return 0;
        if ("non-scaling-stroke".equals(s)) return 2;
        return Integer.MIN_VALUE;
    }

    public int readtyp_featureType(String s) {
        for (int i = 0; i < IOUtils.featureNames.length; i++) {
            if (IOUtils.featureNames[i].equals(s)) return i;
        }
        return Integer.MIN_VALUE;
    }

    public int readactuate_anonymousType_1(String s) {
        if ("other".equals(s)) return 3;
        if ("none".equals(s)) return 0;
        if ("onLoad".equals(s)) return 1;
        if ("onRequest".equals(s)) return 2;
        return Integer.MIN_VALUE;
    }

    public int readshow_anonymousType_3(String s) {
        if ("other".equals(s)) return 3;
        if ("replace".equals(s)) return 4;
        if ("none".equals(s)) return 2;
        if ("new".equals(s)) return 1;
        if ("embed".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtype_anonymousType_2(String s) {
        if ("locator".equals(s)) return 2;
        if ("extended".equals(s)) return 1;
        if ("none".equals(s)) return 3;
        if ("arc".equals(s)) return 0;
        if ("resource".equals(s)) return 4;
        if ("title".equals(s)) return 6;
        if ("simple".equals(s)) return 5;
        return Integer.MIN_VALUE;
    }

    public int readspace_anonymousType_4(String s) {
        if ("default".equals(s)) return 0;
        if ("preserve".equals(s)) return 1;
        return Integer.MIN_VALUE;
    }

    public int readtyp_calcModeType(String s) {
        if ("linear".equals(s)) return 1;
        if ("discrete".equals(s)) return 0;
        if ("paced".equals(s)) return 2;
        if ("spline".equals(s)) return 3;
        return Integer.MIN_VALUE;
    }

/*
    public int readtyp_timeType(String s) {
        if ("indefinite".equals(s)) return 0;
        if ("media".equals(s)) return 1;
        return Integer.MIN_VALUE;
    }

*/
    public int readtyp_animatable(String s) {
        return IOUtils.animatable(s);
    }

    public int readtyp_animFillType(String s) {
        if ("remove".equals(s)) return 1;
        if ("freeze".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_repeatType(String s) {
        if ("indefinite".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_restartType(String s) {
        if ("always".equals(s)) return 0;
        if ("whenNotActive".equals(s)) return 2;
        if ("never".equals(s)) return 1;
        return Integer.MIN_VALUE;
    }

    public int readtyp_rotateType(String s) {
        if ("auto-reverse".equals(s)) return 1;
        if ("auto".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_propagateType(String s) {
        if ("stop".equals(s)) return 1;
        if ("continue".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_defaultActionType(String s) {
        if ("perform".equals(s)) return 1;
        if ("cancel".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_rotscatra(String s) {
        if ("skewY".equals(s)) return 3;
        if ("rotate".equals(s)) return 0;
        if ("skewX".equals(s)) return 2;
        if ("translate".equals(s)) return 4;
        if ("scale".equals(s)) return 1;
        return Integer.MIN_VALUE;
    }

    public int readtyp_syncBehaviorType(String s) {
        if ("locked".equals(s)) return 3;
        if ("default".equals(s)) return 1;
        if ("independent".equals(s)) return 2;
        if ("canSlip".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_syncBehaviorDefaultType(String s) {
        if ("locked".equals(s)) return 3;
        if ("inherit".equals(s)) return 2;
        if ("independent".equals(s)) return 1;
        if ("canSlip".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_syncToleranceType(String s) {
        if ("default".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_syncToleranceDefaultType(String s) {
        if ("inherit".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_choiceType(String s) {
        if ("none".equals(s)) return 1;
        if ("all".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_transformBehaviorType(String s) {
        if ("pinned_90".equals(s)) return 4;
        if ("pinned_270".equals(s)) return 3;
        if ("pinned".equals(s)) return 1;
        if ("pinned_180".equals(s)) return 2;
        if ("geometric".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_gradientUnits(String s) {
        if ("userSpaceOnUse".equals(s)) return 1;
        if ("objectBoundingBox".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_AttributeNameType(String s) {
        return IOUtils.updatable(s);
    }

    public int readtyp_preserveAspectRatioType(String s) {
        if ("none".equals(s)) return 0;
        else if ("xMaxYMax".equals(s)) return 1;
        else if ("xMaxYMax meet".equals(s)) return 1;
        else if ("xMaxYMid".equals(s)) return 2;
        else if ("xMaxYMid meet".equals(s)) return 2;
        else if ("xMaxYMin".equals(s)) return 3;
        else if ("xMaxYMin meet".equals(s)) return 3;
        else if ("xMidYMax".equals(s)) return 4;
        else if ("xMidYMax meet".equals(s)) return 4;
        else if ("xMidYMid".equals(s)) return 5;
        else if ("xMidYMid meet".equals(s)) return 5;
        else if ("xMidYMin".equals(s)) return 6;
        else if ("xMidYMin meet".equals(s)) return 6;
        else if ("xMinYMax".equals(s)) return 7;
        else if ("xMinYMax meet".equals(s)) return 7;
        else if ("xMinYMid".equals(s)) return 8;
        else if ("xMinYMid meet".equals(s)) return 8;
        else if ("xMinYMin".equals(s)) return 9;
        else if ("xMinYMin meet".equals(s)) return 9;
        return Integer.MIN_VALUE;
    }

    public int readtyp_eventType(String s) {
        if (s.startsWith("accessKey(") || s.startsWith("accesskey(")) return 1;
        if ("begin".equals(s) || "beginEvent".equals(s)) return 4;
        if ("end".equals(s) || "endEvent".equals(s)) return 7;
        if (s.startsWith("longAccessKey(") || s.startsWith("longaccesskey(")) return 15;
        if ("repeat".equals(s) || "repeatEvent".equals(s)) return 23;
        for (int i = 0; i < Event.eventNames.length; i++) {
            if (Event.eventNames[i].equals(s)) return i;
        }
        return Integer.MIN_VALUE;
    }

    public int readtyp_keyCode(String s) {
        if ("SHARP".equals(s) || "#".equals(s)) return 17;
        if ("*".equals(s)) return 0;
        if ("SOFT_KEY_2".equals(s)) return 19;
        if ("SOFT_KEY_1".equals(s)) return 18;
        if ("UP".equals(s)) return 20;
        if ("FIRE".equals(s)) return 13;
        if ("NO_KEY".equals(s)) return 15;
        if ("ANY_KEY".equals(s)) return 11;
        if ("RIGHT".equals(s)) return 16;
        if ("LEFT".equals(s)) return 14;
        if ("DOWN".equals(s)) return 12;
        if ("9".equals(s)) return 10;
        if ("8".equals(s)) return 9;
        if ("7".equals(s)) return 8;
        if ("6".equals(s)) return 7;
        if ("5".equals(s)) return 6;
        if ("4".equals(s)) return 5;
        if ("3".equals(s)) return 4;
        if ("2".equals(s)) return 3;
        if ("1".equals(s)) return 2;
        if ("0".equals(s)) return 1;
        System.out.println("unknown key code:"+s);
        return -1;
    }

    public int readtyp_beginEndType(String s) {
        if ("end".equals(s)) return 1;
        if ("begin".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readtyp_overflowType(String s) {
        if ("visible".equals(s)) return 0;
        return Integer.MIN_VALUE;
    }

    public int readcodec_overlayType(String s) {
        if ("none".equals(s)) return 0;
        if ("top".equals(s)) return 1;
        return Integer.MIN_VALUE;
    }

    {
        LASeRElements.put("LASeRHeader", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XLASeRHeader o = new XLASeRHeader();
                String s = null;
                s = atts.getValue("ISO-15938-1-DecoderInit");
                if (s != null) {
                    o.ISO_15938_1_DecoderInit = s;
                }
                s = atts.getValue("ISO-15938-1-DecoderInit-Configuration");
                if (s != null) {
                    o.ISO_15938_1_DecoderInit_Configuration = (byte) xr.readInt(s);
                }
                s = atts.getValue("newSceneIndicator");
                if (s != null) {
                    o.newSceneIndicator = xr.readBoolean(s);
                }
                s = atts.getValue("extensionIDBits");
                if (s != null) {
                    o.extensionIDBits = xr.readInt(s);
                }
                s = atts.getValue("colorComponentBits");
                if (s != null) {
                    o.colorComponentBits_minus_1 = xr.readInt(s) - 1;
                }
                s = atts.getValue("level");
                if (s != null) {
                    o.level = xr.readInt(s);
                }
                s = atts.getValue("pathComponents");
                if (s != null) {
                    o.pathComponents = (byte) xr.readInt(s);
                }
                s = atts.getValue("pointsCodec");
                if (s != null) {
                    o.pointsCodec = ((XMLParserImpl) xr).readtyp_pointsCodecType(s);
                }
                s = atts.getValue("profile");
                if (s != null) {
                    o.profile = ((XMLParserImpl) xr).readtyp_profileType(s);
                }
                s = atts.getValue("timeResolution");
                if (s != null) {
                    o.timeResolution = xr.readInt(s);
                    SDLClasses.timeResolution = o.timeResolution;
                }
                s = atts.getValue("resolution");
                if (s != null) {
                    o.resolution = xr.readInt(s);
                }
                s = atts.getValue("scaleBits_minus_coordBits");
                if (s != null) {
                    o.scaleBits_minus_coordBits = xr.readInt(s);
                }
                s = atts.getValue("coordBits");
                if (s != null) {
                    o.coordBits = xr.readInt(s);
                }
                s = atts.getValue("useFullRequestHost");
                if (s != null) {
                    o.useFullRequestHost = xr.readBoolean(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("a", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xa o = new Xa();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("target");
                if (s != null) {
                    o.target = s;
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("cursorManager", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XcursorManager o = new XcursorManager();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("x");
                if (s != null) {
                    o.x = xr.readFloat(s);
                }
                s = atts.getValue("y");
                if (s != null) {
                    o.y = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    //
    // anticipated implementation of StreamSource
    //
    {
        LASeRElements.put("streamSource", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XstreamSource o = new XstreamSource();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("sources");
                if (s != null) {
                    o.source = xr.readURIs(s);
                }
                s = atts.getValue("sourceIndex");
                if (s != null) {
                    o.sourceIndex = xr.readInt(s);
                }
                s = atts.getValue("mode");
                if (s != null) {
                    o.mode = ((XMLParserImpl) xr).readModeType(s);
                }
                s = atts.getValue("width");
                if (s != null) {
                    o.width = xr.readFloat(s);
                }
                s = atts.getValue("height");
                if (s != null) {
                    o.height = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }


    {
        SVGElements.put("animate", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xanimate o = new Xanimate();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("accumulate");
                if (s != null) {
                    o.accumulate = xr.readEnum(s, "accumulate") == 1;
                }
                s = atts.getValue("additive");
                if (s != null) {
                    o.additive = xr.readEnum(s, "additive") == 1;
                }
                s = atts.getValue("by");
                if (s != null) {
                    o.by = xr.readAnimValue(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("calcMode");
                if (s != null) {
                    o.calcMode = ((XMLParserImpl) xr).readtyp_calcModeType(s);
                }
                s = atts.getValue("from");
                if (s != null) {
                    o.from = xr.readAnimValue(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("min");
                if (s != null) {
                    o.min = xr.readFloat(s);
                }
                s = atts.getValue("max");
                if (s != null) {
                    o.max = xr.readFloat(s);
                }
                s = atts.getValue("keySplines");
                if (s != null) {
                    o.keySplines = xr.readFloatList(s);
                }
                s = atts.getValue("keyTimes");
                if (s != null) {
                    o.keyTimes = xr.readFloatList(s);
                }
                s = atts.getValue("values");
                if (s != null) {
                    o.values = xr.readAnimValues(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("attributeName");
                if (s != null) {
                    o.attributeName = readAttributeName(s);
                }
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                s = atts.getValue("dur");
                if (s != null) {
                    o.dur = xr.readDur(s);
                }
                s = atts.getValue("enabled");
                if (s != null) {
                    o.enabled = xr.readBoolean(s);
                }
                s = atts.getValue("end");
                if (s != null) {
                    o.end = xr.readBeginEnd(s);
                }
                s = atts.getValue("fill");
                if (s != null) {
                    o.fill = ((XMLParserImpl) xr).readtyp_animFillType(s);
                }
                s = atts.getValue("repeatCount");
                if (s != null) {
                    o.repeatCount = xr.readRepeatCount(s, "indefinite");
                }
                s = atts.getValue("repeatDur");
                if (s != null) {
                    o.repeatDur = xr.readRepeatDur(s, "indefinite");
                }
                s = atts.getValue("restart");
                if (s != null) {
                    o.restart = ((XMLParserImpl) xr).readtyp_restartType(s);
                }
                s = atts.getValue("to");
                if (s != null) {
                    o.to = xr.readAnimValue(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("animateColor", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XanimateColor o = new XanimateColor();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("accumulate");
                if (s != null) {
                    o.accumulate = xr.readEnum(s, "accumulate") == 1;
                }
                s = atts.getValue("additive");
                if (s != null) {
                    o.additive =  xr.readEnum(s, "additive") == 1;
                }
                s = atts.getValue("by");
                if (s != null) {
                    o.by = xr.readAnimValue(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("calcMode");
                if (s != null) {
                    o.calcMode = ((XMLParserImpl) xr).readtyp_calcModeType(s);
                }
                s = atts.getValue("from");
                if (s != null) {
                    o.from = xr.readAnimValue(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("min");
                if (s != null) {
                    o.min = xr.readFloat(s);
                }
                s = atts.getValue("max");
                if (s != null) {
                    o.max = xr.readFloat(s);
                }
                s = atts.getValue("keySplines");
                if (s != null) {
                    o.keySplines = xr.readFloatList(s);
                }
                s = atts.getValue("keyTimes");
                if (s != null) {
                    o.keyTimes = xr.readFloatList(s);
                }
                s = atts.getValue("values");
                if (s != null) {
                    o.values = xr.readAnimValues(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("attributeName");
                if (s != null) {
                    o.attributeName = readAttributeName(s);
                }
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                s = atts.getValue("dur");
                if (s != null) {
                    o.dur = xr.readDur(s);
                }
                s = atts.getValue("enabled");
                if (s != null) {
                    o.enabled = xr.readBoolean(s);
                }
                s = atts.getValue("end");
                if (s != null) {
                    o.end = xr.readBeginEnd(s);
                }
                s = atts.getValue("fill");
                if (s != null) {
                    o.fill = ((XMLParserImpl) xr).readtyp_animFillType(s);
                }
                s = atts.getValue("repeatCount");
                if (s != null) {
                    o.repeatCount = xr.readRepeatCount(s, "indefinite");
                }
                s = atts.getValue("repeatDur");
                if (s != null) {
                    o.repeatDur = xr.readRepeatDur(s, "indefinite");
                }
                s = atts.getValue("restart");
                if (s != null) {
                    o.restart = ((XMLParserImpl) xr).readtyp_restartType(s);
                }
                s = atts.getValue("to");
                if (s != null) {
                    o.to = xr.readAnimValue(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("animateMotion", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XanimateMotion o = new XanimateMotion();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("accumulate");
                if (s != null) {
                    o.accumulate = xr.readEnum(s, "accumulate") == 1;
                }
                s = atts.getValue("additive");
                if (s != null) {
                    o.additive =  xr.readEnum(s, "additive") == 1;
                }
                s = atts.getValue("by");
                if (s != null) {
                    o.by = xr.readAnimValue(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("calcMode");
                if (s != null) {
                    o.calcMode = ((XMLParserImpl) xr).readtyp_calcModeType(s);
                }
                s = atts.getValue("from");
                if (s != null) {
                    o.from = xr.readAnimValue(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("min");
                if (s != null) {
                    o.min = xr.readFloat(s);
                }
                s = atts.getValue("max");
                if (s != null) {
                    o.max = xr.readFloat(s);
                }
                s = atts.getValue("keySplines");
                if (s != null) {
                    o.keySplines = xr.readFloatList(s);
                }
                s = atts.getValue("keyTimes");
                if (s != null) {
                    o.keyTimes = xr.readFloatList(s);
                }
                s = atts.getValue("values");
                if (s != null) {
                    o.values = xr.readAnimValues(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("attributeName");
                if (s != null) {
                    o.attributeName = readAttributeName(s);
                }
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                s = atts.getValue("dur");
                if (s != null) {
                    o.dur = xr.readDur(s);
                }
                s = atts.getValue("enabled");
                if (s != null) {
                    o.enabled = xr.readBoolean(s);
                }
                s = atts.getValue("end");
                if (s != null) {
                    o.end = xr.readBeginEnd(s);
                }
                s = atts.getValue("fill");
                if (s != null) {
                    o.fill = ((XMLParserImpl) xr).readtyp_animFillType(s);
                }
                s = atts.getValue("repeatCount");
                if (s != null) {
                    o.repeatCount = xr.readRepeatCount(s, "indefinite");
                }
                s = atts.getValue("repeatDur");
                if (s != null) {
                    o.repeatDur = xr.readRepeatDur(s, "indefinite");
                }
                s = atts.getValue("restart");
                if (s != null) {
                    o.restart = ((XMLParserImpl) xr).readtyp_restartType(s);
                }
                s = atts.getValue("to");
                if (s != null) {
                    o.to = xr.readAnimValue(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("keyPoints");
                if (s != null) {
                    o.keyPoints = xr.readFloatList(s);
                }
                s = atts.getValue("path");
                if (s != null) {
                    o.path = xr.readPath(s);
                }
                s = atts.getValue("rotate");  //TODO
                if (s != null) {
                    o.rotate = xr.readFloatExcept(s, "auto", "auto-reverse");
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("animateTransform", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XanimateTransform o = new XanimateTransform();
                String attName = atts.getValue("attributeName");
                if (attName == null) {
                    attName="transform";
                }
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("accumulate");
                if (s != null) {
                    o.accumulate = xr.readEnum(s, "accumulate") == 1;
                }
                s = atts.getValue("additive");
                if (s != null) {
                    o.additive =  xr.readEnum(s, "additive") == 1;
                }
                s = atts.getValue("by");
                if (s != null) {
                    o.by = xr.readAnimValue(s, o, attName);
                }
                s = atts.getValue("calcMode");
                if (s != null) {
                    o.calcMode = ((XMLParserImpl) xr).readtyp_calcModeType(s);
                }
                s = atts.getValue("from");
                if (s != null) {
                    o.from = xr.readAnimValue(s, o, attName);
                }
                s = atts.getValue("min");
                if (s != null) {
                    o.min = xr.readFloat(s);
                }
                s = atts.getValue("max");
                if (s != null) {
                    o.max = xr.readFloat(s);
                }
                s = atts.getValue("keySplines");
                if (s != null) {
                    o.keySplines = xr.readFloatList(s);
                }
                s = atts.getValue("keyTimes");
                if (s != null) {
                    o.keyTimes = xr.readFloatList(s);
                }
                s = atts.getValue("values");
                if (s != null) {
                    o.values = xr.readAnimValues(s, o, attName);
                }
                o.attributeName = readAttributeName(attName);
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                s = atts.getValue("dur");
                if (s != null) {
                    o.dur = xr.readDur(s);
                }
                s = atts.getValue("enabled");
                if (s != null) {
                    o.enabled = xr.readBoolean(s);
                }
                s = atts.getValue("end");
                if (s != null) {
                    o.end = xr.readBeginEnd(s);
                }
                s = atts.getValue("fill");
                if (s != null) {
                    o.fill = ((XMLParserImpl) xr).readtyp_animFillType(s);
                }
                s = atts.getValue("repeatCount");
                if (s != null) {
                    o.repeatCount = xr.readRepeatCount(s, "indefinite");
                }
                s = atts.getValue("repeatDur");
                if (s != null) {
                    o.repeatDur = xr.readRepeatDur(s, "indefinite");
                }
                s = atts.getValue("restart");
                if (s != null) {
                    o.restart = ((XMLParserImpl) xr).readtyp_restartType(s);
                }
                s = atts.getValue("to");
                if (s != null) {
                    o.to = xr.readAnimValue(s, o, attName);
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("type");
                if (s != null) {
                    o.type = ((XMLParserImpl) xr).readtyp_rotscatra(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("audio", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xaudio o = new Xaudio();
                parseProperties(o, xr, atts);
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("audio-level");
                if (s != null) {
                    o.audio_level = xr.readFloat(s);
                }
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                s = atts.getValue("dur");
                if (s != null) {
                    o.dur = xr.readDur(s);
                }
                s = atts.getValue("end");
                if (s != null) {
                    o.end = xr.readBeginEnd(s);
                }
                s = atts.getValue("min");
                if (s != null) {
                    o.min = xr.readFloat(s);
                }
                s = atts.getValue("max");
                if (s != null) {
                    o.max = xr.readFloat(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("repeatCount");
                if (s != null) {
                    o.repeatCount = xr.readRepeatCount(s, "indefinite");
                }
                s = atts.getValue("repeatDur");
                if (s != null) {
                    o.repeatDur = xr.readRepeatDur(s, "indefinite");
                }
                s = atts.getValue("syncBehavior");
                if (s != null) {
                    o.syncBehavior = ((XMLParserImpl) xr).readtyp_syncBehaviorType(s);
                }
                s = atts.getValue("lsr:syncReference");
                if (s != null) {
                    o.syncReference = xr.readURI(s);
                }
                s = atts.getValue("syncTolerance");
                if (s != null) {
                    o.syncTolerance = xr.readFloatExcept(s, "default");
                }
                s = atts.getValue("syncMaster");
                if (s != null) {
                    o.syncMaster = ("true".equalsIgnoreCase(s) ? 1 : 0);
                }
                s = atts.getValue("smil:clipBegin");
                if (s != null) {
                    o.clipBegin = xr.readClip(s);
                }
                s = atts.getValue("smil:clipEnd");
                if (s != null) {
                    o.clipEnd = xr.readClip(s);
                }
                s = atts.getValue("type");
                if (s != null) {
                    o.type = s;
                }
                s = atts.getValue("restart");
                if (s != null) {
                    o.restart = ((XMLParserImpl) xr).readtyp_restartType(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("circle", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xcircle o = new Xcircle();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("cx");
                if (s != null) {
                    o.cx = xr.readFloat(s);
                }
                s = atts.getValue("cy");
                if (s != null) {
                    o.cy = xr.readFloat(s);
                }
                s = atts.getValue("r");
                if (s != null) {
                    o.r = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("defs", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xdefs o = new Xdefs();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }


    {
        SVGElements.put("ellipse", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xellipse o = new Xellipse();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("cx");
                if (s != null) {
                    o.cx = xr.readFloat(s);
                }
                s = atts.getValue("cy");
                if (s != null) {
                    o.cy = xr.readFloat(s);
                }
                s = atts.getValue("rx");
                if (s != null) {
                    o.rx = xr.readFloat(s);
                }
                s = atts.getValue("ry");
                if (s != null) {
                    o.ry = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }


    {
        SVGElements.put("foreignObject", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XforeignObject o = new XforeignObject();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("height");
                if (s != null) {
                    o.height = xr.readFloat(s);
                }
                s = atts.getValue("width");
                if (s != null) {
                    o.width = xr.readFloat(s);
                }
                s = atts.getValue("x");
                if (s != null) {
                    o.x = xr.readFloat(s);
                }
                s = atts.getValue("y");
                if (s != null) {
                    o.y = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("g", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xg o = new Xg();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("lsr:choice");
                if (s != null) {
                    throw new RuntimeException("parsing error: lsr:choice in a g element");
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("lsr:size");
                if (s != null) {
                    throw new RuntimeException("parsing error: lsr:size in a g element");
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("simpleLayout", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XsimpleLayout o = new XsimpleLayout();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("delta");
                if (s != null) {
                    o.delta = xr.readPoint(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("selector", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xselector o = new Xselector();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("lsr:choice");
                if (s != null) {
                    o.choice = xr.readChoice(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("rectClip", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XrectClip o = new XrectClip();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("size");
                if (s != null) {
                    o.size = xr.readPoint(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("image", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Ximage o = new Ximage();
                parseProperties(o, xr, atts);
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFonts");
                if (s != null) {
                    o.requiredFonts = s;
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("display");
                if (s != null) {
                    o.display = ((XMLParserImpl) xr).readtyp_displayType(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("height");
                if (s != null) {
                    o.height = xr.readFloat(s);
                }
                s = atts.getValue("width");
                if (s != null) {
                    o.width = xr.readFloat(s);
                }
                s = atts.getValue("x");
                if (s != null) {
                    o.x = xr.readFloat(s);
                }
                s = atts.getValue("y");
                if (s != null) {
                    o.y = xr.readFloat(s);
                }
                s = atts.getValue("preserveAspectRatio");
                if (s != null) {
                    o.preserveAspectRatio = ((XMLParserImpl) xr).readtyp_preserveAspectRatioType(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("line", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xline o = new Xline();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("x1");
                if (s != null) {
                    o.x1 = xr.readFloat(s);
                }
                s = atts.getValue("x2");
                if (s != null) {
                    o.x2 = xr.readFloat(s);
                }
                s = atts.getValue("y1");
                if (s != null) {
                    o.y1 = xr.readFloat(s);
                }
                s = atts.getValue("y2");
                if (s != null) {
                    o.y2 = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("linearGradient", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XlinearGradient o = new XlinearGradient();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("gradientUnits");
                if (s != null) {
                    o.gradientUnits = ((XMLParserImpl) xr).readtyp_gradientUnits(s);
                }
                s = atts.getValue("x1");
                if (s != null) {
                    o.x1 = xr.readFloat(s);
                }
                s = atts.getValue("x2");
                if (s != null) {
                    o.x2 = xr.readFloat(s);
                }
                s = atts.getValue("y1");
                if (s != null) {
                    o.y1 = xr.readFloat(s);
                }
                s = atts.getValue("y2");
                if (s != null) {
                    o.y2 = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("mpath", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xmpath o = new Xmpath();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("path", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xpath o = new Xpath();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("d");
                if (s != null) {
                    o.d = xr.readPath(s);
                }
                s = atts.getValue("pathLength");
                if (s != null) {
                    o.pathLength = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("polygon", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xpolygon o = new Xpolygon();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("points");
                if (s != null) {
                    o.points = xr.readFloatList(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("polyline", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xpolyline o = new Xpolyline();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("points");
                if (s != null) {
                    o.points = xr.readFloatList(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("radialGradient", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XradialGradient o = new XradialGradient();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("cx");
                if (s != null) {
                    o.cx = xr.readFloat(s);
                }
                s = atts.getValue("cy");
                if (s != null) {
                    o.cy = xr.readFloat(s);
                }
                s = atts.getValue("gradientUnits");
                if (s != null) {
                    o.gradientUnits = ((XMLParserImpl) xr).readtyp_gradientUnits(s);
                }
                s = atts.getValue("r");
                if (s != null) {
                    o.r = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("rect", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xrect o = new Xrect();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("height");
                if (s != null) {
                    o.height = xr.readFloat(s);
                }
                s = atts.getValue("rx");
                if (s != null) {
                    o.rx = xr.readFloat(s);
                }
                s = atts.getValue("ry");
                if (s != null) {
                    o.ry = xr.readFloat(s);
                }
                s = atts.getValue("width");
                if (s != null) {
                    o.width = xr.readFloat(s);
                }
                s = atts.getValue("x");
                if (s != null) {
                    o.x = xr.readFloat(s);
                }
                s = atts.getValue("y");
                if (s != null) {
                    o.y = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("script", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xscript o = new Xscript();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("begin");
                if (s != null) {
                    throw new RuntimeException("parsing error: lsr:begin on script");
                }
                s = atts.getValue("enabled");
                if (s != null) {
                    throw new RuntimeException("parsing error: lsr:enabled on script");
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("type");
                if (s != null) {
                    o.type = s;
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }


    {
        LASeRElements.put("conditional", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xconditional o = new Xconditional();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                s = atts.getValue("enabled");
                if (s != null) {
                    o.enabled = xr.readBoolean(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("type");
                if (s != null) {
                    o.type = s;
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }


    {
        LASeRElements.put("Add", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XAdd o = new XAdd();
                String s = null;
                s = atts.getValue("attributeName");
                if (s != null) {
                    o.attributeName = readAttributeName(s);
                }
                s = atts.getValue("operandAttribute");
                if (s != null) {
                    o.operandAttribute = readAttributeName(s);
                }
                s = atts.getValue("operandId");
                if (s != null) {
                    o.operandElementId = xr.readID(s);
                }
                s = atts.getValue("ref");
                if (s != null) {
                    o.ref = xr.readID(s);
                }
                s = atts.getValue("value");
                if (s != null) {
                    o.value = xr.readUpdateValue(s, atts.getValue("attributeName"), atts.getValue("index"));
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("Clean", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XClean o = new XClean();
                String s = null;
                s = atts.getValue("groupID");
                if (s != null) {
                    o.groupID = s;
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("Delete", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XDelete o = new XDelete();
                String s = null;
                s = atts.getValue("index");
                if (s != null) {
                    o.index = xr.readInt(s);
                }
                s = atts.getValue("ref");
                if (s != null) {
                    o.ref = xr.readID(s);
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("Insert", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XInsert o = new XInsert();
                String s = null;
                s = atts.getValue("attributeName");
                if (s != null) {
                    o.attributeName = readAttributeName(s);
                }
                s = atts.getValue("index");
                if (s != null) {
                    o.index = xr.readInt(s);
                }
                s = atts.getValue("ref");
                if (s != null) {
                    o.ref = xr.readID(s);
                }
                s = atts.getValue("value");
                if (s != null) {
                    o.value = xr.readUpdateValue(s, atts.getValue("attributeName"), atts.getValue("index"));
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("NewScene", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XNewScene o = new XNewScene();
                String s = null;
                return o;
            }
        });
    }

    {
        SVGElements.put("svg", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xsvg o = new Xsvg();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("height");
                if (s != null) {
                    o.height = xr.readValueWithUnits(s);
                }
                s = atts.getValue("preserveAspectRatio");
                if (s != null) {
                    o.preserveAspectRatio = ((XMLParserImpl) xr).readtyp_preserveAspectRatioType(s);
                }
                s = atts.getValue("syncBehaviorDefault");
                if (s != null) {
                    o.syncBehaviorDefault = ((XMLParserImpl) xr).readtyp_syncBehaviorDefaultType(s);
                }
                s = atts.getValue("syncToleranceDefault");
                if (s != null) {
                    o.syncToleranceDefault = xr.readFloatExcept(s, "inherit");
                }
                s = atts.getValue("viewBox");
                if (s != null) {
                    o.viewBox = xr.readFloatList(s);
                }
                s = atts.getValue("width");
                if (s != null) {
                    o.width = xr.readValueWithUnits(s);
                }
                s = atts.getValue("zoomAndPan");
                if (s != null) {
                    o.zoomAndPan = xr.readEnum(s, "zoomAndPan");
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("contentScriptType");
                if (s != null) {
                    o.contentScriptType = s;
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("Replace", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XReplace o = new XReplace();
                String s = null;
                s = atts.getValue("attributeName");
                if (s != null) {
                    o.attributeName = readAttributeName(s);
                }
                s = atts.getValue("index");
                if (s != null) {
                    o.index = xr.readInt(s);
                }
                s = atts.getValue("ref");
                if (s != null) {
                    o.ref = xr.readID(s);
                }
                s = atts.getValue("value");
                if (s != null) {
                    try {
                        o.value = xr.readUpdateValue(s, atts.getValue("attributeName"), atts.getValue("index"));
                    } catch(ArrayIndexOutOfBoundsException ae) {
                        System.out.println("Replace on "+atts.getValue("attributeName")+" value="+atts.getValue("value"));
                        throw ae;
                    }
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("Restore", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XRestore o = new XRestore();
                String s = null;
                s = atts.getValue("groupID");
                if (s != null) {
                    o.groupID = s;
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("Save", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XSave o = new XSave();
                String s = null;
                s = atts.getValue("attributes");
                if (s != null) {
                    o.attributes = xr.readAttrNameList(s);
                }
                s = atts.getValue("elements");
                if (s != null) {
                    o.elements = xr.readIDList(s);
                }
                s = atts.getValue("groupID");
                if (s != null) {
                    o.groupID = s;
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("SendEvent", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XSendEvent o = new XSendEvent();
                String s = null;
                s = atts.getValue("event");
                if (s != null) {
                    o.event = new Event().read(s);
                }
                s = atts.getValue("intvalue");
                if (s != null) {
                    o.intvalue = xr.readInt(s);
                }
                s = atts.getValue("pointvalue");
                if (s != null) {
                    o.pointvalue = xr.readPoint(s);
                }
                s = atts.getValue("ref");
                if (s != null) {
                    o.ref = xr.readID(s);
                }
                s = atts.getValue("stringvalue");
                if (s != null) {
                    o.stringvalue = s;
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("Activate", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XActivate o = new XActivate();
                String s = null;
                s = atts.getValue("ref");
                if (s != null) {
                    o.ref = readID(s);
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("Deactivate", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XDeactivate o = new XDeactivate();
                String s = null;
                s = atts.getValue("ref");
                if (s != null) {
                    o.ref = readID(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("set", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xset o = new Xset();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("attributeName");
                if (s != null) {
                    o.attributeName = readAttributeName(s);
                }
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                s = atts.getValue("dur");
                if (s != null) {
                    o.dur = xr.readDur(s);
                }
                s = atts.getValue("enabled");
                if (s != null) {
                    o.enabled = xr.readBoolean(s);
                }
                s = atts.getValue("end");
                if (s != null) {
                    o.end = xr.readBeginEnd(s);
                }
                s = atts.getValue("fill");
                if (s != null) {
                    o.fill = ((XMLParserImpl) xr).readtyp_animFillType(s);
                }
                s = atts.getValue("repeatCount");
                if (s != null) {
                    o.repeatCount = xr.readRepeatCount(s, "indefinite");
                }
                s = atts.getValue("repeatDur");
                if (s != null) {
                    o.repeatDur = xr.readRepeatDur(s, "indefinite");
                }
                s = atts.getValue("restart");
                if (s != null) {
                    o.restart = ((XMLParserImpl) xr).readtyp_restartType(s);
                }
                s = atts.getValue("to");
                if (s != null) {
                    o.to = xr.readAnimValue(s, o, atts.getValue("attributeName"));
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("stop", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xstop o = new Xstop();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("offset");
                if (s != null) {
                    o.offset = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("switch", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xswitch o = new Xswitch();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("text", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xtext o = new Xtext();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("editable");
                if (s != null) {
                    o.editable = xr.readBoolean(s);
                }
                s = atts.getValue("rotate");
                if (s != null) {
                    o.rotate = xr.readFloatList(s);
                }
                s = atts.getValue("x");
                if (s != null) {
                    o.x = xr.readFloatList(s);
                }
                s = atts.getValue("y");
                if (s != null) {
                    o.y = xr.readFloatList(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        XMLEventsElements.put("listener", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xlistener o = new Xlistener();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("handler");
                if (s != null) {
                    o.handler = xr.readURI(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("lsr:delay");
                if (s != null) {
                    throw new RuntimeException("parsing exception: found listener.delay");
                    // o.delay = xr.readDur(s);
                }
                s = atts.getValue("enabled");
                if (s != null) {
                    o.enabled = xr.readBoolean(s);
                }
                s = atts.getValue("event");
                if (s != null) {
                    o.event = new Event().read(s);
                    o.keyCode = o.event.keyCode;
                }
                /*s = atts.getValue("keyCodes");
                if (s != null) {
                    o.keyCodes = xr.readKeyList(s);
                }*/
                s = atts.getValue("observer");
                if (s != null) {
                    o.observer = xr.readID(s);
                }
                s = atts.getValue("lsr:timeAttribute");
                if (s != null) {
                    throw new RuntimeException("parsing exception: found listener.timeAttribute");
                    // o.timeAttribute = ((XMLParserImpl) xr).readtyp_beginEndType(s);
                }
                s = atts.getValue("propagate");
                if (s != null) {
                    o.propagate = ((XMLParserImpl) xr).readtyp_propagateType(s);
                }
                s = atts.getValue("defaultAction");
                if (s != null) {
                    o.defaultAction = ((XMLParserImpl) xr).readtyp_defaultActionType(s);
                }
                s = atts.getValue("target");
                if (s != null) {
                    o.target = xr.readID(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("tspan", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xtspan o = new Xtspan();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("use", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xuse o = new Xuse();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("overflow");
                if (s != null) {
                    o.overflow = ((XMLParserImpl) xr).readtyp_overflowType(s);
                }
                s = atts.getValue("x");
                if (s != null) {
                    o.x = xr.readFloat(s);
                }
                s = atts.getValue("y");
                if (s != null) {
                    o.y = xr.readFloat(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("video", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xvideo o = new Xvideo();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("initialVisibility");
                if (s != null) {
                    o.initialVisibility = ("always".equalsIgnoreCase(s) ? 0 :
                            ("whenStarted".equalsIgnoreCase(s) ? 1 : Integer.MIN_VALUE));
                }
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("audio-level");
                if (s != null) {
                    o.audio_level = xr.readFloat(s);
                }
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                s = atts.getValue("dur");
                if (s != null) {
                    o.dur = xr.readDur(s);
                }
                s = atts.getValue("min");
                if (s != null) {
                    o.min = xr.readFloat(s);
                }
                s = atts.getValue("max");
                if (s != null) {
                    o.max = xr.readFloat(s);
                }
                s = atts.getValue("display");
                if (s != null) {
                    o.display = ((XMLParserImpl) xr).readtyp_displayType(s);
                }
                s = atts.getValue("end");
                if (s != null) {
                    o.end = xr.readBeginEnd(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("height");
                if (s != null) {
                    o.height = xr.readFloat(s);
                }
                s = atts.getValue("overlay");
                if (s != null) {
                    o.overlay = ((XMLParserImpl) xr).readcodec_overlayType(s);
                }
                s = atts.getValue("pointer-events");
                if (s != null) {
                    o.pointer_events = ((XMLParserImpl) xr).readtyp_pointer_eventsType(s);
                }
                s = atts.getValue("repeatCount");
                if (s != null) {
                    o.repeatCount = xr.readRepeatCount(s, "indefinite");
                }
                s = atts.getValue("repeatDur");
                if (s != null) {
                    o.repeatDur = xr.readRepeatDur(s, "indefinite");
                }
                s = atts.getValue("syncBehavior");
                if (s != null) {
                    o.syncBehavior = ((XMLParserImpl) xr).readtyp_syncBehaviorType(s);
                }
                s = atts.getValue("syncMaster");
                if (s != null) {
                    o.syncMaster = ("true".equalsIgnoreCase(s) ? 1 : 0);
                }
                s = atts.getValue("lsr:syncReference");
                if (s != null) {
                    o.syncReference = xr.readURI(s);
                }
                s = atts.getValue("syncTolerance");
                if (s != null) {
                    o.syncTolerance = xr.readFloatExcept(s, "default");
                }
                s = atts.getValue("transformBehavior");
                if (s != null) {
                    o.transformBehavior = ((XMLParserImpl) xr).readtyp_transformBehaviorType(s);
                }
                s = atts.getValue("type");
                if (s != null) {
                    o.type = s;
                }
                s = atts.getValue("visibility");
                if (s != null) {
                    o.visibility = ((XMLParserImpl) xr).readtyp_visibilityType(s);
                }
                s = atts.getValue("width");
                if (s != null) {
                    o.width = xr.readFloat(s);
                }
                s = atts.getValue("x");
                if (s != null) {
                    o.x = xr.readFloat(s);
                }
                s = atts.getValue("y");
                if (s != null) {
                    o.y = xr.readFloat(s);
                }
                s = atts.getValue("smil:clipBegin");
                if (s != null) {
                    o.clipBegin = xr.readClip(s);
                }
                s = atts.getValue("smil:clipEnd");
                if (s != null) {
                    o.clipEnd = xr.readClip(s);
                }
                s = atts.getValue("lsr:fullscreen");
                if (s != null) {
                    o.fullscreen = (xr.readBoolean(s) ? 1 : 0);
                }
                s = atts.getValue("preserveAspectRatio");
                if (s != null) {
                    o.preserveAspectRatio = ((XMLParserImpl) xr).readtyp_preserveAspectRatioType(s);
                }
                s = atts.getValue("restart");
                if (s != null) {
                    o.restart = ((XMLParserImpl) xr).readtyp_restartType(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("title", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xtitle o = new Xtitle();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("desc", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xdesc o = new Xdesc();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("metadata", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xmetadata o = new Xmetadata();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("StringIDTable", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XstringIDTable o = new XstringIDTable();
                String s = null;
                s = atts.getValue("strings");
                if (s != null) {
                    o.strings = Utils.strings(s, ' ');
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("animation", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xanimation o = new Xanimation();
                String s = null;
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                s = atts.getValue("initialVisibility");
                if (s != null) {
                    o.initialVisibility = ("always".equalsIgnoreCase(s) ? 0 :
                            ("whenStarted".equalsIgnoreCase(s) ? 1 : Integer.MIN_VALUE));
                }
                s = atts.getValue("dur");
                if (s != null) {
                    o.dur = xr.readDur(s);
                }
                s = atts.getValue("end");
                if (s != null) {
                    o.end = xr.readBeginEnd(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("fill");
                if (s != null) {
                    o.fill = xr.readEnum(s, "fill");
                }
                s = atts.getValue("min");
                if (s != null) {
                    o.min = xr.readFloat(s);
                }
                s = atts.getValue("max");
                if (s != null) {
                    o.max = xr.readFloat(s);
                }
                s = atts.getValue("display");
                if (s != null) {
                    o.display = ((XMLParserImpl) xr).readtyp_displayType(s);
                }
                s = atts.getValue("visibility");
                if (s != null) {
                    o.visibility = ((XMLParserImpl) xr).readtyp_visibilityType(s);
                }
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("audio-level");
                if (s != null) {
                    o.audio_level = xr.readFloat(s);
                }
                s = atts.getValue("height");
                if (s != null) {
                    o.height = xr.readFloat(s);
                }
                s = atts.getValue("pointer-events");
                if (s != null) {
                    o.pointer_events = ((XMLParserImpl) xr).readtyp_pointer_eventsType(s);
                }
                s = atts.getValue("repeatCount");
                if (s != null) {
                    o.repeatCount = xr.readRepeatCount(s, "indefinite");
                }
                s = atts.getValue("repeatDur");
                if (s != null) {
                    o.repeatDur = xr.readRepeatDur(s, "indefinite");
                }
                s = atts.getValue("syncBehavior");
                if (s != null) {
                    o.syncBehavior = ((XMLParserImpl) xr).readtyp_syncBehaviorType(s);
                }
                s = atts.getValue("syncMaster");
                if (s != null) {
                    o.syncMaster = ("true".equalsIgnoreCase(s) ? 1 : 0);
                }
                s = atts.getValue("lsr:syncReference");
                if (s != null) {
                    o.syncReference = xr.readURI(s);
                }
                s = atts.getValue("syncTolerance");
                if (s != null) {
                    o.syncTolerance = xr.readFloatExcept(s, "default");
                }
                s = atts.getValue("type");
                if (s != null) {
                    o.type = s;
                }
                s = atts.getValue("width");
                if (s != null) {
                    o.width = xr.readFloat(s);
                }
                s = atts.getValue("x");
                if (s != null) {
                    o.x = xr.readFloat(s);
                }
                s = atts.getValue("y");
                if (s != null) {
                    o.y = xr.readFloat(s);
                }
                s = atts.getValue("smil:clipBegin");
                if (s != null) {
                    o.clipBegin = xr.readClip(s);
                }
                s = atts.getValue("smil:clipEnd");
                if (s != null) {
                    o.clipEnd = xr.readClip(s);
                }
                s = atts.getValue("preserveAspectRatio");
                if (s != null) {
                    o.preserveAspectRatio = ((XMLParserImpl) xr).readtyp_preserveAspectRatioType(s);
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("restart");
                if (s != null) {
                    o.restart = ((XMLParserImpl) xr).readtyp_restartType(s);
                }
                parseProperties(o, xr, atts);
                return o;
            }
        });
    }

    {
        SVGElements.put("discard", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xdiscard o = new Xdiscard();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                parseProperties(o, xr, atts);
                return o;
            }
        });
    }

    {
        SVGElements.put("font", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xfont o = new Xfont();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("horiz-adv-x");
                if (s != null) {
                    o.horizAdvX = xr.readFloat(s);
                }
                s = atts.getValue("horiz-origin-x");
                if (s != null) {
                    o.horizOriginX = xr.readFloat(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("font-face", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XfontFace o = new XfontFace();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("accent-height");
                if (s != null) {
                    o.accentHeight = xr.readFloat(s);
                }
                s = atts.getValue("alphabetic");
                if (s != null) {
                    o.alphabetic = xr.readFloat(s);
                }
                s = atts.getValue("ascent");
                if (s != null) {
                    o.ascent = xr.readFloat(s);
                }
                s = atts.getValue("bbox");
                if (s != null) {
                    o.bbox = s;
                }
                s = atts.getValue("cap-height");
                if (s != null) {
                    o.capHeight = xr.readFloat(s);
                }
                s = atts.getValue("descent");
                if (s != null) {
                    o.descent = xr.readFloat(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
                s = atts.getValue("font-family");
                if (s != null) {
                    o.fontFamily = s;
                }
                s = atts.getValue("font-stretch");
                if (s != null) {
                    o.fontStretch = s;
                }
                s = atts.getValue("font-style");
                if (s != null) {
                    o.fontStyle = s;
                }
                s = atts.getValue("font-variant");
                if (s != null) {
                    o.fontVariant = s;
                }
                s = atts.getValue("font-weight");
                if (s != null) {
                    o.fontWeight = s;
                }
                s = atts.getValue("hanging");
                if (s != null) {
                    o.hanging = xr.readFloat(s);
                }
                s = atts.getValue("ideographic");
                if (s != null) {
                    o.ideographic = xr.readFloat(s);
                }
                s = atts.getValue("mathematical");
                if (s != null) {
                    o.mathematical = xr.readFloat(s);
                }
                s = atts.getValue("overline-position");
                if (s != null) {
                    o.overlinePosition = xr.readFloat(s);
                }
                s = atts.getValue("overline-thickness");
                if (s != null) {
                    o.overlineThickness = xr.readFloat(s);
                }
                s = atts.getValue("panose-1");
                if (s != null) {
                    o.panose1 = s;
                }
                s = atts.getValue("slope");
                if (s != null) {
                    o.slope = xr.readFloat(s);
                }
                s = atts.getValue("stemh");
                if (s != null) {
                    o.stemh = xr.readFloat(s);
                }
                s = atts.getValue("stemv");
                if (s != null) {
                    o.stemv = xr.readFloat(s);
                }
                s = atts.getValue("strikethrough-position");
                if (s != null) {
                    o.strikethroughPosition = xr.readFloat(s);
                }
                s = atts.getValue("strikethrough-thickness");
                if (s != null) {
                    o.strikethroughThickness = xr.readFloat(s);
                }
                s = atts.getValue("underline-position");
                if (s != null) {
                    o.underlinePosition = xr.readFloat(s);
                }
                s = atts.getValue("underline-thickness");
                if (s != null) {
                    o.underlineThickness = xr.readFloat(s);
                }
                s = atts.getValue("unicode-range");
                if (s != null) {
                    o.unicodeRange = s;
                }
                s = atts.getValue("units-per-em");
                if (s != null) {
                    o.unitsPerEm = xr.readFloat(s);
                }
                s = atts.getValue("widths");
                if (s != null) {
                    o.widths = s;
                }
                s = atts.getValue("x-height");
                if (s != null) {
                    o.xHeight = xr.readFloat(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("font-face-src", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XfontFaceSrc o = new XfontFaceSrc();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("font-face-uri", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XfontFaceUri o = new XfontFaceUri();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
               s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("glyph", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xglyph o = new Xglyph();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("arabic-form");
                if (s != null) {
                    o.arabicForm = s;
                }
                s = atts.getValue("d");
                if (s != null) {
                    o.d = xr.readPath(s);
                }
                s = atts.getValue("glyph-name");
                if (s != null) {
                    o.glyphName = s;
                }
                s = atts.getValue("horiz-adv-x");
                if (s != null) {
                    o.horizAdvX = xr.readFloat(s);
                }
                s = atts.getValue("lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("unicode");
                if (s != null) {
                    o.unicode = s;
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("handler", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xhandler o = new Xhandler();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
               s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("ev:event");
                if (s != null) {
                    o.event = new Event().read(s);
                }
                s = atts.getValue("type");
                if (s != null) {
                    o.type = s;
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("hkern", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xhkern o = new Xhkern();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("g1");
                if (s != null) {
                    o.g1 = s;
                }
                s = atts.getValue("g2");
                if (s != null) {
                    o.g2 = s;
                }
                s = atts.getValue("k");
                if (s != null) {
                    o.k = xr.readFloat(s);
                }
                s = atts.getValue("u1");
                if (s != null) {
                    o.u1 = s;
                }
                s = atts.getValue("u2");
                if (s != null) {
                    o.u2 = s;
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("missing-glyph", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XmissingGlyph o = new XmissingGlyph();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("d");
                if (s != null) {
                    o.d = xr.readPath(s);
                }
                s = atts.getValue("horiz-adv-x");
                if (s != null) {
                    o.horizAdvX = xr.readFloat(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("prefetch", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                Xprefetch o = new Xprefetch();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("externalResourcesRequired");
                if (s != null) {
                    o.externalResourcesRequired = xr.readBoolean(s);
                }
               s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("xlink:type");
                if (s != null) {
                    o.xlink_type = ((XMLParserImpl) xr).readtype_anonymousType_2(s);
                }
                s = atts.getValue("bandwidth");
                if (s != null) {
                    o.bandwidth = xr.readFloat(s);
                }
                s = atts.getValue("mediaCharacterEncoding");
                if (s != null) {
                    o.mediaCharacterEncoding = s;
                }
                s = atts.getValue("mediaContentEncodings");
                if (s != null) {
                    o.mediaContentEncodings = s;
                }
                s = atts.getValue("mediaSize");
                if (s != null) {
                    o.mediaSize = xr.readFloat(s);
                }
                s = atts.getValue("mediaTime");
                if (s != null) {
                    o.mediaTime = xr.readFloat(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("solidColor", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XsolidColor o = new XsolidColor();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                parseProperties(o, xr, atts);
                return o;
            }
        });
    }

    {
        SVGElements.put("tBreak", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XtBreak o = new XtBreak();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                return o;
            }
        });
    }

    {
        SVGElements.put("textArea", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XtextArea o = new XtextArea();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                parseProperties(o, xr, atts);
                s = atts.getValue("requiredExtensions");
                if (s != null) {
                    o.requiredExtensions = s;
                }
                s = atts.getValue("requiredFeatures");
                if (s != null) {
                    o.requiredFeatures = xr.readFeatureList(s);
                }
                s = atts.getValue("requiredFormats");
                if (s != null) {
                    o.requiredFormats = s;
                }
                s = atts.getValue("systemLanguage");
                if (s != null) {
                    o.systemLanguage = s;
                }
                s = atts.getValue("lsr:rotation");
                if (s != null) {
                    o.rotation = xr.readFloat(s);
                }
                s = atts.getValue("lsr:scale");
                if (s != null) {
                    o.scale = xr.readPoint(s);
                }
                s = atts.getValue("transform");
                if (s != null) {
                    o.transform = xr.readMatrix(s);
                }
                s = atts.getValue("lsr:translation");
                if (s != null) {
                    o.translation = xr.readPoint(s);
                }
                s = atts.getValue("editable");
                if (s != null) {
                    o.editable = ("simple".equalsIgnoreCase(s)?1:0);
                }
                s = atts.getValue("width");
                if (s != null) {
                    o.width = xr.readFloat(s);
                }
                s = atts.getValue("height");
                if (s != null) {
                    o.height = xr.readFloat(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("x");
                if (s != null) {
                    o.x = xr.readFloat(s);
                }
                s = atts.getValue("y");
                if (s != null) {
                    o.y = xr.readFloat(s);
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("animateScroll", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XanimateScroll o = new XanimateScroll();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("by");
                if (s != null) {
                    o.by = xr.readPoint(s);
                }
                s = atts.getValue("from");
                if (s != null) {
                    o.from = xr.readScrollStopValue(s);
                }
                s = atts.getValue("to");
                if (s != null) {
                    o.to = xr.readScrollStopValue(s);
                }
                s = atts.getValue("speed");
                if (s != null) {
                    o.speed = xr.readFloat(s);
                }
                s = atts.getValue("direction");
                if (s != null) {
                    if ("left".equalsIgnoreCase(s)) {
                        o.direction = 1;
                    } else if ("right".equalsIgnoreCase(s)) {
                        o.direction = 2;
                    } else if ("up".equalsIgnoreCase(s)) {
                        o.direction = 3;
                    } else if ("down".equalsIgnoreCase(s)) {
                        o.direction = 0;
                    }
                }
                s = atts.getValue("delayAtStart");
                if (s != null) {
                    o.delayAtStart = xr.readFloat(s);
                }
                s = atts.getValue("delayAtEnd");
                if (s != null) {
                    o.delayAtEnd = xr.readFloat(s);
                }
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                s = atts.getValue("dur");
                if (s != null) {
                    o.dur = xr.readDur(s);
                }
                s = atts.getValue("end");
                if (s != null) {
                    o.end = xr.readBeginEnd(s);
                }
                s = atts.getValue("fill");
                if (s != null) {
                    o.fill = ((XMLParserImpl) xr).readtyp_animFillType(s);
                }
                s = atts.getValue("repeatCount");
                if (s != null) {
                    o.repeatCount = xr.readRepeatCount(s, "indefinite");
                }
                s = atts.getValue("repeatDur");
                if (s != null) {
                    o.repeatDur = xr.readRepeatDur(s, "indefinite");
                }
                s = atts.getValue("restart");
                if (s != null) {
                    o.restart = ((XMLParserImpl) xr).readtyp_restartType(s);
                }
                return o;
            }
        });
    }

    {
        LASeRElements.put("setScroll", new LASeRXMLReader() {
            public Object read(XMLParser xr, Attributes atts) {
                XsetScroll o = new XsetScroll();
                String s = null;
                s = atts.getValue("dims:active");
                if (s != null) {
                    o.active = (xr.readBoolean(s)?1:0);
                }
                s = atts.getValue("id");
                if (s != null) {
                    o.id = xr.readID(s);
                }
                s = atts.getValue("xml:base");
                if (s != null) {
                    o.base = s;
                }
                s = atts.getValue("xml:lang");
                if (s != null) {
                    o.lang = s;
                }
                s = atts.getValue("xml:space");
                if (s != null) {
                    o.space = ((XMLParserImpl) xr).readspace_anonymousType_4(s);
                }
                s = atts.getValue("xlink:actuate");
                if (s != null) {
                    o.xlink_actuate = ((XMLParserImpl) xr).readactuate_anonymousType_1(s);
                }
                s = atts.getValue("xlink:arcrole");
                if (s != null) {
                    o.xlink_arcrole = new URI(s);
                }
                s = atts.getValue("xlink:href");
                if (s != null) {
                    o.setHref(xr.readURI(s));
                }
                s = atts.getValue("xlink:role");
                if (s != null) {
                    o.xlink_role = new URI(s);
                }
                s = atts.getValue("xlink:show");
                if (s != null) {
                    o.xlink_show = ((XMLParserImpl) xr).readshow_anonymousType_3(s);
                }
                s = atts.getValue("xlink:title");
                if (s != null) {
                    o.xlink_title = s;
                }
                s = atts.getValue("increment");
                if (s != null) {
                    o.increment = xr.readScrollStopValue(s);
                }
                s = atts.getValue("to");
                if (s != null) {
                    o.to = xr.readScrollStopValue(s);
                }
                s = atts.getValue("direction");
                if (s != null) {
                    o.direction = xr.readEnum(s, "direction");
                }
                s = atts.getValue("begin");
                if (s != null) {
                    o.begin = xr.readBeginEnd(s);
                }
                return o;
            }
        });
    }
}
