/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.io;

import iso.mpeg.streamezzo.sdl.Attribute;
import iso.mpeg.streamezzo.sdl.QName;

import java.io.PrintWriter;

public class XMLWriterUtil {
    public static final int displayAnimatableIndex = IOUtils.animatable("display");
    public static final int visibilityAnimatableIndex = IOUtils.animatable("visibility");
    public static final int typeAnimatableIndex = IOUtils.animatable("type");
    public static final int gradientUnitsAnimatableIndex = IOUtils.animatable("gradientUnits");
    public static final int preserveAspectRatioAnimatableIndex = IOUtils.animatable("preserveAspectRatio");
    public static final int fontStyleAnimatableIndex = IOUtils.animatable("font-style");
    public static final int textAnchorAnimatableIndex = IOUtils.animatable("text-anchor");
    public static final int displayAlignAnimatableIndex = IOUtils.animatable("display-align");
    public static final int choiceAnimatableIndex = IOUtils.animatable("choice");
    public static final int colorRenderingAnimatableIndex = IOUtils.animatable("color-rendering");
    public static final int imageRenderingAnimatableIndex = IOUtils.animatable("image-rendering");
    public static final int editableAnimatableIndex = IOUtils.animatable("editable");
    public static final int fillRuleAnimatableIndex = IOUtils.animatable("fill-rule");
    public static final int fontWeightAnimatableIndex = IOUtils.animatable("font-weight");
    public static final int shapeRenderingAnimatableIndex = IOUtils.animatable("shape-rendering");
    public static final int pointerEventsAnimatableIndex = IOUtils.animatable("pointer-events");
    public static final int strokeLinecapAnimatableIndex = IOUtils.animatable("stroke-linecap");
    public static final int strokeLinejoinAnimatableIndex = IOUtils.animatable("stroke-linejoin");
    public static final int vectorEffectAnimatableIndex = IOUtils.animatable("vector-effect");

    public static void printEnumAnim(PrintWriter pw, int i, int attributeName) {
        if (attributeName == displayAnimatableIndex) {
            printtyp_displayType(pw, i);
        } else if (attributeName == visibilityAnimatableIndex) {
            printtyp_visibilityType(pw, i);
        } else if (attributeName == vectorEffectAnimatableIndex) {
            printtyp_vector_effectType(pw, i);
        } else if (attributeName == typeAnimatableIndex) {
            printtyp_rotscatra(pw, i);
        } else if (attributeName == gradientUnitsAnimatableIndex) {
            printtyp_gradientUnits(pw, i);
        } else if (attributeName == preserveAspectRatioAnimatableIndex) {
            printtyp_preserveAspectRatioType(pw, i);
        } else if (attributeName == fontStyleAnimatableIndex) {
            printtyp_font_styleType(pw, i);
        } else if (attributeName == textAnchorAnimatableIndex) {
            printtyp_textAnchorType(pw, i);
        } else if (attributeName == displayAlignAnimatableIndex) {
            printtyp_displayAlignType(pw, i);
        } else if (attributeName == colorRenderingAnimatableIndex) {
            printtyp_color_renderingType(pw, i);
        } else if (attributeName == imageRenderingAnimatableIndex) {
            printtyp_image_renderingType(pw, i);
        } else if (attributeName == shapeRenderingAnimatableIndex) {
            printtyp_shape_renderingType(pw, i);
        } else if (attributeName == pointerEventsAnimatableIndex) {
            printtyp_pointer_eventsType(pw, i);
        } else if (attributeName == fillRuleAnimatableIndex) {
            printtyp_fill_ruleType(pw, i);
        } else if (attributeName == strokeLinecapAnimatableIndex) {
            printtyp_stroke_linecapType(pw, i);
        } else if (attributeName == strokeLinejoinAnimatableIndex) {
            printtyp_stroke_linejoinType(pw, i);
        } else if (attributeName == fontWeightAnimatableIndex) {
            printtyp_font_weightType(pw, i);
        } else if (attributeName == editableAnimatableIndex) {
            print(pw, i == 1);
        } else if (attributeName == choiceAnimatableIndex) {
            pw.print(i);
        } else {
            throw new RuntimeException("program error in printEnumAnim:"+attributeName+" "+IOUtils.updatableStrings[attributeName]);
        }
    }

    public static void print(PrintWriter pw, boolean i) {
        pw.print(i);
    }

    public static void printEscapeEnumAnim(PrintWriter pw, int i, int attributeName) {
        if (attributeName == choiceAnimatableIndex) {
            printtyp_choiceEscapeType(pw, i);
        } else if (i == 0) {
            pw.print("inherit");
        } else {
            throw new RuntimeException("program error in printEscapeEnumAnim:"+attributeName+" "+IOUtils.updatableStrings[attributeName]);
        }
    }

    public static final int displayUpdatableIndex = IOUtils.updatable("display");
    public static final int visibilityUpdatableIndex = IOUtils.updatable("visibility");
    public static final int vector_effectUpdatableIndex = IOUtils.updatable("vector-effect");
    public static final int calcModeUpdatableIndex = IOUtils.updatable("calcMode");
    public static final int restartUpdatableIndex = IOUtils.updatable("restart");
    public static final int typeUpdatableIndex = IOUtils.updatable("type");
    public static final int syncBehaviorUpdatableIndex = IOUtils.updatable("syncBehavior");
    public static final int syncBehaviorDefaultUpdatableIndex = IOUtils.updatable("syncBehaviorDefault");
    public static final int transformBehaviorUpdatableIndex = IOUtils.updatable("transformBehavior");
    public static final int attributeNameUpdatableIndex = IOUtils.updatable("attributeName");
    public static final int preserveAspectRatioUpdatableIndex = IOUtils.updatable("preserveAspectRatio");
    public static final int eventUpdatableIndex = IOUtils.updatable("event");
    //public static final int timeAttributeUpdatableIndex = IOUtils.updatable("timeAttribute");
    public static final int overflowUpdatableIndex = IOUtils.updatable("overflow");
    public static final int textAnchorUpdatableIndex = IOUtils.updatable("text-anchor");

    public static void printEnumUpdate(PrintWriter pw, int i, int attributeName) {
        if (attributeName == displayUpdatableIndex) {
            printtyp_displayType(pw, i);
        } else if (attributeName == visibilityUpdatableIndex) {
            printtyp_visibilityType(pw, i);
        } else if (attributeName == vector_effectUpdatableIndex) {
            printtyp_vector_effectType(pw, i);
        } else if (attributeName == calcModeUpdatableIndex) {
            printtyp_calcModeType(pw, i);
        } else if (attributeName == restartUpdatableIndex) {
            printtyp_restartType(pw, i);
        } else if (attributeName == typeUpdatableIndex) {
            printtyp_rotscatra(pw, i);
        } else if (attributeName == syncBehaviorUpdatableIndex) {
            printtyp_syncBehaviorType(pw, i);
        } else if (attributeName == syncBehaviorDefaultUpdatableIndex) {
            printtyp_syncBehaviorDefaultType(pw, i);
        } else if (attributeName == transformBehaviorUpdatableIndex) {
            printtyp_transformBehaviorType(pw, i);
        } else if (attributeName == attributeNameUpdatableIndex) {
            print_AttributeID(pw, i);
        } else if (attributeName == preserveAspectRatioUpdatableIndex) {
            printtyp_preserveAspectRatioType(pw, i);
        } else if (attributeName == eventUpdatableIndex) {
            throw new RuntimeException("printEnumUpdate of event not implemented"); // printtyp_eventType(pw, i);
        /*} else if (attributeName == timeAttributeUpdatableIndex) {
            printtyp_beginEndType(pw, i);*/
        } else if (attributeName == overflowUpdatableIndex) {
            printtyp_overflowType(pw, i);
        } else if (attributeName == textAnchorUpdatableIndex) {
            printtyp_textAnchorType(pw, i);
        } else {
            throw new RuntimeException("program error in printEnumUpdate:"+attributeName+" "+IOUtils.updatableStrings[attributeName]);
        }
    }

    public static void printtyp_pointsCodecType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("ExpGolombPointsCodec");
                break;
        }
    }

    public static void printtyp_profileType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("full");
                break;
            case 1:
                pw.print("mini");
                break;
        }
    }

    public static void printtyp_line_increment(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("auto");
                break;
            case 1:
                pw.print("inherit");
                break;
        }
    }

    public static void printcolorType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("inherit");
                break;
            case 1:
                pw.print("none");
                break;
        }
    }

    public static void printtyp_color_renderingType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("inherit");
                break;
            case 2:
                pw.print("optimizeQuality");
                break;
            case 3:
                pw.print("optimizeSpeed");
                break;
            case 0:
                pw.print("auto");
                break;
        }
    }

    public static void printtyp_displayType(PrintWriter pw, int i) {
        switch (i) {
            case 7:
                pw.print("none");
                break;
            case 13:
                pw.print("table-column-group");
                break;
            case 3:
                pw.print("inline");
                break;
            case 15:
                pw.print("table-header-group");
                break;
            case 10:
                pw.print("table-caption");
                break;
            case 5:
                pw.print("list-item");
                break;
            case 4:
                pw.print("inline-table");
                break;
            case 6:
                pw.print("marker");
                break;
            case 0:
                pw.print("block");
                break;
            case 1:
                pw.print("compact");
                break;
            case 14:
                pw.print("table-footer-group");
                break;
            case 11:
                pw.print("table-cell");
                break;
            case 2:
                pw.print("inherit");
                break;
            case 9:
                pw.print("table");
                break;
            case 12:
                pw.print("table-column");
                break;
            case 8:
                pw.print("run-in");
                break;
            case 17:
                pw.print("table-row-group");
                break;
            case 16:
                pw.print("table-row");
                break;
        }
    }

    public static void printtyp_displayAlignType(PrintWriter pw, int i) {
        switch (i) {
            case 4:
                pw.print("inherit");
                break;
            case 3:
                pw.print("center");
                break;
            case 2:
                pw.print("before");
                break;
            case 1:
                pw.print("auto");
                break;
            case 0:
                pw.print("after");
                break;
        }
    }

    public static void printtyp_choiceEscapeType(PrintWriter pw, int i) {
        switch (i) {
            case 3 :
                pw.print("none");
                break;
            case 0 :
                pw.print("all");
                break;
            case 1 :
                pw.print("clip");
                break;
            case 2 :
                pw.print("delta");
                break;
            default:
                pw.print(i);
        }
    }

    public static void printtyp_fill_ruleType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("evenodd");
                break;
            case 1:
                pw.print("inherit");
                break;
            case 2:
                pw.print("nonzero");
                break;
        }
    }

    public static void printtyp_fontFamilyType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("inherit");
                break;
        }
    }

    public static void printtyp_font_styleType(PrintWriter pw, int i) {
        switch (i) {
            case 3:
                pw.print("oblique");
                break;
            case 0:
                pw.print("inherit");
                break;
            case 1:
                pw.print("italic");
                break;
            case 2:
                pw.print("normal");
                break;
        }
    }

    public static void printtyp_text_decorationType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("LASeR_EMBOSS");
                break;
            case 1:
                pw.print("LASeR_ENGRAVE");
                break;
            case 2:
                pw.print("LASeR_LEFTDROPSHADOW");
                break;
            case 3:
                pw.print("LASeR_OUTLINE");
                break;
            case 4:
                pw.print("LASeR_RIGHTDROPSHADOW");
                break;
            case 5:
                pw.print("inherit");
                break;
            case 6:
                pw.print("none");
                break;
            case 7:
                pw.print("underline");
                break;
        }
    }

    public static void printtyp_font_variantType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("inherit");
                break;
            case 1:
                pw.print("normal");
                break;
            case 2:
                pw.print("smallcaps");
                break;
        }
    }

    public static void printtyp_font_weightType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("200");
                break;
            case 6:
                pw.print("700");
                break;
            case 2:
                pw.print("300");
                break;
            case 7:
                pw.print("800");
                break;
            case 11:
                pw.print("inherit");
                break;
            case 3:
                pw.print("400");
                break;
            case 13:
                pw.print("normal");
                break;
            case 8:
                pw.print("900");
                break;
            case 9:
                pw.print("bold");
                break;
            case 4:
                pw.print("500");
                break;
            case 0:
                pw.print("100");
                break;
            case 10:
                pw.print("bolder");
                break;
            case 12:
                pw.print("lighter");
                break;
            case 5:
                pw.print("600");
                break;
        }
    }

    public static void printtyp_image_renderingType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("inherit");
                break;
            case 2:
                pw.print("optimizeQuality");
                break;
            case 3:
                pw.print("optimizeSpeed");
                break;
            case 0:
                pw.print("auto");
                break;
        }
    }

    public static void printtyp_pointer_eventsType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("all");
                break;
            case 1:
                pw.print("boundingBox");
                break;
            case 2:
                pw.print("fill");
                break;
            case 3:
                pw.print("inherit");
                break;
            case 4:
                pw.print("none");
                break;
            case 5:
                pw.print("painted");
                break;
            case 6:
                pw.print("stroke");
                break;
            case 7:
                pw.print("visible");
                break;
            case 8:
                pw.print("visibleFill");
                break;
            case 9:
                pw.print("visiblePainted");
                break;
            case 10:
                pw.print("visibleStroke");
                break;
        }
    }

    public static void printtyp_shape_renderingType(PrintWriter pw, int i) {
        switch (i) {
            case 2:
                pw.print("geometricPrecision");
                break;
            case 1:
                pw.print("crispEdges");
                break;
            case 3:
                pw.print("inherit");
                break;
            case 4:
                pw.print("optimizeSpeed");
                break;
            case 0:
                pw.print("auto");
                break;
        }
    }

    public static void printtyp_stroke_linecapType(PrintWriter pw, int i) {
        switch (i) {
            case 2:
                pw.print("round");
                break;
            case 0:
                pw.print("butt");
                break;
            case 1:
                pw.print("inherit");
                break;
            case 3:
                pw.print("square");
                break;
        }
    }

    public static void printtyp_stroke_linejoinType(PrintWriter pw, int i) {
        switch (i) {
            case 3:
                pw.print("round");
                break;
            case 1:
                pw.print("inherit");
                break;
            case 0:
                pw.print("bevel");
                break;
            case 2:
                pw.print("miter");
                break;
        }
    }

    public static void printtyp_textAnchorType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("end");
                break;
            case 2:
                pw.print("middle");
                break;
            case 1:
                pw.print("inherit");
                break;
            case 3:
                pw.print("start");
                break;
        }
    }

    public static void printtyp_text_renderingType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("geometricPrecision");
                break;
            case 2:
                pw.print("inherit");
                break;
            case 3:
                pw.print("optimizeLegibility");
                break;
            case 4:
                pw.print("optimizeSpeed");
                break;
            case 0:
                pw.print("auto");
                break;
        }
    }

    public static void printtyp_visibilityType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("inherit");
                break;
            case 0:
                pw.print("hidden");
                break;
            case 2:
                pw.print("visible");
                break;
        }
    }

    public static void printtyp_vector_effectType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("inherit");
                break;
            case 0:
                pw.print("none");
                break;
            case 2:
                pw.print("non-scaling-stroke");
                break;
        }
    }

    public static void printtyp_featureType(PrintWriter pw, int i) {
        pw.print(IOUtils.featureNames[i]);
    }

    public static void printactuate_anonymousType_1(PrintWriter pw, int i) {
        switch (i) {
            case 3:
                pw.print("other");
                break;
            case 0:
                pw.print("none");
                break;
            case 1:
                pw.print("onLoad");
                break;
            case 2:
                pw.print("onRequest");
                break;
        }
    }

    public static void printshow_anonymousType_3(PrintWriter pw, int i) {
        switch (i) {
            case 3:
                pw.print("other");
                break;
            case 4:
                pw.print("replace");
                break;
            case 2:
                pw.print("none");
                break;
            case 1:
                pw.print("new");
                break;
            case 0:
                pw.print("embed");
                break;
        }
    }

    public static void printtype_anonymousType_2(PrintWriter pw, int i) {
        switch (i) {
            case 2:
                pw.print("locator");
                break;
            case 1:
                pw.print("extended");
                break;
            case 3:
                pw.print("none");
                break;
            case 0:
                pw.print("arc");
                break;
            case 4:
                pw.print("resource");
                break;
            case 6:
                pw.print("title");
                break;
            case 5:
                pw.print("simple");
                break;
        }
    }

    public static void printspace_anonymousType_4(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("default");
                break;
            case 1:
                pw.print("preserve");
                break;
        }
    }

    public static void printtyp_calcModeType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("linear");
                break;
            case 0:
                pw.print("discrete");
                break;
            case 2:
                pw.print("paced");
                break;
            case 3:
                pw.print("spline");
                break;
        }
    }

    public static void printModeType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("playList");
                break;
            case 0:
                pw.print("keepOld");
                break;
            case 2:
                pw.print("replace");
                break;
            case 3:
                pw.print("useOld");
                break;
        }
    }

/*
    public static void printtyp_timeType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("indefinite");
                break;
            case 1:
                pw.print("media");
                break;
        }
    }

*/
    public static void printtyp_animatable(PrintWriter pw, Attribute i) {
        if (i.isLaser) {
            pw.print(IOUtils.updatableStrings[i.attributeIndex]);
        } else {
            print(pw, i.tag);
            pw.print(".");
            print(pw, i.attr);
        }
    }

    public static void printtyp_animFillType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("remove");
                break;
            case 0:
                pw.print("freeze");
                break;
        }
    }

    public static void printtyp_repeatType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("indefinite");
                break;
        }
    }

    public static void printtyp_restartType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("always");
                break;
            case 2:
                pw.print("whenNotActive");
                break;
            case 1:
                pw.print("never");
                break;
        }
    }

    public static void printtyp_rotateType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("auto-reverse");
                break;
            case 0:
                pw.print("auto");
                break;
        }
    }

    public static void printtyp_rotscatra(PrintWriter pw, int i) {
        switch (i) {
            case 3:
                pw.print("skewY");
                break;
            case 0:
                pw.print("rotate");
                break;
            case 2:
                pw.print("skewX");
                break;
            case 4:
                pw.print("translate");
                break;
            case 1:
                pw.print("scale");
                break;
        }
    }

    public static void printtyp_syncBehaviorType(PrintWriter pw, int i) {
        switch (i) {
            case 3:
                pw.print("locked");
                break;
            case 1:
                pw.print("default");
                break;
            case 2:
                pw.print("independent");
                break;
            case 0:
                pw.print("canSlip");
                break;
        }
    }

    public static void printtyp_syncBehaviorDefaultType(PrintWriter pw, int i) {
        switch (i) {
            case 3:
                pw.print("locked");
                break;
            case 2:
                pw.print("inherit");
                break;
            case 1:
                pw.print("independent");
                break;
            case 0:
                pw.print("canSlip");
                break;
        }
    }

    public static void printtyp_syncToleranceType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("default");
                break;
        }
    }

    public static void printtyp_syncToleranceDefaultType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("inherit");
                break;
        }
    }

    public static void printtyp_transformBehaviorType(PrintWriter pw, int i) {
        switch (i) {
            case 4:
                pw.print("pinned_90");
                break;
            case 3:
                pw.print("pinned_270");
                break;
            case 1:
                pw.print("pinned");
                break;
            case 2:
                pw.print("pinned_180");
                break;
            case 0:
                pw.print("geometric");
                break;
        }
    }

    public static void printtyp_gradientUnits(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("userSpaceOnUse");
                break;
            case 0:
                pw.print("objectBoundingBox");
                break;
        }
    }

    public static void printAttributeName(PrintWriter pw, Attribute i) {
        if (i.isLaser) {
            pw.print(IOUtils.updatableStrings[i.attributeIndex]);
        } else {
            print(pw, i.tag);
            pw.print(".");
            print(pw, i.attr);
        }
    }

    public static void print_AttributeID(PrintWriter pw, int i) {
        pw.print(IOUtils.updatableStrings[i]);
    }

    public static void print(PrintWriter pw, QName i) {
        if (i.ns != null) {
            pw.print(i.ns.prefix);
            pw.print(":");
        }
        pw.print(i.name);
    }

    public static void printtyp_preserveAspectRatioType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("none");
                break;
            case 1:
                pw.print("xMaxYMax");
                break;
            case 2:
                pw.print("xMaxYMid");
                break;
            case 3:
                pw.print("xMaxYMin");
                break;
            case 4:
                pw.print("xMidYMax");
                break;
            case 5:
                pw.print("xMidYMid");
                break;
            case 6:
                pw.print("xMidYMin");
                break;
            case 7:
                pw.print("xMinYMax");
                break;
            case 8:
                pw.print("xMinYMid");
                break;
            case 9:
                pw.print("xMinYMin");
                break;
        }
    }

    public static void printtyp_beginEndType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("end");
                break;
            case 0:
                pw.print("begin");
                break;
        }
    }

    public static void printtyp_overflowType(PrintWriter pw, int i) {
        switch (i) {
            case 0:
                pw.print("visible");
                break;
        }
    }

    public static void printcodec_overlayType(PrintWriter pw, int i) {
        switch (i) {
            case 1:
                pw.print("none");
                break;
            case 0:
                pw.print("fullscreen");
                break;
            case 2:
                pw.print("top");
                break;
        }
    }

    public static void printfullscreen(PrintWriter pw, int i) {
        switch(i) {
            case 1:
                pw.print("true");
                break;
            case 0:
                pw.print("false");
                break;
        }
    }
}
