/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.laser;

import iso.mpeg.streamezzo.sdl.Matrix;
import iso.mpeg.streamezzo.sdl.Path;
import iso.mpeg.streamezzo.sdl.Point;
import iso.mpeg.streamezzo.sdl.Attribute;
import iso.mpeg.streamezzo.util.Visitor;
import iso.mpeg.streamezzo.laser.Ibasic;
import iso.mpeg.streamezzo.laser.Ilocatable;

public class Xpath extends XpropElem implements Ibasic, Ilocatable {
    public Path d;
    public String requiredExtensions;
    public byte[] requiredFeatures;
    public String requiredFormats;
    public int rotation = Integer.MIN_VALUE;
    public Point scale;
    public String systemLanguage;
    public Matrix transform;
    public Point translation;
    public static final Attribute attributeName = new Attribute();
    public int pathLength = Integer.MIN_VALUE;

    public Xpath clonePath() {
        Xpath clone = new Xpath();
        return clone(clone);
    }

    protected Xpath clone(Xpath clone) {
        super.clone(clone);
        clone.d = d;
        clone.requiredExtensions = requiredExtensions;
        clone.requiredFeatures = requiredFeatures;
        clone.requiredFormats = requiredFormats;
        clone.rotation = rotation;
        clone.scale = scale;
        clone.systemLanguage = systemLanguage;
        clone.transform = transform;
        clone.translation = translation;
        return clone;
    }

    public boolean sameAttrs(Xpath other) {
        if (id != Integer.MIN_VALUE) return false;
        if (other == null) return false;
        if (getClass() != other.getClass()) return false;
        if (other.requiredExtensions != requiredExtensions) return false;
        if (other.requiredFeatures != requiredFeatures) return false;
        if (other.requiredFormats != requiredFormats) return false;
        if (other.rotation != rotation) return false;
        if (other.scale != scale) return false;
        if (other.systemLanguage != systemLanguage) return false;
        if (other.transform != transform) return false;
        if (other.translation != translation) return false;
        if (other.audio_level != audio_level) return false;
        if (other.color != color) return false;
        if (other.color_rendering != color_rendering) return false;
        if (other.display != display) return false;
        if (other.display_align != display_align) return false;
        //if (other.fill != fill) return false;
        if (other.fill_opacity != fill_opacity) return false;
        if (other.fill_rule != fill_rule) return false;
        if (other.font_family != font_family) return false;
        if (other.font_size != font_size) return false;
        if (other.font_style != font_style) return false;
        if (other.text_decoration != text_decoration) return false;
        if (other.font_weight != font_weight) return false;
        if (other.image_rendering != image_rendering) return false;
        if (other.pointer_events != pointer_events) return false;
        if (other.shape_rendering != shape_rendering) return false;
        if (other.text_anchor != text_anchor) return false;
        if (other.text_rendering != text_rendering) return false;
        if (other.visibility != visibility) return false;
        if (other.stop_color != stop_color) return false;
        if (other.stop_opacity != stop_opacity) return false;
        //if (other.stroke != stroke) return false;
        if (other.stroke_dasharray != stroke_dasharray) return false;
        if (other.stroke_dashoffset != stroke_dashoffset) return false;
        if (other.stroke_linecap != stroke_linecap) return false;
        if (other.stroke_linejoin != stroke_linejoin) return false;
        if (other.stroke_miterlimit != stroke_miterlimit) return false;
        if (other.stroke_opacity != stroke_opacity) return false;
        if (other.stroke_width != stroke_width) return false;
        if (other.base != base) return false;
        if (other.lang != lang) return false;
        if (other.space != space) return false;
        if (other.privateAttributes != privateAttributes) return false;
        if (other.externalResourcesRequired != externalResourcesRequired) return false;
        return true;
    }

    public String getRequiredExtensions() {
        return requiredExtensions;
    }

    public void setRequiredExtensions(String requiredExtensions) {
        this.requiredExtensions = requiredExtensions;
    }

    public String getRequiredFormats() {
        return requiredFormats;
    }

    public void setRequiredFormats(String requiredFormats) {
        this.requiredFormats = requiredFormats;
    }

    public byte[] getRequiredFeatures() {
        return requiredFeatures;
    }

    public void setRequiredFeatures(byte[] requiredFeatures) {
        this.requiredFeatures = requiredFeatures;
    }

    public String getSystemLanguage() {
        return systemLanguage;
    }

    public void setSystemLanguage(String systemLanguage) {
        this.systemLanguage = systemLanguage;
    }

    public int getRotation() {
        return rotation;
    }

    public void setRotation(int rotation) {
        this.rotation = rotation;
    }

    public Point getScale() {
        return scale;
    }

    public void setScale(Point scale) {
        this.scale = scale;
    }

    public Matrix getTransform() {
        return transform;
    }

    public void setTransform(Matrix transform) {
        this.transform = transform;
    }

    public Point getTranslation() {
        return translation;
    }

    public void setTranslation(Point translation) {
        this.translation = translation;
    }

    public void visit(Visitor v, Object o) throws Exception {
        v.visitXpath(this, o);
    }

    public final static int SEGMENT_C = 0;
    public final static int SEGMENT_H = 1;
    public final static int SEGMENT_L = 2;
    public final static int SEGMENT_M = 3;
    public final static int SEGMENT_Q = 4;
    public final static int SEGMENT_S = 5;
    public final static int SEGMENT_T = 6;
    public final static int SEGMENT_V = 7;
    public final static int SEGMENT_Z = 8;
    public final static int SEGMENT_c = 9;
    public final static int SEGMENT_h = 10;
    public final static int SEGMENT_l = 11;
    public final static int SEGMENT_m = 12;
    public final static int SEGMENT_q = 13;
    public final static int SEGMENT_s = 14;
    public final static int SEGMENT_t = 15;
    public final static int SEGMENT_v = 16;
    public final static int SEGMENT_z = 17;

    public static int nbOfPrintedCoords(int type) {
        switch(type) {
            case SEGMENT_V:
            case SEGMENT_v:
            case SEGMENT_H:
            case SEGMENT_h:
                return 2; // it is 1, but the other is skipped
            case SEGMENT_m:
            case SEGMENT_M:
            case SEGMENT_L:
            case SEGMENT_l:
            case SEGMENT_T:
            case SEGMENT_t:
                return 2;
            case SEGMENT_z:
            case SEGMENT_Z:
                return 0;
            case SEGMENT_C:
            case SEGMENT_c:
                return 6;
            case SEGMENT_S:
            case SEGMENT_s:
            case SEGMENT_Q:
            case SEGMENT_q:
                return 4;
        }
        return -1;
    }

    public static int typeFromString(String type) {
        if ("M".equals(type)) {
            return SEGMENT_M;
        } else if ("m".equals(type)) {
            return SEGMENT_m;
        } else if ("L".equals(type)) {
            return SEGMENT_L;
        } else if ("l".equals(type)) {
            return SEGMENT_l;
        } else if ("H".equals(type)) {
            return SEGMENT_H;
        } else if ("h".equals(type)) {
            return SEGMENT_h;
        } else if ("V".equals(type)) {
            return SEGMENT_V;
        } else if ("v".equals(type)) {
            return SEGMENT_v;
        } else if ("C".equals(type)) {
            return SEGMENT_C;
        } else if ("c".equals(type)) {
            return SEGMENT_c;
        } else if ("S".equals(type)) {
            return SEGMENT_S;
        } else if ("s".equals(type)) {
            return SEGMENT_s;
        } else if ("Q".equals(type)) {
            return SEGMENT_Q;
        } else if ("q".equals(type)) {
            return SEGMENT_q;
        } else if ("T".equals(type)) {
            return SEGMENT_T;
        } else if ("t".equals(type)) {
            return SEGMENT_t;
        } else if ("Z".equals(type)) {
            return SEGMENT_Z;
        } else if ("z".equals(type)) {
            return SEGMENT_z;
        }
        return -1;
    }

}
