/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.laser;

import iso.mpeg.streamezzo.sdl.Matrix;
import iso.mpeg.streamezzo.sdl.Point;
import iso.mpeg.streamezzo.sdl.Attribute;
import iso.mpeg.streamezzo.laser.XpropElem;
import iso.mpeg.streamezzo.laser.Ibasic;
import iso.mpeg.streamezzo.laser.Ilocatable;
import iso.mpeg.streamezzo.laser.XpropElem;
import iso.mpeg.streamezzo.util.Visitor;

import java.util.Vector;

public class Xtext extends XpropElem implements Ibasic, Ilocatable {
    public boolean editable;
    public String requiredExtensions;
    public byte[] requiredFeatures;
    public String requiredFormats;
    public int[] rotate;
    public int rotation = Integer.MIN_VALUE;
    public Point scale;
    public String systemLanguage;
    public Matrix transform;
    public Point translation;
    public int x[] = null;
    public int y[] = null;
    public static final Attribute attributeName = new Attribute();

    public String getRequiredExtensions() {
        return requiredExtensions;
    }

    public void setRequiredExtensions(String requiredExtensions) {
        this.requiredExtensions = requiredExtensions;
    }

    public String getRequiredFormats() {
        return requiredFormats;
    }

    public void setRequiredFormats(String requiredFormats) {
        this.requiredFormats = requiredFormats;
    }

    public byte[] getRequiredFeatures() {
        return requiredFeatures;
    }

    public void setRequiredFeatures(byte[] requiredFeatures) {
        this.requiredFeatures = requiredFeatures;
    }

    public String getSystemLanguage() {
        return systemLanguage;
    }

    public void setSystemLanguage(String systemLanguage) {
        this.systemLanguage = systemLanguage;
    }

    public int getRotation() {
        return rotation;
    }

    public void setRotation(int rotation) {
        this.rotation = rotation;
    }

    public Point getScale() {
        return scale;
    }

    public void setScale(Point scale) {
        this.scale = scale;
    }

    public Matrix getTransform() {
        return transform;
    }

    public void setTransform(Matrix transform) {
        this.transform = transform;
    }

    public Point getTranslation() {
        return translation;
    }

    public void setTranslation(Point translation) {
        this.translation = translation;
    }

    public void visit(Visitor v, Object o) throws Exception {
        v.visitXtext(this, o);
    }

    public void addChild(Object o) {
        if (child == null) child = new Vector();
        if (o instanceof String) {
            XtextContent tc = new XtextContent();
            tc.addChild(o);
            child.addElement(tc);
        } else if (o instanceof Vector) {
            child.addAll((Vector)o);
        } else {
            child.addElement(o);
        }
    }

    public boolean sameAttrs(Xtext other) {
        if (other == null) return false;
        if (getClass() != other.getClass()) return false;
        if (other.requiredExtensions != requiredExtensions) return false;
        if (other.requiredFeatures != requiredFeatures) return false;
        if (other.requiredFormats != requiredFormats) return false;
        if (other.rotation != rotation) return false;
        if (other.scale != scale) return false;
        if (other.systemLanguage != systemLanguage) return false;
        if (other.transform != transform) return false;
        if (other.translation != translation) return false;
        if (other.audio_level != audio_level) return false;
        if (other.color != color) return false;
        if (other.color_rendering != color_rendering) return false;
        if (other.display != display) return false;
        if (other.display_align != display_align) return false;
        if (other.fill != fill) return false;
        if (other.fill_opacity != fill_opacity) return false;
        if (other.fill_rule != fill_rule) return false;
        if (other.font_family != font_family) return false;
        if (other.font_size != font_size) return false;
        if (other.font_style != font_style) return false;
        if (other.text_decoration != text_decoration) return false;
        if (other.font_weight != font_weight) return false;
        if (other.image_rendering != image_rendering) return false;
        if (other.pointer_events != pointer_events) return false;
        if (other.shape_rendering != shape_rendering) return false;
        if (other.text_anchor != text_anchor) return false;
        if (other.text_rendering != text_rendering) return false;
        if (other.visibility != visibility) return false;
        if (other.stop_color != stop_color) return false;
        if (other.stop_opacity != stop_opacity) return false;
        if (other.stroke != stroke) return false;
        if (other.stroke_dasharray != stroke_dasharray) return false;
        if (other.stroke_dashoffset != stroke_dashoffset) return false;
        if (other.stroke_linecap != stroke_linecap) return false;
        if (other.stroke_linejoin != stroke_linejoin) return false;
        if (other.stroke_miterlimit != stroke_miterlimit) return false;
        if (other.stroke_opacity != stroke_opacity) return false;
        if (other.stroke_width != stroke_width) return false;
        if (other.base != base) return false;
        if (other.lang != lang) return false;
        if (other.space != space) return false;
        if (other.privateAttributes != privateAttributes) return false;
        if (other.externalResourcesRequired != externalResourcesRequired) return false;
        if (other.editable != editable) return false;
        if (other.rotate != rotate) return false;
        return true;
    }

    public Xtext cloneText() {
        Xtext clone = new Xtext();
        return clone(clone);
    }

    protected Xtext clone(Xtext clone) {
        super.clone(clone);
        clone.requiredExtensions = requiredExtensions;
        clone.requiredFeatures = requiredFeatures;
        clone.requiredFormats = requiredFormats;
        clone.rotation = rotation;
        clone.scale = scale;
        clone.systemLanguage = systemLanguage;
        clone.transform = transform;
        clone.translation = translation;
        clone.editable = editable;
        clone.rotate = rotate;
        clone.x = x;
        clone.y = y;
        return clone;
    }
}
