/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.laser;

import iso.mpeg.streamezzo.sdl.Matrix;
import iso.mpeg.streamezzo.sdl.Point;
import iso.mpeg.streamezzo.sdl.URI;
import iso.mpeg.streamezzo.sdl.Attribute;
import iso.mpeg.streamezzo.laser.XpropElem;
import iso.mpeg.streamezzo.laser.Ibasic;
import iso.mpeg.streamezzo.laser.Ilocatable;
import iso.mpeg.streamezzo.laser.Ixlink;
import iso.mpeg.streamezzo.util.Visitor;

public class Xuse extends XpropElem implements Ibasic, Ilocatable, Ixlink {
    public int overflow = Integer.MIN_VALUE;
    public String requiredExtensions;
    public byte[] requiredFeatures;
    public String requiredFormats;
    public int rotation = Integer.MIN_VALUE;
    public Point scale;
    public String systemLanguage;
    public Matrix transform;
    public Point translation;
    public int x = Integer.MIN_VALUE;
    public int y = Integer.MIN_VALUE;
    public int xlink_actuate = Integer.MIN_VALUE;
    public URI xlink_arcrole;
    public URI href;
    public URI xlink_role;
    public int xlink_show = Integer.MIN_VALUE;
    public String xlink_title;
    public int xlink_type = Integer.MIN_VALUE;
    public static final Attribute attributeName = new Attribute();

    public String getRequiredExtensions() {
        return requiredExtensions;
    }

    public void setRequiredExtensions(String requiredExtensions) {
        this.requiredExtensions = requiredExtensions;
    }

    public String getRequiredFormats() {
        return requiredFormats;
    }

    public void setRequiredFormats(String requiredFormats) {
        this.requiredFormats = requiredFormats;
    }

    public byte[] getRequiredFeatures() {
        return requiredFeatures;
    }

    public void setRequiredFeatures(byte[] requiredFeatures) {
        this.requiredFeatures = requiredFeatures;
    }

    public String getSystemLanguage() {
        return systemLanguage;
    }

    public void setSystemLanguage(String systemLanguage) {
        this.systemLanguage = systemLanguage;
    }

    public int getRotation() {
        return rotation;
    }

    public void setRotation(int rotation) {
        this.rotation = rotation;
    }

    public Point getScale() {
        return scale;
    }

    public void setScale(Point scale) {
        this.scale = scale;
    }

    public Matrix getTransform() {
        return transform;
    }

    public void setTransform(Matrix transform) {
        this.transform = transform;
    }

    public Point getTranslation() {
        return translation;
    }

    public void setTranslation(Point translation) {
        this.translation = translation;
    }

    public int getActuate() {
        return xlink_actuate;
    }

    public void setActuate(int actuate) {
        this.xlink_actuate = actuate;
    }

    public URI getArcrole() {
        return xlink_arcrole;
    }

    public void setArcrole(URI arcrole) {
        this.xlink_arcrole = arcrole;
    }

    public URI getHref() {
        return href;
    }

    public void setHref(URI href) {
        this.href = href;
    }

    public URI getRole() {
        return xlink_role;
    }

    public void setRole(URI role) {
        this.xlink_role = role;
    }

    public int getShow() {
        return xlink_show;
    }

    public void setShow(int show) {
        this.xlink_show = show;
    }

    public String getTitle() {
        return xlink_title;
    }

    public void setTitle(String title) {
        this.xlink_title = title;
    }

    public int getXlinkType() {
        return xlink_type;
    }

    public void setXlinkType(int type) {
        this.xlink_type = type;
    }

    public void visit(Visitor v, Object o) throws Exception {
        v.visitXuse(this, o);
    }

    public boolean sameAttrs(Xuse other) {
        if (other == null) return false;
        if (id != Integer.MIN_VALUE) return false;
        if (getClass() != other.getClass()) return false;
        if (other.requiredExtensions != requiredExtensions) return false;
        if (other.requiredFeatures != requiredFeatures) return false;
        if (other.requiredFormats != requiredFormats) return false;
        if (other.rotation != rotation) return false;
        if (other.scale != scale) return false;
        if (other.systemLanguage != systemLanguage) return false;
        if (other.transform != transform) return false;
        if (other.translation != translation) return false;
        if (other.audio_level != audio_level) return false;
        if (other.color != color) return false;
        if (other.color_rendering != color_rendering) return false;
        if (other.display != display) return false;
        if (other.display_align != display_align) return false;
        if (other.fill != fill) return false;
        if (other.fill_opacity != fill_opacity) return false;
        if (other.fill_rule != fill_rule) return false;
        if (other.font_family != font_family) return false;
        if (other.font_size != font_size) return false;
        if (other.font_style != font_style) return false;
        if (other.text_decoration != text_decoration) return false;
        if (other.font_weight != font_weight) return false;
        if (other.image_rendering != image_rendering) return false;
        if (other.pointer_events != pointer_events) return false;
        if (other.shape_rendering != shape_rendering) return false;
        if (other.text_anchor != text_anchor) return false;
        if (other.text_rendering != text_rendering) return false;
        if (other.visibility != visibility) return false;
        if (other.overflow != overflow) return false;
        if (other.stop_color != stop_color) return false;
        if (other.stop_opacity != stop_opacity) return false;
        if (other.stroke != stroke) return false;
        if (other.stroke_dasharray != stroke_dasharray) return false;
        if (other.stroke_dashoffset != stroke_dashoffset) return false;
        if (other.stroke_linecap != stroke_linecap) return false;
        if (other.stroke_linejoin != stroke_linejoin) return false;
        if (other.stroke_miterlimit != stroke_miterlimit) return false;
        if (other.stroke_opacity != stroke_opacity) return false;
        if (other.stroke_width != stroke_width) return false;
        if (other.base != base) return false;
        if (other.lang != lang) return false;
        if (other.space != space) return false;
        if (other.privateAttributes != privateAttributes) return false;
        if (other.externalResourcesRequired != externalResourcesRequired) return false;
        if (other.xlink_actuate != xlink_actuate) return false;
        if (URI.equals(other.xlink_arcrole, xlink_arcrole)) return false;
        if (URI.equals(other.xlink_role, xlink_role)) return false;
        if (other.xlink_show != xlink_show) return false;
        if (strcmp(other.xlink_title, xlink_title)) return false;
        if (other.xlink_type != xlink_type) return false;
        if (other.x != x) return false;
        if (other.y != y) return false;
        return true;
    }

    private boolean strcmp(String s1, String s2) {
        if (s1 == null && s2 == null) return false;
        if (s1 != null && s1.equals(s2)) return false;
        return true;
    }

    public Xuse cloneUse() {
        Xuse clone = new Xuse();
        return clone(clone);
    }

    protected Xuse clone(Xuse clone) {
        super.clone(clone);
        clone.requiredExtensions = requiredExtensions;
        clone.requiredFeatures = requiredFeatures;
        clone.requiredFormats = requiredFormats;
        clone.rotation = rotation;
        clone.scale = scale;
        clone.systemLanguage = systemLanguage;
        clone.transform = transform;
        clone.translation = translation;
        clone.x = x;
        clone.y = y;
        clone.overflow = overflow;
        clone.xlink_actuate = xlink_actuate;
        clone.xlink_arcrole = xlink_arcrole;
        clone.href = href;
        clone.xlink_role = xlink_role;
        clone.xlink_show = xlink_show;
        clone.xlink_title = xlink_title;
        clone.xlink_type = xlink_type;
        return clone;
    }
}
