/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.resource;

public class DataType {
    public static final int GIF = 3;
    public static final int SMIL = 4;
    public static final int AMR = 5;
    public static final int WAV = 6;
    public static final int TXT = 7;
    public static final int MP3 = 10;
    public static final int PNG = 11;
    public static final int JPEG = 12;
    public static final int RAW = 13;
    public static final int PCM = 14;
    public static final int SWF = 15;
    public static final int XML = 16;
    public static final int STZ = 17;
    public static final int MIME = 18;
    public static final int WAP = 19;
    public static final int AVI = 20;
    public static final int MOV = 21;
    public static final int MPG = 22;
    public static final int XMLPJ = 23;
    public static final int MMZ = 24;
    public static final int MMS = 25;
    public static final int AAC = 26;
    public static final int UNSUPPORTED = 29241; // un gros nombre sans importance. ( accessoirement mon code postal )
    public static final String JPEG_STR = "jpg";
    public static final String GIF_STR = "gif";
    public static final String SMIL_STR = "smil";
    public static final String AMR_STR = "amr";
    public static final String WAV_STR = "wav";
    public static final String TXT_STR = "txt";
    public static final String MP3_STR = "mp3";
    public static final String PNG_STR = "png";
    public static final String RAW_STR = "raw";
    public static final String PCM_STR = "pcm";
    public static final String SWF_STR = "swf";
    public static final String XML_STR = "xml";
    public static final String XMLPJ_STR = "xmlpj";
    public static final String STZ_STR = "stz";
    public static final String MIME_STR = "mime";
    public static final String WAP_STR = "wap";
    public static final String UNSUPPORTED_STR = "UNSUPPORTED";
    public static final String AVI_STR = "avi";
    public static final String MOV_STR = "mov";
    public static final String MPG_STR = "mpg";
    public static final String MMZ_STR = "mmz";
    public static final String MMS_STR = "mms";
    public static final String AAC_STR = "aac";

    public static String getTypeAString(int myType) {
        switch (myType) {
            case JPEG:
                return JPEG_STR;
            case GIF:
                return GIF_STR;
            case SMIL:
                return SMIL_STR;
            case AMR:
                return AMR_STR;
            case WAV:
                return WAV_STR;
            case TXT:
                return TXT_STR;
            case MP3:
                return MP3_STR;
            case RAW:
                return RAW_STR;
            case PCM:
                return PCM_STR;
            case SWF:
                return SWF_STR;
            case XML:
                return XML_STR;
            case STZ:
                return STZ_STR;
            case MIME:
                return MIME_STR;
            case WAP:
                return WAP_STR;
            case PNG:
                return PNG_STR;
            case AVI:
                return AVI_STR;
            case MOV:
                return MOV_STR;
            case MPG:
                return MPG_STR;
            case XMLPJ:
                return XMLPJ_STR;
            case MMZ:
                return MMZ_STR;
            case MMS:
                return MMS_STR;
            case AAC:
                return AAC_STR;
            default   :
                return UNSUPPORTED_STR;
        }
    }

    private static String getExtension(String s) {
        String ext = null;
        int indice = s.lastIndexOf('.');
        if (indice > 0 && indice < s.length() - 1) {
            ext = s.substring(indice + 1).toLowerCase();
        }
        return ext;
    }

    public static int getType(String dataName) {
        String fileName = "";
        if (dataName != null) {
            fileName = dataName.toLowerCase();
        }
        String extension = getExtension(fileName);
        if (isMMS(extension)) {
            return MMS;
        }
        if (isJPEG(extension)) {
            return JPEG;
        }
        if (isGIF(extension)) {
            return GIF;
        }
        if (isSMIL(extension)) {
            return SMIL;
        }
        if (isAMR(extension)) {
            return AMR;
        }
        if (isWAV(extension)) {
            return WAV;
        }
        if (isTXT(extension)) {
            return TXT;
        }
        if (isMP3(extension)) {
            return MP3;
        }
        if (isPNG(extension)) {
            return PNG;
        }
        if (isRaw(extension)) {
            return RAW;
        }
        if (isSWF(extension)) {
            return SWF;
        }
        if (isPCM(extension)) {
            return PCM;
        }
        if (isXML(extension)) {
            return XML;
        }
        if (isSTZ(extension)) {
            return STZ;
        }
        if (isWAP(extension)) {
            return WAP;
        }
        if (isMIME(extension)) {
            return MIME;
        }
        if (isAVI(extension)) {
            return AVI;
        }
        if (isMOV(extension)) {
            return MOV;
        }
        if (isMPG(extension)) {
            return MPG;
        }
        if (isXMLPJ(extension)) {
            return XMLPJ;
        }
        if (isMMSZ(extension)) {
            return MMZ;
        }
        if (isAAC(extension)) {
            return AAC;
        }
        return DataType.UNSUPPORTED;
        // c'est un format non support
        //throw new UNSUPPORTEDRuntimeException(" format non support :" + fileName);
    }

    private static boolean isAAC(String extension) {
        return "aac".compareTo(extension) == 0;
    }

    private static boolean isMMS(String extension) {
        return "mms".compareTo(extension) == 0;
    }

    private static boolean isMMSZ(String extension) {
        return "mmz".compareTo(extension) == 0;
    }

    private static boolean isXMLPJ(String extension) {
        return "xmlpj".compareTo(extension) == 0;
    }

    private static boolean isMPG(String extension) {
        return ("mpg".compareTo(extension) == 0);
    }

    private static boolean isMOV(String extension) {
        return ("mov".compareTo(extension) == 0);
    }

    private static boolean isAVI(String extension) {
        return ("avi".compareTo(extension) == 0);
    }

    private static boolean isMIME(String extension) {
        return ("mime".compareTo(extension) == 0);
    }

    private static boolean isWAP(String extension) {
        return ("wap".compareTo(extension) == 0);
    }

    private static boolean isSTZ(String extension) {
        return ("stz".compareTo(extension) == 0);
    }

    private static boolean isXML(String extension) {
        return ("xml".compareTo(extension) == 0);
    }

    private static boolean isPCM(String extension) {
        return ("pcm".compareTo(extension) == 0);
    }

    private static boolean isSWF(String extension) {
        return ("swf".compareTo(extension) == 0);
    }

    private static boolean isRaw(String extension) {
        return ("raw".compareTo(extension) == 0);
    }

    private static boolean isJPEG(String extension) {
        return ("jpeg".compareTo(extension) == 0 || "jpg".compareTo(extension) == 0);
    }

    private static boolean isGIF(String extension) {
        return ("gif".compareTo(extension) == 0);
    }

    private static boolean isMP3(String extension) {
        return ("mp3".compareTo(extension) == 0);
    }

    public static boolean isSMIL(String extension) {
        return ("smil".compareTo(extension) == 0 || "sml".compareTo(extension) == 0 || "smi".compareTo(extension) == 0);
    }

    private static boolean isAMR(String extension) {
        return ("amr".compareTo(extension) == 0);
    }

    private static boolean isWAV(String extension) {
        return ("wav".compareTo(extension) == 0);
    }

    private static boolean isTXT(String extension) {
        return ("txt".compareTo(extension) == 0);
    }

    private static boolean isPNG(String extension) {
        return ("png".compareTo(extension) == 0);
    }

    public static String getContentType(int dataType) {
        switch (dataType) {
            case JPEG:
                return "image/jpeg";
            case GIF:
                return "image/gif";
            case SMIL:
                return "application/smil";
            case AMR:
                return "audio/amr";
            case WAV:
                return "audio/wav";
            case TXT:
                return "text/plain";
            case MP3:
                return "audio/mp3";
            case PNG:
                return "image/png";
            case RAW:
                return "audio/raw";
            case PCM:
                return "audio/pcm";
            case SWF:
                return "application/x-shockwave-flash";
            case XML:
                return "text/xml";
            case STZ:
                return "application/octet-stream";
            case WAP:
                return "application/wap";
            case MIME:
                return "multipart/related";
            case AVI:
                return "video/avi";
            case MOV:
                return "video/mov";
            case MPG:
                return "video/mpg";
            case MMZ:
                return "application/octet-stream";
            case MMS:
                return "application/mms-streamezzo";
            case AAC:
                return "audio/aac";
            default :
                return "*/*";
        }
    }
}
