/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.resource;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;

class FileResource extends Resource {
    private File file = null;

    FileResource(String name, String path) {
        this.name = name;
        if (path == null) {
            path = "";
        }
        File thisFilePath = new File(name);
        if (thisFilePath.isAbsolute()) {
            file = thisFilePath;
        } else {
            if (!path.equals("") && !path.endsWith(File.separator)) {
                path = path + File.separator;
            }
            file = new File(path + name);
        }
    }

    public int getSize() {
        return (int)file.length();
    }

    public byte[] getBytes() {
        byte b[] = null;
        try {
            FileInputStream fis = new FileInputStream(file);
            b = new byte[(int)file.length()];
            fis.read(b);
        } catch(Exception e) {
            e.printStackTrace();
        }
        return b;
    }


    public InputStream getInputStream() throws IOException {
        return new FileInputStream(file);
    }

    String getFilePath() {
        if (file != null) {
            return file.getPath();
        }
        return null;
    }
}
