/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.resource;

import iso.mpeg.streamezzo.util.IOMethods;

import java.io.OutputStream;
import java.io.IOException;
import java.io.DataInputStream;

public abstract class Resource implements DataSource {
    protected String name = null;

    public static Resource create(String name, byte data[]) {
        return new MemoryResource(name, data);
    }

    public static Resource create(String name, String path) {
        return new FileResource(name, path);
    }

    public String getName() {
        return name;
    }

    public abstract int getSize();

    public String getContentType() {
        return DataType.getTypeAString(DataType.getType(getName()));
    }

    public abstract byte[] getBytes();

    public static void move(Resource res, OutputStream os, int start, int length) throws IOException {
        if (length <= 0) {
            IOMethods.fillIn(res.getInputStream(), os, 1024);
        } else {
            DataInputStream is = new DataInputStream(res.getInputStream());
            is.skip(start);
            byte data[] = new byte[length];
            is.readFully(data);
            os.write(data);
            os.flush();
        }
    }
}
