/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.sdl;

import java.io.InputStream;

public class BitReaderD extends BitReader {
    public java.io.PrintWriter traceFile;

    public BitReaderD(InputStream in) {
        super(in);
    }

    public boolean readBit(String s) throws java.io.IOException {
        boolean b = readBit();
        traceFile.print(s);
        fill(20 - s.length());
        traceFile.print(" n=1\tvalue=");
        traceFile.println(b);
        return b;
    }

    public int readInt(String s) throws java.io.IOException {
        int b = readInt();
        traceFile.print(s);
        fill(20 - s.length());
        traceFile.print(" n=32\tvalue=");
        traceFile.println(b);
        return b;
    }

    public int readBits(int n, String s) throws java.io.IOException {
        int i = readBits(n);
        traceBits(n, i, s);
        return i;
    }

    public void traceBits(int n, int i, String s) {
        traceFile.print(s);
        fill(20 - s.length());
        traceFile.print(" n=");
        traceFile.print(n);
        traceFile.print("\tvalue=");
        //traceFile.println(Integer.toHexString(i));
        traceFile.println(i);
    }

    private void fill(int n) {
        for (int i = 0; i < n; i++) {
            traceFile.print(" ");
        }
    }

    /*public String readString(String s) throws java.io.IOException {
        int l = readBits(5, "len of len of string");
        int len = readBits(l, "len of string");
        String b = readString(len);
        traceFile.print(s);
        fill(20 - s.length());
        traceFile.print(" n=");
        traceFile.print(len * 8);
        traceFile.print("\tvalue=\"");
        traceFile.print(b);
        traceFile.println("\"");
        return b;
    }*/

    public String readVlcString(String s) throws java.io.IOException {
        int len = readVlcInt(8, "len of string");
        String b = readString(len);
        // System.out.println(b);
        traceFile.print(s);
        fill(20 - s.length());
        traceFile.print(" n=");
        traceFile.print(len * 8);
        traceFile.print("\tvalue=\"");
        traceFile.print(b);
        traceFile.println("\"");
        return b;
    }

    public byte[] readByteArray(int l, String s) throws java.io.IOException {
        traceFile.println("reading " + l + " bytes for " + s);
        return readByteArray(l);
    }

    public byte[] readByteArrayWithLength(String s) throws java.io.IOException {
        int l = readVlcInt(5, "len of byte array");
        traceFile.println("reading " + l + " bytes for " + s);
        return readByteArray(l);
    }

    public void flush(String s) {
        // System.out.println("rflushing " + valid);
        if (valid > 0) {
            traceFile.println("flushing " + valid + " bits: " + s);
            super.flush();
        }
    }

    public int pos() {
        return byteRead;
    }

    public String[] readString1(String msg) throws java.io.IOException {
        int bitsUrlLen = readBits(5, "len of len");
        int l = readBits(bitsUrlLen, "len");
        byte a[] = new byte[l + 2];
        a[0] = (byte) ((l >>> 8) & 0xFF);
        a[1] = (byte) (l & 0xFF);
        for (int i = 2; i < l + 2; i++) {
            a[i] = (byte) (0xFF & readByte());
        }
        java.io.ByteArrayInputStream bais = new java.io.ByteArrayInputStream(a);
        String tmp[] = new String[2];
        tmp[0] = new java.io.DataInputStream(bais).readUTF();
        //this is the number of bytes taken by the encoded string with 5 bits + X (url length in bytes) bits
        tmp[1] = Integer.toString(l + (5 + bitsUrlLen + 8 - (5 + bitsUrlLen) % 8) / 8);
        traceFile.print(msg);
        fill(20 - msg.length());
        traceFile.print(" n=");
        traceFile.print(5 + bitsUrlLen + (l * 8));
        traceFile.print("\tvalue=");
        traceFile.println(tmp[0]);
        return tmp;
    }

    public int readVlcInt(int n, String msg) throws java.io.IOException {
        int k = 1;
        while (readBit()) k++;
        int i = readBits(k * (n - 1));
        traceBits(k * n, i, "vl "+msg);
        return i;
    }
}
