/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.sdl;

import java.io.PrintWriter;

public class Event {
    public int event = Integer.MIN_VALUE;
    public String eventString = null;
    public int keyCode = 15; // NO_KEY

    public void print(PrintWriter pw) {
        if (eventString != null) {
            pw.print(eventString);
        } else {
            printEventEnum(pw, event, keyCode);
        }
    }

    public final static String eventNames[] = new String[] {"abort", "accesskey", "activate", "activatedEvent", "beginEvent",
            "click", "deactivatedEvent", "endEvent", "error", "executionTime", "focusin", "focusout", "keydown", "keyup", "load", "longaccesskey",
            "mousedown", "mousemove", "mouseout", "mouseover", "mouseup", "pause", "pausedEvent", "play", "repeatEvent", "resize",
            "resumedEvent", "scroll", "textinput",
            "unload", "zoom"/*, "lsr:connecting", "lsr:playable", "lsr:buffering", "lsr:endDownload", "lsr:endStream"*/};

    public static void printEventEnum(PrintWriter pw, int i, int j) {
        pw.print(eventNames[i]);
        if (i == 1 || i == 12) {
            pw.print("(");
            printKeyCode(pw, j);
            pw.print(")");
        }
    }

    public static void printKeyCode(PrintWriter pw, int i) {
        switch (i) {
            case 17:
                pw.print("SHARP");
                break;
            case 0:
                pw.print("*");
                break;
            case 19:
                pw.print("SOFT_KEY_2");
                break;
            case 18:
                pw.print("SOFT_KEY_1");
                break;
            case 20:
                pw.print("UP");
                break;
            case 13:
                pw.print("FIRE");
                break;
            case 15:
                pw.print("NO_KEY");
                break;
            case 11:
                pw.print("ANY_KEY");
                break;
            case 16:
                pw.print("RIGHT");
                break;
            case 14:
                pw.print("LEFT");
                break;
            case 12:
                pw.print("DOWN");
                break;
            case 10:
                pw.print("9");
                break;
            case 9:
                pw.print("8");
                break;
            case 8:
                pw.print("7");
                break;
            case 7:
                pw.print("6");
                break;
            case 6:
                pw.print("5");
                break;
            case 5:
                pw.print("4");
                break;
            case 4:
                pw.print("3");
                break;
            case 3:
                pw.print("2");
                break;
            case 2:
                pw.print("1");
                break;
            case 1:
                pw.print("0");
                break;
        }
    }

    public int readKeyCode(String s) {
        if ("SHARP".equals(s) || "#".equals(s)) return 17;
        if ("*".equals(s)) return 0;
        if ("SOFT_KEY_2".equals(s)) return 19;
        if ("SOFT_KEY_1".equals(s)) return 18;
        if ("UP".equals(s)) return 20;
        if ("FIRE".equals(s)) return 13;
        if ("NO_KEY".equals(s)) return 15;
        if ("ANY_KEY".equals(s)) return 11;
        if ("RIGHT".equals(s)) return 16;
        if ("LEFT".equals(s)) return 14;
        if ("DOWN".equals(s)) return 12;
        if ("9".equals(s)) return 10;
        if ("8".equals(s)) return 9;
        if ("7".equals(s)) return 8;
        if ("6".equals(s)) return 7;
        if ("5".equals(s)) return 6;
        if ("4".equals(s)) return 5;
        if ("3".equals(s)) return 4;
        if ("2".equals(s)) return 3;
        if ("1".equals(s)) return 2;
        if ("0".equals(s)) return 1;
        System.out.println("unknown key code:"+s);
        return -1;
    }

    public Event read(String s) {
        if (s.startsWith("accessKey(") || s.startsWith("accesskey(")) {
            event = 1;
            keyCode = readKeyCode(s.substring(10, s.length()-1));
            return this;
        }
        if ("begin".equals(s)) {
            event = 4;
            return this;
        }
        if ("end".equals(s)) {
            event = 7;
            return this;
        }
        if (s.startsWith("longAccessKey(") || s.startsWith("longaccesskey(")) {
            event = 15;
            keyCode = readKeyCode(s.substring(14, s.length()-1));
            return this;
        }
        if ("repeat".equals(s)) {
            event = 23;
            return this;
        }
        for (int i = 0; i < eventNames.length; i++) {
            if (eventNames[i].equals(s)) {
                event = i;
                return this;
            }
        }
        eventString = s;
        return this;
    }

}
