/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.sdl;

import iso.mpeg.streamezzo.util.Utils;
import iso.mpeg.streamezzo.util.Scene;
import iso.mpeg.streamezzo.util.Manager;
import iso.mpeg.streamezzo.laser.*;
import iso.mpeg.streamezzo.io.Decoder;
import iso.mpeg.streamezzo.io.Encoder;
import iso.mpeg.streamezzo.saf.SceneUnit;

import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.util.Vector;

public class SDLClasses {
    public static int scaleBits = 0;
    public static int coordBits = 0;
    public static int resolution = 0;
    public static int colorComponentBits = 0;
    public static int colorIndexBits = 0;
    public static int fontIndexBits = 0;
    public static int nameSpaceIndexBits = 0;
    public static int tagIndexBits = 0;
    public static int codecBits = 0;
    public static int extensionIDBits = 4;
    public static int privateDataIdBits = 0;
    public static Scene currentScene = null;
    public static Manager man = null;
    public static int pointBits = 0;
    public static int fixed_16_8 = 0;
    public static float timeResolution = 1000f;
    //public static int matrixBits = 0;
    public static int currentTimeInTics = 0;

    public static Attribute class_AttributeName(BitReaderD br, String msg) throws IOException {
        Attribute a = new Attribute();
        if (br.readBit(msg)) {
            a.isLaser = false;
            int tagID = br.readVlcInt(5, msg);
            if (tagID != 0) {
                a.tag = man.getTag(tagID);
            }
            a.attr = man.getAttr(tagID, br.readVlcInt(5, msg));
        } else {
            a.isLaser = true;
            a.attributeIndex = br.readBits(8, msg);
        }
        return a;
    }

    public static void class_AttributeName(BitWriter br, Attribute a) throws IOException {
        br.putBit(!a.isLaser);
        if (a.isLaser) {
            br.putBits(8, a.attributeIndex);
        } else {
            int tagID = (a.tag == null ? 0 : man.getTagID(a.tag));
            br.putVlcInt(5, tagID);
            br.putVlcInt(5, man.getAttrID(tagID, a.attr));
        }
    }

    public static void class_anyAttribute(BitReaderD br, String msg, X s) throws IOException {
        if (br.readBit(msg)) {
            do {
                int extID = br.readBits(extensionIDBits, msg);
                int bitlen = br.readVlcInt(5, msg);
                if (extID == 2) {
                    if (s == null) {
                        throw new RuntimeException("decoding AMD1 extension attribute for wrong class");
                    }
                    if (br.readBit("hasRequiredFonts")) {
                        s.requiredFonts = class_byteAlignedString(br, msg);
                    }
                    if (br.readBit("hasX")) {
                        s.x = class_coordinate(br, msg);
                    }
                    if (br.readBit("hasY")) {
                        s.y = class_coordinate(br, msg);
                    }
                } else {
                    if (bitlen % 8 != 0) br.readBits(bitlen % 8, msg);
                    if (bitlen >> 3 != 0) br.readByteArray(bitlen >> 3, msg);
                }
            } while (br.readBit(msg));
        }
    }

    public static void class_anyAttribute(BitWriter bw, String s) throws IOException {
        boolean b = s != null;
        bw.putBit(b);
        if (b) {
            bw.putBits(extensionIDBits, 2);
            int len = bw.bitlength()%8;
            bw.putVlcInt(5, (len == 0 ? 0 : 8 - len) + (s.length()*8));
            class_byteAlignedString(bw, s);
            bw.putBit(false);
        }
    }

    public static void privateChildren(BitWriter br, Vector children, Encoder enc) throws IOException {
        br.putVlcInt(5, children.size());
        for (int i = 0; i < children.size(); i++) {
            Object o = children.elementAt(i);
            boolean b = (o instanceof XtextContent || o instanceof XprivateElement);
            br.putBit(b);
            if (b) {
                b = o instanceof XtextContent;
                br.putBit(b);
                if (b) {
                    SDLClasses.class_byteAlignedString(br, ((XtextContent)o).getContent());
                } else {
                    privateElement(br, (XprivateElement)o, enc);
                }
            } else {
                enc.updatable_elements(br, o);
            }
        }
    }

    public static void privateElement(BitWriter br, XprivateElement pe, Encoder enc) throws IOException {
        int tagIndex = man.getTagID(pe.name);
        br.putBits(tagIndexBits, tagIndex);
        boolean b = pe.id != Integer.MIN_VALUE;
        br.putBit(b);
        if (b) {
            class_ID(br, pe.id);
        }
        br.putVlcInt(5, pe.privateAttributes.length);
        for (int i = 0; i < pe.privateAttributes.length; i++) {
            b = NameSpace.equals(pe.privateAttributes[i].ns, pe.name.ns);
            br.putBit(b);
            if (b) {
                privateAttribute(br, pe.privateAttributes[i], tagIndex);
            } else {
                privateAttribute(br, pe.privateAttributes[i], 0);
            }
        }
        b = pe.child != null;
        br.putBit(b);
        if (b) {
            privateChildren(br, pe.child, enc);
        }
    }

    private static void privateAttribute(BitWriter br, privAttr pa, int tagIndex) throws IOException {
        int bits = Utils.nbBitsForPositiveInteger(man.nbOfAttrs(tagIndex));
        br.putBits(bits, man.getAttrID(tagIndex, pa));
        SDLClasses.class_byteAlignedString(br, pa.value);
    }

    public static void class_opaqueElement(BitReaderD br, XprivateElementContainer pec) throws IOException {
        int len = br.readVlcInt(5, "opaqueLen");
        br.flush("opaque");
        pec.privateDataIdentifier = br.readBits(privateDataIdBits, "privDIdBits");
        br.flush("opaque");
        pec.opaque = br.readByteArray(len - ((privateDataIdBits + 7) >> 3));
    }

    public static int class_overlay(BitReaderD br, String msg) throws IOException {
        if (br.readBit(msg)) {
            return br.readBits(1, msg);
        } else {
            class_extension(br, msg);
        }
        return Integer.MIN_VALUE;
    }

    public static int class_signedInt(BitReaderD br, String msg) throws IOException {
        boolean neg = br.readBit("sign");
        int i = br.readVlcInt(5, msg);
        if (neg) i = -i;
        return i;
    }

    public static void class_signedInt(BitWriter br, int i) throws IOException {
        br.putBit(i < 0);
        if (i < 0) i = -i;
        br.putVlcInt(5, i);
    }

    public static void class_overlay(BitWriter br, int i) throws IOException {
        br.putBit(true);
        br.putBits(1, i);
    }

    private final static String scriptTypes[] = new String []{"application/ecmaScript", "application/jar-archive"};

    public static String class_scriptType(BitReaderD br, String msg) throws IOException {
        if (br.readBit("enumFlag")) {
            return scriptTypes[br.readBits(1, msg)];
        } else {
            return class_byteAlignedString(br, msg);
        }
    }

    public static void class_script(BitWriter bw, String t) throws IOException {
        if ("application/ecmascript".equalsIgnoreCase(t)) {
            bw.putBit(true);
            bw.putBits(2, 0);
        } else if ("application/jar-archive".equalsIgnoreCase(t)) {
            bw.putBit(true);
            bw.putBits(2, 1);
        } else if ("application/laserscript".equalsIgnoreCase(t)) {
            bw.putBit(true);
            bw.putBits(2, 2);
        } else {
            bw.putBit(false);
            class_byteAlignedString(bw, t);
        }
    }

    public static void privateAttributeContainer(BitWriter br, privAttr[] pa) throws IOException {
        br.putBits(4, 0); // anyXML
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        BitWriter bw1 = new BitWriter(baos);
        for (int i = 0; i < pa.length; i++) {
            privateAttribute(br, pa[i], 0);
        }
        bw1.flush();
        br.flush();
        br.putVlcInt(5, bw1.length());
        br.putByteArray(baos.toByteArray());

    }

    public static privAttr[] privateAttributeContainer(BitReaderD br, String msg) throws IOException {
        if (br.readBit("hasPrivAttr")) {
            int ch4 = br.readBits(4);
            switch (ch4) {
                case 0:
                    return class_anyXMLAttribute(br, msg);
                case 1:
                    int skipLen = br.readVlcInt(5, msg);
                    br.flush(msg);
                    br.readBits(SDLClasses.privateDataIdBits, "privateDataIndex");
                    br.flush(msg);
                    br.readByteArray(skipLen - ((SDLClasses.privateDataIdBits + 7) >> 3), "privateData");
                    break;
                case 2:
                    class_anyAttribute(br, msg, null);
                    break;
                default:
                    class_extension(br, msg);
                    break;
            }
        }
        return null;
    }

    private static privAttr[] class_anyXMLAttribute(BitReaderD br, String msg) throws IOException {
        int nb = br.readVlcInt(5, "nb of attrs");
        privAttr pa[] = new privAttr[nb];
        int bits = Utils.nbBitsForPositiveInteger(SDLClasses.man.nbOfAttrs(0));
        for (int i = 0; i < nb; i++) {
            pa[i] = new privAttr();
            pa[i].set(SDLClasses.man.getAttr(0, br.readBits(bits, "attr idx")));
            pa[i].value = SDLClasses.class_byteAlignedString(br, "priv attr val");
        }
        return pa;
    }

    public static int class_preserveAspectRatio(BitReaderD br, String msg) throws IOException {
        // this code is maintained as is because it is correct
        // the complexity of the SDL type is artificial
        return br.readBits(6, msg);
    }

    public static void class_preserveAspectRatio(BitWriter bw, int i) throws IOException {
        // this code is maintained as is because it is correct
        // the complexity of the SDL type is artificial
        bw.putBits(6, i);
    }

    public static int[] coordinateListType(BitReaderD br, String msg) throws IOException {
        int len = br.readVlcInt(5, msg);
        int res[] = new int[len];
        for (int i = 0; i < len; i++) {
            res[i] = class_coordinate(br, msg);
        }
        return res;
    }

    public static void coordinateListType(BitWriter br, int i[]) throws IOException {
        br.putVlcInt(5, i.length);
        for (int j = 0; j < i.length; j++) {
            class_coordinate(br, i[j]);
        }
    }

    public static int repeatCountType(BitReaderD br, String msg) throws IOException {
        if (br.readBit(msg)) {
            return Integer.MAX_VALUE;
        } else {
            return class_fixed_16_8(br, msg);
        }
    }

    public static int repeatDurType(BitReaderD br, String msg) throws IOException {
        if (br.readBit(msg)) {
            return Integer.MAX_VALUE;
        } else {
            return translateTimeUponReading(br.readVlcInt(5, msg), true);
        }
    }

    public static void repeatCountType(BitWriter bw, int i) throws IOException {
        if (i == Integer.MAX_VALUE) {
            bw.putBit(true);
        } else {
            bw.putBit(false);
            class_fixed_16_8(bw, i);
        }
    }

    public static void repeatDurType(BitWriter bw, int i) throws IOException {
        if (i == Integer.MAX_VALUE) {
            bw.putBit(true);
        } else {
            bw.putBit(false);
            bw.putVlcInt(5, translateTimeUponWriting(i, true));
        }
    }

    public static SmilTime[] timesType(BitReaderD br, String msg) throws IOException {
        if (br.readBit(msg)) {
            return new SmilTime[] {new SmilTime()};
        } else {
            int len = br.readVlcInt(5, "nbTimes");
            SmilTime res[] = new SmilTime[len];
            for (int i = 0; i < len; i++) {
                res[i] = smilTime(br, msg);
            }
            return res;
        }
    }

    public static SmilTime smilTime(BitReaderD br, String msg) throws IOException {
        SmilTime res = new SmilTime();
        if (br.readBit("hasEvent")) {
            if (br.readBit("hasId")) {
                res.id = class_IDREF(br, msg);
            }
            res.event = class_event(br, msg);
        }
        if (br.readBit("hasClock")) {
            boolean neg = br.readBit("sign");
            res.delay = translateTimeUponReading(br.readVlcInt(5, msg), false);
            if (neg) res.delay = - res.delay;
        }
        return res;
    }

    public static void timesType(BitWriter br, SmilTime i[]) throws IOException {
        if (i.length == 1 && i[0].isIndefinite()) {
            br.putBit(true);
        } else {
            br.putBit(false);
            br.putVlcInt(5, i.length);
            for (int j = 0; j < i.length; j++) {
                smilTime(br, i[j]);
            }
        }
    }

    public static void smilTime(BitWriter bw, SmilTime t) throws IOException {
        bw.putBit(t.event != null);
        if (t.event != null) {
            bw.putBit(t.id != Integer.MIN_VALUE);
            if (t.id != Integer.MIN_VALUE) {
                class_IDREF(bw, t.id);
            }
            class_event(bw, t.event);
        }
        bw.putBit(t.delay != Integer.MIN_VALUE);
        if (t.delay != Integer.MIN_VALUE) {
            bw.putBit(t.delay < 0);
            bw.putVlcInt(5, translateTimeUponWriting(Math.abs(t.delay), false));
        }
    }

    public static void class_rare(BitWriter bw, X o) throws IOException {
        // find the number of rares
        int nbOfRares = 0;
        boolean rare2 = false;
        int nbOfRare2 = 0, bitLenRare2 = 0;
        if (o.classe != null) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).audio_level != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).color != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).color_rendering != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Idpv && ((Idpv) o).getDisplay() != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).display_align != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).fill_opacity != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).fill_rule != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).image_rendering != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).line_increment != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Idpv && ((Idpv) o).getPointer_events() != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).shape_rendering != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).solid_color != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).solid_opacity != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).stop_color != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).stop_opacity != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).stroke_dasharray != null) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).stroke_dashoffset != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).stroke_linecap != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).stroke_linejoin != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).stroke_miterlimit != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).stroke_opacity != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).stroke_width != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).text_anchor != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).text_rendering != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).viewport_fill != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).viewport_fill_opacity != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).vector_effect != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Idpv && ((Idpv) o).getVisibility() != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Ibasic && ((Ibasic) o).getRequiredExtensions() != null) nbOfRares++;
        if (o instanceof Ibasic && ((Ibasic) o).getRequiredFeatures() != null) nbOfRares++;
        if (o instanceof Ibasic && ((Ibasic) o).getRequiredFormats() != null) nbOfRares++;
        if (o instanceof Ibasic && ((Ibasic) o).getSystemLanguage() != null) nbOfRares++;
        if (o.base != null) nbOfRares++;
        if (o.lang != null) nbOfRares++;
        if (o.space != Integer.MIN_VALUE) nbOfRares++;
        if (o.navNext != Integer.MIN_VALUE) nbOfRares++;
        if (o.navUp != Integer.MIN_VALUE) nbOfRares++;
        if (o.navUpRight != Integer.MIN_VALUE) nbOfRares++;
        if (o.navUpLeft != Integer.MIN_VALUE) nbOfRares++;
        if (o.navPrev != Integer.MIN_VALUE) nbOfRares++;
        if (o.navDown != Integer.MIN_VALUE) nbOfRares++;
        if (o.navDownRight != Integer.MIN_VALUE) nbOfRares++;
        if (o.navDownLeft != Integer.MIN_VALUE) nbOfRares++;
        if (o.navLeft != Integer.MIN_VALUE) nbOfRares++;
        if (o.focusable != Integer.MIN_VALUE) nbOfRares++;
        if (o.navRight != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Ilocatable && ((Ilocatable) o).getTransform() != null) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).font_variant != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).font_family != null) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).font_size != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).font_style != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).text_decoration != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof XpropElem && ((XpropElem) o).font_weight != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Ixlink && ((Ixlink) o).getTitle() != null) nbOfRares++;
        if (o instanceof Ixlink && ((Ixlink) o).getXlinkType() != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Ixlink && ((Ixlink) o).getRole() != null) nbOfRares++;
        if (o instanceof Ixlink && ((Ixlink) o).getArcrole() != null) nbOfRares++;
        if (o instanceof Ixlink && ((Ixlink) o).getActuate() != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Ixlink && ((Ixlink) o).getShow() != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Xset && ((Xset) o).end != null) nbOfRares++;
        if (o instanceof Xaudio && ((Xaudio) o).end != null) nbOfRares++;
        if (o instanceof Xset && ((Xset) o).max != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Xset && ((Xset) o).min != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Xaudio && ((Xaudio) o).max != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Xaudio && ((Xaudio) o).min != Integer.MIN_VALUE) nbOfRares++;
        if (o instanceof Xaudio && ((Xaudio) o).syncMaster != Integer.MIN_VALUE) {
            rare2 = true;
            nbOfRare2++;
            bitLenRare2 += 4;
        }
        if (o instanceof XpropElem && ((XpropElem) o).focusHighlight != Integer.MIN_VALUE) {
            rare2 = true;
            nbOfRare2++;
            bitLenRare2 += 5;
        }
        if (o instanceof Xvideo && ((Xvideo)o).initialVisibility != Integer.MIN_VALUE) {
            rare2 = true;
            nbOfRare2++;
            bitLenRare2 += 5;
        }
        if (o instanceof Xanimation && ((Xanimation)o).initialVisibility != Integer.MIN_VALUE) {
            rare2 = true;
            nbOfRare2++;
            bitLenRare2 += 5;
        }
        if (o instanceof Xvideo && ((Xvideo)o).fullscreen != Integer.MIN_VALUE) {
            rare2 = true;
            nbOfRare2++;
            bitLenRare2 += 4;
        }
        if (o instanceof Xsvg && ((Xsvg)o).fullscreen != Integer.MIN_VALUE) {
            rare2 = true;
            nbOfRare2++;
            bitLenRare2 += 4;
        }
        if (o.active != Integer.MIN_VALUE) {
            rare2 = true;
            nbOfRare2++;
            bitLenRare2 += 3;
        }
        if (o.requiredFonts != null) {
            rare2 = true;
            nbOfRare2++;
            int length_of_requiredFonts = o.requiredFonts.length()*8;
            // compute advance of bitstream, and add other rare2
            // then there will be a byte alignment
            int align = 8 - ((bw.bitlength()+1+6+bitLenRare2)%8);
            if (align==8) align = 0;
            bitLenRare2 += 3+length_of_requiredFonts+align;
        }
        if (rare2) nbOfRares++;
        bw.putBit(nbOfRares > 0);
        if (nbOfRares == 0) return;
        // encode the rares
        bw.putBits(6, nbOfRares);
        // rare2
        if (rare2) {
            bw.putBits(6, /* RARE_EXTENSION */ 49);
            bw.putBits(extensionIDBits, 2);
            bw.putVlcInt(5, bitLenRare2+2);
            bw.putBits(2, nbOfRare2);
            if (o instanceof Xaudio && ((Xaudio)o).syncMaster != Integer.MIN_VALUE) {
                bw.putBits(3, 0);
                bw.putBit(((Xaudio)o).syncMaster == 1);
            }
            if (o instanceof XpropElem && ((XpropElem)o).focusHighlight != Integer.MIN_VALUE) {
                bw.putBits(3, 1);
                bw.putBits(2, ((XpropElem)o).focusHighlight);
            }
            if (o instanceof Xvideo && ((Xvideo) o).initialVisibility != Integer.MIN_VALUE) {
                bw.putBits(3, 2);
                bw.putBits(2, ((Xvideo)o).initialVisibility);
            }
            if (o instanceof Xanimation && ((Xanimation) o).initialVisibility != Integer.MIN_VALUE) {
                bw.putBits(3, 2);
                bw.putBits(2, ((Xanimation)o).initialVisibility);
            }
            if (o instanceof Xvideo && ((Xvideo)o).fullscreen != Integer.MIN_VALUE) {
                bw.putBits(3, 3);
                bw.putBits(2, ((Xvideo)o).fullscreen);
            }
            if (o instanceof Xsvg && ((Xsvg)o).fullscreen != Integer.MIN_VALUE) {
                bw.putBits(3, 3);
                bw.putBits(2, ((Xsvg)o).fullscreen);
            }
            if (o.requiredFonts != null) {
                bw.putBits(3, 4);
                SDLClasses.class_byteAlignedString(bw, o.requiredFonts);
            }
            if (o.active != Integer.MIN_VALUE) {
                bw.putBits(3, 5);
                bw.putBits(1, o.active);
            }
            bw.putBit(false);
        }
        if (o.classe != null) {
            bw.putBits(6, /* RARE_CLASS */ 0);
            SDLClasses.class_byteAlignedString(bw, o.classe);
        }
        if (o instanceof XpropElem && ((XpropElem) o).audio_level != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_AUDIO_LEVEL */ 1);
            SDLClasses.class_0to1float(bw, ((XpropElem) o).audio_level);
        }
        if (o instanceof XpropElem && ((XpropElem) o).color != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_COLOR */ 2);
            SDLClasses.class_paint(bw, ((XpropElem) o).color);
        }
        if (o instanceof XpropElem && ((XpropElem) o).color_rendering != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_COLOR_RENDERING */ 3);
            bw.putBits(2, ((XpropElem) o).color_rendering);
        }
        if (o instanceof Idpv && ((Idpv) o).getDisplay() != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_DISPLAY */ 4);
            bw.putBits(5, ((Idpv) o).getDisplay());
        }
        if (o instanceof XpropElem && ((XpropElem) o).display_align != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_DISPLAY_ALIGN */ 5);
            bw.putBits(3, ((XpropElem) o).display_align);
        }
        if (o instanceof XpropElem && ((XpropElem) o).fill_opacity != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FILL_OPACITY */ 6);
            SDLClasses.class_0to1float(bw, ((XpropElem) o).fill_opacity);
        }
        if (o instanceof XpropElem && ((XpropElem) o).fill_rule != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FILL_RULE */ 7);
            bw.putBits(2, ((XpropElem) o).fill_rule);
        }
        if (o instanceof XpropElem && ((XpropElem) o).image_rendering != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_IMAGE_RENDERING */ 8);
            bw.putBits(2, ((XpropElem) o).image_rendering);
        }
        if (o instanceof XpropElem && ((XpropElem) o).line_increment != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_LINE_INCREMENT */ 9);
            SDLClasses.lineIncrementType(bw, ((XpropElem) o).line_increment);
        }
        if (o instanceof Idpv && ((Idpv) o).getPointer_events() != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_POINTER_EVENTS */ 10);
            bw.putBits(4, ((Idpv) o).getPointer_events());
        }
        if (o instanceof XpropElem && ((XpropElem) o).shape_rendering != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_SHAPE_RENDERING */ 11);
            bw.putBits(3, ((XpropElem) o).shape_rendering);
        }
        if (o instanceof XpropElem && ((XpropElem) o).solid_color != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_SOLID_COLOR */ 12);
            SDLClasses.class_paint(bw, ((XpropElem) o).solid_color);
        }
        if (o instanceof XpropElem && ((XpropElem) o).solid_opacity != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_SOLID_OPACITY */ 13);
            SDLClasses.class_0to1float(bw, ((XpropElem) o).solid_opacity);
        }
        if (o instanceof XpropElem && ((XpropElem) o).stop_color != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_STOP_COLOR */ 14);
            SDLClasses.class_paint(bw, ((XpropElem) o).stop_color);
        }
        if (o instanceof XpropElem && ((XpropElem) o).stop_opacity != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_STOP_OPACITY */ 15);
            SDLClasses.class_0to1float(bw, ((XpropElem) o).stop_opacity);
        }
        if (o instanceof XpropElem && ((XpropElem) o).stroke_dasharray != null) {
            bw.putBits(6, /* RARE_STROKE_DASHARRAY */ 16);
            bw.putBit(false); // isInherit bit
            int max = ((XpropElem)o).stroke_dasharray.length;
            bw.putVlcInt(5, max);
            for (int j = 0; j < max; j++) {
                SDLClasses.class_fixed_16_8(bw, ((XpropElem) o).stroke_dasharray[j]);
            }
        }
        if (o instanceof XpropElem && ((XpropElem) o).stroke_dashoffset != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_STROKE_DASHOFFSET */ 17);
            SDLClasses.class_fixed_16_8i(bw, ((XpropElem) o).stroke_dashoffset);
        }
        if (o instanceof XpropElem && ((XpropElem) o).stroke_linecap != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_STROKE_LINECAP */ 18);
            bw.putBits(2, ((XpropElem) o).stroke_linecap);
        }
        if (o instanceof XpropElem && ((XpropElem) o).stroke_linejoin != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_STROKE_LINEJOIN */ 19);
            bw.putBits(2, ((XpropElem) o).stroke_linejoin);
        }
        if (o instanceof XpropElem && ((XpropElem) o).stroke_miterlimit != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_STROKE_MITERLIMIT */ 20);
            SDLClasses.class_fixed_16_8i(bw, ((XpropElem) o).stroke_miterlimit);
        }
        if (o instanceof XpropElem && ((XpropElem) o).stroke_opacity != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_STROKE_OPACITY */ 21);
            SDLClasses.class_0to1float(bw, ((XpropElem) o).stroke_opacity);
        }
        if (o instanceof XpropElem && ((XpropElem) o).stroke_width != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_STROKE_WIDTH */ 22);
            SDLClasses.class_fixed_16_8i(bw, ((XpropElem) o).stroke_width);
        }
        if (o instanceof XpropElem && ((XpropElem) o).text_anchor != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_TEXT_ANCHOR */ 23);
            bw.putBits(2, ((XpropElem) o).text_anchor);
        }
        if (o instanceof XpropElem && ((XpropElem) o).text_rendering != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_TEXT_RENDERING */ 24);
            bw.putBits(3, ((XpropElem)o).text_rendering);
        }
        if (o instanceof XpropElem && ((XpropElem) o).viewport_fill != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_VIEWPORT_FILL */ 25);
            SDLClasses.class_paint(bw, ((XpropElem) o).viewport_fill);
        }
        if (o instanceof XpropElem && ((XpropElem) o).viewport_fill_opacity != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_VIEWPORT_FILL_OPACITY */ 26);
            SDLClasses.class_0to1float(bw, ((XpropElem) o).viewport_fill_opacity);
        }
        if (o instanceof XpropElem && ((XpropElem) o).vector_effect != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_VECTOR_EFFECT */ 27);
            bw.putBits(4, ((XpropElem) o).vector_effect);
        }
        if (o instanceof Idpv && ((Idpv) o).getVisibility() != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_VISIBILITY */ 28);
            bw.putBits(2, ((Idpv) o).getVisibility());
        }
        if (o instanceof Ibasic && ((Ibasic) o).getRequiredExtensions() != null) {
            bw.putBits(6, /* RARE_REQUIREDEXTENSIONS */ 29);
            SDLClasses.class_byteAlignedString(bw, ((Ibasic) o).getRequiredExtensions());
        }
        if (o instanceof Ibasic && ((Ibasic) o).getRequiredFeatures() != null) {
            bw.putBits(6, /* RARE_REQUIREDFEATURES */ 30);
            byte fe[] = ((Ibasic) o).getRequiredFeatures();
            bw.putVlcInt(5, fe.length);
            for (int i = 0; i < fe.length; i++) {
                bw.putBits(6, fe[i]);
            }
        }
        if (o instanceof Ibasic && ((Ibasic) o).getRequiredFormats() != null) {
            bw.putBits(6, /* RARE_REQUIREDFORMATS */ 31);
            SDLClasses.class_byteAlignedString(bw, ((Ibasic) o).getRequiredFormats());
        }
        if (o instanceof Ibasic && ((Ibasic) o).getSystemLanguage() != null) {
            bw.putBits(6, /* RARE_SYSTEMLANGUAGE */ 32);
            SDLClasses.class_byteAlignedString(bw, ((Ibasic) o).getSystemLanguage());
        }
        if (o.base != null) {
            bw.putBits(6, /* RARE_XML_BASE */ 33);
            SDLClasses.class_byteAlignedString(bw, o.base);
        }
        if (o.lang != null) {
            bw.putBits(6, /* RARE_XML_LANG */ 34);
            SDLClasses.class_byteAlignedString(bw, o.lang);
        }
        if (o.space != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_XML_SPACE */ 35);
            bw.putBits(1, o.space);
        }
        if (o.navNext != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FOCUSNEXT */ 36);
            SDLClasses.focusType(bw, o.navNext);
        }
        if (o.navUp != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FOCUSNORTH */ 37);
            SDLClasses.focusType(bw, o.navUp);
        }
        if (o.navUpRight != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FOCUSNORTHEAST */ 38);
            SDLClasses.focusType(bw, o.navUpRight);
        }
        if (o.navUpLeft != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FOCUSNORTHWEST */ 39);
            SDLClasses.focusType(bw, o.navUpLeft);
        }
        if (o.navPrev != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FOCUSPREV */ 40);
            SDLClasses.focusType(bw, o.navPrev);
        }
        if (o.navDown != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FOCUSSOUTH */ 41);
            SDLClasses.focusType(bw, o.navDown);
        }
        if (o.navDownRight != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FOCUSSOUTHEAST */ 42);
            SDLClasses.focusType(bw, o.navDownRight);
        }
        if (o.navDownLeft != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FOCUSSOUTHWEST */ 43);
            SDLClasses.focusType(bw, o.navDownLeft);
        }
        if (o.navLeft != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FOCUSWEST */ 44);
            SDLClasses.focusType(bw, o.navLeft);
        }
        if (o.focusable != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FOCUSABLE */ 45);
            bw.putBits(2, o.focusable);
        }
        if (o.navRight != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FOCUSEAST */ 46);
            SDLClasses.focusType(bw, o.navRight);
        }
        if (o instanceof Ilocatable && ((Ilocatable) o).getTransform() != null) {
            bw.putBits(6, /* RARE_TRANSFORM */ 47);
            SDLClasses.transformType(bw, ((Ilocatable) o).getTransform());
        }
        if (o instanceof XpropElem && ((XpropElem) o).font_variant != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FONT_VARIANT */ 50);
            bw.putBits(2, ((XpropElem) o).font_variant);
        }
        if (o instanceof XpropElem && ((XpropElem) o).font_family != null) {
            bw.putBits(6, /* RARE_FONT_FAMILY */ 51);
            SDLClasses.class_fontFamily(bw, ((XpropElem) o).font_family);
        }
        if (o instanceof XpropElem && ((XpropElem) o).font_size != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FONT_SIZE */ 52);
            SDLClasses.class_fixed_16_8i(bw, ((XpropElem) o).font_size);
        }
        if (o instanceof XpropElem && ((XpropElem) o).font_style != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FONT_STYLE */ 53);
            bw.putBits(3, ((XpropElem) o).font_style);
        }
        if (o instanceof XpropElem && ((XpropElem) o).text_decoration != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_TEXT_DECORATION */ 48);
            bw.putBits(4, ((XpropElem) o).text_decoration);
        }
        if (o instanceof XpropElem && ((XpropElem) o).font_weight != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_FONT_WEIGHT */ 54);
            bw.putBits(4, ((XpropElem) o).font_weight);
        }
        if (o instanceof Ixlink && ((Ixlink) o).getTitle() != null) {
            bw.putBits(6, /* RARE_HREF_TITLE */ 55);
            SDLClasses.class_byteAlignedString(bw, ((Ixlink) o).getTitle());
        }
        if (o instanceof Ixlink && ((Ixlink) o).getXlinkType() != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_HREF_TYPE */ 56);
            bw.putBits(3, ((Ixlink) o).getXlinkType());
        }
        if (o instanceof Ixlink && ((Ixlink) o).getRole() != null) {
            bw.putBits(6, /* RARE_HREF_ROLE */ 57);
            SDLClasses.class_anyURI(bw, ((Ixlink) o).getRole());
        }
        if (o instanceof Ixlink && ((Ixlink) o).getArcrole() != null) {
            bw.putBits(6, /* RARE_HREF_ARCROLE */ 58);
            SDLClasses.class_anyURI(bw, ((Ixlink) o).getArcrole());
        }
        if (o instanceof Ixlink && ((Ixlink) o).getActuate() != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_HREF_ACTUATE */ 59);
            bw.putBits(2, ((Ixlink) o).getActuate());
        }
        if (o instanceof Ixlink && ((Ixlink) o).getShow() != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_HREF_SHOW */ 60);
            bw.putBits(3, ((Ixlink) o).getShow());
        }
        if (o instanceof Xset && ((Xset) o).end != null) {
            bw.putBits(6, /* RARE_END */ 61);
            SDLClasses.timesType(bw, ((Xset) o).end);
        }
        if (o instanceof Xaudio && ((Xaudio) o).end != null) {
            bw.putBits(6, /* RARE_END */ 61);
            SDLClasses.timesType(bw, ((Xaudio) o).end);
        }
        if (o instanceof Xset && ((Xset) o).max != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_MAX */ 62);
            SDLClasses.class_time(bw, ((Xset) o).max, true);
        }
        if (o instanceof Xset && ((Xset) o).min != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_MIN */ 63);
            SDLClasses.class_time(bw, ((Xset) o).min, true);
        }
        if (o instanceof Xaudio && ((Xaudio) o).max != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_MAX */ 62);
            SDLClasses.class_time(bw, ((Xaudio) o).max, true);
        }
        if (o instanceof Xaudio && ((Xaudio) o).min != Integer.MIN_VALUE) {
            bw.putBits(6, /* RARE_MIN */ 63);
            SDLClasses.class_time(bw, ((Xaudio) o).min, true);
        }
    }

    public static void class_extension(BitReaderD br, String msg) throws IOException {
        int i = br.readVlcInt(5, msg);
        br.readByteArray(i, msg);
    }

    public static void class_anyURI(BitWriter bw, URI uri) throws IOException {
        boolean b = uri.uri != null;
        bw.putBit(b);
        if (b) {
            class_byteAlignedString(bw, uri.uri);
            // optional binary data part of the URL
            bw.putBit(uri.data != null);
            if (uri.data != null) { // data part of the URL
                bw.putVlcInt(5, uri.data.length);
                bw.putByteArray(uri.data);
            }
            // end of optional binary data part of the URL
        }
        b = uri.id != Integer.MIN_VALUE;
        bw.putBit(b);
        if (b) class_IDREF(bw, uri.id);
        b = uri.streamID != Integer.MIN_VALUE;
        bw.putBit(b);
        if (b) class_IDREF(bw, uri.streamID);
    }

    public static void class_anyURI(BitWriter bw, UpdateValue uv) throws IOException {
        boolean b = uv.string != null;
        bw.putBit(b);
        if (b) {
            class_byteAlignedString(bw, uv.string);
            // optional binary data part of the URL
            // a data url is not updatable
            bw.putBit(false);
            // end of optional binary data part of the URL
        }
        b = uv.x != Integer.MIN_VALUE;
        bw.putBit(b);
        if (b) class_IDREF(bw, uv.x);
        b = uv.y != Integer.MIN_VALUE;
        bw.putBit(b);
        if (b) class_IDREF(bw, uv.y);
    }

    public static URI class_anyURI(BitReaderD br, String msg) throws IOException {
        URI uri = new URI();
        boolean b = br.readBit("hasURL");
        if (b) {
            uri.uri = class_byteAlignedString(br, msg);
            // optional binary data part of the URL
            if (br.readBit("hasData")) { //data part of the URL
                int len = br.readVlcInt(5, "data len");
                uri.data = br.readByteArray(len, "url data");
            }
            // end of optional binary data part of the URL
        }
        b = br.readBit("hasID");
        if (b) uri.id = class_IDREF(br, "ID");
        b = br.readBit("hasStreamID");
        if (b) uri.streamID = class_IDREF(br, "streamID");
        return uri;
    }

    public static void class_anyURI(BitReaderD br, UpdateValue uv, String msg) throws IOException {
        boolean b = br.readBit("hasURL");
        if (b) {
            uv.string = class_byteAlignedString(br, msg);
            // optional binary data part of the URL
            br.readBit("hasData(f!)"); // is false in this case, no data URL in replace
            // end of optional binary data part of the URL
        }
        b = br.readBit("hasID");
        if (b) uv.x = class_IDREF(br, "ID");
        b = br.readBit("hasStreamID");
        if (b) uv.y = class_IDREF(br, "streamID");
    }

    public static UpdateValue class_updateValue(int type, BitReaderD br, String msg) throws IOException {
        UpdateValue uv = new UpdateValue(type);
        int max = 0;
        switch (type) {
            case TYPE_boolean:
                uv.x = (br.readBit(msg) ? 1 : 0);
                break;
            case TYPE_enum:
                uv.isDefault = br.readBit("isDefault");
                if (!uv.isDefault) {
                    uv.x = br.readVlcInt(5, msg);
                }
                break;
            case TYPE_index:
            case TYPE_ID:
                uv.isDefault = br.readBit("isDefault");
                if (!uv.isDefault) {
                    if (br.readBit("enum")) {
                        uv.escapeEnum = br.readBits(2, msg);
                    } else {
                        uv.x = br.readVlcInt(5, msg);
                    }
                }
                break;
            case TYPE_color:
                uv.x = class_paint(br, msg);
                break;
            case TYPE_fraction:
                uv.isDefault = br.readBit("isDefault");
                if (!uv.isDefault) {
                    uv.x = class_0to1float(br, msg);
                }
                break;
            case TYPE_float:
                uv.isDefault = br.readBit("isDefault");
                if (!uv.isDefault) {
                    if (br.readBit("enum")) {
                        uv.escapeEnum = br.readBits(2, msg);
                    } else {
                        uv.x = class_fixed_16_8(br, msg);
                    }
                }
                break;
            case TYPE_time:
                uv.x = class_time(br, msg, true);
                break;
            case TYPE_point:
                pointType(br, uv, msg);
                break;
            case TYPE_matrix:
                uv.matrix = class_transform(br, msg);
                break;
            case TYPE_string:
                uv.string = class_byteAlignedString(br, msg);
                break;
            case TYPE_points:
                uv.list = class_pointSequence(br, msg);
                break;
            case TYPE_path:
                uv.path = class_path(br, msg);
                break;
            case TYPE_ints:
                max = br.readVlcInt(5, msg);
                uv.list = new int[max];
                for (int i = 0; i < max; i++) {
                    uv.list[i] = br.readBits(8, msg);
                }
                break;
            case TYPE_floats: // floatValues
                uv.isDefault = br.readBit("isDefault");
                if (!uv.isDefault) {
                    if (br.readBit("enum")) {
                        uv.escapeEnum = br.readBits(2, msg);
                    } else {
                        max = br.readVlcInt(5, msg);
                        uv.list = new int[max];
                        for (int i = 0; i < max; i++) {
                            uv.list[i] = class_fixed_16_8(br, msg);
                        }
                    }
                }
                break;
            case TYPE_keyTimes:
                uv.list = class_fraction12List(br, msg);
                break;
            case TYPE_smil_times: // timeValues
                uv.times = timesType(br, msg);
                break;
            case TYPE_unit: // unitValue
                uv.list = class_valueWithUnits(br, msg);
                break;
            case TYPE_URI:
                class_anyURI(br, uv, msg);
                break;
            case TYPE_event:
                uv.event = class_event(br, msg);
                break;
            case TYPE_scale:
                uv.x = class_fixed_16_8(br, msg);
                uv.y = class_fixed_16_8(br, msg);
                break;
            case TYPE_coordinate:
                uv.x = class_coordinate(br, msg);
                break;
            case ITYPE_point:
                pointType(br, uv, msg);
                break;
            case ITYPE_int:
                // small int
                uv.x = br.readBits(8, msg);
                break;
            case ITYPE_float:
                // float
                uv.x = class_fixed_16_8(br, msg);
                break;
            case ITYPE_smil_time:
                uv.time = smilTime(br, msg);
                break;
            case ITYPE_keytime:
                if (br.readBit("hasShort")) {
                    if (br.readBit("isZero")) {
                        uv.x = 0;
                    } else {
                        uv.x = 4096;
                    }
                } else {
                    uv.x = br.readBits(12 , msg);
                }
                break;
            case ITYPE_0to1:
                uv.x = class_0to1float(br, msg);
                uv.y = class_0to1float(br, msg);
                break;
            case -1: // error
                throw new RuntimeException("type error in update");
            default:
                throw new RuntimeException("unimplemented type error in update:" + type);
        }
        return uv;
    }

    public static void class_updateValue(BitWriter bw, UpdateValue uv) throws IOException {
        switch (uv.type) {
            case TYPE_boolean:
                bw.putBit(uv.x == 1);
                break;
            case TYPE_enum:
                if (uv.isDefault) {
                    bw.putBit(true);
                } else {
                    bw.putBit(false);
                    bw.putVlcInt(5, uv.x);
                }
                break;
            case TYPE_index:
            case TYPE_ID:
                if (uv.isDefault) {
                    bw.putBit(true);
                } else {
                    bw.putBit(false);
                    if (uv.escapeEnum != -1) {
                        bw.putBit(true);
                        bw.putBits(2, uv.escapeEnum);
                    } else {
                        bw.putBit(false);
                        bw.putVlcInt(5, uv.x);
                    }
                }
                break;
            case TYPE_color:
                class_paint(bw, uv.x);
                break;
            case TYPE_fraction:
                if (uv.isDefault) {
                    bw.putBit(true);
                } else {
                    bw.putBit(false);
                    class_0to1float(bw, uv.x);
                }
                break;
            case TYPE_float:
                if (uv.isDefault) {
                    bw.putBit(true);
                } else {
                    bw.putBit(false);
                    if (uv.escapeEnum != -1) {
                        bw.putBit(true);
                        bw.putBits(2, uv.escapeEnum);
                    } else {
                        bw.putBit(false);
                        class_fixed_16_8(bw, uv.x);
                    }
                }
                break;
            case TYPE_time:
                class_time(bw, uv.x, true);
                break;
            case TYPE_point:
                pointType(bw, uv);
                break;
            case TYPE_matrix:
                transformType(bw, uv.matrix);
                break;
            case TYPE_string:
                class_byteAlignedString(bw, uv.string);
                break;
            case TYPE_points:
                class_pointSequence(bw, uv.list);
                break;
            case TYPE_path:
                class_path(bw, uv.path);
                break;
            case TYPE_ints:
                bw.putVlcInt(5, uv.list.length);
                for (int i = 0; i < uv.list.length; i++) {
                    bw.putBits(8, uv.list[i]);
                }
                break;
            case TYPE_floats: // floatValues
                if (uv.isDefault) {
                    bw.putBit(true);
                } else {
                    bw.putBit(false);
                    if (uv.escapeEnum != -1) {
                        bw.putBit(true);
                        bw.putBits(2, uv.escapeEnum);
                    } else {
                        bw.putBit(false);
                        bw.putVlcInt(5, uv.list.length);
                        for (int i = 0; i < uv.list.length; i++) {
                            class_fixed_16_8(bw, uv.list[i]);
                        }
                    }
                }
                break;
            case TYPE_smil_times: // timeValues
                timesType(bw, uv.times);
                break;
            case TYPE_unit: // unitValue
                class_valueWithUnits(bw, uv.list);
                break;
            case TYPE_URI:
                class_anyURI(bw, uv);
                break;
            case TYPE_event:
                class_event(bw, uv.event);
                break;
            case TYPE_scale:
                class_fixed_16_8(bw, uv.x);
                class_fixed_16_8(bw, uv.y);
                break;
            case TYPE_coordinate:
                class_coordinate(bw, uv.x);
                break;
            case ITYPE_point:
                pointType(bw, uv);
                break;
            case ITYPE_int:
                bw.putBits(8, uv.x);
                break;
            case ITYPE_float:
                class_fixed_16_8(bw, uv.x);
                break;
            case ITYPE_keytime:
                class_time(bw, uv.x, false);
                break;
            case ITYPE_0to1:
                class_0to1float(bw, uv.x);
                class_0to1float(bw, uv.y);
                break;
            default:
                throw new RuntimeException("unimplemented type error in update:" + uv.type);
        }
    }

    public static int typeOf(Attribute a, int index) {
        if (a.isLaser) {
            if (index >= 0) {
                // indexed
                return updateIndexedType[a.attributeIndex];
            } else {
                // not indexed
                return updateType[a.attributeIndex];
            }
        } else {
            return TYPE_string;
        }
    }

    public final static int TYPE_boolean = 0;
    public final static int TYPE_enum = 1;
    public final static int TYPE_color = 2;
    public final static int TYPE_fraction = 3;
    public final static int TYPE_float = 4;
    public final static int TYPE_time = 5;
    public final static int TYPE_point = 6;
    public final static int TYPE_matrix = 7;
    public final static int TYPE_string = 8;
    public final static int TYPE_points = 9;
    public final static int TYPE_path = 10;
    public final static int TYPE_ints = 11;
    public final static int TYPE_floats = 12;
    public final static int TYPE_smil_times = 13;
    public final static int TYPE_unit = 14;
    public final static int TYPE_index = 15;
    public final static int TYPE_URI = 16;
    public final static int TYPE_ID = 17;
    public final static int TYPE_event = 18;
    public final static int TYPE_scale = 19;
    public final static int TYPE_coordinate = 20;
    public final static int TYPE_keyTimes = 21;

    public static int updateType(Attribute a) {
        if (a.isLaser) {
            return updateType[a.attributeIndex];
        } else {
            return TYPE_string;
        }
    }

    public static int updateIType(Attribute a) {
        if (a.isLaser) {
            return updateIndexedType[a.attributeIndex];
        } else {
            throw new RuntimeException("cannot update anyXML attribute with index");
        }
    }

    private static final int updateType[] = {
            TYPE_string, // a.target
            TYPE_boolean, // accumulate
            TYPE_boolean, // additive
            TYPE_fraction, // audio-level
            TYPE_float, // bandwidth
            TYPE_smil_times, // begin
            TYPE_enum, // calcMode
            -1, // children
            TYPE_index, // choice
            TYPE_time, // clipBegin
            TYPE_time, // clipEnd
            TYPE_color, // color
            TYPE_enum, // color-rendering
            TYPE_coordinate, // cx
            TYPE_coordinate, // cy
            TYPE_path, // d
            TYPE_floats, // delta
            TYPE_enum, // display
            TYPE_enum, // display-align
            TYPE_time, // dur
            TYPE_boolean, // editable
            TYPE_boolean, // enabled
            TYPE_smil_times, // end
            TYPE_event, // event
            TYPE_boolean, // externalResourcesRequired
            TYPE_color, // fill
            TYPE_fraction, // fill-opacity
            TYPE_enum, // fill-rule
            TYPE_boolean, // focusable
            TYPE_index, // font-family
            TYPE_float, // font-size
            TYPE_enum, // font-style
            TYPE_enum, // font-variant
            TYPE_enum, // font-weight
            TYPE_boolean, // fullscreen
            TYPE_enum, // gradient-units
            TYPE_ID, // handler
            TYPE_coordinate, // height
            TYPE_enum, // image-rendering
            TYPE_floats, // keyPoints
            TYPE_floats, // keySplines
            TYPE_keyTimes, // keyTimes
            TYPE_float, // line-increment
            TYPE_ID, // listener.target
            TYPE_string, // mediaCharacterEncoding
            TYPE_string, // mediaContentEncodings
            TYPE_float, // mediaSize
            TYPE_time, // mediaTime
            TYPE_ID, // nav-down
            TYPE_ID, // nav-down-left
            TYPE_ID, // nav-down-right
            TYPE_ID, // nav-left
            TYPE_ID, // nav-next
            TYPE_ID, // nav-prev
            TYPE_ID, // nav-right
            TYPE_ID, // nav-up
            TYPE_ID, // nav-up-left
            TYPE_ID, // nav-up-right
            TYPE_ID, // observer
            TYPE_fraction, // offset
            TYPE_fraction, // opacity
            TYPE_enum, // overflow
            TYPE_enum, // overlay
            TYPE_path, // path
            TYPE_float, // pathLength
            TYPE_enum, // pointer-events
            TYPE_points, // points
            -1, // preserveAspectRatio
            TYPE_coordinate, // r
            TYPE_float, // repeatCount
            TYPE_time, // repeatDur
            TYPE_string, // requiredExtensions
            TYPE_ints, // requiredFeatures
            TYPE_string, // requiredFormats
            TYPE_enum, // restart
            TYPE_floats, // rotate
            TYPE_float, // rotation
            TYPE_coordinate, // rx
            TYPE_coordinate, // ry
            TYPE_scale, // scale
            TYPE_enum, // shape-rendering
            TYPE_point, // size
            TYPE_color, // solid-color
            TYPE_fraction, // solid-opacity
            TYPE_color, // stop-color
            TYPE_fraction, // stop-opacity
            TYPE_color, // stroke
            TYPE_floats, // stroke-dasharray
            TYPE_float, // stroke-dashoffset
            TYPE_enum, // stroke-linecap
            TYPE_enum, // stroke-linejoin
            TYPE_float, // stroke-miterlimit
            TYPE_fraction, // stroke-opacity
            TYPE_float, // stroke-width
            TYPE_unit, // svg.height
            TYPE_unit, // svg.width
            TYPE_enum, // syncBehavior
            TYPE_enum, // syncBehaviorDefault
            TYPE_URI, // syncReference
            TYPE_float, // syncTolerance
            TYPE_float, // syncToleranceDefault
            TYPE_string, // systemLanguage
            TYPE_enum, // text-align
            TYPE_enum, // text-anchor
            TYPE_enum, // text-decoration
            TYPE_enum, // text-display
            TYPE_enum, // text-rendering
            TYPE_string, // textContent
            TYPE_matrix, // transform
            TYPE_enum, // transformBehavior
            TYPE_point, // translation
            TYPE_enum, // vector-effect,
            TYPE_floats, // viewBox
            TYPE_color, // viewport-fill
            TYPE_fraction, // viewport-fill-opacity
            TYPE_enum, // visibility
            TYPE_coordinate, // width
            TYPE_coordinate, // x
            TYPE_coordinate, // x1
            TYPE_coordinate, // x2
            TYPE_enum, // xlink:actuate
            TYPE_string, // xlink:arcrole
            TYPE_URI, // xlink:href
            TYPE_string, // xlink:role
            TYPE_enum, // xlink:show
            TYPE_string, // xlink:title
            TYPE_enum, // xlink:type
            TYPE_string, // xml:base
            TYPE_string, // xml:lang
            TYPE_coordinate, // y
            TYPE_coordinate, // y1
            TYPE_coordinate, // y2
            TYPE_boolean // zoomAndPan
    };

    // ITYPE_* codes have been changed to fix a bug
    public static final int ITYPE_point = 100;
    public static final int ITYPE_int = 101;
    public static final int ITYPE_float = 102;
    public static final int ITYPE_keytime = 103;
    public static final int ITYPE_0to1 = 104;
    public static final int ITYPE_smil_time = 105;

    public static final int updateIndexedType[] = {
            -1, // a.target
            -1, // accumulate
            -1, // additive
            -1, // audio-level
            -1, // bandwidth
            ITYPE_smil_time, // begin
            -1, // calcMode
            -1, // children
            -1, // choice
            -1, // clipBegin
            -1, // clipEnd
            -1, // color
            -1, // color-rendering
            -1, // cx
            -1, // cy
            -1, // d
            -1, // delta
            -1, // display
            -1, // display-align
            -1, // dur
            -1, // editable
            -1, // enabled
            ITYPE_smil_time, // end
            -1, // event
            -1, // externalResourcesRequired
            -1, // fill
            -1, // fill-opacity
            -1, // fill-rule
            -1, // focusable
            -1, // font-family
            -1, // font-size
            -1, // font-style
            -1, // font-variant
            -1, // font-weight
            -1, // fullscreen
            -1, // gradient-units
            -1, //handler
            -1, // height
            -1, // image-rendering
            ITYPE_0to1, // keyPoints
            ITYPE_float, // keySplines
            ITYPE_keytime, // keyTimes
            -1, // line-increment
            -1, // listener.target
            -1, // mediaCharacterEncoding
            -1, // mediaContentEncodings
            -1, // mediaSize
            -1, // mediaTime
            -1, // nv-down
            -1, // nav-down-left
            -1, // nav-down-right
            -1, // nav-left
            -1, // nav-next
            -1, // nav-prev
            -1, // nav-right
            -1, // nav-up
            -1, // nav-up-left
            -1, // nav-up-right
            -1, // observer
            -1, // offset
            -1, // opacity
            -1, // overflow
            -1, // overlay
            -1, // path
            -1, // pathLength
            -1, // pointer-events
            ITYPE_point, // points
            -1, // preserveAspectRatio
            -1, // r
            -1, // repeatCount
            -1, // repeatDur
            -1, // requiredExtensions
            ITYPE_int, // requiredFeatures
            -1, // requiredFormats
            -1, // restart
            -1, // rotate
            -1, // rotation
            -1, // rx
            -1, // ry
            -1, // scale
            -1, // shape-rendering
            -1, // delta
            -1, // solid-color
            -1, // solid-opacity
            -1, // stop-color
            -1, // stop-opacity
            -1, // stroke
            ITYPE_float, // stroke-dasharray
            -1, // stroke-dashoffset
            -1, // stroke-linecap
            -1, // stroke-linejoin
            -1, // stroke-miterlimit
            -1, // stroke-opacity
            -1, // stroke-width
            -1, // svg.height
            -1, // svg.width
            -1, // syncBehavior
            -1, // syncBehaviorDefault
            -1, // syncReference
            -1, // syncTolerance
            -1, // syncToleranceDefault
            -1, // systemLanguage
            -1, // text-align
            -1, // text-anchor
            -1, // text-decoration
            -1, // text-display
            -1, // text-rendering
            -1, // textContent
            -1, // transform
            -1, // transformBehavior
            -1, // translation
            -1, // vector-effect
            ITYPE_float, // viewBox
            -1, // viewport-fill
            -1, // viewport-fill-opacity
            -1, // visibility
            -1, // width
            -1, // x
            -1, // x1
            -1, // x2
            -1, // xlink:actuate
            -1, // xlink:arcrole
            -1, // xlink:href
            -1, // xlink:role
            -1, // xlink:show
            -1, // xlink:title
            -1, // xlink:type
            -1, // xml:base
            -1, // xml:lang
            -1, // y
            -1, // y1
            -1, // y2
            -1 // zoomAndPan
    };

    public static AnimValue[] class_AnimatedValues(BitReaderD br, String msg) throws IOException {
        int choice = br.readBits(4, msg);
        int max = br.readVlcInt(5, msg);
        AnimValue u[] = new AnimValue[max];
        for (int i = 0; i < max; i++) {
            AnimValue av = new AnimValue();
            av.type = choice;
            u[i] = animValue1(av, br, msg);
        }
        return u;
    }

    public static void class_AnimatedValues(BitWriter br, AnimValue av[]) throws IOException {
        int start = br.bitlength();
        br.putBits(4, av[0].type);
        br.putVlcInt(5, av.length);
        codecBits += br.bitlength() - start;
        for (int i = 0; i < av.length; i++) {
            animValue1(br, av[i]);
        }
    }

    public final static int ANIMTYPE_string = 0;
    public final static int ANIMTYPE_float = 1;
    public final static int ANIMTYPE_path = 2;
    public final static int ANIMTYPE_pointSeq = 3;
    public final static int ANIMTYPE_fraction = 4;
    public final static int ANIMTYPE_color = 5;
    public final static int ANIMTYPE_enum = 6;
    //public final static int ANIMTYPE_ints = 7; now reserved
    public final static int ANIMTYPE_floats = 8;
    public final static int ANIMTYPE_point = 9;
    public final static int ANIMTYPE_id = 10;
    public final static int ANIMTYPE_font = 11;
    public final static int ANIMTYPE_uri = 12;

    public static int animateType(Attribute a) {
        if (a.isLaser) {
            return animType[a.attributeIndex];
        } else {
            return ANIMTYPE_string;
        }
    }

    public final static int animType[] = new int[]{
            ANIMTYPE_string, // a.target
            -1, // accumulate
            -1, // additive
            ANIMTYPE_fraction, //audio-level
            -1, // bandwidth
            -1, // begin
            ANIMTYPE_enum, // calcMode
            -1, // children
            ANIMTYPE_enum, //choice
            -1, // clipBegin
            -1, // clipEnd
            ANIMTYPE_color, //color
            ANIMTYPE_enum, // color-rendering
            ANIMTYPE_float, //cx
            ANIMTYPE_float, //cy
            ANIMTYPE_path, //d
            ANIMTYPE_point, // delta
            ANIMTYPE_enum, //display
            ANIMTYPE_enum, //display-align
            -1, // dur
            ANIMTYPE_enum, //editable
            -1, // enabled
            -1, // end
            -1, // event
            -1, // externalResourcesRequired
            ANIMTYPE_color, //fill
            ANIMTYPE_fraction, //fill-opacity
            ANIMTYPE_enum, //fill-rule
            ANIMTYPE_enum, //focusable
            ANIMTYPE_font, //font-family
            ANIMTYPE_float, //font-size
            ANIMTYPE_enum, //font-style
            ANIMTYPE_enum, // font-variant
            ANIMTYPE_enum, //font-weight
            -1, // fullscreen
            ANIMTYPE_enum, //gradientUnits
            -1, // handler
            ANIMTYPE_float, //height
            ANIMTYPE_enum, //image-rendering
            -1, // keyPoints
            -1, // keySplines
            -1, // keyTimes
            ANIMTYPE_float, // line-increment
            -1, // listener.target
            -1, // mediaCharacterEncoding
            -1, // mediaContentEncodings
            -1, // mediaSize
            -1, // mediaTime
            ANIMTYPE_id, // nav*
            ANIMTYPE_id, // nav*
            ANIMTYPE_id, // nav*
            ANIMTYPE_id, // nav*
            ANIMTYPE_id, // nav*
            ANIMTYPE_id, // nav*
            ANIMTYPE_id, // nav*
            ANIMTYPE_id, // nav*
            ANIMTYPE_id, // nav*
            ANIMTYPE_id, // nav*
            -1, // observer
            -1, // offset
            ANIMTYPE_fraction, //opacity
            -1, // overflow
            -1, // overlay
            -1, // path
            ANIMTYPE_float, //pathLength
            ANIMTYPE_enum, //pointer-events
            ANIMTYPE_pointSeq, //points
            ANIMTYPE_enum, //preserveAspectRatio
            ANIMTYPE_float, //r
            -1, // repeatCount
            -1, // repeatDur
            -1, // requiredExtensions
            -1, // requiredFeatures
            -1, // requiredFormats
            -1, // restart
            ANIMTYPE_float, //rotate
            -1, // rotation
            ANIMTYPE_float, //rx
            ANIMTYPE_float, //ry
            -1, // scale
            ANIMTYPE_enum, //shape-rendering
            ANIMTYPE_point, //size
            ANIMTYPE_color, // solid-color
            ANIMTYPE_fraction, // solid-opacity
            ANIMTYPE_color, //stop-color
            ANIMTYPE_fraction, //stop-opacity
            ANIMTYPE_color, //stroke
            ANIMTYPE_floats, //stroke-dasharray
            ANIMTYPE_float, //stroke-dashoffset
            ANIMTYPE_enum, //stroke-linecap
            ANIMTYPE_enum, //stroke-linejoin
            ANIMTYPE_float, //stroke-miterlimit
            ANIMTYPE_fraction, //stroke-opacity
            ANIMTYPE_float, //stroke-width
            ANIMTYPE_float, // svg.height
            ANIMTYPE_float, // svg.width
            -1, // syncBehavior
            -1, // syncBehaviorDefault
            -1, // syncReference
            -1, // syncTolerance
            -1, // syncToleranceDefault
            -1, // systemLanguage
            -1, // text-align
            ANIMTYPE_enum, //text-anchor
            ANIMTYPE_enum, //text-decoration
            ANIMTYPE_enum, //text-display
            -1, // text-rendering
            -1, // textContent
            ANIMTYPE_floats, //transform
            -1, // transformBehavior
            -1, // translation
            ANIMTYPE_enum, //vector-effect
            ANIMTYPE_floats, //viewBox
            ANIMTYPE_color, //viewport-fill
            ANIMTYPE_fraction, //viewport-fill-opacity
            ANIMTYPE_enum, //visibility
            ANIMTYPE_float, //width
            ANIMTYPE_float, //x
            ANIMTYPE_float, //x1
            ANIMTYPE_float, //x2
            -1, // xlink:actuate
            -1, // xlink:arcrole
            ANIMTYPE_uri, //xlink:href
            -1, // xlink:role
            -1, // xlink:show
            -1, // xlink:title
            -1, // xlink:type
            -1, // xml:base
            -1, // xml:lang
            ANIMTYPE_float, //y
            ANIMTYPE_float, //y1
            ANIMTYPE_float, //y2
            -1 // zoomAndPan
    };

    public static int[] class_valueWithUnits(BitReaderD br, String msg) throws IOException {
        int u[] = new int[2];
        u[0] = br.readInt(msg); // float represented as fixed point with 24 bits mantissa
        u[1] = br.readBits(3, msg); // 0 no unit, 1 in, 2 cm, 3 mm, 4 pt, 5 pc, 6 'px', 7 %
        return u;
    }

    public static void class_valueWithUnits(BitWriter br, int i[]) throws IOException {
        int start = br.bitlength();
        br.putInt(i[0]); // float represented as fixed point with 24 bits mantissa
        br.putBits(3, i[1]); // 0 no unit, 1 in, 2 cm, 3 mm, 4 pt, 5 pc, 6 'px', 7 %
        codecBits += br.bitlength() - start;
    }

    public static String class_fontFamily(BitReaderD br, String msg) throws IOException {
        if (br.readBit(msg)) return "inherit";
        return man.getFont(br.readBits(fontIndexBits, msg));
    }

    public static void class_fontFamily(BitWriter br, String f) throws IOException {
        int start = br.bitlength();
        if ("inherit".equals(f)) {
            br.putBit(true);
        } else {
            br.putBit(false);
            br.putBits(fontIndexBits, man.getFontID(f));
        }
        codecBits += br.bitlength() - start;
    }

    public static Path class_path(BitReaderD br, String msg) throws IOException {
        Path path = new Path();
        path.pointSeq = class_pointSequence(br, msg);
        path.types = new byte[br.readVlcInt(5, msg)];
        for (int i = 0; i < path.types.length; i++) {
            path.types[i] = (byte) br.readBits(5, msg);
        }
        if ( (0 == path.pointSeq.length) || (0 == path.types.length) ) {
            path = null;            
        }
        return path;
    }

    public static void class_path(BitWriter br, Path path) throws IOException {
        if (null == path) {
            class_pointSequence(br, null);
        } else {
            class_pointSequence(br, path.pointSeq);
        }
        int start = br.bitlength();
        if (null == path || path.types == null) {
            br.putVlcInt(5, 0);
        } else {
            br.putVlcInt(5, path.types.length);
            for (int i = 0; i < path.types.length; i++) {
                br.putBits(5, path.types[i]);
            }
        }
        codecBits += br.bitlength() - start;
    }

    public static int[] dasharrayType(BitReaderD br, String msg) throws IOException {
        if (br.readBit(msg)) {
            return new int[0]; // inherit is an empty but non null array
        }
        int max = br.readVlcInt(5, msg);
        int u[] = new int[max];
        for (int i = 0; i < max; i++) {
            u[i] = class_fixed_16_8(br, msg);
        }
        return u;
    }

    public static void dasharrayType(BitWriter br, int da[]) throws IOException {
        int start = br.bitlength();
        if (da.length == 0) {
            br.putBit(true);
            codecBits += br.bitlength() - start;
        } else {
            br.putBit(false);
            br.putVlcInt(5, da.length);
            codecBits += br.bitlength() - start;
            for (int i = 0; i < da.length; i++) {
                class_fixed_16_8(br, da[i]);
            }
        }
    }

    public static int[] class_pointSequence(BitReaderD br, String msg) throws IOException {
        int len = br.readVlcInt(5, "nbOfPoints");
        int res[] = new int[2 * len];
        if (br.readBit("newCodec")) {
            int k = br.readBits(4, "k");
            int bits = br.readBits(5, "nbBitsFirst");
            int firstX = translateUponReading(br.readBits(bits, "firstX"), bits);
            res[0] = firstX;
            int firstY = translateUponReading(br.readBits(bits, "firstY"), bits);
            res[1] = firstY;
            for (int i = 2; i < len * 2; i += 2) {
                int mvalue = 0;
                while (!br.readBit("mvalue bits x")) mvalue++;
                int info = getInfo(br.readBits(mvalue + k, "dx"), k, mvalue);
                int dx = extract(info);
                firstX = firstX + translateUponReading(dx, mvalue + k + 1);
                res[i] = firstX;
                mvalue = 0;
                while (!br.readBit("mvalue bits x")) mvalue++;
                info = getInfo(br.readBits(mvalue + k, "dy"), k, mvalue);
                int dy = extract(info);
                firstY = firstY + translateUponReading(dy, mvalue + k + 1);
                res[i + 1] = firstY;
            }
        } else {
            read1(br, res, len);
        }
        return res;
    }

    public static void class_pointSequence(BitWriter bw, int seq[]) throws IOException {
        int start = bw.bitlength();
        if (null == seq) {
            bw.putVlcInt(5, 0);
            bw.putBit(false);
            bw.putBits(5, 0);
            return;
        }
        bw.putVlcInt(5, seq.length / 2);
        bw.putBit(false); //TODO add new codec
        if (seq.length < 6) {
            int bits = 0;
            for (int i = 0; i < seq.length; i++) {
                bits = Math.max(bits, Utils.nbBitsForInteger(translateUponWriting(seq[i])));
            }
            bw.putBits(5, bits);
            for (int i = 0; i < seq.length; i++) {
                bw.putBits(bits, translateUponWriting(seq[i]));
            }
            codecBits += bw.bitlength() - start;
            return;
        }
        int firstX = seq[0], firstY = seq[1];
        int curX = translateUponWriting(firstX), curY = translateUponWriting(firstY);
        int fbits = Math.max(Utils.nbBitsForInteger(curX), Utils.nbBitsForInteger(curY));
        bw.putBits(5, fbits);
        int f = translateUponWriting(firstX);
        bw.putBits(fbits, f);
        f = translateUponWriting(firstY);
        bw.putBits(fbits, f);
        int maxDX = -1, maxDY = -1, minDX = 1, minDY = 1;
        for (int i = 2; i < seq.length; i += 2) {
            int x = seq[i], y = seq[i + 1];
            int dx = translateUponWriting(x) - curX, dy = translateUponWriting(y) - curY;
            maxDX = Math.max(maxDX, dx);
            maxDY = Math.max(maxDY, dy);
            minDX = Math.min(minDX, dx);
            minDY = Math.min(minDY, dy);
            curX += dx;
            curY += dy;
        }
        int dxbits = Math.max(Utils.nbBitsForInteger(minDX), Utils.nbBitsForInteger(maxDX));
        int dybits = Math.max(Utils.nbBitsForInteger(minDY), Utils.nbBitsForInteger(maxDY));
        bw.putBits(5, dxbits);
        bw.putBits(5, dybits);
        curX = translateUponWriting(firstX);
        curY = translateUponWriting(firstY);
        for (int i = 2; i < seq.length; i += 2) {
            int x = seq[i], y = seq[i + 1];
            int dx = translateUponWriting(x) - curX, dy = translateUponWriting(y) - curY;
            bw.putBits(dxbits, dx);
            bw.putBits(dybits, dy);
            curX += dx;
            curY += dy;
        }
        codecBits += bw.bitlength() - start;
    }

    private static int extract(int i) {
        if (i == 0) return i;
        if ((i & 1) == 0) {
            return -(i >> 1);
        }
        return (i + 1) >> 1;
    }

    private static int getInfo(int codeNum, int kvalue, int Mvalue) {
        return codeNum - power2[kvalue] + power2[kvalue + Mvalue];
    }

    private static final int power2[] = {
            0x00000001, 0x00000002, 0x00000004, 0x00000008, 0x00000010, 0x00000020, 0x00000040, 0x00000080,
            0x00000100, 0x00000200, 0x00000400, 0x00000800, 0x00001000, 0x00002000, 0x00004000, 0x00008000,
            0x00010000, 0x00020000, 0x00040000, 0x00080000, 0x00100000, 0x00200000, 0x00400000, 0x00800000,
            0x01000000, 0x02000000, 0x04000000, 0x08000000, 0x10000000, 0x20000000, 0x40000000, 0x80000000
    };

    private static void read1(BitReaderD br, int fv[], int len) throws IOException {
        /*
        If the number of points is less than 3, a simple encoding is used.
        A number of 5 bits give the number of bits on which the coordinates are encoded.
        */
        if (len < 3) {
            int bits = br.readBits(5, "nbBits");
            for (int i = 0; i < len * 2; i++) {
                fv[i] = translateUponReading(br.readBits(bits, "coord"), bits);
            }
        } else {
            /*
            If the number of points LEN in the polygon are more than 3, then:
            - the number N1 is read on 5 bits
            - the first two coordinates are read from the stream using N1 bits
            - the number NX and NY are read on 5 bits
            - then LEN-1 points are read, using NX points for the X value and NY bits for the Y value, both X and Y
            being differences from the previous point
            */
            int bits = br.readBits(5, "nbBitsFirst");
            int firstX = translateUponReading(br.readBits(bits, "firstX"), bits);
            fv[0] = firstX;
            int firstY = translateUponReading(br.readBits(bits, "firstY"), bits);
            fv[1] = firstY;
            int bitsX = br.readBits(5, "nbBitsX");
            int bitsY = br.readBits(5, "nbBitsY");
            for (int i = 2; i < len * 2; i += 2) {
                int cur = translateUponReading(br.readBits(bitsX, "nextX"), bitsX);
                fv[i] = firstX + cur;
                firstX = firstX + cur;
                cur = translateUponReading(br.readBits(bitsY, "nextY"), bitsY);
                fv[i + 1] = firstY + cur;
                firstY = firstY + cur;
            }
        }
    }

    public static void class_namespace(BitWriter br, NameSpace n) throws IOException {
        int start = br.bitlength();
        br.putBits(nameSpaceIndexBits, man.getNameSpaceID(n));
        codecBits += br.bitlength() - start;
    }

    public static NameSpace class_namespace(BitReaderD br, String msg) throws IOException {
        return man.getNameSpace(br.readBits(nameSpaceIndexBits, msg));
    }

    public static byte[] byteList(BitReaderD br, String msg) throws IOException {
        int max = br.readVlcInt(5, msg);
        byte u[] = new byte[max];
        for (int i = 0; i < max; i++) {
            u[i] = (byte) br.readBits(8, msg);
        }
        return u;
    }

    public static void byteList(BitWriter br, byte b[]) throws IOException {
        int start = br.bitlength();
        br.putVlcInt(5, b.length);
        for (int i = 0; i < b.length; i++) {
            br.putBits(8, b[i]);
        }
        codecBits += br.bitlength() - start;
    }

    public static byte[] featureList(BitReaderD br, String msg) throws IOException {
        int max = br.readVlcInt(5, msg);
        byte u[] = new byte[max];
        for (int i = 0; i < max; i++) {
            u[i] = (byte) br.readBits(6, msg);
        }
        return u;
    }

    public static void featureList(BitWriter br, byte u[]) throws IOException {
        int start = br.bitlength();
        br.putVlcInt(5, u.length);
        for (int i = 0; i < u.length; i++) {
            br.putBits(6, u[i]);
        }
        codecBits += br.bitlength() - start;
    }

    public static int class_0to1float(BitReaderD br, String msg) throws IOException {
        //if (br.readBit(msg)) return Integer.MAX_VALUE; // inherit
        return (br.readBits(8, msg) << 12) / 255;
    }

    public static void class_0to1float(BitWriter br, int frac) throws IOException {
        int start = br.bitlength();
        //boolean b = (frac == Integer.MAX_VALUE);
        //br.putBit(b);
        //if (!b)
        br.putBits(8, 0xFF & ((frac * 255) >> 12));
        codecBits += br.bitlength() - start;
    }

    public static int class_fixed_16_8(BitReaderD br, String msg) throws IOException {
        int i = br.readBits(24, msg) << 4;
        if ((i & 0x08000000) != 0) i |= 0xF0000000;
        return i;
    }

    public static void class_fixed_16_8(BitWriter br, int f) throws IOException {
        int start = br.bitlength();
        fixed_16_8 += 3;
        br.putBits(24, f >> 4);
        codecBits += br.bitlength() - start;
    }

    public static int class_fixed_16_8i(BitReaderD br, String msg) throws IOException {
        boolean b = br.readBit(msg);
        if (b) return Integer.MAX_VALUE; // inherit
        int i = br.readBits(24, msg) << 4;
        if ((i & 0x08000000) != 0) i |= 0xF0000000;
        return i;
    }

    public static void class_fixed_16_8i(BitWriter br, int f) throws IOException {
        int start = br.bitlength();
        if (f == Integer.MAX_VALUE) {
            br.putBit(true); // inherit
            return; // return was missing with respect to SDL of 10/08/05
        }
        br.putBit(false);
        fixed_16_8 += 3;
        br.putBits(24, f >> 4);
        codecBits += br.bitlength() - start;
    }

    public static Point pointType(BitReaderD br, String msg) throws IOException {
        Point p = new Point();
        pointType(br, p, msg);
        return p;
    }

    public static void pointType(BitReaderD br, Point p, String msg) throws IOException {
        p.x = translateUponReading(br.readBits(coordBits, msg + " x"), coordBits);
        p.y = translateUponReading(br.readBits(coordBits, msg + " y"), coordBits);
    }

    public static void pointType(BitWriter br, Point p) throws IOException {
        out(br, p.x);
        out(br, p.y);
    }

    public static Point shortPointType(BitReaderD br, String msg) throws IOException {
        Point p = new Point();
        p.x = translateUponReading(br.readBits(coordBits, msg + " x"), coordBits);
        p.y = translateUponReading(br.readBits(coordBits, msg + " y"), coordBits);
        return p;
    }

    public static int class_coordinate(BitReaderD br, String msg) throws IOException {
        return translateUponReading(br.readBits(coordBits, msg), coordBits);
    }

    public static void shortPointType(BitWriter br, Point p) throws IOException {
        out(br, p.x);
        out(br, p.y);
    }

    public static void class_coordinate(BitWriter br, int i) throws IOException {
        out(br, i);
    }

    public static Matrix class_transform(BitReaderD br, String msg) throws IOException {
        if (!br.readBit("isNotMatrix")) { // more explicit log message
            Matrix m = new Matrix();
            if (br.readBit(msg + " flag1")) {
                m.xx = translateScale(br.readBits(coordBits + scaleBits, msg + " xx"));
                m.yy = translateScale(br.readBits(coordBits + scaleBits, msg + " yy"));
                m.xxValid = true;
                m.yyValid = true;
            }
            if (br.readBit(msg + " flag2")) {
                m.xy = translateScale(br.readBits(coordBits + scaleBits, msg + " xy"));
                m.yx = translateScale(br.readBits(coordBits + scaleBits, msg + " yx"));
                m.xyValid = true;
                m.yxValid = true;
            }
            if (br.readBit(msg + " flag3")) {
                m.xz = translateUponReading(br.readBits(coordBits + scaleBits, msg + " xz"), coordBits + scaleBits);
                m.yz = translateUponReading(br.readBits(coordBits + scaleBits, msg + " yz"), coordBits + scaleBits);
                m.xzValid = true;
                m.yzValid = true;
            }
            return m;
        } else {
            // to take into account ref(svg[,x,y]), another case is inserted here
            if (br.readBit("isRef")) {
                Matrix m = new Matrix();
                m.isRefSVG = true;
                if (br.readBit("hasXY")) {
                    m.xz = class_fixed_16_8(br, msg);
                    m.xzValid = true;
                    m.yz = class_fixed_16_8(br, msg);
                    m.yzValid = true;
                }
                return m;
            } else {
                // end insertion of ref(svg)
                class_extension(br, msg);
            }
            return null;
        }
    }

    public static void transformType(BitWriter br, Matrix m) throws IOException {
        int start = br.bitlength();
        // to take into account ref(svg[,x,y]), another case is inserted here
        br.putBit(m.isRefSVG);
        if (m.isRefSVG) {
            br.putBit(true);
            if ((m.xzValid && m.xz != 0) || (m.yzValid && m.yz != 0)) {
                class_fixed_16_8(br, m.xz);
                class_fixed_16_8(br, m.yz);
            }
            // end ref(svg)
        } else {
            if (m.xxValid || m.yyValid) {
                br.putBit(true);
                outScale(br, m.xx);
                outScale(br, m.yy);
            } else {
                br.putBit(false);
            }
            if (m.xyValid || m.yxValid) {
                br.putBit(true);
                outScale(br, m.xy);
                outScale(br, m.yx);
            } else {
                br.putBit(false);
            }
            if (m.xzValid || m.yzValid) {
                br.putBit(true);
                outZ(br, m.xz);
                outZ(br, m.yz);
            } else {
                br.putBit(false);
            }
        }
        codecBits += br.bitlength() - start;
    }

    private static void outZ(BitWriter bw, int f) throws IOException {
        int start = bw.bitlength();
        int i = f >> (12 - resolution);
        bw.putBits(coordBits + scaleBits, i);
        codecBits += bw.bitlength() - start;
    }

    public static void out(BitWriter bw, int f) throws IOException {
        int start = bw.bitlength();
        int i = f >> (12 - resolution);
        bw.putBits(coordBits, i);
        codecBits += bw.bitlength() - start;
    }

    private static void outScale(BitWriter bw, int i) throws IOException {
        int start = bw.bitlength();
        bw.putBits(coordBits + scaleBits, i >> 4);
        codecBits += bw.bitlength() - start;
    }

    public static int class_ID(BitReaderD br, String msg) throws IOException {
        int i = br.readVlcInt(5, msg);
        if (br.readBit("reserved")) {
            int len = br.readVlcInt(5, "len");
            if (len % 8 > 0) br.readBits(len%8, msg);
            if (len >> 3 > 0) br.readByteArray(len>>3, msg);
        }
        return i;
    }

    public static void class_ID(BitWriter br, int id) throws IOException {
        int start = br.bitlength();
        br.putVlcInt(5, id);
        codecBits += br.bitlength() - start;
        br.putBit(false); // reserved
    }

    public static int class_IDREF(BitReaderD br, String msg) throws IOException {
        int i = br.readVlcInt(5, msg);
        if (br.readBit("reserved")) {
            int len = br.readVlcInt(5, "len");
            if (len % 8 > 0) br.readBits(len%8, msg);
            if (len >> 3 > 0) br.readByteArray(len>>3, msg);
        }
        return i;
    }

    public static void class_IDREF(BitWriter br, int i) throws IOException {
        int start = br.bitlength();
        br.putVlcInt(5, i);
        codecBits += br.bitlength() - start;
        br.putBit(false); // reserved
    }

    public static void class_ElementAttributeList(BitReaderD br, XSave self) throws IOException {
        int max = br.readVlcInt(5, "element/attribute list");
        self.elements = new int[max];
        self.attributes = new Attribute[max];
        for (int i = 0; i < max; i++) {
            self.elements[i] = br.readVlcInt(5, "element");
            self.attributes[i] = SDLClasses.class_AttributeName(br, "attribute");
        }
    }

    public static void class_ElementAttributeList(BitWriter br, XSave self) throws IOException {
        int start = br.bitlength();
        br.putVlcInt(5, self.elements.length);
        for (int j = 0; j < self.elements.length; j++) {
            br.putVlcInt(5, self.elements[j]);
            SDLClasses.class_AttributeName(br, self.attributes[j]);
        }
        codecBits += br.bitlength() - start;
    }

    public static String class_byteAlignedString(BitReaderD br, String msg) throws IOException {
        br.flush("aligned");
        return br.readVlcString(msg);
    }

    public static void class_byteAlignedString(BitWriter br, String s) throws IOException {
        int start = br.bitlength();
        br.flush();
        br.putVlcString(s);
        codecBits += br.bitlength() - start;
    }

    public static int class_paint(BitReaderD br, String msg) throws IOException {
        if (br.readBit("isColorIndex")) { // more explicit log message
            return man.getColor(br.readBits(colorIndexBits, msg));
        } else {
            switch(br.readBits(2, "ch2")) {
              case 0:
                  //enum
                  switch (br.readBits(2, msg)) {
                      case 0:
                          return -3;
                      case 1:
                          return -1;
                      case 2:
                          return -2;
                  }
                  break;
              case 1:
                  // URI
                  return -4 - currentScene.idMan.registerPaintURI(class_anyURI(br, msg));
              case 2:
                  //System Paint Server
                  class_byteAlignedString(br, msg);
                  break;
              case 3:
                  // extension
                  class_extension(br, msg);
                  break;
            }
        }
        return Integer.MIN_VALUE;
    }

    public static void class_paint(BitWriter br, int i) throws IOException {
        int start = br.bitlength();
        if (i < 0) {
            // case of system paint server not implemented
            if (i < -3) {
                // URI
                br.putBit(false);
                br.putBits(2, 1);
                class_anyURI(br, currentScene.idMan.getPaintURI(-4 - i));
            } else {
                // enum
                br.putBit(false);
                br.putBits(2, 0);
                switch (i) {
                    case -1:
                        br.putBits(2, 1);
                        break;
                    case -2:
                        br.putBits(2, 2);
                        break;
                    case -3:
                        br.putBits(2, 0);
                        break;
                }
            }
        } else {
            br.putBit(true);
            br.putBits(colorIndexBits, man.getColorID(i));
        }
        codecBits += br.bitlength() - start;
    }

    public static int syncTolerance(BitReaderD br, String msg) throws IOException {
        int sw = br.readBits(1, msg);
        switch (sw) {
            case 0:
                return translateTimeUponReading(br.readVlcInt(5, msg), true);
            case 1:
                return -1;
        }
        return Integer.MIN_VALUE;
    }

    public static void syncTolerance(BitWriter br, int i) throws IOException {
        int start = br.bitlength();
        if (i == -1) {
            br.putBit(true);
        } else {
            br.putBit(false);
            br.putVlcInt(5, translateTimeUponWriting(i, true));
        }
        codecBits += br.bitlength() - start;
    }

    public static int choiceType(BitReaderD br, String msg) throws IOException {
        int sw = br.readBits(1, msg + " choice");
        switch (sw) {
            case 0:
                return br.readBits(8, msg);
            case 1:
                return br.readBits(1) - 2;
        }
        return Integer.MIN_VALUE;
    }

    public static void choiceType(BitWriter br, int i) throws IOException {
        int start = br.bitlength();
        br.putBit(i < 0);
        if (i < 0) {
            br.putBits(1, i + 2);
        } else {
            br.putBits(8, i);
        }
        codecBits += br.bitlength() - start;
    }

    public static int rotateType(BitReaderD br, String msg) throws IOException {
        int sw = br.readBits(1, msg + " rotate");
        switch (sw) {
            case 0:
                return class_fixed_16_8(br, msg);
            case 1:
                return Integer.MAX_VALUE - br.readBits(1, msg);
        }
        return Integer.MIN_VALUE;
    }

    public static void rotateType(BitWriter br, int rt) throws IOException {
        int start = br.bitlength();
        if (rt > Integer.MAX_VALUE - 2) {
            br.putBit(true);
            br.putBit(rt == Integer.MAX_VALUE - 1);
            codecBits += br.bitlength() - start;
        } else {
            br.putBit(false);
            codecBits += br.bitlength() - start;
            class_fixed_16_8(br, rt);
        }
    }

    public static int lineIncrementType(BitReaderD br, String msg) throws IOException {
        int sw = br.readBits(1, msg + " lineIncrement");
        switch (sw) {
            case 0:
                return class_fixed_16_8(br, msg);
            case 1:
                return Integer.MAX_VALUE - br.readBits(1, msg);
        }
        return Integer.MIN_VALUE;
    }

    public static void lineIncrementType(BitWriter br, int rt) throws IOException {
        int start = br.bitlength();
        if (rt > Integer.MAX_VALUE - 2) {
            br.putBit(true);
            br.putBit(rt == Integer.MAX_VALUE - 1);
            codecBits += br.bitlength() - start;
        } else {
            br.putBit(false);
            codecBits += br.bitlength() - start;
            class_fixed_16_8(br, rt);
        }
    }

    public static int focusType(BitReaderD br, String msg) throws IOException {
        int sw = br.readBits(1, msg + " focus");
        switch (sw) {
            // MGR : When the focus value is not a local idref, the first bit must be 1 (= isEnum)
            case 0:
                return class_IDREF(br, msg);
            case 1:
                return Integer.MAX_VALUE - br.readBits(1, msg);
        }
        return Integer.MIN_VALUE;
    }

    public static void focusType(BitWriter br, int rt) throws IOException {
        int start = br.bitlength();
        if (rt > Integer.MAX_VALUE - 2) {
            // MGR : When the focus value is not a local idref, the first bit must be 1 (= isEnum)
            br.putBit(true);
            br.putBit(rt == Integer.MAX_VALUE - 1);
            codecBits += br.bitlength() - start;
        } else {
            br.putBit(false);
            codecBits += br.bitlength() - start;
            class_IDREF(br, rt);
        }
    }

    public static Event class_event(BitReaderD br, String msg) throws IOException {
        Event ev = new Event();
        boolean b = br.readBit("isEnum");
        if (b) {
            ev.event = br.readBits(6, "event");
            switch (ev.event) {
                case 1: //accessKey
                case 15: //longAccessKey
                case 25: //repeatKey
                case 29: //shortAccessKey
                    ev.keyCode = br.readVlcInt(5, "keyCode");
                    break;
            }
        } else {
            ev.eventString = class_byteAlignedString(br, msg);
        }
        return ev;
    }

    public static void class_event(BitWriter br, Event rt) throws IOException {
        int start = br.bitlength();
        if (rt.eventString == null) {
            br.putBit(true);
            br.putBits(6, rt.event);
            switch (rt.event) {
                case 1: //accessKey
                case 12: //longAccessKey
                    br.putVlcInt(5, rt.keyCode);
                    break;
            }
            codecBits += br.bitlength() - start;
        } else {
            br.putBit(true);
            class_byteAlignedString(br, rt.eventString);
            codecBits += br.bitlength() - start;
        }
    }

    public static AnimValue class_AnimatedValue(BitReaderD br, String msg) throws IOException {
        AnimValue av = new AnimValue();
        av.type = br.readBits(4, msg);
        return animValue1(av, br, msg);
    }

    public static void class_AnimatedValue(BitWriter br, AnimValue av) throws IOException {
        br.putBits(4, av.type);
        animValue1(br, av);
    }

    public static AnimValue animValue1(AnimValue av, BitReaderD br, String msg) throws IOException {
        if (br.readBit("escapeFlag")) {
            av.escapeEnum = br.readBits(2, "escapeEnum");
            return av;
        }
        switch (av.type) {
            case ANIMTYPE_string:
                av.string = class_byteAlignedString(br, msg);
                break;
            case ANIMTYPE_float:
                av.fractionOrFloatOrColor = class_fixed_16_8(br, msg);
                break;
            case ANIMTYPE_uri:
                av.uri = class_anyURI(br, msg);
                break;
            case ANIMTYPE_path:
                av.path = class_path(br, msg);
                break;
            case ANIMTYPE_pointSeq:
                av.pointSeq = class_pointSequence(br, msg);
                break;
            case ANIMTYPE_fraction:
                av.fractionOrFloatOrColor = class_0to1float(br, msg);
                break;
            case ANIMTYPE_color:
                av.fractionOrFloatOrColor = class_paint(br, msg);
                break;
            case ANIMTYPE_enum:
                av.fractionOrFloatOrColor = br.readVlcInt(5, msg);
                break;
            case ANIMTYPE_id:
                av.fractionOrFloatOrColor = class_IDREF(br, msg);
                break;
            case ANIMTYPE_font:
                av.string = man.getFont(br.readVlcInt(5, msg));
                break;
            case ANIMTYPE_floats:
                int max = br.readVlcInt(5, msg);
                av.tab = new int[max];
                for (int i = 0; i < max; i++) {
                    av.tab[i] = class_fixed_16_8(br, msg);
                }
                break;
            /*case ANIMTYPE_ints:
                int ma = br.readVlcInt(5, msg);
                av.tab = new int[ma];
                for (int i = 0; i < ma; i++) {
                    av.tab[i] = br.readVlcInt(5, msg);
                }
                break;*/
            case ANIMTYPE_point:
                av.tab = new int[2];
                av.tab[0] = class_coordinate(br, msg);
                av.tab[1] = class_coordinate(br, msg);
                break;
            case 13:
                class_extension(br, msg);
                break;
        }
        return av;
    }

    public static void animValue1(BitWriter bw, AnimValue av) throws IOException {
        if (av.escapeEnum != -1) {
            bw.putBit(true);
            bw.putBits(2, av.escapeEnum);
            return;
        }
        bw.putBit(false);
        switch (av.type) {
            case ANIMTYPE_path:
                class_path(bw, av.path);
                break;
            case ANIMTYPE_pointSeq:
                class_pointSequence(bw, av.pointSeq);
                break;
            case ANIMTYPE_fraction:
                class_0to1float(bw, av.fractionOrFloatOrColor);
                break;
            case ANIMTYPE_color:
                class_paint(bw, av.fractionOrFloatOrColor);
                break;
            case ANIMTYPE_float:
                class_fixed_16_8(bw, av.fractionOrFloatOrColor);
                break;
            case ANIMTYPE_enum:
                bw.putVlcInt(5, av.fractionOrFloatOrColor);
                break;
            case ANIMTYPE_id:
                class_IDREF(bw, av.fractionOrFloatOrColor);
                break;
            case ANIMTYPE_font:
                bw.putVlcInt(5, man.getFontID(av.string));
                break;
            case ANIMTYPE_string:
                class_byteAlignedString(bw, av.string);
                break;
            case ANIMTYPE_floats:
                bw.putVlcInt(5, av.tab.length);
                for (int i = 0; i < av.tab.length; i++) {
                    class_fixed_16_8(bw, av.tab[i]);
                }
                break;
            /*case ANIMTYPE_ints:
                bw.putVlcInt(5, av.tab.length);
                for (int i = 0; i < av.tab.length; i++) {
                    bw.putVlcInt(5, av.tab[i]);
                }
                break;*/
            case ANIMTYPE_uri:
                class_anyURI(bw, av.uri);
                break;
            case ANIMTYPE_point:
                class_coordinate(bw, av.tab[0]);
                class_coordinate(bw, av.tab[1]);
                break;
            case 13:
                bw.putVlcString(av.string);
                break;
        }
    }

    public static int class_time(BitReaderD br, String msg, boolean dur) throws IOException {
        if (!br.readBit(msg)) {
            // time values need to be negative sometimes, thus the addition of a sign bit
            boolean b = br.readBit("sign");
            int a = br.readVlcInt(5, msg);
            return translateTimeUponReading((b ? -a : a), dur);
        } else {
            switch(br.readBits(2, "time")) {
                case 1:
                    return Integer.MIN_VALUE; // indefinite
                case 2:
                    return Integer.MAX_VALUE; // media
                case 0:
                    return Integer.MIN_VALUE+1; // auto
            }
        }
        return Integer.MAX_VALUE-1; // illegal
    }

    public static void class_time(BitWriter bw, int t, boolean dur) throws IOException {
        int start = bw.bitlength();
        if (t == Integer.MAX_VALUE || t == Integer.MIN_VALUE+1 || t == Integer.MIN_VALUE) {
            bw.putBit(true);
            bw.putBits(2, (t == Integer.MAX_VALUE ? 2 : (t == Integer.MIN_VALUE+1 ? 0 : 1)));
        } else {
            bw.putBit(false);
            bw.putBit(t < 0);
            bw.putVlcInt(5, translateTimeUponWriting(Math.abs(t), dur));
        }
        codecBits += bw.bitlength() - start;
    }

    /*public static int[] class_timeList(BitReaderD br, String msg, boolean dur) throws IOException {
        int max = br.readVlcInt(5, msg);
        int u[] = new int[max];
        for (int i = 0; i < max; i++) {
            u[i] = class_time(br, msg, dur);
        }
        return u;
    }*/

    /*public static void class_timeList(BitWriter br, int tl[], boolean dur) throws IOException {
        int start = br.bitlength();
        br.putVlcInt(5, tl.length);
        codecBits += br.bitlength() - start;
        for (int i = 0; i < tl.length; i++) {
            class_time(br, tl[i], dur);
        }
    }*/

    public static int[] class_floatList(BitReaderD br, String msg) throws IOException {
        return class_floatList(br.readVlcInt(5, msg), br, msg);
    }

    public static void class_floatList(BitWriter br, int fl[]) throws IOException {
        int start = br.bitlength();
        br.putVlcInt(5, fl.length);
        codecBits += br.bitlength() - start;
        floatListType1(br, fl);
    }

    public static void class_floatListNoLength(BitWriter br, int fl[]) throws IOException {
        floatListType1(br, fl);
    }

    public static int[] class_floatList(int max, BitReaderD br, String msg) throws IOException {
        int u[] = new int[max];
        for (int i = 0; i < max; i++) {
            u[i] = class_fixed_16_8(br, msg);
        }
        return u;
    }

    public static void floatListType1(BitWriter br, int fl[]) throws IOException {
        for (int i = 0; i < fl.length; i++) {
            class_fixed_16_8(br, fl[i]);
        }
    }

    protected static int negp(int i, int nbBits) {
        if (i > tab[nbBits]) {
            i -= tab1[nbBits];
        }
        return i;
    }

    private final static int[] tab1 = new int[]{1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768,
            65536, 131072, 262144, 524288, 1048576, 2097152, 4194304, 8388608, 16777216,
            33554432, 67108864, 134217728, 268435456};
    protected final static int[] tab = new int[]{0, 0, 1, 3, 7, 15, 31, 63, 127, 255, 511, 1023, 2047, 4095, 8191, 16383, 32767,
            65535, 131071, 262143, 524287, 1048575, 2097151, 4194303, 8388607, 16777215,
            33554431, 67108863, 134217727, 268435455};

    public static int translateUponReading(int n, int bits) {
        int a = negp(n, bits);
        return a << (12 - resolution);
    }

    public static int translateUponReading(int n, boolean sign) {
        int a = (sign ? -n : n);
        return a << (12 - resolution);
    }

    public static int translateUponReadingAVlc5Int(int n) {
        int bits = 0, nn = n;
        while ((nn >>= 4) > 0) bits += 4;
        return translateUponReading(n, bits);
    }

    public static int translateUponWriting(int n) {
        // try to do rounding on fixed point
        n = n >> (11 - resolution);
        n++;
        return n >> 1;
    }

    public static int translateScale(int n) {
        if (n >= tab1[scaleBits + coordBits - 1]) {
            n -= tab1[scaleBits + coordBits];
        }
        return (scaleBits > 4 ? n >>> (scaleBits - 4) : n << (4 - scaleBits));
    }

    public static int twoBitsType(BitReaderD br, String msg) throws IOException {
        int sw = br.readBits(1, msg);
        switch (sw) {
            case 0:
                return br.readBits(2, msg);
            case 1:
                class_extension(br, msg);
                return -1;
        }
        return Integer.MIN_VALUE;
    }

    public static void twoBitsType(BitWriter br, int i) throws IOException {
        int start = br.bitlength();
        br.putBit(false);
        br.putBits(2, i);
        codecBits += br.bitlength() - start;
    }

    private static void outColor(BitWriter bw, int comp) throws IOException {
        int start = bw.bitlength();
        if (SDLClasses.colorComponentBits > 8) {
            bw.putBits(SDLClasses.colorComponentBits, comp << (SDLClasses.colorComponentBits - 8));
        } else {
            bw.putBits(SDLClasses.colorComponentBits, comp >> (8 - SDLClasses.colorComponentBits));
        }
        codecBits += bw.bitlength() - start;
    }

    public final static boolean DEBUG_ANYXML_TABLE_ENCODING = false;
    public static void class_initialisation(BitWriter bw1, XstringIDTable stringIDTable) throws IOException {
        boolean b = man.nbOfColors() > 0;
        bw1.putBit(b);
        if (b) {
            bw1.putVlcInt(5, man.nbOfColors() - man.colorsAlreadySaved());
            for (int i = man.colorsAlreadySaved(); i < man.nbOfColors(); i++) {
                int col = man.getColor(i);
                outColor(bw1, col >> 16);
                outColor(bw1, 0xFF & (col >> 8));
                outColor(bw1, 0xFF & col);
            }
            SDLClasses.colorIndexBits = Utils.nbBitsForPositiveInteger(man.nbOfColors());
        }
        b = man.nbOfFonts() > 0;
        bw1.putBit(b);
        if (b) {
            bw1.putVlcInt(5, man.nbOfFonts() - man.fontsAlreadySaved());
            for (int i = man.fontsAlreadySaved(); i < man.nbOfFonts(); i++) {
                String font = man.getFont(i);
                SDLClasses.class_byteAlignedString(bw1, font);
            }
            SDLClasses.fontIndexBits = Utils.nbBitsForPositiveInteger(man.nbOfFonts());
        }
        b = man.nbOfNameSpaces() > 0;
        bw1.putBit(b);
        if (b) {
            bw1.putVlcInt(5, man.nbOfNameSpaces() - man.nameSpacesAlreadySaved());
            for (int i = man.nameSpacesAlreadySaved(); i < man.nbOfNameSpaces(); i++) {
                SDLClasses.class_byteAlignedString(bw1, man.getNameSpace(i).uri);
            }
            SDLClasses.nameSpaceIndexBits = Utils.nbBitsForPositiveInteger(man.nbOfNameSpaces());
        }
        b = man.nbOfTags() > 0;
        bw1.putBit(b);
        if (b) {
            bw1.putVlcInt(5, man.nbOfTags() - man.tagsAlreadySaved());
            for (int i = man.tagsAlreadySaved(); i < man.nbOfTags(); i++) {
                if (i != 0) {
                    bw1.putBits(nameSpaceIndexBits, man.getTagNSID(i));
                    SDLClasses.class_byteAlignedString(bw1, man.getTag(i).name);
                }
                int max = man.nbOfAttrs(i);
                if (DEBUG_ANYXML_TABLE_ENCODING) {
                    System.out.println("Tag"+i+":"+man.getTag(i).toString()+", "+max);
                }
                bw1.putBit(max > 0);
                if (max > 0) {
                    bw1.putVlcInt(5, max);
                    for (int j = 0; j < max; j++) {
                        QName a = man.getAttr(i, j);
                        if (DEBUG_ANYXML_TABLE_ENCODING) {
                            System.out.println("Attr:"+a.toString());
                        }
                        if (i == 0) {
                            bw1.putBits(nameSpaceIndexBits, man.getNameSpaceID(a.ns));
                        }
                        class_byteAlignedString(bw1, a.name);
                    }
                }
            }
            SDLClasses.tagIndexBits = Utils.nbBitsForPositiveInteger(man.nbOfTags());
        }
        // extendedInitialisation
        int len = 0;
        if (stringIDTable != null && stringIDTable.strings != null) {
            len = stringIDTable.strings.size();
        }
        bw1.putVlcInt(5, len);
        for (int i = 0; i < len; i++) {
            String s = (String)stringIDTable.strings.elementAt(i);
            bw1.putVlcInt(5, currentScene.idMan.getOrCreateNodeID(s));
            class_byteAlignedString(bw1, s);
        }
        bw1.putBit(false); // no extension
        //TODO, here should go the global stream table
        man.saved();
    }

    private static int reframeComponent(int comp) {
        if (SDLClasses.colorComponentBits > 8) {
            return 0xFF & (comp >> SDLClasses.colorComponentBits - 8);
        } else {
            return 0xFF & (comp << (8 - SDLClasses.colorComponentBits));
        }
    }

    public static void class_initialisation(BitReaderD br, String msg, SceneUnit su) throws IOException {
        boolean b = br.readBit("hasColors");
        if (b) {
            int len = br.readVlcInt(5, "nb of colors");
            for (int i = 0; i < len; i++) {
                int col = 0;
                col += reframeComponent(br.readBits(SDLClasses.colorComponentBits, "red")) << 16;
                col += reframeComponent(br.readBits(SDLClasses.colorComponentBits, "green")) << 8;
                col += reframeComponent(br.readBits(SDLClasses.colorComponentBits, "blue"));
                man.addColor(col);
            }
            SDLClasses.colorIndexBits = Utils.nbBitsForPositiveInteger(man.nbOfColors());
        }
        b = br.readBit("hasFonts");
        if (b) {
            int len = br.readVlcInt(5, "nb of fonts");
            for (int i = 0; i < len; i++) {
                man.registerFont(SDLClasses.class_byteAlignedString(br, "font name"));
            }
            SDLClasses.fontIndexBits = Utils.nbBitsForPositiveInteger(man.nbOfFonts());
        }
        b = br.readBit("hasPrivateDataIdentifiers");
        if (b) {
            int len = br.readVlcInt(5, "nb of namespaces");
            for (int i = 0; i < len; i++) {
                NameSpace n = NameSpace.get(SDLClasses.class_byteAlignedString(br, "name space uri"));
                man.registerNamespace(n);
            }
            SDLClasses.nameSpaceIndexBits = Utils.nbBitsForPositiveInteger(man.nbOfNameSpaces());
        }
        b = br.readBit("hasTags");
        if (b) {
            int len = br.readVlcInt(5, "nb of tags");
            for (int i = 0; i < len; i++) {
                QName t = new QName();
                if (i != 0) {
                    t.ns = man.getNameSpace(br.readBits(nameSpaceIndexBits, "ns"));
                    t.name = SDLClasses.class_byteAlignedString(br, "tag");
                }
                QName attrs[] = null;
                if (br.readBit("anyAttrs?")) {
                    int l = br.readVlcInt(5, "nb of attrs");
                    attrs = new QName[l];
                    for (int j = 0; j < l; j++) {
                        attrs[j] = new QName();
                        if (i == 0) {
                            attrs[j].ns = man.getNameSpace(br.readBits(nameSpaceIndexBits, "ns"));
                        }
                        attrs[j].name = SDLClasses.class_byteAlignedString(br, "attr name");
                    }
                }
                man.registerTag(t, attrs);
            }
            SDLClasses.tagIndexBits = Utils.nbBitsForPositiveInteger(man.nbOfTags());
        }


        // extendedInitialisation
        // stringIDTable: external string ID references
        int len = br.readVlcInt(5, "countG");
        if (len != 0) {
            su.stringIDTable = new XstringIDTable();
            su.stringIDTable.strings = new Vector();
        }
        for (int i = 0; i < len; i++) {
            int id = br.readVlcInt(5, "binaryIdForThisStringID");
            String s = SDLClasses.class_byteAlignedString(br, "stringID");
            currentScene.idMan.setID(s, id);
            su.stringIDTable.strings.addElement(s);
        }
        if (br.readBit("ext")) {
            int i = br.readVlcInt(5, "ext");
            br.readBits(i%8, "ext");
            br.readByteArray(i>>>3, msg);
        }
    }

    /**
     * Computes the number of bits necessary to represent a positive number
     */
    public static int nbBitsForPositiveInteger(int i) {
        if (i < 0) {
            throw new RuntimeException("A negative number was passed to nbBitForPositiveInteger:" + i);
        }
        int n = 0;
        for (; i > 0; i >>>= 1) {
            n++;
        }
        return n;
    }

    /**
     * Computes the number of bits to represent a number that can be positive or negative.
     * The result is one more than the above because for a positive number N, you need to be able to
     * discriminate it from its opposite -N
     */
    public static int nbBitsForInteger(int i) {
        return nbBitsForPositiveInteger(i < 0 ? 1 - i : i) + 1;
    }

    public static int[] class_fraction12List(BitReaderD br, String msg) throws IOException {
        int nb = br.readVlcInt(5, msg);
        int u[] = new int[nb];
        boolean shortcut;
        for (int i = 0; i < nb; i++) {
            shortcut = br.readBit();
            if (shortcut) {
                if (br.readBit())
                    u[i] = 0;
                else
                    u[i] = 4096;
            } else
                u[i] = br.readBits(12);
        }
        return u;
    }


    public static void _(String s) {
        System.out.print(s);
    }

    public static void class_fraction12List(BitWriter br, int fl[]) throws IOException {
        br.putVlcInt(5, fl.length);
        for (int i = 0; i < fl.length; i++) {
            if (fl[i] == 0) {
                br.putBit(true);
                br.putBit(true);
            } else if (fl[i] == 4096) {
                br.putBit(true);
                br.putBit(false);
            } else {
                br.putBit(false);
                br.putBits(12, fl[i]);
            }
        }
    }

    public static void contentType(BitWriter bw, String type) throws Exception {
        bw.putBit(type != null);
        if (type != null) {
            if (type.equalsIgnoreCase("application/laserScript")) {
                bw.putBit(true);
                bw.putBits(2, 0);
            } else if (type.equalsIgnoreCase("application/ecmaScript")) {
                bw.putBit(true);
                bw.putBits(2, 1);
            } else if (type.equalsIgnoreCase("application/java-archive")) {
                bw.putBit(true);
                bw.putBits(2, 2);
            } else {
                bw.putBit(false);
                class_byteAlignedString(bw, type);
            }
        }
    }

    public static String contentType(BitReaderD br) throws Exception {
        if (br.readBit("hasType")) {
            if (br.readBit("hasEnum")) {
                int i = br.readBits(2, "type");
                switch (i) {
                    case 0:
                        return "application/laserScript";
                    case 1:
                        return "application/ecmaScript";
                    case 2:
                        return "application/java-archive";
                    default:
                        throw new RuntimeException("decoding error: unknown content type " + i);
                }
            } else {
                return class_byteAlignedString(br, "stype");
            }
        } else {
            return null;
        }
    }

    public static int translateTimeUponReading(int i, boolean dur) {
        // read time as ticks
        // return time as 20.12 float
        return ((int)(((float)i)*4096f/timeResolution)) + (dur ? 0 : currentTimeInTics);
    }

    public static int translateTimeUponWriting(int i, boolean dur) {
        // get time as 20.12 float
        // return time as ticks
        return (((int)(i*timeResolution) >> 12)) - (dur ? 0 : currentTimeInTics);
    }

    public static ScrollStop class_scrollStop(BitReaderD br, String msg) throws IOException {
        ScrollStop ss = new ScrollStop();
        if (br.readBit("scrollStopChoice")) {
            Point p = new Point();
            ss.value = p;
            p.x = class_fixed_16_8(br, msg);
            p.y = class_fixed_16_8(br, msg);
        } else {
            ss.idRef = class_IDREF(br, msg);
        }
        return ss;
    }

    public static void class_scrollStop(BitWriter bw, ScrollStop ss) throws IOException {
        if (ss.value != null) {
            bw.putBit(true);
            class_fixed_16_8(bw, ss.value.x);
            class_fixed_16_8(bw, ss.value.y);
        } else {
            bw.putBit(false);
            class_IDREF(bw, ss.idRef);
        }
    }
}
