/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.sdl;

import iso.mpeg.streamezzo.util.IDManager;

import java.io.PrintWriter;

public class SmilTime {
    public int id = Integer.MIN_VALUE;
    public int delay = Integer.MIN_VALUE;
    public Event event = null;

    public boolean isIndefinite() {
        return (id == Integer.MIN_VALUE) && (event == null) && (delay == Integer.MIN_VALUE);
    }

    public void print(PrintWriter pw, IDManager idMan) {
        if (id == Integer.MIN_VALUE) {
            if (event == null) {
                if (delay == Integer.MIN_VALUE) {
                    pw.print("indefinite");
                } else {
                    String s = Float.toString(((float) delay) / 4096f);
                    if (s.endsWith(".0")) s = s.substring(0, s.length() - 2);
                    pw.print(s);
                }
            } else {
                if (delay == Integer.MIN_VALUE) {
                    event.print(pw);
                } else {
                    event.print(pw);
                    pw.print("+");
                    String s = Float.toString(((float) delay) / 4096f);
                    if (s.endsWith(".0")) s = s.substring(0, s.length() - 2);
                    pw.print(s);
                }
            }
        } else {
            if (event == null) {
                throw new RuntimeException("invalid smil time: id and no event");
            } else {
                if (idMan != null) {
                    String s = idMan.getID(id);
                    if (s != null) pw.print(s);
                    else pw.print(id);
                }
                pw.print(id);
                pw.print(".");
                if (delay == Integer.MIN_VALUE) {
                    event.print(pw);
                } else {
                    event.print(pw);
                    pw.print("+");
                    String s = Float.toString(((float) delay) / 4096f);
                    if (s.endsWith(".0")) s = s.substring(0, s.length() - 2);
                    pw.print(s);
                }
            }
        }
    }

    public SmilTime read(String s, IDManager idMan) {
        // receiving either a number or x.y or x.y+number
        // if number, return the number
        s = s.trim();
        float factor = 1f;
        if (s.endsWith("ms")) {
            s = s.substring(0, s.length()-2);
            factor = 0.001f;
        }
        if (s.endsWith("s")) {
            s = s.substring(0, s.length()-1);
        }
        try {
            delay = (int) (Float.parseFloat(s) * factor * 4096f);
            return this;
        } catch(NumberFormatException e) {}
        // otherwise, create a listener in self and return null
        // if has a +
        int plus = s.indexOf("+");
        int dot = s.indexOf(".");
        if (plus >= 0) {
            // plus
            if (dot >= 0) {
                // dot
                id = idMan.getOrCreateNodeID(s.substring(0, dot));
                event = new Event().read(s.substring(dot+1, plus).trim());
            } else {
                // no dot
                event = new Event().read(s.substring(0, plus));
            }
            delay = (int) (Float.parseFloat(s.substring(plus+1).trim()) * factor * 4096);
        } else {
            // no plus
            if (dot >= 0) {
                // dot
                id = idMan.getOrCreateNodeID(s.substring(0, dot));
                event = new Event().read(s.substring(dot+1));
            } else {
                // no dot
                event = new Event().read(s);
            }
        }
        return this;
    }
}