/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.util;

import iso.mpeg.streamezzo.laser.*;
import iso.mpeg.streamezzo.saf.*;

public class BasicVisitor implements Visitor {
    public void visitXAU(XAU o, Object obj) throws Exception { }
    public void visitXAdd(XAdd o, Object obj) throws Exception { }
    public void visitXDeactivate(XDeactivate o, Object obj) throws Exception { }

    public void visitXReleaseResource(XReleaseResource o, Object obj) throws Exception { }

    public void visitXActivate(XActivate o, Object obj) throws Exception { }
    public void visitXClean(XClean o, Object obj) throws Exception { }
    public void visitXDelete(XDelete o, Object obj) throws Exception { }
    public void visitXInsert(XInsert o, Object obj) throws Exception {
        if (o.child instanceof Visitable) ((Visitable)o.child).visit(this, obj);
    }
    public void visitXLASeRHeader(XLASeRHeader o, Object obj) throws Exception { }

    public void visitXNewScene(XNewScene o, Object obj) throws Exception {
        if (o.child instanceof Visitable) ((Visitable)o.child).visit(this, obj);
    }

    public void visitXRefreshScene(XRefreshScene o, Object obj) throws Exception {
        if (o.child instanceof Visitable) ((Visitable)o.child).visit(this, obj);
    }

    public void visitXReplace(XReplace o, Object obj) throws Exception {
        if (o.child != null) {
            for (int i = 0; i < o.child.size(); i++) {
                Object oo = o.child.elementAt(i);
                if (oo instanceof Visitable) ((Visitable)oo).visit(this, obj);
            }
        }
    }
    public void visitXRestore(XRestore o, Object obj) throws Exception { }
    public void visitXSave(XSave o, Object obj) throws Exception { }
    public void visitXSendEvent(XSendEvent o, Object obj) throws Exception { }
    public void visitX(X o, Object obj) throws Exception {
        if (o.child == null) return;
        for (int i = 0; i < o.child.size(); i++) {
            Object oo = o.child.elementAt(i);
            if (oo instanceof Visitable) ((Visitable)oo).visit(this, obj);
        }
    }
    public void visitXa(Xa o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }

    public void visitXcursor(XcursorManager o, Object obj) throws Exception {
        visitX((X)o, obj);
    }

    public void visitXupdateSource(XupdateSource o, Object obj) throws Exception { }

    //
    // anticipated implementation of StreamSource
    //
    public void visitXstreamSource(XstreamSource o, Object obj) throws Exception {
        visitX((X)o, obj);
    }

    public void visitXanimate(Xanimate o, Object obj) throws Exception {
        visitXset((Xset)o, obj);
    }
    public void visitXanimateColor(XanimateColor o, Object obj) throws Exception {
        visitXanimate((Xanimate)o, obj);
    }
    public void visitXanimateMotion(XanimateMotion o, Object obj) throws Exception {
        visitXanimate((Xanimate)o, obj);
    }
    public void visitXanimateTransform(XanimateTransform o, Object obj) throws Exception {
        visitXanimate((Xanimate)o, obj);
    }
    public void visitXaudio(Xaudio o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitXcircle(Xcircle o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXdefs(Xdefs o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXdesc(Xdesc o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitXdtm(Xdesc_metaData_title o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitXellipse(Xellipse o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXforeignObject(XforeignObject o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXg(Xg o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXrectClip(XrectClip o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXselector(Xselector o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXsimpleLayout(XsimpleLayout o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXimage(Ximage o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXline(Xline o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXlinearGradient(XlinearGradient o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXmetadata(Xmetadata o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitXmpath(Xmpath o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitXpath(Xpath o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXpolygon(Xpolygon o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXpolyline(Xpolyline o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXprivateElement(XprivateElement o, Object obj) throws Exception {
        visitX((X)o, obj);
    }

    public void visitXprivateElementContainer(XprivateElementContainer o, Object obj) throws Exception {
        visitX((X)o, obj);
    }

    public void visitXpropElem(XpropElem o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitXradialGradient(XradialGradient o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXrect(Xrect o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXscript(Xscript o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitXconditional(Xconditional o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitXset(Xset o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitXstop(Xstop o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXsvg(Xsvg o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXswitch(Xswitch o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXtext(Xtext o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXtextContent(XtextContent o, Object obj) throws Exception {
    }
    public void visitXtitle(Xtitle o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitXlistener(Xlistener o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitXtspan(Xtspan o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXuse(Xuse o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitXvideo(Xvideo o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitCacheObject(CacheObject o, Object obj) throws Exception {
        visitStzPacket((SAFPacket)o, obj);
    }
    public void visitEndOfSAFSession(EndOfSAFSession o, Object obj) throws Exception {
        visitStzPacket((SAFPacket)o, obj);
    }
    public void visitEndOfStream(EndOfStream o, Object obj) throws Exception {
        visitStzPacket((SAFPacket)o, obj);
    }
    public void visitFontHeader(FontHeader o, Object obj) throws Exception {
        visitStzHeader((SAFHeader)o, obj);
    }
    public void visitFontUnit(FontUnit o, Object obj) throws Exception {
        visitStzPacket((SAFPacket)o, obj);
    }
    public void visitImageHeader(ImageHeader o, Object obj) throws Exception {
        visitStzHeader((SAFHeader)o, obj);
    }
    public void visitImageUnit(ImageUnit o, Object obj) throws Exception {
        visitStzPacket((SAFPacket)o, obj);
    }
    public void visitMediaHeader(MediaHeader o, Object obj) throws Exception {
        visitStzHeader((SAFHeader)o, obj);
    }
    public void visitMediaUnit(MediaUnit o, Object obj) throws Exception {
        visitStzPacket((SAFPacket)o, obj);
    }
    public void visitRemoteStreamHeader(RemoteStreamHeader o, Object obj) throws Exception {
        visitStzHeader((SAFHeader)o, obj);
    }
    public void visitSAFSession(Scene o, Object obj) throws Exception {
        for (int i = 0; i < o.size(); i++) {
            Object oo = o.elementAt(i);
            if (oo instanceof Visitable) ((Visitable)oo).visit(this, obj);
        }
    }
    public void visitSceneHeader(SceneHeader o, Object obj) throws Exception {
        visitStzHeader((SAFHeader)o, obj);
    }
    public void visitSceneUnit(SceneUnit o, Object obj) throws Exception {
        if (o.updates == null) return;
        for (int i = 0; i < o.updates.size(); i++) {
            Object oo = o.updates.elementAt(i);
            if (oo instanceof Visitable) ((Visitable)oo).visit(this, obj);
        }
    }
    public void visitStzHeader(SAFHeader o, Object obj) throws Exception {
        visitStzPacket((SAFPacket)o, obj);
    }
    public void visitStzPacket(SAFPacket o, Object obj) throws Exception { }
    public void visitScene(Scene o, Object obj) throws Exception {
        for (int i = 0; i < o.size(); i++) {
            Object oo = o.elementAt(i);
            if (oo instanceof Visitable) ((Visitable)oo).visit(this, obj);
        }
    }
    public void visitStringIDTable(XstringIDTable o, Object obj) throws Exception { }

    // AMD1
    public void visitAnimateScroll(XanimateScroll o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitScrollStop(XsetScroll o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitAnimation(Xanimation o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitPrefetch(Xprefetch o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitDiscard(Xdiscard o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitHandler(Xhandler o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitFont(Xfont o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitSolidColor(XsolidColor o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitTextArea(XtextArea o, Object obj) throws Exception {
        visitXpropElem((XpropElem)o, obj);
    }
    public void visitTBreak(XtBreak o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitFontFace(XfontFace o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitFontFaceSrc(XfontFaceSrc o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitFontFaceUri(XfontFaceUri o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitGlyph(Xglyph o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitHkern(Xhkern o, Object obj) throws Exception {
        visitX((X)o, obj);
    }
    public void visitMissingGlyph(XmissingGlyph o, Object obj) throws Exception {
        visitX((X)o, obj);
    }

}
