/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.util;

public class IntAndStringListReader {
    private String stringBuffer;
    private int index, len;

    public IntAndStringListReader(String s) {
        stringBuffer = s;
        index = 0;
        len = stringBuffer.length();
    }

    public boolean nextIsString() throws java.io.EOFException {
        out:while (true) {
            if (index >= len) {
                throw new java.io.EOFException();
            }
            char c = stringBuffer.charAt(index);
            switch (c) {
                case ' ':
                    index++;
                    break;
                default:
                    break out;
            }
        }
        return Character.isLetter(stringBuffer.charAt(index));
    }

    public String nextString() throws java.io.EOFException {
        out:while (true) {
            if (index >= len) {
                throw new java.io.EOFException();
            }
            char c = stringBuffer.charAt(index);
            switch (c) {
                case ' ':
                    index++;
                    break;
                default:
                    break out;
            }
        }
        int nextIndex = len;
        int nextIndex1 = stringBuffer.indexOf(' ', index);
        if (nextIndex1 > 0) {
            nextIndex = Math.min(nextIndex, nextIndex1);
        }
        String returnString = new String(stringBuffer.substring(index, nextIndex));
        index = nextIndex + 1;
        return returnString;
    }

    public int nextInt() throws java.io.EOFException {
        out:while (true) {
            if (index >= len) {
                throw new java.io.EOFException();
            }
            char c = stringBuffer.charAt(index);
            switch (c) {
                case 'L':
                case '(':
                case ')':
                case ';':
                case ',':
                case '\t':
                case ' ':
                    index++;
                    break;
                default:
                    break out;
            }
        }
        int nextIndex = len;
        int nextIndex1 = stringBuffer.indexOf(' ', index);
        if (nextIndex1 > 0) {
            nextIndex = Math.min(nextIndex, nextIndex1);
        }
        nextIndex1 = stringBuffer.indexOf('\t', index);
        if (nextIndex1 > 0) {
            nextIndex = Math.min(nextIndex, nextIndex1);
        }
        nextIndex1 = stringBuffer.indexOf(',', index);
        if (nextIndex1 > 0) {
            nextIndex = Math.min(nextIndex, nextIndex1);
        }
        nextIndex1 = stringBuffer.indexOf(';', index);
        if (nextIndex1 > 0) {
            nextIndex = Math.min(nextIndex, nextIndex1);
        }
        nextIndex1 = stringBuffer.indexOf(')', index);
        if (nextIndex1 > 0) {
            nextIndex = Math.min(nextIndex, nextIndex1);
        }
        nextIndex1 = stringBuffer.indexOf('(', index);
        if (nextIndex1 > 0) {
            nextIndex = Math.min(nextIndex, nextIndex1);
        }
        nextIndex1 = stringBuffer.indexOf('L', index);
        if (nextIndex1 > 0) {
            nextIndex = Math.min(nextIndex, nextIndex1);
        }
        int f = Integer.parseInt(stringBuffer.substring(index, nextIndex));
        index = nextIndex + 1;
        return f;
    }
}