/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.util;

import iso.mpeg.streamezzo.sdl.NameSpace;
import iso.mpeg.streamezzo.sdl.QName;
import iso.mpeg.streamezzo.sdl.privAttr;

import java.io.Serializable;
import java.util.Vector;

public final class Manager implements Serializable {
    /**
     * the vector containing the colors already defined
     */
    private Vector colors = new Vector();
    private int nbColorsAlreadySaved = 0;
    /**
     * the vector containing the colors already defined
     */
    private Vector fonts = new Vector();
    private int nbFontsAlreadySaved = 0;

    private Vector nameSpaces = new Vector();
    private int nbNameSpacesAlreadySaved = 0;

    private Vector privateTags = new Vector();
    private int nbTagsAlreadySaved = 0;

    private static final boolean debug = false;

    public void saved() {
        nbColorsAlreadySaved = colors.size();
        nbFontsAlreadySaved = fonts.size();
        nbNameSpacesAlreadySaved = nameSpaces.size();
        nbTagsAlreadySaved = privateTags.size();
    }

    public int colorsAlreadySaved() { return nbColorsAlreadySaved; }
    public int fontsAlreadySaved() { return nbFontsAlreadySaved; }
    public int nameSpacesAlreadySaved() { return nbNameSpacesAlreadySaved; }
    public int tagsAlreadySaved() { return nbTagsAlreadySaved; }

    public int nbOfColors() {
        return colors.size();
    }

    public int nbOfFonts() {
        return fonts.size();
    }

    /**
     * to register a fill when encoding it
     *
     * @param c         the fill
     */
    public int registerColor(int c) {
        for (int i = 0; i < colors.size(); i++) {
            int cc = ((Integer)colors.elementAt(i)).intValue();
            if (c == cc) {
                return i;
            }
        }
        colors.addElement(new Integer(c));
        // System.out.println("color "+Integer.toHexString(c)+" "+(colors.delta()-1));
        return colors.size()-1;
    }

    public int addColor(int c) {
        colors.addElement(new Integer(c));
        return colors.size()-1;
    }

    public int getColorID(int c) {
        for (int i = 0; i < colors.size(); i++) {
            int cc = ((Integer)colors.elementAt(i)).intValue();
            if (c == cc) {
                return i;
            }
        }
        throw new RuntimeException("unknown color: "+c);
    }

    public int registerFont(String s) {
        if (s == null) {
            System.out.println();
        }
        for (int i = 0; i < fonts.size(); i++) {
            String ss = (String)fonts.elementAt(i);
            if (s.equals(ss)) {
                return i;
            }
        }
        fonts.addElement(s);
        //System.out.println("register font "+s+" "+(fonts.delta()-1)+" man:"+this);
        return fonts.size()-1;
    }

    public int getFontID(String s) {
        for (int i = 0; i < fonts.size(); i++) {
            String ss = (String)fonts.elementAt(i);
            if (s.equals(ss)) {
                return i;
            }
        }
        throw new RuntimeException("unknown font: "+s+" man:"+this);
    }

    public String getFont(int i) {
        return (String)fonts.elementAt(i);
    }

    public int getColor(int i) {
        return ((Integer)colors.elementAt(i)).intValue();
    }

    public int registerNamespace(NameSpace n) {
        if (n == null || nameSpaces.indexOf(n) >= 0) return -1;
        nameSpaces.addElement(n);
        return nameSpaces.size()-1;
    }

    public int nbOfNameSpaces() {
        return nameSpaces.size();
    }

    public int getNameSpaceID(NameSpace n) {
        for (int i = 0; i < nameSpaces.size(); i++) {
            NameSpace p = (NameSpace)nameSpaces.elementAt(i);
            if (p == n) return i;
        }
        return registerNamespace(n);
    }

    public NameSpace getNameSpace(String prefix) {
        for (int i = 0; i < nameSpaces.size(); i++) {
            NameSpace p = (NameSpace)nameSpaces.elementAt(i);
            if (prefix.equals(p.prefix)) return p;
        }
        return null;
    }

    public NameSpace getNameSpace(int i) {
        return (NameSpace)nameSpaces.elementAt(i);
    }

    public void registerTag(QName name, QName privAttrs[]) {
        if (debug) {
            System.out.println("register {" + name.ns.uri + "}:" + name.name + " " + privAttrs.length);
            for (int i = 0; i < privAttrs.length; i++) {
                System.out.println("  {" + (privAttrs[i].ns == null ? "null" : privAttrs[i].ns.uri) + "}:" + privAttrs[i].name);
            }
        }
        if (privateTags.size() == 0 && name.name != null) {
            // if the first registered QName has a non null name, then we are parsing an XML
            // and we should add the dummy
            Manager.Tag t = new Manager.Tag();
            t.name = new QName();
            privateTags.addElement(t);
        }
        Manager.Tag t = null;
        for (int i = 0; i < privateTags.size(); i++) {
            Manager.Tag tag = (Manager.Tag)privateTags.elementAt(i);
            if (tag.name.equals(name)) {
                t = tag;
                break;
            }
        }
        if (t == null) {
            t = new Manager.Tag();
            t.name = name;
            privateTags.addElement(t);
        }
        // then merge the attrs
        Manager.Tag t0 = (Manager.Tag)(privateTags.elementAt(0));
        if (privAttrs != null) {
            int max = t.attrNames.size(), max0 =  t0.attrNames.size();
            for (int i = 0; i < privAttrs.length; i++) {
                boolean add = true;
                if (privAttrs[i].ns == null || NameSpace.equals(t.name.ns, privAttrs[i].ns)) {
                    // 1. if the namespace is the same, add to the tag
                    for (int j = 0; j < max; j++) {
                        if (privAttrs[i].equals(t.attrNames.elementAt(j))) {
                            add = false;
                            break;
                        }
                    }
                    if (add) {
                        t.attrNames.addElement(privAttrs[i]);
                    }
                } else {
                    // 2. if the namespace is different, add to the 0 tag
                    for (int j = 0; j < max0; j++) {
                        if (privAttrs[i].equals(t0.attrNames.elementAt(j))) {
                            add = false;
                            break;
                        }
                    }
                    if (add) {
                        t0.attrNames.addElement(privAttrs[i]);
                    }
                }
            }
        }
    }

    public int nbOfTags() {
        return privateTags.size();
    }

    public int getTagID(QName name) {
        for (int i = 0; i < privateTags.size(); i++) {
            if (name.equals(((Manager.Tag)privateTags.elementAt(i)).name)) return i;
        }
        throw new RuntimeException("unregistered tag: {"+(name.ns == null ? "null" : name.ns.uri)+"}:"+name.name);
    }

    public int getAttrID(int tagID, QName attrName) {
        Manager.Tag t = (Manager.Tag)privateTags.elementAt(tagID);
        for (int i = 0; i < t.attrNames.size(); i++) {
            privAttr p = (privAttr)(t.attrNames.elementAt(i));
            if (p.name.equals(attrName.name)) return i;
        }
        if (tagID != 0) {
            t = (Manager.Tag) privateTags.elementAt(0);
            for (int i = 0; i < t.attrNames.size(); i++) {
                privAttr p = (privAttr) (t.attrNames.elementAt(i));
                if (p.equals(attrName)) return i;
            }
        }
        throw new RuntimeException("unregistered attr: "+attrName.toString()+" tagID:"+tagID+" "+getTag(tagID).toString());
    }

    public QName getTag(int i) {
        return ((Manager.Tag)privateTags.elementAt(i)).name;
    }

    public NameSpace getTagNS(int i) {
        return ((Manager.Tag)privateTags.elementAt(i)).name.ns;
    }

    public int getTagNSID(int i) {
        return getNameSpaceID(((Manager.Tag)privateTags.elementAt(i)).name.ns);
    }

    public QName getAttr(int i, int j) {
        return ((QName)((Manager.Tag)privateTags.elementAt(i)).attrNames.elementAt(j));
    }

    public int getAttrNSID(int i, int j) {
        throw new RuntimeException("not implemented:getAttrNSID");
    }

    public int nbOfAttrs(int i) {
        return ((Manager.Tag)privateTags.elementAt(i)).attrNames.size();
    }

    private static class Tag {
        QName name = null;
        Vector attrNames = new Vector();
    }
}
