/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.util;

import iso.mpeg.streamezzo.laser.Xanimate;
import iso.mpeg.streamezzo.laser.XReplace;
import iso.mpeg.streamezzo.laser.*;
import iso.mpeg.streamezzo.sdl.SDLClasses;
import iso.mpeg.streamezzo.io.IOUtils;

public class PerUnit extends BasicVisitor {
    public Manager man = null;

    public PerUnit(Manager m) {
        man = m;
    }

    public void visitXpropElem(XpropElem o, Object obj) throws Exception {
        super.visitXpropElem(o, obj);
        if (o.font_family != null) {
            man.registerFont(o.font_family);
        }
        if (o.fill != Integer.MIN_VALUE) {
            man.registerColor(o.fill);
        }
        if (o.stroke != Integer.MIN_VALUE) {
            man.registerColor(o.stroke);
        }
        if (o.stop_color != Integer.MIN_VALUE) {
            man.registerColor(o.stop_color);
        }
        if (o.solid_color != Integer.MIN_VALUE) {
            man.registerColor(o.solid_color);
        }
        if (o.viewport_fill != Integer.MIN_VALUE) {
            man.registerColor(o.viewport_fill);
        }
        if (o.color != Integer.MIN_VALUE) {
            man.registerColor(o.color);
        }
    }

    public void visitXsvg(Xsvg o, Object obj) throws Exception {
        super.visitXsvg(o, obj);
        if (o.viewport_fill != Integer.MIN_VALUE) man.registerColor(o.viewport_fill);
    }

    public void visitXforeignObject(XforeignObject o, Object obj) throws Exception {
        super.visitXforeignObject(o, obj);
        if (o.viewport_fill != Integer.MIN_VALUE) man.registerColor(o.viewport_fill);
    }

    public void visitXimage(Ximage o, Object obj) throws Exception {
        super.visitXimage(o, obj);
        if (o.viewport_fill != Integer.MIN_VALUE) man.registerColor(o.viewport_fill);
    }

    public void visitXprivateElement(XprivateElement o, Object obj) throws Exception {
        man.registerNamespace(o.name.ns);
        man.registerTag(o.name, o.privateAttributes);
        super.visitXprivateElement(o, obj);
    }

    public void visitXset(Xset o, Object obj) throws Exception {
        super.visitXset(o, obj);
        if (o.attributeName == null) return;
        int u = SDLClasses.animateType(o.attributeName);
        if (u == SDLClasses.ANIMTYPE_color) {
            if (o.to != null) man.registerColor(o.to.fractionOrFloatOrColor);
        } else if (u == SDLClasses.ANIMTYPE_font) {
            if (o.to != null) man.registerFont(o.to.string);
        }
    }

    public void visitXanimate(Xanimate o, Object obj) throws Exception {
        super.visitXanimate(o, obj);
        if (o.attributeName == null) return;
        int u = SDLClasses.animateType(o.attributeName);
        if (u == SDLClasses.ANIMTYPE_color) {
            if (o.from != null) man.registerColor(o.from.fractionOrFloatOrColor);
            if (o.by != null) man.registerColor(o.by.fractionOrFloatOrColor);
            if (o.values != null) {
                for (int i = 0; i < o.values.length; i++) {
                    man.registerColor(o.values[i].fractionOrFloatOrColor);
                }
            }
        } else if (u == SDLClasses.ANIMTYPE_font) {
            if (o.from != null) man.registerFont(o.from.string);
            if (o.by != null) man.registerFont(o.by.string);
            if (o.values != null) {
                for (int i = 0; i < o.values.length; i++) {
                    if (o.values[i].string != null) {
                        man.registerFont(o.values[i].string);
                    }
                }
            }
        }
    }

    public void visitXReplace(XReplace o, Object obj) throws Exception {
        super.visitXReplace(o, obj);
        if (o.attributeName != null && SDLClasses.updateType(o.attributeName) == SDLClasses.TYPE_color) {
            man.registerColor(o.value.x);
//        } else if (o.attributeName != null && o.attributeName.isLaser && o.attributeName.attributeIndex == 29 /* font-family */) {
//            man.registerFont(o.value.string);
        }
    }
}
