/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.util;

import iso.mpeg.streamezzo.io.Parent;
import iso.mpeg.streamezzo.saf.SAFHeader;

import java.util.Vector;
import java.io.File;

public class Scene extends Vector implements Parent {
    public String name = null;
    public String dir = "."+File.separator;
    public IDManager idMan = new IDManager();
    public Manager man = new Manager();

    public Scene(String name, String dir) {
        this.name = name;
        this.dir = dir;
    }

    public void addChild(Object o) {
        if (o instanceof String) return;
        addElement(o);
    }

    /**
     * returns the last header with the specified href
     */
    public SAFHeader lastHeaderWithThisStreamID(String sid) {
        SAFHeader res = null;
        for (int i = 0; i < size(); i++) {
            Object o = elementAt(i);
            if (o instanceof SAFHeader) {
                SAFHeader h = (SAFHeader)o;
                if (sid.equals(h.streamID)) {
                    res = h;
                }
            }
        }
        return res;
    }

    public void toMatrix() throws Exception {
        ToMatrix tm = new ToMatrix();
        for (int i = 0; i < size(); i++) {
            ((Visitable)elementAt(i)).visit(tm, null);
        }
    }

    public void fromMatrix() throws Exception {
        FromMatrix tm = new FromMatrix();
        for (int i = 0; i < size(); i++) {
            ((Visitable)elementAt(i)).visit(tm, null);
        }
    }
}
