/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.util;

import iso.mpeg.streamezzo.laser.*;
import iso.mpeg.streamezzo.sdl.Matrix;
import iso.mpeg.streamezzo.sdl.Point;

public class ToMatrix extends BasicVisitor {
    public void process(Ilocatable i) {
        if (i.getTransform() != null) {
            if (i.getRotation() != Integer.MIN_VALUE || i.getScale() != null || i.getTranslation() != null) {
                throw new RuntimeException(i+" has matrix and rotation or scale or translation");
            }
        } else {
            Matrix m = new Matrix();
            boolean b = false;
            if (i.getScale() != null) {
                Point p = i.getScale();
                m.concatScale(p.x, p.y);
                b = true;
            }
            if (i.getRotation() != Integer.MIN_VALUE) {
                m.concatRotation(i.getRotation());
                b = true;
            }
            if (i.getTranslation() != null) {
                Point p = i.getTranslation();
                m.concatTranslation(p.x, p.y);
                b = true;
            }
            if (b) {
                i.setTransform(m);
                i.setRotation(Integer.MIN_VALUE);
                i.setScale(null);
                i.setTranslation(null);
            }
        }
    }

    public void visitXa(Xa o, Object obj) throws Exception {
        super.visitXa(o, obj);
        process(o);
    }
    public void visitXcircle(Xcircle o, Object obj) throws Exception {
        super.visitXcircle(o, obj);
        process(o);
    }
    public void visitXellipse(Xellipse o, Object obj) throws Exception {
        super.visitXellipse(o, obj);
        process(o);
    }
    public void visitXg(Xg o, Object obj) throws Exception {
        super.visitXg(o, obj);
        process(o);
    }
    public void visitXimage(Ximage o, Object obj) throws Exception {
        super.visitXimage(o, obj);
        process(o);
    }
    public void visitXline(Xline o, Object obj) throws Exception {
        super.visitXline(o, obj);
        process(o);
    }
    public void visitXpath(Xpath o, Object obj) throws Exception {
        super.visitXpath(o, obj);
        process(o);
    }
    public void visitXpolygon(Xpolygon o, Object obj) throws Exception {
        super.visitXpolygon(o, obj);
        process(o);
    }
    public void visitXpolyline(Xpolyline o, Object obj) throws Exception {
        super.visitXpolyline(o, obj);
        process(o);
    }
    public void visitXrect(Xrect o, Object obj) throws Exception {
        super.visitXrect(o, obj);
        process(o);
    }
    public void visitXtext(Xtext o, Object obj) throws Exception {
        super.visitXtext(o, obj);
        process(o);
    }
    public void visitXuse(Xuse o, Object obj) throws Exception {
        super.visitXuse(o, obj);
        process(o);
    }
    public void visitXvideo(Xvideo o, Object obj) throws Exception {
        super.visitXvideo(o, obj);
        process(o);
    }
}
