/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.util;

import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeIterator;

import java.util.Vector;
import java.io.File;
import java.io.FileInputStream;

public class Utils {
    public static String removeEndDotZero(String s1) {
        if (s1.endsWith(".0")) {
            return s1.substring(0, s1.length()-2);
        }
        return s1;
    }

    public static boolean endsWith(String s1, String s2) {
        return s1.endsWith(s2);
    }

    public static int indexOf(NodeIterator list, String search) {
        if (search == null) {
            return -1;
        }
        int index = 0;
        Node node = list.nextNode();
        while (node != null) {
            if (search.equals(node.getNodeValue())) {
                return index;
            }
            index++;
            node = list.nextNode();
        }
        return -1;
    }

    public static String substByIndex(NodeIterator list, String search) {
        if (search == null) {
            return "";
        }
        Vector ids = strings(search);
        Vector nodeValues = new Vector();
        Node node = list.nextNode();
        while (node != null) {
            nodeValues.addElement(node.getNodeValue());
            node = list.nextNode();
        }
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < ids.size(); i++) {
            String si = (String)ids.elementAt(i);
            for (int j = 0; j < nodeValues.size(); j++) {
                String sj = (String)nodeValues.elementAt(j);
                if (sj.equals(si)) {
                    if (i > 0) sb.append(" ");
                    sb.append(j);
                    break;
                }
            }
        }
        return sb.toString();
    }

    public static String colorsToRgb(String colors) {
        if (colors == null) {
            return "";
        }
        Vector ids = strings(colors);
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < ids.size(); i++) {
            String si = (String)ids.elementAt(i);
            FloatAndStringListReader fsr = new FloatAndStringListReader(si);
            float r = 0f, g = 0f, b = 0f;
            try {
                r = fsr.nextFloat();
                g = fsr.nextFloat();
                b = fsr.nextFloat();
            } catch(Exception e) {}
            if (i > 0) sb.append(";");
            sb.append("rgb(");
            sb.append((int)(r*255f));
            sb.append(",");
            sb.append((int)(g*255f));
            sb.append(",");
            sb.append((int)(b*255f));
            sb.append(")");
        }
        return sb.toString();
    }

    public static String roundFloatList(String list, int precision) {
        if (list == null) {
            return "";
        }
        list = list.trim();
        boolean addPercent = false;
        if (list.endsWith("%")) {
            addPercent = true;
            list = list.substring(0, list.length()-1);
        }
        float m1 = (float)Math.pow(10f, precision), m2 = (float)Math.pow(10f, -precision);
        FloatAndStringListReader fsr = new FloatAndStringListReader(list);
        StringBuffer s = new StringBuffer();
        try {
            boolean first = true;
            while(true) {
                if (!first) s.append(" ");
                float f = fsr.nextFloat();
                String ss = Float.toString(Math.round(f*m1)*m2);
                if (ss.endsWith(".0")) ss = ss.substring(0, ss.length()-2);
                s.append(ss);
                first = false;
            }
        } catch(Exception e) {}
        return (addPercent ? s.toString() + "%" : s.toString());
    }

    public static String floatListToChar(String list, int precision) {
        if (list == null) {
            return "";
        }
        FloatAndStringListReader fsr = new FloatAndStringListReader(list);
        StringBuffer s = new StringBuffer();
        try {
            while(true) {
                fsr.nextFloat();
                s.append(" *");
            }
        } catch(Exception e) {}
        return s.toString();
    }

    public static String roundPath(String list, int precision) {
        if (list == null) {
            return "";
        }
        float m1 = (float)Math.pow(10f, precision), m2 = (float)Math.pow(10f, -precision);
        FloatAndStringListReader fsr = new FloatAndStringListReader(list);
        StringBuffer s = new StringBuffer();
        try {
            boolean first = true;
            while(true) {
                if (fsr.nextIsString()) {
                    if (!first) s.append(" ");
                    s.append(fsr.nextString());
                } else {
                    if (!first) s.append(" ");
                    float f = fsr.nextFloat();
                    String ss = Float.toString(Math.round(f * m1) * m2);
                    if (ss.endsWith(".0")) ss = ss.substring(0, ss.length() - 2);
                    s.append(ss);
                }
                first = false;
            }
        } catch(Exception e) {}
        return s.toString();
    }

/*
    public static String pathToChar(String list, int precision) {
        if (list == null) {
            return "";
        }
        float m1 = (float)Math.pow(10f, precision), m2 = (float)Math.pow(10f, -precision);
        FloatAndStringListReader fsr = new FloatAndStringListReader(list);
        StringBuffer s = new StringBuffer();
        try {
            while(true) {
                if (fsr.nextIsString()) {
                    s.append(" ");
                    s.append(fsr.nextString());
                } else {
                    fsr.nextFloat();
                    s.append(" *");
                }
            }
        } catch(Exception e) {}
        return s.toString();
    }
*/

    public static String rgbizeValues(String values) {
        Vector v = strings(values);
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < v.size(); i++) {
            String s = (String)v.elementAt(i);
            if (i != 0) sb.append(";");
            if (s.startsWith("#") && s.length() == 7) {
                s = s.substring(1);
                sb.append("rgb(");
                sb.append(Integer.parseInt(s.substring(0, 2), 16));
                sb.append(",");
                sb.append(Integer.parseInt(s.substring(2, 4), 16));
                sb.append(",");
                sb.append(Integer.parseInt(s.substring(4), 16));
                sb.append(")");
            } else if (s.startsWith("#") && s.length() == 4) {
                s = s.substring(1);
                sb.append("rgb(");
                sb.append(Integer.parseInt(s.substring(0, 1), 16) << 4);
                sb.append(",");
                sb.append(Integer.parseInt(s.substring(1, 2), 16) << 4);
                sb.append(",");
                sb.append(Integer.parseInt(s.substring(2), 16) << 4);
                sb.append(")");
            } else {
                sb.append(s);
            }
        }
        return sb.toString();
    }

    public static String rgbizeColor(String s) {
        StringBuffer sb = new StringBuffer();
        if (s.startsWith("#") && s.length() == 7) {
            s = s.substring(1);
            sb.append("rgb(");
            sb.append(Integer.parseInt(s.substring(0, 2), 16));
            sb.append(",");
            sb.append(Integer.parseInt(s.substring(2, 4), 16));
            sb.append(",");
            sb.append(Integer.parseInt(s.substring(4), 16));
            sb.append(")");
        } else if (s.startsWith("#") && s.length() == 4) {
            s = s.substring(1);
            sb.append("rgb(");
            sb.append(Integer.parseInt(s.substring(0, 1), 16) << 4);
            sb.append(",");
            sb.append(Integer.parseInt(s.substring(1, 2), 16) << 4);
            sb.append(",");
            sb.append(Integer.parseInt(s.substring(2), 16) << 4);
            sb.append(")");
        } else {
            sb.append(s);
        }
        return sb.toString();
    }

    public static Vector strings(String s) {
        return strings(s, ';');
    }

    public static Vector strings(String s, char c) {
        if (s == null) return null;
        Vector v = new Vector();
        int index = 0;
        int nextSemiColon = s.indexOf(c, index);
        while (nextSemiColon != -1) {
            v.addElement(s.substring(index, nextSemiColon).trim());
            index = nextSemiColon+1;
            nextSemiColon = s.indexOf(c, index);
        }
        if (index < s.length()) {
            v.addElement(s.substring(index).trim());
        }
        return v;
    }
    /**
     * Computes the number of bits necessary to represent a positive number
     * @param i
     */
    public static int nbBitsForPositiveInteger(int i) {
        if (i < 0) {
            throw new RuntimeException("A negative number was passed to nbBitForPositiveInteger:"+i);
        }
        int n = 0;
        for (; i > 0; i >>>= 1) {
            n++;
        }
        return n;
    }

    /**
     * Computes the number of bits to represent a number that can be positive or negative.
     * The result is one more than the above because for a positive number N, you need to be able to
     * discriminate it from its opposite -N
     * @param i
     */
    public static int nbBitsForInteger(int i) {
        return nbBitsForPositiveInteger(i < 0 ? 1-i : i) + 1;
    }
    public static String translateTransform(String s) {
        AMatrix m = new AMatrix();
        m.initFromString(s);
        return "matrix("+m.a + "," + m.b + "," + m.c + "," + m.d + "," + m.x + "," + m.y+")";
    }

    public static class AMatrix {
        public float a, b, d, c, x, y;

        public void makeIdentity() {
            a = 1f;
            b = 0f;
            c = 0f;
            d = 1f;
            x = 0f;
            y = 0f;
        }

        public void initFromString(String str) {
            this.makeIdentity();
            int k = 0;
            while (true) {
                int p = str.indexOf(")", k) + 1;
                if (p <= 0) break;
                AMatrix m = new AMatrix();
                m.parseTransform(str.substring(k, p));
                k = p;
                this.concat(m);
            }
        }

        public void concat(AMatrix m) {
            float A = m.a * this.a + m.b * this.c;
            float B = m.a * this.b + m.b * this.d;
            float C = m.c * this.a + m.d * this.c;
            float D = m.c * this.b + m.d * this.d;
            this.x += m.x * this.a + m.y * this.c;
            this.y += m.x * this.b + m.y * this.d;
            this.a = A;
            this.b = B;
            this.c = C;
            this.d = D;
        }

        public void parseTransform(String str) {
            int p = 0;
            {
                char c = 0;
                while (p < str.length() && ((c = str.charAt(p)) == ' ' || c == '\t' || c == '\n' || c == '\r' || c == ',')) {
                    p++;
                }
                str = str.substring(p);
                int fin = str.indexOf(")");
                if (fin <= 0) {
                    this.makeIdentity();
                    return;
                }
            }
            int k = str.indexOf("(");
            int close = str.indexOf(")");
            if (k == 6 && str.startsWith("matrix")) {
                FloatVector vals = new FloatVector();
                splitParams(vals, str.substring(k + 1, close));
                if (vals.size() != 6) {
                    this.makeIdentity();
                } else {
                    this.a = vals.elementAt(0) - 0;
                    this.b = vals.elementAt(1) - 0;
                    this.c = vals.elementAt(2) - 0;
                    this.d = vals.elementAt(3) - 0;
                    this.x = vals.elementAt(4) - 0;
                    this.y = vals.elementAt(5) - 0;
                }
            } else if (k == 9 && str.startsWith("translate")) {
                FloatVector vals = new FloatVector();
                splitParams(vals, str.substring(k + 1, close));
                if (vals.size() != 2)
                    this.makeIdentity();
                else {
                    this.a = 1;
                    this.b = 0;
                    this.c = 0;
                    this.d = 1;
                    this.x = vals.elementAt(0) - 0;
                    this.y = vals.elementAt(1) - 0;
                }
            } else if (k == 5 && str.startsWith("scale")) {
                FloatVector vals = new FloatVector();
                splitParams(vals, str.substring(k + 1, close));
                if (vals.size() == 1 || vals.size() == 2) {
                    this.a = vals.elementAt(0) - 0;
                    if (vals.size() == 1)
                        this.d = this.a;
                    else
                        this.d = vals.elementAt(1) - 0;
                    this.b = 0;
                    this.c = 0;
                    this.x = 0;
                    this.y = 0;
                } else
                    makeIdentity();
            } else if (k == 6 && str.startsWith("rotate")) {
                FloatVector vals = new FloatVector();
                splitParams(vals, str.substring(k + 1, close));
                float v = vals.elementAt(0) * 0.017453293f;
                this.a = (float) Math.cos(v);
                this.b = (float) Math.sin(v);
                this.c = -this.b;
                this.d = this.a;
                if (vals.size() == 3) {
                    this.x = vals.elementAt(1) * (1 - this.a) + vals.elementAt(2) * this.b;
                    this.y = vals.elementAt(2) * (1 - this.a) - vals.elementAt(1) * this.b;
                } else {
                    this.x = 0;
                    this.y = 0;
                }
            } else if (k == 5 && str.startsWith("skewX")) {
                float v = Float.parseFloat(str.substring(k + 1, close)) * 0.017453293f;
                this.a = 1;
                this.b = 0;
                this.c = (float) Math.tan(v);
                this.d = 1;
                this.x = 0;
                this.y = 0;
            } else if (k == 5 && str.startsWith("skewY")) {
                float v = Float.parseFloat(str.substring(k + 1, close)) * 0.017453293f;
                this.a = 1;
                this.b = (float) Math.tan(v);
                this.c = 0;
                this.d = 1;
                this.x = 0;
                this.y = 0;
            } else
                this.makeIdentity();
        }

        public void splitParams(FloatVector res, String str) {
            int length = str.length();
            int k = 0;
            int i = 0;
            while (k < length) {
                int p1 = str.indexOf(" ", k);
                int p2 = str.indexOf(",", k);
                int p3 = str.indexOf("\t", k);
                if (p1 < k)
                    p1 = length;
                if (p2 < k)
                    p2 = length;
                if (p3 < k)
                    p3 = length;
                int p = Math.min(p1, Math.min(p2, p3));
                if (p > k)
                    res.addElement(Float.parseFloat(str.substring(k, p)));
                k = p + 1;
            }
        }
    }

    public static boolean testForTrigger(String s) {
        if (s == null || "".equals(s)) return false;
        if (s.indexOf("+") > 0 || s.indexOf(";") > 0) return true;
        s = s.trim();
        if (s.endsWith("s")) s = s.substring(0, s.length() - 1);
        float f = Float.NEGATIVE_INFINITY;
        try {
            f = Float.parseFloat(s);
        } catch (Exception e) {
        }
        return (f == Float.NEGATIVE_INFINITY);
    }

    public static String beginForTrigger(String s) {
        int i = s.indexOf(";"), begin = 0;
        while (i > 0) {
            String ss = s.substring(begin, i);
            if (!"".equals(ss) && ss.indexOf("+") < 0) {
                ss = ss.trim();
                if (ss.endsWith("s")) ss = ss.substring(0, ss.length() - 1);
                float f = Float.NEGATIVE_INFINITY;
                try {
                    f = Float.parseFloat(ss);
                } catch (Exception e) {
                }
                if (f != Float.NEGATIVE_INFINITY) {
                    return Integer.toString((int) (f * 1000f));
                }
            }
            begin = i + 1;
            i = s.indexOf(";", begin);
        }
        return "indefinite";
    }

    public static String translatePath(String s) {
        StringBuffer res = new StringBuffer();
        int max = s.length();
        char previous = ' ', c;
        for (int i = 0; i < max; i++) {
            c = s.charAt(i);
            if (c == 'e' || c == 'E') {
                res.append(c);
            } else if (Character.isLetter(c)) {
                res.append(" ");
                res.append(c);
                res.append(" ");
            } else if (c == ',') {
                res.append(" ");
            } else if (c == '-' && previous != 'e' && previous != 'E') {
                res.append(" ");
                res.append(c);
            } else
                res.append(c);
            previous = c;
        }
        return res.toString();
    }

    public static String translateColorList(String s) {
        StringBuffer res = new StringBuffer();
        s = s.trim();
        int indexLow = 0, indexHigh = 0, max = s.length();
        for (; indexLow < max; indexLow = indexHigh + 1, indexHigh += 2) {
            indexHigh = s.indexOf(";", indexHigh);
            if (indexHigh <= 0) indexHigh = max;
            res.append(translateColors(s.substring(indexLow, indexHigh)));
            if (indexHigh != max) res.append(";");
        }
        return res.toString();
    }

    public static String translateColors(String s) {
        s = s.trim();
        if ("aliceblue".equalsIgnoreCase(s)) return "rgb(240,248,255f)";
        if ("antiquewhite".equalsIgnoreCase(s)) return "rgb(250,235,215)";
        if ("aqua".equalsIgnoreCase(s)) return "rgb(0,255,255)";
        if ("aquamarine".equalsIgnoreCase(s)) return "rgb(127,255,212)";
        if ("azure".equalsIgnoreCase(s)) return "rgb(240,255,255)";
        if ("beige".equalsIgnoreCase(s)) return "rgb(245,245,220)";
        if ("bisque".equalsIgnoreCase(s)) return "rgb(255,228,196)";
        if ("black".equalsIgnoreCase(s)) return "rgb(0,0,0)";
        if ("blanchedalmond".equalsIgnoreCase(s)) return "rgb(255,235,205)";
        if ("blue".equalsIgnoreCase(s)) return "rgb(0,0,255)";
        if ("blueviolet".equalsIgnoreCase(s)) return "rgb(138,43,226)";
        if ("brown".equalsIgnoreCase(s)) return "rgb(165,42,42)";
        if ("burlywood".equalsIgnoreCase(s)) return "rgb(222,184,135)";
        if ("cadetblue".equalsIgnoreCase(s)) return "rgb(95,158,160)";
        if ("chartreuse".equalsIgnoreCase(s)) return "rgb(127,255,0)";
        if ("chocolate".equalsIgnoreCase(s)) return "rgb(210,105,30)";
        if ("coral".equalsIgnoreCase(s)) return "rgb(255,127,80)";
        if ("cornflowerblue".equalsIgnoreCase(s)) return "rgb(100,149,237)";
        if ("cornsilk".equalsIgnoreCase(s)) return "rgb(255,248,220)";
        if ("crimson".equalsIgnoreCase(s)) return "rgb(220,20,60)";
        if ("cyan".equalsIgnoreCase(s)) return "rgb(0,255,255)";
        if ("darkblue".equalsIgnoreCase(s)) return "rgb(0,0,139)";
        if ("darkcyan".equalsIgnoreCase(s)) return "rgb(0,139,139)";
        if ("darkgoldenrod".equalsIgnoreCase(s)) return "rgb(184,134,11)";
        if ("darkgray".equalsIgnoreCase(s)) return "rgb(169,169,169)";
        if ("darkgreen".equalsIgnoreCase(s)) return "rgb(0,100,0)";
        if ("darkgrey".equalsIgnoreCase(s)) return "rgb(169,169,169)";
        if ("darkkhaki".equalsIgnoreCase(s)) return "rgb(189,183,107)";
        if ("darkmagenta".equalsIgnoreCase(s)) return "rgb(139,0,139)";
        if ("darkolivegreen".equalsIgnoreCase(s)) return "rgb(85,107,47)";
        if ("darkorange".equalsIgnoreCase(s)) return "rgb(255,140,0)";
        if ("darkorchid".equalsIgnoreCase(s)) return "rgb(153,50,204)";
        if ("darkred".equalsIgnoreCase(s)) return "rgb(139,0,0)";
        if ("darksalmon".equalsIgnoreCase(s)) return "rgb(233,150,122)";
        if ("darkseagreen".equalsIgnoreCase(s)) return "rgb(143,188,143)";
        if ("darkslateblue".equalsIgnoreCase(s)) return "rgb(72,61,139)";
        if ("darkslategray".equalsIgnoreCase(s)) return "rgb(47,79,79)";
        if ("darkslategrey".equalsIgnoreCase(s)) return "rgb(47,79,79)";
        if ("darkturquoise".equalsIgnoreCase(s)) return "rgb(0,206,209)";
        if ("darkviolet".equalsIgnoreCase(s)) return "rgb(148,0,211)";
        if ("deeppink".equalsIgnoreCase(s)) return "rgb(255,20,147)";
        if ("deepskyblue".equalsIgnoreCase(s)) return "rgb(0,191,255)";
        if ("dimgray".equalsIgnoreCase(s)) return "rgb(105,105,105)";
        if ("dimgrey".equalsIgnoreCase(s)) return "rgb(105,105,105)";
        if ("dodgerblue".equalsIgnoreCase(s)) return "rgb(30,144,255)";
        if ("firebrick".equalsIgnoreCase(s)) return "rgb(178,34,34)";
        if ("floralwhite".equalsIgnoreCase(s)) return "rgb(255,250,240)";
        if ("forestgreen".equalsIgnoreCase(s)) return "rgb(34,139,34)";
        if ("fuchsia".equalsIgnoreCase(s)) return "rgb(255,0,255)";
        if ("gainsboro".equalsIgnoreCase(s)) return "rgb(220,220,220)";
        if ("ghostwhite".equalsIgnoreCase(s)) return "rgb(248,248,255)";
        if ("gold".equalsIgnoreCase(s)) return "rgb(255,215,0)";
        if ("goldenrod".equalsIgnoreCase(s)) return "rgb(218,165,32)";
        if ("gray".equalsIgnoreCase(s)) return "rgb(128,128,128)";
        if ("grey".equalsIgnoreCase(s)) return "rgb(128,128,128)";
        if ("green".equalsIgnoreCase(s)) return "rgb(0,128,0)";
        if ("greenyellow".equalsIgnoreCase(s)) return "rgb(173,255,47)";
        if ("honeydew".equalsIgnoreCase(s)) return "rgb(240,255,240)";
        if ("hotpink".equalsIgnoreCase(s)) return "rgb(255,105,180)";
        if ("indianred".equalsIgnoreCase(s)) return "rgb(205,92,92)";
        if ("indigo".equalsIgnoreCase(s)) return "rgb(75,0,130)";
        if ("ivory".equalsIgnoreCase(s)) return "rgb(255,255,240)";
        if ("khaki".equalsIgnoreCase(s)) return "rgb(240,230,140)";
        if ("lavender".equalsIgnoreCase(s)) return "rgb(230,230,250)";
        if ("lavenderblush".equalsIgnoreCase(s)) return "rgb(255,240,245)";
        if ("lawngreen".equalsIgnoreCase(s)) return "rgb(124,252,0)";
        if ("lemonchiffon".equalsIgnoreCase(s)) return "rgb(255,250,205)";
        if ("lightblue".equalsIgnoreCase(s)) return "rgb(173,216,230)";
        if ("lightcoral".equalsIgnoreCase(s)) return "rgb(240,128,128)";
        if ("lightcyan".equalsIgnoreCase(s)) return "rgb(224,255,255)";
        if ("lightgoldenrodyellow".equalsIgnoreCase(s)) return "rgb(250,250,210)";
        if ("lightgray".equalsIgnoreCase(s)) return "rgb(211,211,211)";
        if ("lightgreen".equalsIgnoreCase(s)) return "rgb(144,238,144)";
        if ("lightgrey".equalsIgnoreCase(s)) return "rgb(211,211,211)";
        if ("lightpink".equalsIgnoreCase(s)) return "rgb(255,182,193)";
        if ("lightsalmon".equalsIgnoreCase(s)) return "rgb(255,160,122)";
        if ("lightseagreen".equalsIgnoreCase(s)) return "rgb(32,178,170)";
        if ("lightskyblue".equalsIgnoreCase(s)) return "rgb(135,206,250)";
        if ("lightslategray".equalsIgnoreCase(s)) return "rgb(119,136,153)";
        if ("lightslategrey".equalsIgnoreCase(s)) return "rgb(119,136,153)";
        if ("lightsteelblue".equalsIgnoreCase(s)) return "rgb(176,196,222)";
        if ("lightyellow".equalsIgnoreCase(s)) return "rgb(255,255,224)";
        if ("lime".equalsIgnoreCase(s)) return "rgb(0,255,0)";
        if ("limegreen".equalsIgnoreCase(s)) return "rgb(50,205,50)";
        if ("linen".equalsIgnoreCase(s)) return "rgb(250,240,230)";
        if ("magenta".equalsIgnoreCase(s)) return "rgb(255,0,255)";
        if ("maroon".equalsIgnoreCase(s)) return "rgb(128,0,0)";
        if ("mediumaquamarine".equalsIgnoreCase(s)) return "rgb(102,205,170)";
        if ("mediumblue".equalsIgnoreCase(s)) return "rgb(0,0,205)";
        if ("mediumorchid".equalsIgnoreCase(s)) return "rgb(186,85,211)";
        if ("mediumpurple".equalsIgnoreCase(s)) return "rgb(147,112,219)";
        if ("mediumseagreen".equalsIgnoreCase(s)) return "rgb(60,179,113)";
        if ("mediumslateblue".equalsIgnoreCase(s)) return "rgb(123,104,238)";
        if ("mediumspringgreen".equalsIgnoreCase(s)) return "rgb(0,250,154)";
        if ("mediumturquoise".equalsIgnoreCase(s)) return "rgb(72,209,204)";
        if ("mediumvioletred".equalsIgnoreCase(s)) return "rgb(199,21,133)";
        if ("midnightblue".equalsIgnoreCase(s)) return "rgb(25,25,112)";
        if ("mintcream".equalsIgnoreCase(s)) return "rgb(245,255,250)";
        if ("mistyrose".equalsIgnoreCase(s)) return "rgb(255,228,225)";
        if ("moccasin".equalsIgnoreCase(s)) return "rgb(255,228,181)";
        if ("navajowhite".equalsIgnoreCase(s)) return "rgb(255,222,173)";
        if ("navy".equalsIgnoreCase(s)) return "rgb(0,0,128)";
        if ("oldlace".equalsIgnoreCase(s)) return "rgb(253,245,230)";
        if ("olive".equalsIgnoreCase(s)) return "rgb(128,128,0)";
        if ("olivedrab".equalsIgnoreCase(s)) return "rgb(107,142,35)";
        if ("orange".equalsIgnoreCase(s)) return "rgb(255,165,0)";
        if ("orangered".equalsIgnoreCase(s)) return "rgb(255,69,0)";
        if ("orchid".equalsIgnoreCase(s)) return "rgb(218,112,214)";
        if ("palegoldenrod".equalsIgnoreCase(s)) return "rgb(238,232,170)";
        if ("palegreen".equalsIgnoreCase(s)) return "rgb(152,251,152)";
        if ("paleturquoise".equalsIgnoreCase(s)) return "rgb(175,238,238)";
        if ("palevioletred".equalsIgnoreCase(s)) return "rgb(219,112,147)";
        if ("papayawhip".equalsIgnoreCase(s)) return "rgb(255,239,213)";
        if ("peachpuff".equalsIgnoreCase(s)) return "rgb(255,218,185)";
        if ("peru".equalsIgnoreCase(s)) return "rgb(205,133,63)";
        if ("pink".equalsIgnoreCase(s)) return "rgb(255,192,203)";
        if ("plum".equalsIgnoreCase(s)) return "rgb(221,160,221)";
        if ("powderblue".equalsIgnoreCase(s)) return "rgb(176,224,230)";
        if ("purple".equalsIgnoreCase(s)) return "rgb(128,0,128)";
        if ("red".equalsIgnoreCase(s)) return "rgb(255,0,0)";
        if ("rosybrown".equalsIgnoreCase(s)) return "rgb(188,143,143)";
        if ("royalblue".equalsIgnoreCase(s)) return "rgb(65,105,225)";
        if ("saddlebrown".equalsIgnoreCase(s)) return "rgb(139,69,19)";
        if ("salmon".equalsIgnoreCase(s)) return "rgb(250,128,114)";
        if ("sandybrown".equalsIgnoreCase(s)) return "rgb(244,164,96)";
        if ("seagreen".equalsIgnoreCase(s)) return "rgb(46,139,87)";
        if ("seashell".equalsIgnoreCase(s)) return "rgb(255,245,238)";
        if ("sienna".equalsIgnoreCase(s)) return "rgb(160,82,45)";
        if ("silver".equalsIgnoreCase(s)) return "rgb(192,192,192)";
        if ("skyblue".equalsIgnoreCase(s)) return "rgb(135,206,235)";
        if ("slateblue".equalsIgnoreCase(s)) return "rgb(106,90,205)";
        if ("slategray".equalsIgnoreCase(s)) return "rgb(112,128,144)";
        if ("slategrey".equalsIgnoreCase(s)) return "rgb(112,128,144)";
        if ("snow".equalsIgnoreCase(s)) return "rgb(255,250,250)";
        if ("springgreen".equalsIgnoreCase(s)) return "rgb(0,255,127)";
        if ("steelblue".equalsIgnoreCase(s)) return "rgb(70,130,180)";
        if ("tan".equalsIgnoreCase(s)) return "rgb(210,180,140)";
        if ("teal".equalsIgnoreCase(s)) return "rgb(0,128,128)";
        if ("thistle".equalsIgnoreCase(s)) return "rgb(216,191,216)";
        if ("tomato".equalsIgnoreCase(s)) return "rgb(255,99,71)";
        if ("turquoise".equalsIgnoreCase(s)) return "rgb(64,224,208)";
        if ("violet".equalsIgnoreCase(s)) return "rgb(238,130,238)";
        if ("wheat".equalsIgnoreCase(s)) return "rgb(245,222,179)";
        if ("white".equalsIgnoreCase(s)) return "rgb(255,255,255)";
        if ("whitesmoke".equalsIgnoreCase(s)) return "rgb(245,245,245)";
        if ("yellow".equalsIgnoreCase(s)) return "rgb(255,255,0)";
        if ("yellowgreen".equalsIgnoreCase(s)) return "rgb(154,205,50)";
        /*if (s.startsWith("rgb(")) {
            FloatAndStringListReader fslr = new FloatAndStringListReader(s.substring(4));
            try {
                float r = fslr.nextFloat() / 255f;
                float g = fslr.nextFloat() / 255f;
                float b = fslr.nextFloat() / 255f;
                return r + " " + g + " " + b;
            } catch (Exception e) {
                return "error";
            }
        } else */if (s.startsWith("#") && s.length() == 7) {
            int i = Integer.parseInt(s.substring(1), 16);
            float b = (i & 0xff);
            i = i >> 8;
            float g = (i & 0xff);
            i = i >> 8;
            float r = (i & 0xff);
            return "rgb("+(int)r + "," +(int)g + "," +(int)b+")";
        } else if (s.startsWith("#")) {
            int i = Integer.parseInt(s.substring(1), 16);
            float b = (i & 0xf) * 16f;
            i = i >> 4;
            float g = (i & 0xf) * 16f;
            i = i >> 4;
            float r = (i & 0xf) * 16f;
            return "rgb("+(int)r + "," +(int)g + "," +(int)b+")";
        } else
            return s;
    }

    public static String toMilliseconds(String s) {
        if (s.endsWith("s")) s = s.substring(0, s.length() - 1);
        float f = Float.parseFloat(s);
        f = Math.round(f);
        return Integer.toString((int) f);
    }

    public static byte[] get(String s) throws java.io.IOException {
        File f = new File(s);
        FileInputStream fis = new FileInputStream(s);
        byte b[] = new byte[(int)f.length()];
        fis.read(b);
        return b;
    }

}
