<?xml version="1.0"  encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:java="http://xml.apache.org/xalan/java" xmlns:saf="urn:mpeg:mpeg4:SAF:2005"
    xmlns:lsr="urn:mpeg:mpeg4:LASeR:2005"
    xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:svg="http://www.w3.org/2000/svg"
    xmlns:test="http://www.w3.org/2000/02/svg/testsuite/description/" xmlns:ev="http://www.w3.org/2001/xml-events" 
    exclude-result-prefixes="java test">
    <xsl:output encoding="UTF-8" indent="yes" method="xml"/>
    <xsl:template match="*|@*|text()">
        <xsl:copy>
            <xsl:apply-templates select="*|@*|text()"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="svg:svg">
        <saf:SAFSession xmlns="http://www.w3.org/2000/svg"
                        xmlns:lsr="urn:mpeg:mpeg4:LASeR:2005" xmlns:saf="urn:mpeg:mpeg4:SAF:2005" 
                        xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:xlink="http://www.w3.org/1999/xlink">
            <saf:sceneHeader>
                <lsr:LASeRHeader/>
            </saf:sceneHeader>
            <xsl:for-each select="//svg:image[starts-with(@xlink:href, 'data:')]">
                <saf:imageHeader>
                    <xsl:attribute name="streamID">
                        <xsl:value-of select="generate-id()"/>
                    </xsl:attribute>
                    <xsl:attribute name="objectTypeIndication">
                        <xsl:choose>
                            <xsl:when test="contains(@xlink:href, 'jpeg')">108</xsl:when>
                            <xsl:otherwise>109</xsl:otherwise>
                        </xsl:choose>
                    </xsl:attribute>
                </saf:imageHeader>
                <saf:imageUnit>
                    <xsl:attribute name="ref">
                        <xsl:value-of select="generate-id()"/>
                    </xsl:attribute>
                    <xsl:attribute name="source">
                        <xsl:value-of select="@xlink:href"/>
                    </xsl:attribute>
                </saf:imageUnit>
                <saf:endOfStream>
                    <xsl:attribute name="ref">
                        <xsl:value-of select="generate-id()"/>
                    </xsl:attribute>
                </saf:endOfStream>
            </xsl:for-each>
            <saf:sceneUnit>
                <lsr:NewScene>
                    <svg>
                        <xsl:apply-templates select="*|@*"/>
                    </svg>
                </lsr:NewScene>
            </saf:sceneUnit>
            <saf:endOfSAFSession/>
        </saf:SAFSession>
    </xsl:template>
    <xsl:template match="@transform">
        <xsl:attribute name="transform">
            <xsl:value-of select="java:iso.mpeg.streamezzo.util.Utils.translateTransform(.)"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="svg:path/@d">
        <xsl:attribute name="d">
            <xsl:value-of select="normalize-space(java:iso.mpeg.streamezzo.util.Utils.translatePath(.))"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="svg:glyph/@d">
        <xsl:attribute name="d">
            <xsl:value-of select="normalize-space(java:iso.mpeg.streamezzo.util.Utils.translatePath(.))"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="svg:missing-glyph/@d">
        <xsl:attribute name="d">
            <xsl:value-of select="normalize-space(java:iso.mpeg.streamezzo.util.Utils.translatePath(.))"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="svg:animateMotion/@path">
        <xsl:attribute name="path">
            <xsl:value-of select="normalize-space(java:iso.mpeg.streamezzo.util.Utils.translatePath(.))"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="svg:animate[@attributeName = 'd']/@values">
        <xsl:attribute name="values">
            <xsl:value-of select="normalize-space(java:iso.mpeg.streamezzo.util.Utils.translatePath(.))"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="svg:animate[@attributeName = 'd']/@from">
        <xsl:attribute name="from">
            <xsl:value-of select="normalize-space(java:iso.mpeg.streamezzo.util.Utils.translatePath(.))"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="svg:animate[@attributeName = 'd']/@to">
        <xsl:attribute name="to">
            <xsl:value-of select="normalize-space(java:iso.mpeg.streamezzo.util.Utils.translatePath(.))"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="svg:set[@attributeName = 'd']/@to">
        <xsl:attribute name="to">
            <xsl:value-of select="normalize-space(java:iso.mpeg.streamezzo.util.Utils.translatePath(.))"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="@xlink:href">
        <xsl:attribute name="xlink:href">
            <xsl:choose>
                <xsl:when test="starts-with(.,'data:')">
                    <xsl:value-of select="generate-id(..)"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="."/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:attribute>
    </xsl:template>

    <xsl:template match="@style">
        <xsl:call-template name="processStyle">
            <xsl:with-param name="style" select="."/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template name="processStyle1">
        <xsl:param name="style"/>
    </xsl:template>

    <xsl:template name="processStyle">
        <xsl:param name="style"/>
        <xsl:variable name="process" select="substring-before($style, ';')"/>
        <xsl:choose>
            <xsl:when test="$process">
                <xsl:attribute name="{normalize-space(substring-before($process, ':'))}">
                    <xsl:value-of select="normalize-space(substring-after($process, ':'))"/>
                </xsl:attribute>
                <xsl:call-template name="processStyle">
                    <xsl:with-param name="style" select="substring-after($style, ';')"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="v" select="normalize-space(substring-before($style, ':'))"/>
                <xsl:if test="$v != ''">
                    <xsl:attribute name="{$v}">
                        <xsl:value-of select="normalize-space(substring-after($style, ':'))"/>
                    </xsl:attribute>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>

