/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */

package iso.mpeg.streamezzo.player.node;

import iso.mpeg.streamezzo.player.library.Matrix2D;
import iso.mpeg.streamezzo.player.tool.GraphObjectManager;
import iso.mpeg.streamezzo.player.tool.SceneLoader;
import iso.mpeg.streamezzo.player.util.EventListener;


public class A extends GenericGroup
{
    //newly added attrs
	public Xlink xlink = new Xlink();
	
	/** URL of this hyperlink element */
    public String target = null;

////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////// Properties management  ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Default constructor, it sets the transform matrix to the identity matrix.
     *
     * @see RenderedGraphObject
     */
    public A(GraphObjectManager graphObjectManager)
    {
        super(graphObjectManager);
        m_actualTransform = Matrix2D.createIdentityMatrix();
        m_definedTransform = Matrix2D.createIdentityMatrix();
    }

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////// Rendering operations ////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * This method computes all the tranform in the graph according to the
     * stored transform matrix. If the node did not change, only propagation is needed
     */
    public void specificRender(int a_sceneTimeInMillis, boolean parentHasChanged, GraphObject useContext)
    {
        // Copy the flag that indicates if something was changed and reset the original flag
        // We have to use this copy because a child node can modify the original flag
        boolean wasUpdatedCurrent = m_wasUpdated;
        m_wasUpdated = false;

        // Ignore visibility because even if this grouping element is hidden, one of its children can be visible
        // and have to be rendered.
        // Furthermore inheritable properties and matrix must be spread
        if (parentHasChanged)
        {
            wasUpdatedCurrent = true;
        }

        // Process on children
        int nbChildren = m_children.size();
        GraphObject child = null;
        for (int i = 0; i < nbChildren; i++) {
            child = (GraphObject) m_children.elementAt(i);
            child.render(a_sceneTimeInMillis, wasUpdatedCurrent, m_actualTransform, m_actualProperties, useContext);
        }
    }
   
    public void postRender()
    {
        // Hyperlinking
        if (xlink.getHref() != null)
        {
            // Load a new scene
            SceneLoader sceneLoader = m_graphObjectManager.m_renderer.getSceneLoader();
            sceneLoader.reLoadScene(xlink.getHref().uri);
        }
    }

////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////// OnEvent //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Is called when receiving an event
     */
    public void onEvent(Event event, Object eventData)
    {
        if (event.event == EventListener.UI_ACTIVATE)
        {
            if (xlink.getHref() != null)
            {
                // Load a new scene in the Post Render
               m_graphObjectManager.registerForPostRender(this);
            }
        }
    }
}
