/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */

package iso.mpeg.streamezzo.player.node;

import iso.mpeg.streamezzo.player.tool.Debug;
import iso.mpeg.streamezzo.player.tool.GraphObjectManager;

public class Audio extends Multimedia
{
    //newly added attrs
	public Xlink xlink = new Xlink(); // related streamId
	public int dur = Integer.MIN_VALUE;
	public int syncBehavior = Integer.MIN_VALUE;
    public URI syncReference;
    public int syncTolerance = Integer.MIN_VALUE;
    public int audio_level = Integer.MIN_VALUE;
    
    private Object m_oMedia = null;    // Device specific media

    public Audio(GraphObjectManager graphObjectManager)
    {
        super(graphObjectManager);
    }

    public int getAudioStreamId() {
    	if (xlink.getHref() != null)
    		return xlink.getHref().streamID;
    	else 
    		throw new RuntimeException("Null href, could not get audio stream id");
    }

    
    public void specificRender(int sceneTimeInMillis, boolean parentHasChanged, GraphObject useContext)
    {
        switch (m_state)
        {
            case STATE_STARTPLAYING:
                if (Debug.DEBUG_MULTIMEDIA) System.out.println("MULTIMEDIA : sceneTime="+sceneTimeInMillis+ " - state = START PLAYING");
                if (m_oMedia != null)
                {
                    // release previous media
                    m_graphObjectManager.m_renderer.cleanMedia(m_oMedia);
                    m_oMedia = null;
                }
                Object oStream = m_graphObjectManager.getSingleObject(getAudioStreamId());
                if ((oStream != null) && (oStream instanceof String))
                {
                    m_oMedia = m_graphObjectManager.m_renderer.createRemoteAudio((String)oStream, 1);
                    m_graphObjectManager.m_renderer.playMedia(m_oMedia);
                }
                break;

            case STATE_PLAYING:
                if (! isStarted())
                {
                    if (m_graphObjectManager.m_renderer.isMediaStarted(m_oMedia))
                    {
                        long offset = m_graphObjectManager.m_renderer.getCurrentMediaTime(m_oMedia);
                        onStart(sceneTimeInMillis, offset == -1 ? -1 : (int)(offset / (long) 1000));
                    }
                }
                if (isDurationNeeded())
                {
                    long nDuration = m_graphObjectManager.m_renderer.getMediaDuration(m_oMedia);
                    if (nDuration != -1)
                    {
                        // round up
                        nDuration += 500;
                        // convert to milliseconds
                        onDurationAvailable(sceneTimeInMillis, (int)(nDuration / (long) 1000));
                    }
                }
                if ( m_graphObjectManager.m_renderer.isEndOfMedia(m_oMedia))
                {
                    onEnd(sceneTimeInMillis);
                }
                break;

            case STATE_ENDPLAYING:
                if (Debug.DEBUG_MULTIMEDIA) System.out.println("MULTIMEDIA : sceneTime="+sceneTimeInMillis+ " - state = END PLAYING");
                if (m_oMedia != null)
                {
                    // release media
                    m_graphObjectManager.m_renderer.cleanMedia(m_oMedia);
                    m_oMedia = null;
                }
                break;
        }
    }


}