/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.player.node;

import iso.mpeg.streamezzo.player.library.Matrix2D;
import iso.mpeg.streamezzo.player.library.Point2D;
import iso.mpeg.streamezzo.player.tool.GraphObjectManager;

import java.util.Vector;

public abstract class GenericGroup extends RenderedGraphObject {

    public GenericGroup(GraphObjectManager graphObjectManager) {
        super(graphObjectManager);
        m_children = new Vector();
    }

    public void setTransformMatrixValues(Matrix2D a_matrix)
    {
        m_wasUpdated = true;
        Matrix2D.copyMatrixVariables(m_definedTransform, a_matrix);
        if (m_parent == null)
        {
            //It is the root node
            Matrix2D.copyMatrixVariables(m_actualTransform, m_definedTransform);
        }
    }

    public Matrix2D getTransformMatrixByRef()
    {
        return m_definedTransform;
    }

    /**
     * Updates the presentation values of the matrix
     * @param newTransform New transformation values
     */
    public void updateActualTransformation(Matrix2D newTransform)
    {
        Matrix2D.copyMatrixVariables(m_actualTransform, newTransform);

        // Notify if something changes
        if (! m_actualTransform.equals(m_prevActualTransform))
        {
            wasUpdated();
        }
    }

    public void setTranslation(Point2D translation)
    {
        m_wasUpdated = true;
        m_definedTransform.setTranslation(translation);
        if (m_parent == null)
        {
            //It is the root node
            Matrix2D.copyMatrixVariables(m_actualTransform, m_definedTransform);
        }
    }

    public void setScale(Point2D scale)
    {
        m_wasUpdated = true;
        m_definedTransform.setScale(scale);
        if (m_parent == null)
        {
            //It is the root node
            Matrix2D.copyMatrixVariables(m_actualTransform, m_definedTransform);
        }
    }

}
