/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.player.node;

import iso.mpeg.streamezzo.player.tool.GraphObjectManager;
import iso.mpeg.streamezzo.player.tool.SceneLoader;

public class Svg extends GenericGroup
{
	//newly added attributes
    public int syncBehaviorDefault = Integer.MIN_VALUE;
    public int syncToleranceDefault = Integer.MIN_VALUE;
    public int playbackOrder = Integer.MIN_VALUE;
    public int timelineBegin = Integer.MIN_VALUE;
    public int snapshotTime = Integer.MIN_VALUE;
    public String version = null;
    public String baseProfile = null;
    public String contentScriptType = null;
    
	//TODO now there are more types of units
    /** If units for width is expressed in percents */
    public boolean m_wUnitsIsPercents = false;
    /** If units for height is expressed in percents */
    public boolean m_hUnitsIsPercents = false;
    /** The scene width */
    public int m_rSceneWidth = -1;
    /** The scene height */
    public int m_rSceneHeight = -1;
    /** The DOM viewbox {x, y, w, h}. Values are RealInt20.12 */
    public int[] m_rViewbox = {0, 0, 0, 0};
    /** The actual viewbox {x, y, w, h}. Values are RealInt20.12 */
    public int[] m_rActualViewbox = {0, 0, 0, 0};
    /** The Previous actual viewbox {x, y, w, h}. Values are RealInt20.12 */
    public int[] m_rPrevActualViewbox = {0, 0, 0, 0};
    /** preserveAspectRatio mode on X [0 : none / 1 : xMin / 2 : xMid / 3 : xMax] **/
    public int m_preserveAspectRatioX = 2;
    /** preserveAspectRatio mode on Y [0 : none / 1 : yMin / 2 : yMid / 3 : yMax] **/
    public int m_preserveAspectRatioY = 2;
    /** preserveAspectRatio scale mode [0 : meet / 1 : slice] **/
    public int m_preserveAspectRatioScale = 0;
    /** ZoomAndPan [false : disable / true : magnify]  **/
    public boolean m_zoomAndPan = true;
    /** Background color */
    public int m_backgroundFill = Value.COLOR_NONE;
    /** Background fill opacity */
    public byte m_backgroundFillOpacity = (byte)0xFF;
    /** Buffering time */
    public int m_bufferingTime = 0;
    /** The sceneloader instance */
    public SceneLoader m_sceneLoader = null;

    /**
     * Constructor
     * @param graphObjectManager
     * @see iso.mpeg.streamezzo.player.node.RenderedGraphObject
     */
    public Svg(GraphObjectManager graphObjectManager)
    {
        super(graphObjectManager);
    }

    /**
     * This operation is responsible to reset presentation values with the DOM Values
     * Doesn't apply on base properties (stroke, fill, opacity, ...)
     */
    public void resetDOMValues()
    {
        // Changed ?
        if ( (m_rViewbox[0] != m_rActualViewbox[0]) ||
             (m_rViewbox[1] != m_rActualViewbox[1]) ||
             (m_rViewbox[2] != m_rActualViewbox[2]) ||
             (m_rViewbox[3] != m_rActualViewbox[3]) )
        {
            // Store previous values
            m_rPrevActualViewbox[0] = m_rActualViewbox[0];
            m_rPrevActualViewbox[1] = m_rActualViewbox[1];
            m_rPrevActualViewbox[2] = m_rActualViewbox[2];
            m_rPrevActualViewbox[3] = m_rActualViewbox[3];

            // Set DOM values
            m_rActualViewbox[0] = m_rViewbox[0];
            m_rActualViewbox[1] = m_rViewbox[1];
            m_rActualViewbox[2] = m_rViewbox[2];
            m_rActualViewbox[3] = m_rViewbox[3];

            m_wasUpdated = true;
        }
    }

    public void setDOMBackgroundFill(int color)
    {
        m_backgroundFill = color;
        m_sceneLoader.setBackGroundColor(m_backgroundFill);
    }

    public void setDOMWidth(int r_width)
    {
        m_rSceneWidth = r_width;
        m_wUnitsIsPercents = false;
        m_sceneLoader.setSceneSize(m_rSceneWidth, m_wUnitsIsPercents, m_rSceneHeight, m_hUnitsIsPercents);
    }

    public void setDOMHeight(int r_height)
    {
        m_rSceneHeight = r_height;
        m_hUnitsIsPercents = false;
        m_sceneLoader.setSceneSize(m_rSceneWidth, m_wUnitsIsPercents, m_rSceneHeight, m_hUnitsIsPercents);
    }

    public void setDOMZoomAndPan(boolean isTrue)
    {
        m_zoomAndPan = isTrue;
        m_sceneLoader.setZoomAndPan(m_zoomAndPan);
    }

    public void setDOMViewBox(int[] viewbox)
    {
        m_rViewbox[0] = viewbox[0];
        m_rViewbox[1] = viewbox[1];
        m_rViewbox[2] = viewbox[2];
        m_rViewbox[3] = viewbox[3];
    }

    public void updateActualViewbox(int[] rViewbox)
    {
        // Sanity check
        if (rViewbox.length < 4)
        {
            return;
        }

        // Check changes
        if ( (rViewbox[0] != m_rPrevActualViewbox[0]) ||
             (rViewbox[1] != m_rPrevActualViewbox[1]) ||
             (rViewbox[2] != m_rPrevActualViewbox[2]) ||
             (rViewbox[3] != m_rPrevActualViewbox[3]) )
        {
            m_rActualViewbox[0] = rViewbox[0];
            m_rActualViewbox[1] = rViewbox[1];
            m_rActualViewbox[2] = rViewbox[2];
            m_rActualViewbox[3] = rViewbox[3];
            m_wasUpdated = true;
        }
        else
        {
            m_rActualViewbox[0] = m_rPrevActualViewbox[0];
            m_rActualViewbox[1] = m_rPrevActualViewbox[1];
            m_rActualViewbox[2] = m_rPrevActualViewbox[2];
            m_rActualViewbox[3] = m_rPrevActualViewbox[3];
            m_wasUpdated = false;
        }
    }

////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////// Renderingt operations ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
    /**
     * This method computes all the tranform in the graph according to the
     * stored transform matrix. If the node did not change, only propagation is needed
     */
    public void specificRender(int sceneTimeInMillis, boolean parentHasChanged, GraphObject useContext)
    {
        // update the scene dimension factors
        m_sceneLoader.setViewbox(m_rActualViewbox);

        // Children count
        int nbChildren = m_children.size();

        // Render chidren
        GraphObject child = null;
        for (int i = 0; i < nbChildren; i++)
        {
            child = (GraphObject)m_children.elementAt(i);
            if (child != null)
            {
                child.render(sceneTimeInMillis, false, m_actualTransform, m_actualProperties, useContext);
            }
        }
    }


}
