/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.player.node;

import iso.mpeg.streamezzo.player.library.Matrix2D;
import iso.mpeg.streamezzo.player.library.Point2D;
import iso.mpeg.streamezzo.player.library.Polygon2D;
import iso.mpeg.streamezzo.player.library.Real;
import iso.mpeg.streamezzo.player.util.NodeProperties;

import java.util.Vector;

public class Value {
    
	public int m_updateValueType = Integer.MIN_VALUE;
	public int m_animValueType = Integer.MIN_VALUE;
	
	public int m_escapeEnum = Integer.MIN_VALUE;
	public URI m_uri = null;
	public Path m_path = null;
	public Event m_event = null;
	
	public Polygon2D m_polygon = null;
    public byte[] m_types = null;
    public int m_enumOrColorOrRealOrID = -1;
    public String m_string = null;
    public Matrix2D m_matrix = null;
    public int[] m_floatsOrInts = null;
    public boolean m_boolean = false;
    public Point2D m_point = null;
    public Object m_node = null;
    public Vector m_textContent = null;
    public Vector m_nodes = null;
	
	// animatableAttributeIndex constants
    
    public final static int ANIM_CX = 3;
    public final static int ANIM_CY = 4;
    public final static int ANIM_D = 5;
    public final static int ANIM_DISPLAY = 6;
    public final static int ANIM_DISPLAYALIGN = 7;
    public final static int ANIM_FILL = 9;
    public final static int ANIM_FILLOPACITY = 10;
    public final static int ANIM_FILLRULE = 11;
    public final static int ANIM_FOCUSABLE = 20;
    public final static int ANIM_FONTFAMILY = 21;
    public final static int ANIM_FONTSIZE = 22;
    public final static int ANIM_FONTSTYLE = 23;
    public final static int ANIM_HEIGHT = 26;
    public final static int ANIM_POINTS = 30;
	public final static int ANIM_R = 32;
	public final static int ANIM_RX = 34;
	public final static int ANIM_RY = 35;
	public final static int ANIM_SIZE = 37; 
	public final static int ANIM_STROKE = 40;
	public final static int ANIM_STROKEOPACITY = 46;
	public final static int ANIM_STROKEWIDTH = 47;
	public final static int ANIM_TEXTALIGN = 49;
	public final static int ANIM_TRANSFORM = 50;
	public final static int ANIM_VIEWBOX = 53;
	public final static int ANIM_VISIBILITY = 56;
	public final static int ANIM_WIDTH = 57;
	public final static int ANIM_X = 58;
	public final static int ANIM_X1 = 59;
	public final static int ANIM_X2 = 60;
	public final static int ANIM_HREF = 61;
	public final static int ANIM_Y = 62;
	public final static int ANIM_Y1 = 63;
	public final static int ANIM_Y2 = 64;
	
	
	// AnimValue constants
        
    public final static int ANIMTYPE_string = 0;
	public final static int ANIMTYPE_float = 1;
	public final static int ANIMTYPE_path = 2;
	public final static int ANIMTYPE_pointSeq = 3;
	public final static int ANIMTYPE_fraction = 4;
	public final static int ANIMTYPE_color = 5;
	public final static int ANIMTYPE_enum = 6;
	public final static int ANIMTYPE_ints = 7;
	public final static int ANIMTYPE_floats = 8;
	public final static int ANIMTYPE_point = 9;
	public final static int ANIMTYPE_id = 10;
	public final static int ANIMTYPE_font = 11;
	public final static int ANIMTYPE_uri = 12;
	
	// TML : TODO : Find it in spec....
	public final static int animName[] = new int[] {
		-1, // "a.target",
		-1, // "accumulate",
		-1, // "additive",
		-1, // "audio-level",
		-1, // "bandwidth",
		-1, // "begin",
		-1, // "calcMode",
		-1, // "children",
		-1, // "choice",
		-1, // "clipBegin",
		-1, // "clipEnd",
		-1, // "color",
		-1, // "color-rendering",
		ANIM_CX,
		ANIM_CY,
		ANIM_D,
		-1, // "delta",
		ANIM_DISPLAY,
		ANIM_DISPLAYALIGN,
		-1, // "dur",
		-1, // "editable",
		-1, // "enabled",
		-1, // "end",
		-1, // "event",
		-1, // "externalResourcesRequired",
		ANIM_FILL, 
		ANIM_FILLOPACITY,
		ANIM_FILLRULE,
		ANIM_FOCUSABLE,
		ANIM_FONTFAMILY,// 30
		ANIM_FONTSIZE,
		ANIM_FONTSTYLE,
		-1, // "font-variant",
		-1, // "font-weight",
		-1, // "fullscreen",
		-1, // "gradientUnits",
		-1, // "handler",
		ANIM_HEIGHT,
		-1, // "image-rendering",
		-1, // "keyPoints",
		-1, // "keySplines",
		-1, // "keyTimes",
		-1, // "line-increment",
		-1, // "listener.target",
		-1, // "mediaCharacterEncoding",
		-1, // "mediaContentEncodings",
		-1, // "mediaSize",
		-1, // "mediaTime",
		-1, // "nav-down",
		-1, // "nav-down-left",
		-1, // "nav-down-right",
		-1, // "nav-left",
		-1, // "nav-next",
		-1, // "nav-prev",
		-1, // "nav-right",
		-1, // "nav-up",
		-1, // "nav-up-left",
		-1, // "nav-up-right",
		-1, // "observer",
		-1, // "offset",
		-1, // "opacity",
		-1, // "overflow",
		-1, // "overlay",
		-1, // "path",
		-1, // "pathLength",
		-1, // "pointer-events",
		ANIM_POINTS,
		-1, // "preserveAspectRatio",
		ANIM_R,
		-1, // "repeatCount",
		-1, // "repeatDur",
		-1, // "requiredExtensions",
		-1, // "requiredFeatures",
		-1, // "requiredFormats",
		-1, // "restart",
		-1, // "rotate",
		-1, // "rotation",
		ANIM_RX,
		ANIM_RY,
		-1, // "scale",
		-1, // "shape-rendering",
		ANIM_SIZE,
		-1, // "solid-color",
		-1, // "solid-opacity",
		-1, // "stop-color",
		-1, // "stop-opacity",
		ANIM_STROKE,
		-1, // "stroke-dasharray",
		-1, // "stroke-dashoffset",
		-1, // "stroke-linecap",
		-1, // "stroke-linejoin",
		-1, // "stroke-miterlimit",
		ANIM_STROKEOPACITY,
		ANIM_STROKEWIDTH,
		-1, // "svg.height",
		-1, // "svg.width",
		-1, // "syncBehavior",
		-1, // "syncBehaviorDefault",
		-1, // "syncReference",
		-1, // "syncTolerance",
		-1, // "syncToleranceDefault",
		-1, // "systemLanguage",
		-1, // "text-align",
		-1, // "text-anchor",
		-1, // "text-decoration",
		-1, // "text-rendering",
		-1, // "textContent",
		ANIM_TRANSFORM,
		-1, // "transformBehavior",
		-1, // "translation",
		-1, // "vector-effect",
		ANIM_VIEWBOX,
		-1, // "viewport-fill",
		-1, // "viewport-fill-opacity",
		ANIM_VISIBILITY,
		ANIM_WIDTH,
		ANIM_X,
		ANIM_X1,
		ANIM_X2,
		-1, // "xlink:actuate",
		-1, // "xlink:arcrole",
		ANIM_HREF,
		-1, // "xlink:role",
		-1, // "xlink:show",
		-1, // "xlink:title",
		-1, // "xlink:type",
		-1, // "xml:base",
		-1, // "xml:lang",
		ANIM_Y,
		ANIM_Y1,                  
		ANIM_Y2,
		-1 // "zoomAndPan"
	};
	
	public final static int animType[] = new int[] {
		-1,
		-1,
		-1,
	    ANIMTYPE_float, //cx
	    ANIMTYPE_float, //cy
	    ANIMTYPE_path, //d
	    ANIMTYPE_enum, //display
	    ANIMTYPE_enum, //display-align
	    ANIMTYPE_enum, //editable
	    ANIMTYPE_color, //fill
	    ANIMTYPE_fraction, //fill-opacity
	    ANIMTYPE_enum, //fill-rule
	    ANIMTYPE_id, //focusEast
	    ANIMTYPE_id, //focusNorth
	    ANIMTYPE_id, //focusNorthEast
	    ANIMTYPE_id, //focusNorthWest
	    ANIMTYPE_id, //focusSouth
	    ANIMTYPE_id, //focusSouthEast
	    ANIMTYPE_id, //focusSouthWest
	    ANIMTYPE_id, //focusWest
	    ANIMTYPE_id, //focusable
	    ANIMTYPE_font, //font-family
	    ANIMTYPE_float, //font-size
	    ANIMTYPE_enum, //font-style
	    ANIMTYPE_enum, //font-weight
	    ANIMTYPE_enum, //gradientUnits
	    ANIMTYPE_float, //height
	    ANIMTYPE_enum, //image-rendering
	    ANIMTYPE_float, //pathLength
	    ANIMTYPE_ints, //pointer-events
	    ANIMTYPE_pointSeq, //points
	    ANIMTYPE_enum, //preserveAspectRatio
	    ANIMTYPE_float, //r
	    ANIMTYPE_float, //rotate
	    ANIMTYPE_float, //rx
	    ANIMTYPE_float, //ry
	    ANIMTYPE_enum, //shape-rendering
	    ANIMTYPE_point, //size
	    ANIMTYPE_color, //stop-color
	    ANIMTYPE_fraction, //stop-opacity
	    ANIMTYPE_color, //stroke
	    ANIMTYPE_floats, //stroke-dasharray
	    ANIMTYPE_float, //stroke-dashoffset
	    ANIMTYPE_enum, //stroke-linecap
	    ANIMTYPE_enum, //stroke-linejoin
	    ANIMTYPE_float, //stroke-miterlimit
	    ANIMTYPE_fraction, //stroke-opacity
	    ANIMTYPE_float, //stroke-width
	    ANIMTYPE_string, //target
	    ANIMTYPE_enum, //text-anchor
	    ANIMTYPE_floats, //transform
	    ANIMTYPE_enum, //type
	    ANIMTYPE_enum, //vector-effect
	    ANIMTYPE_floats, //viewBox
	    ANIMTYPE_color, //viewport-fill
	    ANIMTYPE_fraction, //viewport-fill-opacity
	    ANIMTYPE_enum, //visibility
	    ANIMTYPE_float, //width
	    ANIMTYPE_float, //x
	    ANIMTYPE_float, //x1
	    ANIMTYPE_float, //x2
	    ANIMTYPE_uri, //xlink:href
	    ANIMTYPE_float, //y
	    ANIMTYPE_float, //y1
	    ANIMTYPE_float //y2	
/*		ANIMTYPE_fraction, //audio-level
        ANIMTYPE_enum, //choice
        ANIMTYPE_color, //color
	    ANIMTYPE_enum, // color-rendering
	    ANIMTYPE_float, //cx
	    ANIMTYPE_float, //cy
	    ANIMTYPE_path, //d
	    ANIMTYPE_enum, //display
	    ANIMTYPE_enum, //display-align
	    ANIMTYPE_enum, //editable
	    ANIMTYPE_color, //fill
	    ANIMTYPE_fraction, //fill-opacity
	    ANIMTYPE_enum, //fill-rule
	    ANIMTYPE_id, //focusEast
	    ANIMTYPE_id, //focusNorth
	    ANIMTYPE_id, //focusNorthEast
	    ANIMTYPE_id, //focusNorthWest
	    ANIMTYPE_id, //focusSouth
	    ANIMTYPE_id, //focusSouthEast
	    ANIMTYPE_id, //focusSouthWest
	    ANIMTYPE_id, //focusWest
	    ANIMTYPE_id, //focusable
	    ANIMTYPE_font, //font-family
	    ANIMTYPE_float, //font-size
	    ANIMTYPE_enum, //font-style
	    ANIMTYPE_enum, //font-weight
	    ANIMTYPE_enum, //gradientUnits
	    ANIMTYPE_float, //height
	    ANIMTYPE_enum, //image-rendering
	    ANIMTYPE_float, //pathLength
	    ANIMTYPE_ints, //pointer-events
	    ANIMTYPE_pointSeq, //points
	    ANIMTYPE_enum, //preserveAspectRatio
	    ANIMTYPE_float, //r
	    ANIMTYPE_float, //rotate
	    ANIMTYPE_float, //rx
	    ANIMTYPE_float, //ry
	    ANIMTYPE_enum, //shape-rendering
	    ANIMTYPE_point, //size
	    ANIMTYPE_color, //stop-color
	    ANIMTYPE_fraction, //stop-opacity
	    ANIMTYPE_color, //stroke
	    ANIMTYPE_floats, //stroke-dasharray
	    ANIMTYPE_float, //stroke-dashoffset
	    ANIMTYPE_enum, //stroke-linecap
	    ANIMTYPE_enum, //stroke-linejoin
	    ANIMTYPE_float, //stroke-miterlimit
	    ANIMTYPE_fraction, //stroke-opacity
	    ANIMTYPE_float, //stroke-width
	    ANIMTYPE_string, //target
	    ANIMTYPE_enum, //text-anchor
	    ANIMTYPE_floats, //transform
	    ANIMTYPE_enum, //type
	    ANIMTYPE_enum, //vector-effect
	    ANIMTYPE_floats, //viewBox
	    ANIMTYPE_color, //viewport-fill
	    ANIMTYPE_fraction, //viewport-fill-opacity
	    ANIMTYPE_enum, //visibility
	    ANIMTYPE_float, //width
	    ANIMTYPE_float, //x
	    ANIMTYPE_float, //x1
	    ANIMTYPE_float, //x2
	    ANIMTYPE_uri, //xlink:href
	    ANIMTYPE_float, //y
	    ANIMTYPE_float, //y1
	    ANIMTYPE_float //y2	*/
	};
	
    // updateblaAttributeIndex constants
	
	public final static char UPDATE_ATARGET = 0;
	public final static char UPDATE_ACCUMULATE = 1;
	public final static char UPDATE_ADDITIVE = 2;
	public final static char UPDATE_AUDIOLEVEL = 3;
	public final static char UPDATE_BANDWIDTH = 4;
	public final static char UPDATE_BEGIN = 5;
	public final static char UPDATE_CALCMODE = 6;
	public final static char UPDATE_CHILDREN = 7;
	public final static char UPDATE_CHOICE = 8;
	public final static char UPDATE_CLIPBEGIN = 9;
	public final static char UPDATE_CLIPEND = 10;
	public final static char UPDATE_COLOR = 11;
	public final static char UPDATE_COLORRENDERING = 12;
	public final static char UPDATE_CX = 13;
	public final static char UPDATE_CY = 14;
	public final static char UPDATE_D = 15;
	public final static char UPDATE_DELAY = 16;
	public final static char UPDATE_DISPLAY = 17;
	public final static char UPDATE_DISPLAYALIGN = 18;
	public final static char UPDATE_DUR = 19;
	public final static char UPDATE_EDITABLE = 20;
	public final static char UPDATE_ENABLED = 21;
	public final static char UPDATE_END = 22;
	public final static char UPDATE_EVENT = 23;
	public final static char UPDATE_EXTERNALRESOURCESREQUIRED = 24;
	public final static char UPDATE_COLOR_FILL = 25;
	public final static char UPDATE_FILLOPACITY = 26;
	public final static char UPDATE_FILLRULE = 27;
	public final static char UPDATE_FOCUSABLE = 28;
	public final static char UPDATE_FONTFAMILY = 29;
	public final static char UPDATE_FONTSIZE = 30;
	public final static char UPDATE_FONTSTYLE = 31;
	public final static char UPDATE_FONTVARIANT = 32;
	public final static char UPDATE_FONTWEIGHT = 33;
	public final static char UPDATE_FULLSCREEN = 34;
	public final static char UPDATE_GRADIENTUNITS = 35;
	public final static char UPDATE_HANDLER = 36;
	public final static char UPDATE_HEIGHT = 37;
	public final static char UPDATE_IMAGERENDERING = 38;
	public final static char UPDATE_KEYPOINTS = 39;
	public final static char UPDATE_KEYSPLINES = 40;
	public final static char UPDATE_KEYTIMES = 41;
	public final static char UPDATE_LINEINCREMENT = 42;
	public final static char UPDATE_LISTENERTARGET = 43;
	public final static char UPDATE_MEDIACHARACTERENCODING = 44;
	public final static char UPDATE_MEDIACONTENTENCODINGS = 45;
	public final static char UPDATE_MEDIASIZE = 46;
	public final static char UPDATE_MEDIATIME = 47;
	public final static char UPDATE_FOCUSSOUTH = 48;
	public final static char UPDATE_FOCUSSOUTHWEST = 49;
	public final static char UPDATE_FOCUSSOUTHEAST = 50;
	public final static char UPDATE_FOCUSWEST = 51;
	public final static char UPDATE_FOCUSNEXT = 52;
	public final static char UPDATE_FOCUSPREV = 53;
	public final static char UPDATE_FOCUSEAST = 54;
	public final static char UPDATE_FOCUSNORTH = 55;
	public final static char UPDATE_FOCUSNORTHWEST = 56;
	public final static char UPDATE_FOCUSNORTHEAST = 57;
	public final static char UPDATE_OBSERVER = 58;
	public final static char UPDATE_OFFSET = 59;
	public final static char UPDATE_OPACITY = 60;
	public final static char UPDATE_OVERFLOW = 61;
	public final static char UPDATE_OVERLAY = 62;
	public final static char UPDATE_PATH = 63;
	public final static char UPDATE_PATHLENGTH = 64;
	public final static char UPDATE_POINTEREVENTS = 65;
	public final static char UPDATE_POINTS = 66;
	public final static char UPDATE_PRESERVEASPECTRATIO = 67;
	public final static char UPDATE_R = 68;
	public final static char UPDATE_REPEATCOUNT = 69;
	public final static char UPDATE_REPEATDUR = 70;
	public final static char UPDATE_REQUIREDEXTENSIONS = 71;
	public final static char UPDATE_REQUIREDFEATURES = 72;
	public final static char UPDATE_REQUIREDFORMATS = 73;
	public final static char UPDATE_RESTART = 74;
	public final static char UPDATE_ROTATE = 75;
	public final static char UPDATE_ROTATION = 76;
	public final static char UPDATE_RX = 77;
	public final static char UPDATE_RY = 78;
	public final static char UPDATE_SCALE = 79;
	public final static char UPDATE_SHAPERENDERING = 80;
	public final static char UPDATE_SIZE = 81;
	public final static char UPDATE_SOLIDCOLOR = 82;
	public final static char UPDATE_SOLIDOPACITY = 83;
	public final static char UPDATE_STOPCOLOR = 84;
	public final static char UPDATE_STOPOPACITY = 85;
	public final static char UPDATE_STROKE = 86;
	public final static char UPDATE_STROKEDASHARRAY = 87;
	public final static char UPDATE_STROKEDASHOFFSET = 88;
	public final static char UPDATE_STROKELINECAP = 89;
	public final static char UPDATE_STROKELINEJOIN = 90;
	public final static char UPDATE_STROKEMITERLIMIT = 91;
	public final static char UPDATE_STROKEOPACITY = 92;
	public final static char UPDATE_STROKEWIDTH = 93;
	public final static char UPDATE_SVGHEIGHT = 94;
	public final static char UPDATE_SVGWIDTH = 95;
	public final static char UPDATE_SYNCBEHAVIOR = 96;
	public final static char UPDATE_SYNCBEHAVIORDEFAULT = 97;
	public final static char UPDATE_SYNCREFERENCE = 98;
	public final static char UPDATE_SYNCTOLERANCE = 99;
	public final static char UPDATE_SYNCTOLERANCEDEFAULT = 100;
	public final static char UPDATE_SYSTEMLANGUAGE = 101;
	public final static char UPDATE_TEXTALIGH = 102;
	public final static char UPDATE_TEXTANCHOR = 103;
	public final static char UPDATE_TEXTDECORATION = 104;
	public final static char UPDATE_TEXTRENDERING = 105;
	public final static char UPDATE_TEXTCONTENT = 106;
	public final static char UPDATE_TRANSFORM = 107;
	public final static char UPDATE_TRANSFORMBEHAVIOR = 108;
	public final static char UPDATE_TRANSLATION = 109;
	public final static char UPDATE_VECTOREFFECT = 110;
	public final static char UPDATE_VIEWBOX = 111;
	public final static char UPDATE_VIEWPORTFILL = 112;
	public final static char UPDATE_VIEWPORTFILLOPACITY = 113;
	public final static char UPDATE_VISIBILITY = 114;
	public final static char UPDATE_WIDTH = 115;
	public final static char UPDATE_X = 116;
	public final static char UPDATE_X1 = 117;
	public final static char UPDATE_X2 = 118;
	public final static char UPDATE_XLINK_ACTUATE = 119;
	public final static char UPDATE_XLINK_ARCROLE = 120;
	public final static char UPDATE_XLINK_HREF = 121;
	public final static char UPDATE_XLINK_ROLE = 122;
	public final static char UPDATE_XLINK_SHOW = 123;
	public final static char UPDATE_XLINK_TITLE = 124;
	public final static char UPDATE_XLINK_TYPE = 125;
	public final static char UPDATE_XML_BASE = 126;
	public final static char UPDATE_XML_LANG = 127;
	public final static char UPDATE_Y = 128;
	public final static char UPDATE_Y1 = 129;
	public final static char UPDATE_Y2 = 130;
	public final static char UPDATE_ZOOMANDPAN = 131;
	
	
/*	
	public final static char UPDATE_ANIMFILL = 3;
	public final static char UPDATE_APPEND = 4;
	public final static char UPDATE_ATTRIBUTENAME = 4;
	public final static char UPDATE_BY = 8;
	public final static char UPDATE_KEYCODES = 39;
	public final static char UPDATE_FROM = 45;	
	public final static char UPDATE_TIMEATTRIBUTE = 103;
	public final static char UPDATE_TO = 104;
	public final static char UPDATE_TYPE = 108;
	public final static char UPDATE_VALUES = 109;
	public final static char UPDATE_XML_SPACE = 128;
*/	
	
// UpdateValue constants
/*  TML : TODO : remove after verification 
    public static final int ITYPE_float = 102;
	public static final int ITYPE_int = 101;
	public static final int ITYPE_point = 100;
	public static final int ITYPE_time = 103;
*/
	public static final int ITYPE_point = 100;
	public static final int ITYPE_int = 101;
	public static final int ITYPE_float = 102;
	public static final int ITYPE_keyTime = 103;
	public static final int ITYPE_0to1 = 104;
	public static final int ITYPE_smil_time = 105;

	
	public static final int TYPE_boolean = 0;
	public static final int TYPE_enum = 1;
	public static final int TYPE_color = 2;
	public static final int TYPE_fraction = 3;
	public static final int TYPE_float = 4;
	public static final int TYPE_time = 5;
	public static final int TYPE_point = 6;
	public static final int TYPE_matrix = 7;
	public static final int TYPE_string = 8;
	public static final int TYPE_points = 9;
	public static final int TYPE_path = 10;
	public static final int TYPE_ints = 11;
	public static final int TYPE_floats = 12;
	public static final int TYPE_smil_times = 13;
	public static final int TYPE_unit = 14;
	public static final int TYPE_index = 15;
	public static final int TYPE_URI = 16;
	public static final int TYPE_ID = 17;
	public static final int TYPE_event = 18;
	public static final int TYPE_scale = 19;
	public static final int TYPE_coordinate = 20;
	public static final int TYPE_keyTimes = 21;



	public static final byte updateTypeOfAttribute[] = {
		TYPE_string, // a.target
        TYPE_boolean, // accumulate
        TYPE_boolean, // additive
        TYPE_fraction, // audio-level
        TYPE_float, // bandwidth
        TYPE_smil_times, // begin
        TYPE_enum, // calcMode
        -1, // children
        TYPE_index, // choice
        TYPE_time, // clipBegin
        TYPE_time, // clipEnd
        TYPE_color, // color
        TYPE_enum, // color-rendering
        TYPE_coordinate, // cx
        TYPE_coordinate, // cy
        TYPE_path, // d
        TYPE_floats, //delta
        TYPE_enum, // display
        TYPE_enum, // display-align
        TYPE_time, // dur
        TYPE_boolean, // editable
        TYPE_boolean, // enabled
        TYPE_smil_times, // end
        TYPE_event, // event
        TYPE_boolean, // externalResourcesRequired
        TYPE_color, // fill
        TYPE_fraction, // fill-opacity
        TYPE_enum, // fill-rule
        TYPE_enum, // focusable
        TYPE_index, // font-family
        TYPE_float, // font-size
        TYPE_enum, // font-style
        TYPE_enum, // font-variant
        TYPE_enum, // font-weight
        TYPE_boolean, // fullscreen
        TYPE_enum, // gradient-units
        TYPE_ID, // handler
        TYPE_coordinate, // height
        TYPE_enum, // image-rendering
        TYPE_floats, // keyPoints
        TYPE_floats, // keySplines
        TYPE_keyTimes, // keyTimes
        TYPE_float, // line-increment
        TYPE_ID, // listener.target
        TYPE_string, // mediaCharacterEncoding
        TYPE_string, // mediaContentEncodings
        TYPE_float, // mediaSize
        TYPE_time, // mediaTime
        TYPE_ID, // nav-down
        TYPE_ID, // nav-down-left
        TYPE_ID, // nav-down-right
        TYPE_ID, // nav-left
        TYPE_ID, // nav-next
        TYPE_ID, // nav-prev
        TYPE_ID, // nav-right
        TYPE_ID, // nav-up
        TYPE_ID, // nav-up-left
        TYPE_ID, // nav-up-right
        TYPE_ID, // observer
        TYPE_fraction, // offset
        TYPE_fraction, // opacity
        TYPE_enum, // overflow
        TYPE_enum, // overlay
        TYPE_path, // path
        TYPE_float, // pathLength
        TYPE_enum, // pointer-events
        TYPE_points, // points
        -1, // preserveAspectRatio
        TYPE_coordinate, // r
        TYPE_float, // repeatCount
        TYPE_time, // repeatDur
        TYPE_string, // requiredExtensions
        TYPE_ints, // requiredFeatures
        TYPE_string, // requiredFormats
        TYPE_enum, // restart
        TYPE_floats, // rotate
        TYPE_float, // rotation
        TYPE_coordinate, // rx
        TYPE_coordinate, // ry
        TYPE_scale, // scale
        TYPE_enum, // shape-rendering
        TYPE_point, // size
        TYPE_color, // solid-color
        TYPE_fraction, // solid-opacity
        TYPE_color, // stop-color
        TYPE_fraction, // stop-opacity
        TYPE_color, // stroke
        TYPE_floats, // stroke-dasharray
        TYPE_float, // stroke-dashoffset
        TYPE_enum, // stroke-linecap
        TYPE_enum, // stroke-linejoin
        TYPE_float, // stroke-miterlimit
        TYPE_fraction, // stroke-opacity
        TYPE_float, // stroke-width
        TYPE_unit, // svg.height
        TYPE_unit, // svg.width
        TYPE_enum, // syncBehavior
        TYPE_enum, // syncBehaviorDefault
        TYPE_URI, // syncReference
        TYPE_float, // syncTolerance
        TYPE_float, // syncToleranceDefault
        TYPE_string, // systemLanguage
        TYPE_enum, // text-align
        TYPE_enum, // text-anchor
        TYPE_enum, // text-decoration
        TYPE_enum, // text-rendering
        TYPE_string, // textContent
        TYPE_matrix, // transform
        TYPE_enum, // transformBehavior
        TYPE_point, // translation
        TYPE_enum, // vector-effect,
        TYPE_floats, // viewBox
        TYPE_color, // viewport-fill
        TYPE_fraction, // viewport-fill-opacity
        TYPE_enum, // visibility
        TYPE_coordinate, // width
        TYPE_coordinate, // x
        TYPE_coordinate, // x1
        TYPE_coordinate, // x2
        TYPE_enum, // xlink:actuate
        TYPE_string, // xlink:arcrole
        TYPE_URI, // xlink:href
        TYPE_string, // xlink:role
        TYPE_enum, // xlink:show
        TYPE_string, // xlink:title
        TYPE_enum, // xlink:type
        TYPE_string, // xml:base
        TYPE_string, // xml:lang
        TYPE_coordinate, // y
        TYPE_coordinate, // y1
        TYPE_coordinate, // y2
        TYPE_boolean // zoomAndPan
	};

	public static final byte indexedTypeOfAttribute[] = {
		-1, // a.target
        -1, // accumulate
        -1, // additive
        -1, // audio-level
        -1, // bandwidth
        ITYPE_smil_time, // begin
        -1, // calcMode
        -1, // children
        -1, // choice
        -1, // clipBegin
        -1, // clipEnd
        -1, // color
        -1, // color-rendering
        -1, // cx
        -1, // cy
        -1, // d
        -1, // delta
        -1, // display
        -1, // display-align
        -1, // dur
        -1, // editable
        -1, // enabled
        ITYPE_smil_time, // end
        -1, // event
        -1, // externalResourcesRequired
        -1, // fill
        -1, // fill-opacity
        -1, // fill-rule
        -1, // focusable
        -1, // font-family
        -1, // font-size
        -1, // font-style
        -1, // font-variant
        -1, // font-weight
        -1, // fullscreen
        -1, // gradient-units
        -1, // handler
        -1, // height
        -1, // image-rendering
        ITYPE_0to1, // keyPoints
        ITYPE_float, // keySplines
        ITYPE_keyTime, // keyTimes
        -1, // line-increment
        -1, // listener.target
        -1, // mediaCharacterEncoding
        -1, // mediaContentEncodings
        -1, // mediaSize
        -1, // mediaTime
        -1, // nav-down
        -1, // nav-down-left
        -1, // nav-down-right
        -1, // nav-left
        -1, // nav-next
        -1, // nav-prev
        -1, // nav-right
        -1, // nav-up
        -1, // nav-up-left
        -1, // nav-up-right
        -1, // observer
        -1, // offset
        -1, // opacity
        -1, // overflow
        -1, // overlay
        -1, // path
        -1, // pathLength
        -1, // pointer-events
        ITYPE_point, // points
        -1, // preserveAspectRatio
        -1, // r
        -1, // repeatCount
        -1, // repeatDur
        -1, // requiredExtensions
        ITYPE_int, // requiredFeatures
        -1, // requiredFormats
        -1, // restart
        -1, // rotate
        -1, // rotation
        -1, // rx
        -1, // ry
        -1, // scale
        -1, // shape-rendering
        -1, // size
        -1, // solid-color
        -1, // solid-opacity
        -1, // stop-color
        -1, // stop-opacity
        -1, // stroke
        ITYPE_float, // stroke-dasharray
        -1, // stroke-dashoffset
        -1, // stroke-linecap
        -1, // stroke-linejoin
        -1, // stroke-miterlimit
        -1, // stroke-opacity
        -1, // stroke-width
        -1, // svg.height
        -1, // svg.width
        -1, // syncBehavior
        -1, // syncBehaviorDefault
        -1, // syncReference
        -1, // syncTolerance
        -1, // syncToleranceDefault
        -1, // systemLanguage
        -1, // text-align
        -1, // text-anchor
        -1, // text-decoration
        -1, // text-rendering
        -1, // textContent
        -1, // transform
        -1, // transformBehavior
        -1, // translation
        -1, // vector-effect
        ITYPE_float, // viewBox
        -1, // viewport-fill
        -1, // viewport-fill-opacity
        -1, // visibility
        -1, // width
        -1, // x
        -1, // x1
        -1, // x2
        -1, // xlink:actuate
        -1, // xlink:arcrole
        -1, // xlink:href
        -1, // xlink:role
        -1, // xlink:show
        -1, // xlink:title
        -1, // xlink:type
        -1, // xml:base
        -1, // xml:lang
        -1, // y
        -1, // y1
        -1, // y2
        -1 // zoomAndPan
	};

	/** No color */
    public final static int COLOR_NONE = -2;
    /** No stroke */
    public final static int STROKE_NONE = -2;

    /** No index*/
    public static final int NO_INDEX = -2;
    public static final int END_INDEX = -1;


    
    // Value types
    public static final byte UNKNOWN = -1;
    public static final byte BOOLEAN = 0;
    public static final byte INT3 = 1;
    public static final byte INT8 = 2;
    public static final byte ID = 3;
    public static final byte COLOR = 4;
    public static final byte OPACITY = 5;
    public static final byte FLOAT = 6;
    public static final byte TIME = 7;
    public static final byte POINT = 8;
    public static final byte MATRIX = 9;
    public static final byte STRING = 10;
    public static final byte POINTS = 11;
    public static final byte PATH = 12;
    public static final byte VIEWBOX = 13;
    public static final byte INTS = 14;
    public static final byte FLOATS = 15;
    public static final byte TIMES = 16;
    public static final byte ELEMENT = 17;
    public static final byte TEXTCONTENT = 18;

    
    /**
     * @param type an ANIMTYPE_* constant
     * @return the corresponding universal type constant
     */
    public static byte convertTypeFromAnimValue(int type) {
        switch (type) {
        case Value.ANIMTYPE_path:
        	return Value.PATH;
        case Value.ANIMTYPE_pointSeq:
        	return Value.POINTS;
        case Value.ANIMTYPE_fraction:
            return Value.OPACITY;
        case Value.ANIMTYPE_color:
        	return Value.COLOR;
        case Value.ANIMTYPE_float:
        	return Value.FLOAT;
        case Value.ANIMTYPE_enum:
        	return Value.INT3;
        case Value.ANIMTYPE_id:
        	return Value.ID;
        case Value.ANIMTYPE_font:
        	return Value.STRING;
        case Value.ANIMTYPE_string:
        	return Value.STRING;
        case Value.ANIMTYPE_floats:
            return Value.FLOATS;
        case Value.ANIMTYPE_ints:
        	return Value.INTS;
        case Value.ANIMTYPE_point:
            return Value.POINT;
        case Value.ANIM_HREF:
        	return Value.ID; // TML : TODO : NOT the right return value
        default:
            return Value.UNKNOWN;
        }
    }

    /**
     * Check if the attribute is discrete and cannot be interpolated
     * @param attributeType
     * @return
     */
    public static boolean isDiscreteAttribute(byte attributeType)
    {
        if ((attributeType >= NodeProperties.FIRST_STRING &&
             attributeType <= NodeProperties.LAST_STRING) ||
            (attributeType >= NodeProperties.FIRST_ID &&
             attributeType <= NodeProperties.LAST_ID))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public static byte getAttributeType(byte attribute)
    {
        if (attribute <= NodeProperties.LAST_BOOLEAN)
        {
            return BOOLEAN;
        }
        else if (attribute <= NodeProperties.LAST_INT3)
        {
            return INT3;
        }
        else if (attribute <= NodeProperties.LAST_INT8)
        {
            return INT8;
        }
        else if (attribute <= NodeProperties.LAST_ID)
        {
            return ID;
        }
        else if (attribute <= NodeProperties.LAST_COLOR)
        {
            return COLOR;
        }
        else if (attribute <= NodeProperties.LAST_OPACITY)
        {
            return OPACITY;
        }
        else if (attribute <= NodeProperties.LAST_FLOAT)
        {
            return FLOAT;
        }
        else if (attribute <= NodeProperties.LAST_TIME)
        {
            return TIME;
        }
        else if (attribute <= NodeProperties.LAST_POINT)
        {
            return POINT;
        }
        else if (attribute <= NodeProperties.LAST_MATRIX)
        {
            return MATRIX;
        }
        else if (attribute <= NodeProperties.LAST_STRING)
        {
            return STRING;
        }
        else if (attribute <= NodeProperties.LAST_POINTS)
        {
            return POINTS;
        }
        else if (attribute <= NodeProperties.LAST_PATH)
        {
            return PATH;
        }
        else if (attribute <= NodeProperties.LAST_VIEWBOX)
        {
            return VIEWBOX;
        }
        else if (attribute <= NodeProperties.LAST_INTS)
        {
            return INTS;
        }
        else if (attribute <= NodeProperties.LAST_FLOATS)
        {
            return FLOATS;
        }
        else if (attribute <= NodeProperties.LAST_TIMES)
        {
            return TIMES;
        }
        else if (attribute <= NodeProperties.LAST_ELEMENT)
        {
            return ELEMENT;
        }
        else if (attribute <= NodeProperties.LAST_TEXTCONTENT)
        {
            return TEXTCONTENT;
        }
        else if (attribute <= NodeProperties.LAST_FLOATS2)
        {
            return FLOATS;
        }

        return -1;
    }

    /**
     * Adds a value to this value of same type
     * The result of the addition is same as this if addition cannot be performed
     * @param valueToAdd
     * @return The result of addition
     */
    public Value Add(Value valueToAdd, byte attributeType)
    {
        // Check types
        Value valueToReturn = this;
        switch (attributeType)
        {
            case COLOR:
                int redToAdd = ((valueToAdd.m_enumOrColorOrRealOrID >> 16) & 255);
                int greenToAdd = ((valueToAdd.m_enumOrColorOrRealOrID >> 8) & 255);
                int blueToAdd = ((valueToAdd.m_enumOrColorOrRealOrID) & 255);
                int red = ((m_enumOrColorOrRealOrID >> 16) & 255);
                int green = ((m_enumOrColorOrRealOrID >> 8) & 255);
                int blue = ((m_enumOrColorOrRealOrID) & 255);
                red = red + redToAdd;
                red = (red > 255) ? 255 : red;
                red = (red < 0) ? 0 : red;
                green = green + greenToAdd;
                green = (green > 255) ? 255 : green;
                green = (green < 0) ? 0 : green;
                blue = blue + blueToAdd;
                blue = (blue > 255) ? 255 : blue;
                blue = (blue < 0) ? 0 : blue;
                valueToReturn = new Value();
                valueToReturn.m_enumOrColorOrRealOrID = (red << 16) + (green << 8) + blue;
                break;

            case FLOAT:
                valueToReturn = new Value();
                valueToReturn.m_enumOrColorOrRealOrID = m_enumOrColorOrRealOrID + valueToAdd.m_enumOrColorOrRealOrID;
                break;
                
            case OPACITY:
                valueToReturn = new Value();
                valueToReturn.m_enumOrColorOrRealOrID = m_enumOrColorOrRealOrID + valueToAdd.m_enumOrColorOrRealOrID;
                valueToReturn.m_enumOrColorOrRealOrID = (valueToReturn.m_enumOrColorOrRealOrID > 255) ? 255 : valueToReturn.m_enumOrColorOrRealOrID;
                valueToReturn.m_enumOrColorOrRealOrID = (valueToReturn.m_enumOrColorOrRealOrID < 0) ? 0 : valueToReturn.m_enumOrColorOrRealOrID;
                break;

            case POINT:
                valueToReturn = new Value();
                int rX, rY;
                rX = m_point.x_r + valueToAdd.m_point.x_r;
                rY = m_point.y_r + valueToAdd.m_point.y_r;
                valueToReturn.m_point = new Point2D(rX, rY);
                break;

            case PATH:
            case POINTS:
                if (valueToAdd.m_polygon == null) return null;
                if (valueToAdd.m_polygon.m_points == null) return null;
                if (valueToAdd.m_polygon.m_points.size() < 1) return null;

                // Init temp vars
                Vector valueToAddPoints = valueToAdd.m_polygon.m_points;
                Vector points = m_polygon.m_points;
                Point2D toAddPoint = null;
                Point2D point = null;
                Point2D newPoint = null;

                // Allocate a polygon as result
                valueToReturn = new Value();
                valueToReturn.m_polygon = new Polygon2D();

                // Types : same for down, up and result
                valueToReturn.m_types = valueToAdd.m_types;

                // Interpolate each point
                for (int i = 0; (i < valueToAddPoints.size()) && (i < points.size()); i++)
                {
                    // Get current pair of points
                    point = (Point2D)points.elementAt(i);
                    toAddPoint = (Point2D)valueToAddPoints.elementAt(i);

                    // Compute x et y
                    rX = point.x_r + toAddPoint.x_r;
                    rY = point.y_r + toAddPoint.y_r;

                    // Allocate a new point for result and attach it to result polygon
                    newPoint = new Point2D(rX, rY);
                    valueToReturn.m_polygon.m_points.addElement(newPoint);
                }
                break;
            case FLOATS:
            case VIEWBOX :
                if (this.m_floatsOrInts == null) return null;
                if (valueToAdd.m_floatsOrInts == null) return null;
                if (this.m_floatsOrInts.length != valueToAdd.m_floatsOrInts.length) return null;

                valueToReturn = new Value();
                valueToReturn.m_floatsOrInts = new int[this.m_floatsOrInts.length];
                for(int i=0;i<valueToReturn.m_floatsOrInts.length;i++)
                {
                    valueToReturn.m_floatsOrInts[i] = this.m_floatsOrInts[i] + valueToAdd.m_floatsOrInts[i];
                }
                break;

            // Cannot perform addition
            case INT3:
            case INT8:
            case ID:
            case STRING:
                valueToReturn = new Value();
                valueToReturn.m_string = this.m_string;
                valueToReturn.m_enumOrColorOrRealOrID = this.m_enumOrColorOrRealOrID;
                break;
        }
        return valueToReturn;
    }

    /**
     * Multiplies this value by a factor
     * The result of the multiplication is same as this if multiplication cannot be performed
     * @param factor
     * @return The result of multiplication
     */
    public Value Mul(int factor, byte attributeType)
    {
        // Check types
        Value valueToReturn = this;
        switch (attributeType)
        {
            case COLOR:
                int red = ((m_enumOrColorOrRealOrID >> 16) & 255);
                int green = ((m_enumOrColorOrRealOrID >> 8) & 255);
                int blue = ((m_enumOrColorOrRealOrID) & 255);
                red = red * factor;
                red = (red > 255) ? 255 : red;
                red = (red < 0) ? 0 : red;
                green = green * factor;
                green = (green > 255) ? 255 : green;
                green = (green < 0) ? 0 : green;
                blue = blue * factor;
                blue = (blue > 255) ? 255 : blue;
                blue = (blue < 0) ? 0 : blue;
                valueToReturn = new Value();
                valueToReturn.m_enumOrColorOrRealOrID = (red << 16) + (green << 8) + blue;
                break;

            case FLOAT:
                valueToReturn = new Value();
                valueToReturn.m_enumOrColorOrRealOrID = Real.applyMul(m_enumOrColorOrRealOrID,
                        Real.convertIntToRealInt(factor));
                break;

            case OPACITY:
                valueToReturn = new Value();
                valueToReturn.m_enumOrColorOrRealOrID = m_enumOrColorOrRealOrID * factor;
                valueToReturn.m_enumOrColorOrRealOrID = (valueToReturn.m_enumOrColorOrRealOrID > 255) ? 255 : valueToReturn.m_enumOrColorOrRealOrID;
                valueToReturn.m_enumOrColorOrRealOrID = (valueToReturn.m_enumOrColorOrRealOrID < 0) ? 0 : valueToReturn.m_enumOrColorOrRealOrID;
                break;

            case POINT:
                valueToReturn = new Value();
                int rX, rY;
                rX = Real.applyMul(m_point.x_r, Real.convertIntToRealInt(factor));
                rY = Real.applyMul(m_point.y_r, Real.convertIntToRealInt(factor));
                valueToReturn.m_point = new Point2D(rX, rY);
                break;

            case PATH:
            case POINTS:
                // Init temp vars
                Vector points = m_polygon.m_points;
                Point2D point = null;
                Point2D newPoint = null;

                // Allocate a polygon as result
                valueToReturn = new Value();
                valueToReturn.m_polygon = new Polygon2D();

                // Types : same for down, up and result
                valueToReturn.m_types = this.m_types;

                // Interpolate each point
                for (int i = 0; i < points.size(); i++)
                {
                    // Get current point
                    point = (Point2D)points.elementAt(i);

                    // Compute x et y
                    rX = Real.applyMul(point.x_r, Real.convertIntToRealInt(factor));
                    rY = Real.applyMul(point.y_r, Real.convertIntToRealInt(factor));

                    // Allocate a new point for result and attach it to result polygon
                    newPoint = new Point2D(rX, rY);
                    valueToReturn.m_polygon.m_points.addElement(newPoint);
                }
                break;
            case FLOATS:
            case VIEWBOX :
                if (this.m_floatsOrInts == null) return null;

                valueToReturn = new Value();
                valueToReturn.m_floatsOrInts = new int[this.m_floatsOrInts.length];
                for(int i=0;i<valueToReturn.m_floatsOrInts.length;i++)
                {
                    valueToReturn.m_floatsOrInts[i] = Real.applyMul(this.m_floatsOrInts[i], Real.convertIntToRealInt(factor));
                }
                break;

            // Cannot perform multiplication
            case INT3:
            case INT8:
            case ID:
            case STRING:
                valueToReturn = new Value();
                valueToReturn.m_string = this.m_string;
                valueToReturn.m_enumOrColorOrRealOrID = this.m_enumOrColorOrRealOrID;
                break;
        }

        return valueToReturn;
    }

    public static byte updateValueType(int attributeEnum, int index) {
	    if (index >= 0) {
	        // indexed
	        return Value.indexedTypeOfAttribute[attributeEnum];
	    } else {
	        // not indexed
	        return Value.updateTypeOfAttribute[attributeEnum];
	    }
	}

	/**
     * Performs a linear interpolation between the passed-in 2 values
     * @param downValue
     * @param upValue
     * @param rCoeff (20.12 fixed point)
     * @param attributeType
     * @return
     */
    public static Value linearInterpolation(Value downValue, Value upValue, int rCoeff, byte attributeType)
    {
        // Temp var declaration
        int rUp = 0;
        int rDown = 0;
        Vector downValuePoints = null;
        Vector upValuePoints = null;
        Point2D downPoint = null;
        Point2D upPoint = null;
        Point2D newPoint = null;
        int rX, rY;

        // Value to return
        Value computedValue = new Value();

        // Invert coeff
        int rInvCoeff = Real.VALUE1 - rCoeff;

        // Depends on attribute type
        switch (attributeType)
        {
            case COLOR:
                // Check color=none case : no interpolation
                if ( (downValue.m_enumOrColorOrRealOrID == COLOR_NONE) || (upValue.m_enumOrColorOrRealOrID == COLOR_NONE) )
                {
                    if (rCoeff > Real.div2(Real.VALUE1, 1))
                    {
                        return upValue;
                    }
                    else
                    {
                        return downValue;
                    }
                }

                // get r,g,b components
                int rRed = Real.convertIntToRealInt( (upValue.m_enumOrColorOrRealOrID >> 16) & 255);
                int rGreen = Real.convertIntToRealInt( (upValue.m_enumOrColorOrRealOrID >> 8) & 255);
                int rBlue = Real.convertIntToRealInt( (upValue.m_enumOrColorOrRealOrID) & 255);

                // Interpolate
                rRed = Real.applyMul(rRed, rCoeff);
                rGreen = Real.applyMul(rGreen, rCoeff);
                rBlue = Real.applyMul(rBlue, rCoeff);
                rRed = rRed + Real.applyMul( Real.convertIntToRealInt( (downValue.m_enumOrColorOrRealOrID >> 16) & 255), rInvCoeff);
                rGreen = rGreen + Real.applyMul( Real.convertIntToRealInt( (downValue.m_enumOrColorOrRealOrID >> 8) & 255), rInvCoeff);
                rBlue = rBlue + Real.applyMul( Real.convertIntToRealInt( (downValue.m_enumOrColorOrRealOrID) & 255), rInvCoeff);

                // convert real to int
                int red = Real.round(rRed);
                int green = Real.round(rGreen);
                int blue = Real.round(rBlue);
                red = (red > 255) ? 255 : red;
                red = (red < 0) ? 0 : red;
                green = (green > 255) ? 255 : green;
                green = (green < 0) ? 0 : green;
                blue = (blue > 255) ? 255 : blue;
                blue = (blue < 0) ? 0 : blue;

                // computed value
                computedValue.m_enumOrColorOrRealOrID = (red << 16) + (green << 8) + blue;
                return computedValue;

            case FLOAT:
                rUp = Real.applyMul(upValue.m_enumOrColorOrRealOrID, rCoeff);
                rDown = Real.applyMul(downValue.m_enumOrColorOrRealOrID, rInvCoeff);
                computedValue.m_enumOrColorOrRealOrID = rUp + rDown;
                return computedValue;

            case OPACITY:
                rUp = Real.applyMul(Real.convertIntToRealInt(upValue.m_enumOrColorOrRealOrID), rCoeff);
                rDown = Real.applyMul(Real.convertIntToRealInt(downValue.m_enumOrColorOrRealOrID), rInvCoeff);
                int result = Real.round(rUp + rDown);
                result = (result > 255) ? 255 : result;
                result = (result < 0) ? 0 : result;
                computedValue.m_enumOrColorOrRealOrID = result;
                return computedValue;

            case PATH:
                // Path data animation is only possible when each path data specification within an animation
                // specification has exactly the same list of path data commands as the d attribute. The animation
                // engine interpolates each parameter to each path data command separately.

                // Init temp vars
                downValuePoints = downValue.m_polygon.m_points;
                upValuePoints = upValue.m_polygon.m_points;

                // Allocate a polygon as result
                computedValue.m_polygon = new Polygon2D();

                // Types : same for down, up and result
                computedValue.m_types = downValue.m_types;

                // Interpolate each point
                for (int i = 0; (i < downValuePoints.size()) && (i < upValuePoints.size()); i++)
                {
                    // Get current pair of points
                    downPoint = (Point2D)downValuePoints.elementAt(i);
                    upPoint = (Point2D)upValuePoints.elementAt(i);

                    // Compute x et y
                    rX = Real.applyMul(upPoint.x_r, rCoeff) + Real.applyMul(downPoint.x_r, rInvCoeff);
                    rY = Real.applyMul(upPoint.y_r, rCoeff) + Real.applyMul(downPoint.y_r, rInvCoeff);

                    // Allocate a new point for result and attach it to result polygon
                    newPoint = new Point2D(rX, rY);
                    computedValue.m_polygon.m_points.addElement(newPoint);
                }
                return computedValue;

            case POINT:
                // Init temp vars
                downPoint = downValue.m_point;
                upPoint = upValue.m_point;

                // Compute x et y
                rX = Real.applyMul(upPoint.x_r, rCoeff) + Real.applyMul(downPoint.x_r, rInvCoeff);
                rY = Real.applyMul(upPoint.y_r, rCoeff) + Real.applyMul(downPoint.y_r, rInvCoeff);

                // Allocate a new point for result and attach it to result polygon
                computedValue.m_point = new Point2D(rX, rY);

                return computedValue;

            case POINTS:
                // Init temp vars
                downValuePoints = downValue.m_polygon.m_points;
                upValuePoints = upValue.m_polygon.m_points;

                // Allocate a polygon as result
                computedValue.m_polygon = new Polygon2D();

                // Types : same for down, up and result
                computedValue.m_types = downValue.m_types;

                // Interpolate each point
                for (int i = 0; (i < downValuePoints.size()) && (i < upValuePoints.size()); i++)
                {
                    // Get current pair of points
                    downPoint = (Point2D)downValuePoints.elementAt(i);
                    upPoint = (Point2D)upValuePoints.elementAt(i);

                    // Compute x et y
                    rX = Real.applyMul(upPoint.x_r, rCoeff) + Real.applyMul(downPoint.x_r, rInvCoeff);
                    rY = Real.applyMul(upPoint.y_r, rCoeff) + Real.applyMul(downPoint.y_r, rInvCoeff);

                    // Allocate a new point for result and attach it to result polygon
                    newPoint = new Point2D(rX, rY);
                    computedValue.m_polygon.m_points.addElement(newPoint);
                }
                return computedValue;

            case FLOATS:
            case VIEWBOX :
                // init tmp vars
                int[] downValueFloats = downValue.m_floatsOrInts;
                int[] upValueFloats = upValue.m_floatsOrInts;

                // allocate an array for the result
                computedValue.m_floatsOrInts = new int[downValueFloats.length];

                // Interpolate each point
                for (int i = 0; (i < downValueFloats.length) && (i < upValueFloats.length); i++)
                {
                    rUp = upValueFloats[i];
                    rDown = downValueFloats[i];

                    computedValue.m_floatsOrInts[i] = Real.applyMul(rUp, rCoeff) + Real.applyMul(rDown, rInvCoeff);
                }
                return computedValue;

            // Cannot perform linear interpolation
            case INT3:
            case INT8:
            case STRING:
            case ID:
                return (rCoeff < Real.div2(Real.VALUE1, 1)) ? downValue : upValue;
        }

        return null;
    }
}
