/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */

package iso.mpeg.streamezzo.player.node;

import iso.mpeg.streamezzo.player.library.Point2D;
import iso.mpeg.streamezzo.player.library.Real;
import iso.mpeg.streamezzo.player.tool.GraphObjectManager;

public class Video extends Multimedia
{
    //newly added attrs
	public Xlink xlink = new Xlink(); // related streamId
	public int dur = Integer.MIN_VALUE;
	public int transformBehavior = Integer.MIN_VALUE;
	public int syncBehavior = Integer.MIN_VALUE;
    public URI syncReference;
    public int syncTolerance = Integer.MIN_VALUE;
    public int audio_level = Integer.MIN_VALUE;


    public int m_rWidth = -1;             // DOM width
    private int m_rActualWidth;           // current width

    public int m_rHeight = -1;            // DOM height
    private int m_rActualHeight;          // current width

    public int m_rX = 0;             // DOM X coordinate
    private int m_rActualX;          // current X coordinate

    public int m_rY = 0;             // Y coordinate
    private int m_rActualY;          // current Y coordinate

    private Object m_oMedia = null;    // Device specific media
    private int[] m_oMediaArea = null; // Current media area set

    public Video(GraphObjectManager graphObjectManager)
    {
        super(graphObjectManager);
    }

    public int getVideoStreamId() {
    	if (xlink.getHref() != null)
    		return xlink.getHref().streamID;
    	else 
    		throw new RuntimeException("Null href, could not get video stream id");
    }

    public void specificRender(int sceneTimeInMillis, boolean parentHasChanged, GraphObject useContext)
    {
        switch (m_state)
        {
            case STATE_STARTPLAYING:
            {
                if (m_oMedia != null)
                {
                    // release previous media
                    m_graphObjectManager.m_renderer.cleanMedia(m_oMedia);
                    m_oMedia = null;
                    m_oMediaArea = null;
                }
                Object oStream = m_graphObjectManager.getSingleObject(getVideoStreamId());
                if ((oStream != null) && (oStream instanceof String))
                {
                    m_oMediaArea = calculateDisplayRect();
                    int[] oDisplayRect = new int[4];
                    System.arraycopy(m_oMediaArea, 0, oDisplayRect, 0, m_oMediaArea.length);
                    m_oMedia = m_graphObjectManager.m_renderer.createRemoteVideo((String)oStream, oDisplayRect, m_repeatCount);
                    m_graphObjectManager.m_renderer.playMedia(m_oMedia);
                }

                break;
            }

            case STATE_PLAYING:
            {
                if (! isStarted())
                {
                    if (m_graphObjectManager.m_renderer.isMediaStarted(m_oMedia))
                    {
                        long offset = m_graphObjectManager.m_renderer.getCurrentMediaTime(m_oMedia);
                        onStart(sceneTimeInMillis, offset == -1 ? -1 : (int)(offset / (long) 1000));
                    }
                }
                if (isDurationNeeded())
                {
                    long nDuration = m_graphObjectManager.m_renderer.getMediaDuration(m_oMedia);
                    if (nDuration != -1)
                    {
                        // round up
                        nDuration += 500;
                        // convert to milliseconds
                        onDurationAvailable(sceneTimeInMillis, (int)(nDuration / (long) 1000));
                    }
                }
                if ( m_graphObjectManager.m_renderer.isEndOfMedia(m_oMedia))
                {
                    onEnd(sceneTimeInMillis);
                }
                int[] oDisplayRect = calculateDisplayRect();
                // compare with current one and check if updates are needed
                if (oDisplayRect[0] != m_oMediaArea[0] || oDisplayRect[1] != m_oMediaArea[1])
                {
                    m_graphObjectManager.m_renderer.setVideoMediaLocation(m_oMedia, oDisplayRect[0], oDisplayRect[1]);
                }
                if (oDisplayRect[2] != m_oMediaArea[2] || oDisplayRect[3] != m_oMediaArea[3])
                {
                    m_graphObjectManager.m_renderer.setVideoMediaLocation(m_oMedia, oDisplayRect[2], oDisplayRect[3]);
                }
                // update current rect
                System.arraycopy(oDisplayRect, 0, m_oMediaArea, 0, oDisplayRect.length);
                break;
            }

            case STATE_ENDPLAYING:
            {
                if (m_oMedia != null)
                {
                    // release media
                    m_graphObjectManager.m_renderer.cleanMedia(m_oMedia);
                    m_oMedia = null;
                    m_oMediaArea = null;
                }
                break;
            }
        }
    }

    /**
     * This operation is responsible to reset presentation values with the DOM Values
     * Doesn't apply on base properties (stroke, fill, opacity, ...)
     */
    public void resetDOMValues()
    {
        // Set DOM values
        m_rActualX = m_rX;
        m_rActualY = m_rY;
        m_rActualWidth = m_rWidth;
        m_rActualHeight = m_rHeight;
    }


    /**
     * Calculates the current display rect
     * @return the display rect (x, y, width, height)
     */
    private int[] calculateDisplayRect()
    {
        // apply x/y translation
        Point2D tmp = m_actualTransform.apply(new Point2D(m_rActualX, m_rActualY));
        Point2D videoXY = new Point2D(tmp.x_r, tmp.y_r);
        tmp = m_actualTransform.apply(new Point2D(m_rActualWidth, m_rActualHeight));
        Point2D videoWH = new Point2D(tmp.x_r, tmp.y_r);

        // apply scale factor
        videoXY.x_r = Real.applyMul(videoXY.x_r, m_graphObjectManager.m_renderer.getXScaleFactor());
        videoXY.y_r = Real.applyMul(videoXY.y_r, m_graphObjectManager.m_renderer.getYScaleFactor());

        videoWH.x_r = Real.applyMul(videoWH.x_r, m_graphObjectManager.m_renderer.getXScaleFactor());
        videoWH.y_r = Real.applyMul(videoWH.y_r, m_graphObjectManager.m_renderer.getYScaleFactor());

        // generates calculated display rect and normalize
        int[] oDisplayRect = new int[4];
        oDisplayRect[0] = m_graphObjectManager.m_renderer.normalizeX(videoXY.x_r);
        oDisplayRect[1] = m_graphObjectManager.m_renderer.normalizeX(videoXY.y_r);
        oDisplayRect[2] = m_graphObjectManager.m_renderer.normalizeX(videoWH.x_r);
        oDisplayRect[3] = m_graphObjectManager.m_renderer.normalizeX(videoWH.y_r);

        return oDisplayRect;
    }
}