/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */

package iso.mpeg.streamezzo.player.tool;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.midlet.MIDlet;

/**
 */
public class LASeRPlayer extends MIDlet implements CommandListener
{
	private Display m_display = null;
	private Command m_exitCommand = new Command("Exit", Command.ITEM, 1);
	private Renderer renderer = null;
	private SceneLoader sceneLoader = null;
	private static boolean m_isLoaded = false;

	/*
	 * Construct a new MIDLET.
	 */
	public LASeRPlayer()
	{
		m_display = Display.getDisplay(this);
		renderer = new Renderer(m_display, this);
        GraphObjectManager manager = new GraphObjectManager();
		manager.setRenderer(renderer);
        sceneLoader = new LASeRBinParser(manager);
		renderer.setSceneLoader(sceneLoader);
		sceneLoader.init(null);
		renderer.setFontForGraphics(Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_PLAIN, Font.SIZE_MEDIUM));
	}

	public void startApp()
	{
		if (!m_isLoaded)
		{
            renderer.addCommand(m_exitCommand);
			renderer.setCommandListener(this);
			m_display.setCurrent(renderer);
		}
		m_isLoaded = true;
	}

	/**
	 * Time to pause
	 */
	public void pauseApp()
	{
		m_display.setCurrent(null);
	}

	/**
	 * Destroy must cleanup everything.
	 */
	public void destroyApp(boolean unconditional)
	{
		m_display.setCurrent(null);
	}

	public void commandAction(Command c, Displayable s)
	{
        if (c == m_exitCommand)
        {
            sceneLoader.reLoadScene("cmd://quit");
        }
	}
}
