/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */

package iso.mpeg.streamezzo.player.tool;

import iso.mpeg.streamezzo.player.library.BasicRenderer;

import java.io.DataInputStream;
import java.io.IOException;

import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;
import javax.microedition.io.StreamConnection;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.TextField;


/**
 * This class provides a higher level interface above the Canvas object and the
 * Graphics object. It is double-buffered and works on integer objects. The scene
 * to be rendered is a Group node that is the root node of the graph of
 * graphics objects.
 * The root node (hence the scene) is provided to the renderer via the default
 * constructor.
 * @see javax.microedition.lcdui.Graphics
 * @see iso.mpeg.streamezzo.player.node.Group
 */

public class Renderer extends BasicRenderer implements CommandListener, Rendering
{
    private Display m_display;
    private static LASeRPlayer callerClass;

    public static boolean isNotDisplay = false;
    private static boolean quitMainLoop = false;

    /**
     * TextInput management *
     */
    public Command m_okCommand = new Command("OK", Command.OK, 1);
    public TextField m_textBox;
    public Form m_textInputForm;
    public String m_textInput = null;

    /** The root of the scene */
    private SceneLoader sceneLoader = null;

////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////// Base operations ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Default constructor. The parameter is the root node of the scene.
     * @param display it is the Display shared with all MIDlet objects
     * @see iso.mpeg.streamezzo.player.node.RenderedGraphObject
     */
    public Renderer(Display display, LASeRPlayer aClass)
    {
        callerClass = aClass;
        m_display = display;

        m_imageX = 0;
        m_imageY = 0;

        //Trace will be removed during compilation
        if (Debug.DEBUG_TRACE)
        {
            System.out.println("Renderer Constructor: Canvas width= " + m_canvasWidth + " height= " + m_canvasHeight);
        }

        clearImage();
    }

    /**
     * Runs the animation and makes the repaint requests.
     * The thread will exit when it is no longer the current Animation thread.
     * @see iso.mpeg.streamezzo.player.node.GraphObject#render
     */
    public void run()
    {
        Thread mythread = Thread.currentThread();
        long startTime;
        long delta = 1000;

        startTime = System.currentTimeMillis();
        paintSystemTimeInMillis = startTime;
        long time = 0;

        //FOR FPS TEST
        Runtime runtime = Runtime.getRuntime();
        runtime.gc();
        long minMem = runtime.totalMemory();
        long maxMem = 0;
        long instantMen = -1;
        boolean bufferEmpty = true;
        boolean displayStats = false;

        Graphics g = getGraphics();
        int[] oVideoOverlay = new int[4];
        /*int[] oOverlay = {10, 10, 96, 64};
        //Object oMedia = createRemoteVideo("rtsp://stream.streamezzo.com/public/jt.3gp", oOverlay, 1);
        Object oMedia = createRemoteVideo("lres://jt.3gp", oOverlay, 1);
        playMedia(oMedia);*/

        while (render_thread == mythread && quitMainLoop == false)
        {
            synchronized (this)
            {
                if(BasicRenderer.DISPLAY_FPS)
                {
                    if(sceneLoader.m_units.size() == 0 && bufferEmpty == false && displayStats == false )
                    {
                        displayStats = true;
                    }

                    if(sceneLoader.m_units.size() != 0)
                    {
                        bufferEmpty = false;
                    }
                }

                if(BasicRenderer.SYNCHRONIZE_HTTP)
                {
                    synchronized( sceneLoader )
                    {
                        render(); //SceneLoader computes its own scene time
                    }
                }
                else
                {
                    render(); //SceneLoader computes its own scene time
                }

                if (!mediaTick(this, oVideoOverlay))
                {
                    paint(g);
                    flushGraphics(0, 0, m_canvasWidth, m_canvasHeight);
                }
                else
                {
                    paint(g);

                    int x, y, width, height;
                    // TOP
                    x = 0;
                    y = 0;
                    width = m_canvasWidth;
                    height = oVideoOverlay[1];
                    if (height > 0)
                    {
                        flushGraphics(x, y, width, height);
                    }

                    // LEFT
                    x = 0;
                    y = oVideoOverlay[1];
                    width = oVideoOverlay[0];
                    height = oVideoOverlay[3];
                    if ((width > 0) && (height > 0))
                    {
                        flushGraphics(x, y, width, height);
                    }

                    // RIGHT
                    x = oVideoOverlay[0] + oVideoOverlay[2];
                    y = oVideoOverlay[1];
                    width = m_canvasWidth - x;
                    height = oVideoOverlay[3];
                    if ((width > 0) && (height > 0))
                    {
                        flushGraphics(x, y, width, height);
                    }

                    // BOTTOM
                    x = 0;
                    y = oVideoOverlay[1] + oVideoOverlay[3];
                    width = m_canvasWidth;
                    height = m_canvasHeight - y;
                    if ((width > 0) && (height > 0))
                    {
                        flushGraphics(x, y, width, height);
                    }
                }


                //Wait for the next frame
                time = System.currentTimeMillis();
                delta = time - paintSystemTimeInMillis;
                paintSystemTimeInMillis = System.currentTimeMillis();

                if(BasicRenderer.DISPLAY_FPS)
                {
                    instantMen = runtime.freeMemory();
                    if(instantMen <= minMem || instantMen == -1)
                    {
                        minMem = instantMen;
                    }
                    if(instantMen >= maxMem )
                    {
                        maxMem = instantMen;
                    }
                }

                if(BasicRenderer.LIMIT_FPS)
                {
                    try
                    {
                        if (delta < BasicRenderer.TIME_MIN_PER_RENDER)
                        {
                            this.wait(BasicRenderer.TIME_MIN_PER_RENDER-delta);
                        }
                    }
                    catch (InterruptedException e)
                    {
                    }
                }
            }
        }
        if (quitMainLoop)
        {
            callerClass.destroyApp(false);
            callerClass.notifyDestroyed();
        }
    }

    public void quit()
    {
        quitMainLoop = true;
    }

    public int getSceneHeight()
    {
        return sceneLoader.getSceneHeight();
    }

    public int getSceneWidth()
    {
        return sceneLoader.getSceneWidth();
    }

    /**
     * Viewbox getter
     *
     * @return the viewbox as {x, y, width, height}
     */
    public int[] getViewbox()
    {
        return sceneLoader.getViewbox();
    }


///////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////// Interactivity ////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////
    public String textInput(String title, String msg)
    {
        m_textBox = new TextField(title, msg, 1000, TextField.ANY);
        m_textInputForm = new Form("Text Input");
        m_textInputForm.insert(0, m_textBox);
        m_textInputForm.addCommand(m_okCommand);

        m_textInputForm.setCommandListener(this);
        m_display.setCurrent(m_textInputForm);

        sceneLoader.pauseScene();
        synchronized (this)
        {
            try
            {
                wait();
            }
            catch (InterruptedException ie)
            {
                ie.printStackTrace();
            }
        }

        m_display.setCurrent(this);
        return m_textInput;
    }


////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////// Painting /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
    /**
     * Notified when Canvas is made visible.
     * If there is more than one image to m_display
     * create the render_thread to run the animation timing.
     */
    protected void showNotify()
    {
        if(render_thread == null)
        {
            render_thread = new Thread(this);
            render_thread.start();
        }
    }

    private void render()
    {
        //init the pointer
        synchronized (this)
        {
            if (wasPointerPressed)
            {
                m_lastPointerPressed = m_oldLastPointerPressed;
                wasPointerPressed = false;
            }
            else
            {
                m_lastPointerPressed = null;
            }

            //init the key
            if (wasKeyPressed)
            {
                m_lastKeyPressed = m_oldLastKeyPressed;
                wasKeyPressed = false;

                // Feed the navigator with the pressed key
                sceneLoader.m_navigator.setPressedKey(m_oldLastKeyPressed);
            }
            else
            {
                m_lastKeyPressed = BasicRenderer.NO_KEY;
            }
        }
        sceneLoader.render();

        // Reset navigator key pressed state
        sceneLoader.m_navigator.resetKeyPressed();

        // Reset scale factor state
        m_ScaleFactorUpdated = false;
    }

////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////// Loading //////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
    public void closeConnector()
            throws IOException
    {
        if(sceneLoader.m_byteIS != null)
        {
            try
            {
                sceneLoader.m_byteIS.close();
            }catch(Exception e)
            {
                System.out.println("    Cannot close : m_byteIS");
            }
        }
        sceneLoader.m_byteIS = null;

        if (connection instanceof HttpConnection)
        {
            ((HttpConnection) connection).close();
        }
        else if (connection instanceof StreamConnection)
        {
            ((StreamConnection) connection).close();
        }
        connection = null;
    }

    public SceneLoader getSceneLoader()
    {
        return sceneLoader;
    }

    public void setSceneLoader(SceneLoader loader)
    {
        sceneLoader = loader;
        loader.m_graphObjectManager.setRenderer(this);
    }

    public DataInputStream openURL(String url) throws IOException
    {
        try
        {
            //if it's a distant file
            if (url.startsWith("http://"))
            {
                connection = openHTTPConnector(url, m_canvasWidth, getHeightLASeR());
                if (connection == null)
                {
                    return null;
                }
                return getDataInputStreamFromConnector(connection);
                // if it's a local file
            }
            else if( url.startsWith("cmd://quit") )
            {
                quit();
                return null;
            }
            else
            {
                url = url.substring(6,url.length());
                return new DataInputStream(this.getClass().getResourceAsStream(url));
            }
        }
        catch (Exception npe)
        {
            npe.printStackTrace();
            throw new NullPointerException(npe.getMessage());
        }
        catch (OutOfMemoryError ome)
        {
            throw new OutOfMemoryError(ome.getMessage());
        }
    }

    private Object openHTTPConnector(String url, int width, int height) throws IOException, OutOfMemoryError
    {
        HttpConnection c = null;
        int status = -1;

        c = (HttpConnection) Connector.open(url,Connector.READ, true);

            //SET REQUEST HEADER
            String conf = System.getProperty("microedition.configuration");
            String prof = System.getProperty("microedition.profiles");
            String locale = System.getProperty("microedition.locale");

            String ua = "Profile/" + prof +
                " Configuration/" + conf + " Screen/w" + width + "h" + height;

            ua = ua + "t" + System.getProperty("microedition.platform");

            c.setRequestProperty("HTTP-Version", "HTTP/1.1");

            c.setRequestProperty("User-Agent", ua);

            if (locale != null)
            {
                c.setRequestProperty("Content-Language", locale);
            }

            // Get the status code, causing the connection to be made
            try
            {
                status = c.getResponseCode();
            }
        catch (IOException ioe) { }

            if (BasicRenderer.PHONE_DEBUG)
            {
                textInput("Response Code", String.valueOf(status));

            String contentType = c.getType();
            if (contentType == null) contentType = "";

            String transf = c.getHeaderField("Transfer-Encoding");
            if (transf == null) transf = "";

            String date = c.getHeaderField("Date");
            if (date == null) date = "";

            String server = c.getHeaderField("Server");
            if (server == null) server = "";

            String toprint = "ContentType " + contentType + "\nTransfer-Encoding " + transf + "\nDate " + date + "\nServer " + server;
            textInput("Header", toprint);
        }
        return c;
    }

    /*************************************
     CommandListener Implementation*******
     *************************************/

    public void commandAction(Command c, Displayable s)
    {
        //Ok selected in an input text Form
        if (c.equals(m_okCommand) && s.equals(m_textInputForm))
        {
            m_textInput = m_textBox.getString();
            sceneLoader.resumeScene();
            synchronized (this)
            {
                notifyAll();
            }
        }
    }
}
