/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.player.util;

import iso.mpeg.streamezzo.player.node.SVGset;

import java.util.Enumeration;
import java.util.Vector;

public class AnimationManager
{
    // list of animations
    private Vector animations = new Vector(1);

    /**
     * Registers an animation
     * @param anim the animation node
     */
    public void attach(SVGset anim)
    {
        // animations are added in a sorted list
        TimedElement te = null;
        int i = 0;
        for(i=0;i<animations.size();i++)
        {
            te = (TimedElement)animations.elementAt(i);
            if (te.getBeginTime() > anim.getBeginTime())
                break;
        }
        animations.insertElementAt(anim, i);
    }

    /**
     * Removes an animation from the list
     * @param anim the animation node
     */
    public void detach(SVGset anim)
    {
        animations.removeElement(anim);
    }


    /**
     * Do all animations
     * @param sceneTimeInMillis
     */
    public void animateAll(int sceneTimeInMillis)
    {
        SVGset anim = null;

        // Go through the listeners list and send the new event
        for (Enumeration e = animations.elements() ; e.hasMoreElements() ;)
        {
            anim = (SVGset)e.nextElement();
            anim.animate(sceneTimeInMillis);
        }

        // reorder animations in case of a restart
        sortAnimations();
    }

    /**
     * Sorts all animations by begin time
     */
    public void sortAnimations()
    {
        // todo find a faster algorithm

        // number of elements
        int n = animations.size();
        // flag for no swap occured in a pass
        boolean noSwap = true;
        TimedElement tmp = null;

        for (int i=0; i<n-1; i++)
        {
            for (int j=0; j<n-1-i; j++)
            {
                if (((TimedElement)animations.elementAt(j+1)).getBeginTime()
                  < ((TimedElement)animations.elementAt(j)).getBeginTime())
                {
                    // compare the two neighbors
                    tmp = (TimedElement)animations.elementAt(j); // swap elem[j] and elem[j+1]
                    animations.setElementAt(animations.elementAt(j+1), j);
                    animations.setElementAt(tmp, j+1);
                    noSwap = false;
                }
            }
            // if it's sorted before all the passes, stop the process
            if (noSwap) break;
        }
    }
}
