/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */

package iso.mpeg.streamezzo.player.util;

import iso.mpeg.streamezzo.player.node.Event;


/**
 * This Interface defines a listener of events (Observer pattern)
 * Every object which implement this Interface and are registered to an EvenManager
 * will be notified of new events via the method notify().
 */
public interface EventListener
{
    /////////////////
    // EVENT TYPES //
    /////////////////

    // encoded events are first
   
	// TML : changed like parser XML
/*
0	abort
	accesskey
	activate
	activatedEvent
	beginEvent
5	click
	deactivatedEvent
	endEvent
	error
	executionTime
10	focusin	
	focusout
	keydown
	keyup
	load
15	longaccesskey
	mousedown
	mousemove
	mouseout
	mouseover
20	mouseup
	pause
	pausedEvent
	play
	repeatEvent
25	resize
	resume
	resumedEvent
	scroll
	textinput
30	unload
	zoom
*/	
    // User Interface Events
    public static final byte UI_FOCUS_IN      = 10;
    public static final byte UI_FOCUS_OUT     = 11;
    public static final byte UI_ACTIVATE      = 2;
    public static final byte UI_ACTIVATE_EVENT	= 3;
    public static final byte UI_DESACTIVATE_EVENT	= 6;
    public static final Event _UI_FOCUS_IN = new Event(10);
    public static final Event _UI_FOCUS_OUT = new Event(11);
    public static final Event _UI_ACTIVATE = new Event(2);
    public static final Event _UI_ACTIVATE_EVENT = new Event(3);
    public static final Event _UI_DESACTIVATE_EVENT = new Event(6);

    // Mouse Events
    public static final byte MOUSE_DOWN       = 16;
    public static final byte MOUSE_UP         = 20;
    public static final byte MOUSE_CLICK      = 5;
    public static final byte MOUSE_OVER       = 19;
    public static final byte MOUSE_OUT        = 18;
    public static final byte MOUSE_MOVE       = 17;

    // Time based Events
    public static final byte TIME_BEGIN       = 4;
    public static final byte TIME_END         = 7;
    public static final byte TIME_REPEAT      = 24;

    // Zoom Events
    public static final byte ZOOM_EVENT       = 31;

    // Key events
    public static final byte KEY_DOWN         = 12;
    public static final byte KEY_UP           = 13;
    public static final byte KEY_PRESS        = 1;
    public static final byte KEY_LONG_PRESS   = 15;

    public static final byte PAUSE	 		  = 21;
    public static final byte PAUSE_EVENT	  = 22;
    public static final byte PLAY			  = 23;
    public static final byte RESUME			  = 26;
    public static final byte RESUME_EVENT	  = 27;
    // Misc Events
    
    public static final byte EVENT_SCROLL     = 28;
    public static final byte EVENT_RESIZE     = 25;
    public static final byte EVENT_ERROR      = 8;
    public static final byte EVENT_ABORT      = 0;
    public static final byte EVENT_LOAD       = 14;
    public static final byte EVENT_UNLOAD     = 30;
    public static final byte EVENT_TEXT_INPUT = 29;

    // Internal event
    public static final byte INTERNAL_TARGET_DELETED = 100;
    public static final byte INTERNAL_UNDEFINED = -1;


    /**
     * Method called back to inform the listener when a new event is generated
     * @param initiator the object which initiated the event
     * @param eventType the event type
     * @param eventData the associated event data
     * @return true if the event has been handled
     */
    public boolean update(Object initiator, byte eventType, Object eventData);
}
