/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */

package iso.mpeg.streamezzo.player.util;

import java.util.Enumeration;
import java.util.Vector;

/**
 * EventManager class manages a list of EventListeners and notifies all of them
 * when a new event is generated.
 * @see EventListener interface
 */

public class EventManager
{
    public Vector listeners;

    /**
     * Constructor
     */
    public EventManager()
    {
        listeners = new Vector(1);
    }

    /**
     * Registers a listener
     * @param listener
     */
    public void attach(EventListener listener)
    {
        listeners.addElement(listener);
    }

    /**
     * Removes a listener from the list
     * @param listener
     */
    public void detach(EventListener listener)
    {
        listeners.removeElement(listener);
    }

    /**
     * Removes all the attached listeners
     */
    public void clean()
    {
        listeners.removeAllElements();
    }

    /**
     * Notifies all the regitered listeners
     * @param initiator the Object which initiated the event
     * @param eventType the event type
     * @param eventData the associated event data
     * @return true if the event has been handled
     */
    public boolean notify(Object initiator, byte eventType, Object eventData)
    {
        boolean isHandled = false;
        EventListener listener = null;

        // Go through the listeners list and send the new event
        for (Enumeration e = listeners.elements() ; e.hasMoreElements() ;)
        {
            listener = (EventListener)e.nextElement();
            if (listener.update(initiator, eventType, eventData))
            {
                isHandled = true;
            }
        }

        return isHandled;
    }
}
