/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.player.util;

import iso.mpeg.streamezzo.player.BasicRendering;
import iso.mpeg.streamezzo.player.library.Real;
import iso.mpeg.streamezzo.player.node.Value;

public class NodeProperties
{    
	public static final byte DISPLAY_BLOCK = 0; // TML
	public static final byte DISPLAY_COMPACT = 1; // TML
    public static final byte DISPLAY_INHERIT = 2; // TML
    public static final byte DISPLAY_INLINE = 3; // TML
    public static final byte DISPLAY_INLINETABLE = 4; // TML
	public static final byte DISPLAY_LISTITEM = 5; // TML
    public static final byte DISPLAY_MARKER = 6; // TML
    public static final byte DISPLAY_NONE = 7; // TML
	public static final byte DISPLAY_RUNIN = 8; // TML
	public static final byte DISPLAY_TABLE = 9; // TML
    public static final byte DISPLAY_TABLECAPTION = 10; // TML
    public static final byte DISPLAY_TABLECELL = 11; // TML
    public static final byte DISPLAY_TABLECOLUMN = 12; // TML
	public static final byte DISPLAY_TABLECOLUMNGROUP = 13; // TML
    public static final byte DISPLAY_TABLEFOOTERGROUP = 14; // TML
    public static final byte DISPLAY_TABLEHEADERGROUP = 15; // TML
    public static final byte DISPLAY_TABLEROW = 16; // TML
    public static final byte DISPLAY_TABLEROWGROUP = 17; // TML
    
    
    public static final byte VISIBILITY_HIDDEN = 0;
    public static final byte VISIBILITY_COLLAPSE = 0; // TML : TODO : not present in XML parser
    public static final byte VISIBILITY_INHERIT = 1;
    public static final byte VISIBILITY_VISIBLE = 2;

    // TML : like parser Xml
    public static final byte DISPLAYALIGN_AFTER = 0;
    public static final byte DISPLAYALIGN_AUTO = 1;
    public static final byte DISPLAYALIGN_BEFORE = 2;
    public static final byte DISPLAYALIGN_CENTER = 3;
    public static final byte DISPLAYALIGN_INHERIT = 4;
    
//  TML : like parser Xml
    public static final byte TEXTALIGN_END = 0;
    public static final byte TEXTALIGN_INHERIT = 1;
    public static final byte TEXTALIGN_CENTER = 2;
    public static final byte TEXTALIGN_START = 3;
    
    public byte m_display = DISPLAY_INLINE;
    public int m_fill = 0x00000000;
    public int m_color = 0x00000000;
    public int m_stopColor = 0x00000000;
    public int m_stroke = Value.STROKE_NONE;
    public byte m_fillOpacity = (byte)0xFF;
    public byte m_strokeOpacity = (byte)0xFF;
    public int m_strokeWidth = Real.VALUE1;
    public int m_strokeDashoffset = 0;
    public int m_strokeMiterlimit = 0;
    public int m_strokeDasharray[] = null;
    public String m_fontFamily = null;
    public int m_fontSize = BasicRendering.limitBetween0And8; // 20.12 fixed point
    public byte m_fontStyle = -1;
    public byte m_visibility = VISIBILITY_VISIBLE;
    public byte m_fillRule = 1;
    public byte m_focusable = 0; // no heritance
    // public int m_navIndex = -1; // no heritance
    // public int m_navLeft = -1; // no heritance
    // public int m_navRight = -1; // no heritance
    // public int m_navUp = -1; // no heritance
    // public int m_navDown = -1; // no heritance
    public byte m_text_anchor = TEXTALIGN_START;
    public byte m_displayAlign = DISPLAYALIGN_BEFORE;
    public byte m_strokeLinecap = 0;
    public byte m_strokeLinejoin = 0;
    public byte m_pointerEvents = 0;
    public byte m_overflow = 0;
    public byte m_fontWeight = 0;
    public byte m_fontVariant = 0;
    public String m_requiredExtensions = null;
    public String m_systemLanguage = null;
    public String m_xmlbase = null;
    public String m_xmllang = null;
    public String m_requiredFormats = null;
    public boolean m_externalResourcesRequired = false;
    public byte m_requiredFeatures[] = null;
    public byte m_stopOpacity = (byte)0xFF;
    
    //
    // TML : Activate and desactivate noeud
    //
    public boolean m_Deactivated = false;
    public boolean m_DeactivatedOwner = false; 
    public boolean m_Deactivating = false;

    ////// Inheritance management. See NodeBaseProperties and codec//////

    // BOOLEAN
    public static final byte ACCUMULATE            =  0;
    public static final byte ADDITIVE              =  1;
    public static final byte APPEND                =  2;
    public static final byte EDITABLE              =  3;
    public static final byte FOCUSABLE             =  4;
    public static final byte ANIMFILL              =  5;
    public static final byte EXTERNALRESOURCESREQUIRED  =  6;
    public static final byte ZOOMANDPAN            =  7;

    public static final byte FIRST_BOOLEAN = ACCUMULATE;
    public static final byte LAST_BOOLEAN = ZOOMANDPAN;

    // INT3
    public static final byte CALCMODE              =  8;
    public static final byte DISPLAY               =  9;
    public static final byte FILLRULE              = 10;
    public static final byte RESTART               = 11;
    public static final byte DISPLAYALIGN          = 12;
    public static final byte VISIBILITY            = 13;
    public static final byte TEXTALIGN             = 14;
    public static final byte TYPE                  = 15;
    public static final byte STROKELINECAP         = 16;
    public static final byte STROKELINEJOIN        = 17;
    public static final byte OVERFLOW              = 18;
    public static final byte FONTVARIANT           = 19;
    public static final byte COLORRENDERING        = 20;
    public static final byte IMAGERENDERING        = 21;
    public static final byte SHAPERENDERING        = 22;
    public static final byte TEXTRENDERING         = 23;
    public static final byte TIMEATTRIBUTE         = 24;

    public static final byte FIRST_INT3 = CALCMODE;
    public static final byte LAST_INT3 = TIMEATTRIBUTE;

    // INT8
    public static final byte ATTRIBUTENAME         = LAST_INT3+1;
    public static final byte EVENT                 = LAST_INT3+2;
    public static final byte CHOICE                = LAST_INT3+3;
    public static final byte FONTSTYLE             = LAST_INT3+4;
    public static final byte KEYCODE               = LAST_INT3+5;
    public static final byte PRESERVEASPECTRATIO   = LAST_INT3+6;
    public static final byte REPEATCOUNT           = LAST_INT3+7;
    public static final byte POINTEREVENTS         = LAST_INT3+8;
    public static final byte FONTWEIGHT            = LAST_INT3+9;
    public static final byte NAVINDEX              = LAST_INT3+10;
    public static final byte FONTFAMILY            = LAST_INT3+11;

    public static final byte FIRST_INT8 = ATTRIBUTENAME;
    public static final byte LAST_INT8 = FONTFAMILY;

    // ID
    public static final byte ID                    = LAST_INT8+1;
    public static final byte MPATH                 = LAST_INT8+2;
    public static final byte NAVUP                 = LAST_INT8+3;
    public static final byte NAVDOWN               = LAST_INT8+4;
    public static final byte NAVLEFT               = LAST_INT8+5;
    public static final byte NAVRIGHT              = LAST_INT8+6;
    public static final byte HREF                  = LAST_INT8+7;
    public static final byte OBSERVER              = LAST_INT8+8;

    public static final byte FIRST_ID = ID;
    public static final byte LAST_ID = OBSERVER;

    // COLOR
    public static final byte COLORFILL             = LAST_ID+1;
    public static final byte STROKE                = LAST_ID+2;
    public static final byte STOPCOLOR             = LAST_ID+3;
    public static final byte COLOR                 = LAST_ID+4;
    public static final byte BACKGROUNDFILL        = LAST_ID+5;

    public static final byte FIRST_COLOR = COLORFILL;
    public static final byte LAST_COLOR = BACKGROUNDFILL;

    // OPACITY
    public static final byte FILLOPACITY           = LAST_COLOR+1;
    public static final byte STROKEOPACITY         = LAST_COLOR+2;
    public static final byte STOPOPACITY           = LAST_COLOR+3;
    public static final byte BACKGROUNDFILLOPACITY = LAST_COLOR+4;

    public static final byte FIRST_OPACITY = FILLOPACITY;
    public static final byte LAST_OPACITY = BACKGROUNDFILLOPACITY;

    // FLOAT
    public static final byte STROKEWIDTH           = LAST_OPACITY+1;
    public static final byte STROKEDASHOFFSET      = LAST_OPACITY+2;
    public static final byte STROKEMITERLIMIT      = LAST_OPACITY+3;
    public static final byte FONTSIZE              = LAST_OPACITY+4;
    public static final byte ROTATION              = LAST_OPACITY+5;
    public static final byte ROTATE                = LAST_OPACITY+6;
    public static final byte HEIGHT                = LAST_OPACITY+7;
    public static final byte WIDTH                 = LAST_OPACITY+8;
    public static final byte R                     = LAST_OPACITY+9;
    public static final byte RX                    = LAST_OPACITY+10;
    public static final byte RY                    = LAST_OPACITY+11;
    public static final byte X2                    = LAST_OPACITY+12;
    public static final byte Y2                    = LAST_OPACITY+13;
    public static final byte CX                    = LAST_OPACITY+14;
    public static final byte CY                    = LAST_OPACITY+15;
    public static final byte X1                    = LAST_OPACITY+16;
    public static final byte Y1                    = LAST_OPACITY+17;
    public static final byte X                     = LAST_OPACITY+18;
    public static final byte Y                     = LAST_OPACITY+19;

    public static final byte FIRST_FLOAT = STROKEWIDTH;
    public static final byte LAST_FLOAT = Y;

    // TIME
    public static final byte BEGIN                 = LAST_FLOAT+1;
    public static final byte END                   = LAST_FLOAT+2;
    public static final byte DUR                   = LAST_FLOAT+3;
    public static final byte DELAY                 = LAST_FLOAT+4;
    public static final byte REPEATDUR             = LAST_FLOAT+5;

    public static final byte FIRST_TIME = BEGIN;
    public static final byte LAST_TIME = REPEATDUR;

    // POINT
    public static final byte SCALE                 = LAST_TIME+1;
    public static final byte SIZE                  = LAST_TIME+2;
    public static final byte TRANSLATION           = LAST_TIME+3;

    public static final byte FIRST_POINT = SCALE;
    public static final byte LAST_POINT = TRANSLATION;

    // MATRIX
    public static final byte TRANSFORM             = LAST_POINT+1;

    public static final byte FIRST_MATRIX = TRANSFORM;
    public static final byte LAST_MATRIX = TRANSFORM;

    // STRING
    public static final byte REQUIREDFORMATS       = LAST_MATRIX+1;
    public static final byte REQUIREDEXTENSIONS    = LAST_MATRIX+2;
    public static final byte SYSTEMLANGUAGE        = LAST_MATRIX+3;
    public static final byte XMLLANG               = LAST_MATRIX+4;
    public static final byte XMLBASE               = LAST_MATRIX+5;
    public static final byte URL                   = LAST_MATRIX+6;

    public static final byte FIRST_STRING = XMLBASE;
    public static final byte LAST_STRING = URL;

    // POINTS
    public static final byte POINTS                = LAST_STRING+1;

    public static final byte FIRST_POINTS = POINTS;
    public static final byte LAST_POINTS = POINTS;

    // PATH
    public static final byte D                     = LAST_POINTS+1;

    public static final byte FIRST_PATH = D;
    public static final byte LAST_PATH = D;

    // VIEWBOX
    public static final byte VIEWBOX               = LAST_PATH+1;

    public static final byte FIRST_VIEWBOX = VIEWBOX;
    public static final byte LAST_VIEWBOX = VIEWBOX;

    // INTS
    public static final byte REQUIREDFEATURES      = LAST_VIEWBOX+1;
    public static final byte KEYCODES              = LAST_VIEWBOX+2;

    public static final byte FIRST_INTS = KEYCODES;
    public static final byte LAST_INTS = KEYCODES;

    // FLOATS
    public static final byte STROKEDASHARRAY       = LAST_INTS+1;
    public static final byte KEYPOINTS             = LAST_INTS+2;
    public static final byte KEYSPLINES            = LAST_INTS+3;

    public static final byte FIRST_FLOATS = KEYPOINTS;
    public static final byte LAST_FLOATS = KEYSPLINES;

    // TIMES
    public static final byte KEYTIMES              = LAST_FLOATS+1;

    public static final byte FIRST_TIMES = KEYTIMES;
    public static final byte LAST_TIMES = KEYTIMES;

    // ELEMENT
    public static final byte CHILDREN              = LAST_TIMES+1;

    public static final byte FIRST_ELEMENT = CHILDREN;
    public static final byte LAST_ELEMENT = CHILDREN;

    // TEXTCONTENT
    public static final byte TEXTCONTENT           = LAST_ELEMENT+1;

    public static final byte FIRST_TEXTCONTENT = TEXTCONTENT;
    public static final byte LAST_TEXTCONTENT = TEXTCONTENT;

    // FLOATS
    public static final byte TRANSFORMROTATE       = LAST_TEXTCONTENT+1;
    public static final byte TRANSFORMSCALE        = LAST_TEXTCONTENT+2;
    public static final byte TRANSFORMSKEWX        = LAST_TEXTCONTENT+3;
    public static final byte TRANSFORMSKEWY        = LAST_TEXTCONTENT+4;
    public static final byte TRANSFORMTRANSLATE    = LAST_TEXTCONTENT+5;

    public static final byte LAST_FLOATS2 = TRANSFORMSKEWY; // todo


    /** inheritance flag : each bit tells if property is inherited or not */
    protected int m_propertyInheritanceL0 = 0xFFFFFFFF;
    protected int m_propertyInheritanceL1 = 0xFFFFFFFF;
    protected int m_propertyInheritanceH0 = 0xFFFFFFFF;
    protected int m_propertyInheritanceH1 = 0xFFFFFFFF;
    
    protected int m_propertyInheritanceExt = 0xFFFFFFFF;


    /**
     * Return a copy of the current instance
     * @return
     */
    public NodeProperties getCopy()
    {
        NodeProperties copyObject = new NodeProperties();

        copyObject.m_fill = m_fill;
        copyObject.m_stroke = m_stroke;
        copyObject.m_fillOpacity = m_fillOpacity;
        copyObject.m_strokeOpacity = m_strokeOpacity;
        copyObject.m_strokeWidth = m_strokeWidth;
        copyObject.m_fontFamily = m_fontFamily;
        copyObject.m_fontSize = m_fontSize;
        copyObject.m_fontStyle = m_fontStyle;
        copyObject.m_display = m_display;
        copyObject.m_visibility = m_visibility;
        copyObject.m_fillRule = m_fillRule;
        copyObject.m_focusable = m_focusable;
        /*copyObject.m_navIndex = m_navIndex;
        copyObject.m_navLeft = m_navLeft;
        copyObject.m_navRight = m_navRight;
        copyObject.m_navUp = m_navUp;
        copyObject.m_navDown = m_navDown;*/
        copyObject.m_text_anchor = m_text_anchor;
        copyObject.m_displayAlign = m_displayAlign;
        copyObject.m_propertyInheritanceL0 = m_propertyInheritanceL0;
        copyObject.m_propertyInheritanceL1 = m_propertyInheritanceL1;
        copyObject.m_propertyInheritanceH0 = m_propertyInheritanceH0;
        copyObject.m_propertyInheritanceH1 = m_propertyInheritanceH1;
        
        copyObject.m_Deactivated = m_Deactivated;
        copyObject.m_DeactivatedOwner = m_DeactivatedOwner; 
        copyObject.m_Deactivating = m_Deactivating;

        return copyObject;
    }

    /**
     * Copies the current instance to passed-in
     */
    public void copyTo(NodeProperties target)
    {
        target.m_display = m_display;
        target.m_fill = m_fill;
        target.m_stroke = m_stroke;
        target.m_fillOpacity = m_fillOpacity;
        target.m_strokeOpacity = m_strokeOpacity;
        target.m_strokeWidth = m_strokeWidth;
        target.m_fontFamily = m_fontFamily;
        target.m_fontSize = m_fontSize;
        target.m_fontStyle = m_fontStyle;
        target.m_display = m_display;
        target.m_visibility = m_visibility;
        target.m_fillRule = m_fillRule;
        target.m_focusable = m_focusable;
        /*target.m_navIndex = m_navIndex;
        target.m_navLeft = m_navLeft;
        target.m_navRight = m_navRight;
        target.m_navUp = m_navUp;
        target.m_navDown = m_navDown;*/
        target.m_text_anchor = m_text_anchor;
        target.m_displayAlign = m_displayAlign;
        target.m_propertyInheritanceL0 = m_propertyInheritanceL0;
        target.m_propertyInheritanceL1 = m_propertyInheritanceL1;
        target.m_propertyInheritanceH0 = m_propertyInheritanceH0;
        target.m_propertyInheritanceH1 = m_propertyInheritanceH1;
        
        target.m_Deactivated = m_Deactivated;
        target.m_DeactivatedOwner = m_DeactivatedOwner; 
        target.m_Deactivating = m_Deactivating;
    }

    public boolean equals(Object o)
    {
        if (this == o) return true;
        if (!(o instanceof NodeProperties)) return false;

        final NodeProperties nodeProperties = (NodeProperties) o;

        if (! super.equals(o))
        {
            return false;
        }
        if (m_displayAlign != nodeProperties.m_displayAlign) return false;
        if (m_fill != nodeProperties.m_fill) return false;
        if (m_fillOpacity != nodeProperties.m_fillOpacity) return false;
        if (m_fillRule != nodeProperties.m_fillRule) return false;
        if (m_focusable != nodeProperties.m_focusable) return false;
        if (m_fontSize != nodeProperties.m_fontSize) return false;
        if (m_fontStyle != nodeProperties.m_fontStyle) return false;
        /*if (m_navDown != nodeProperties.m_navDown) return false;
        if (m_navIndex != nodeProperties.m_navIndex) return false;
        if (m_navLeft != nodeProperties.m_navLeft) return false;
        if (m_navRight != nodeProperties.m_navRight) return false;
        if (m_navUp != nodeProperties.m_navUp) return false;*/
        if (m_stroke != nodeProperties.m_stroke) return false;
        if (m_strokeOpacity != nodeProperties.m_strokeOpacity) return false;
        if (m_strokeWidth != nodeProperties.m_strokeWidth) return false;
        if (m_text_anchor != nodeProperties.m_text_anchor) return false;
        if (m_visibility != nodeProperties.m_visibility) return false;
        if (m_fontFamily != null ? !m_fontFamily.equals(nodeProperties.m_fontFamily) : nodeProperties.m_fontFamily != null) return false;
        
        if (m_Deactivated != nodeProperties.m_Deactivated) return false;
        if (m_DeactivatedOwner != nodeProperties.m_DeactivatedOwner) return false;
        if (m_Deactivating != nodeProperties.m_Deactivating) return false;

        return true;
    }

    /**
     * Checks if a property have to be inherited
     * @param propertyIndex Index of the property to check (see NodeProperties constants)
     * @return true if inherited
     */
    public boolean isInherited(char propertyIndex)
    {
    	// Extract bit
        int nBit;
        if (propertyIndex < 32)
        {
            nBit = m_propertyInheritanceL0 >> propertyIndex;
        }
        else if (propertyIndex < 64)
        {
            nBit = m_propertyInheritanceL1 >> (propertyIndex - 32);
        }
        else if (propertyIndex < 96)
        {
            nBit = m_propertyInheritanceH0 >> (propertyIndex - 64);
        }
        else if (propertyIndex < 128)
        {
            nBit = m_propertyInheritanceH1 >> (propertyIndex - 96);
        }
        else
        {
            nBit = m_propertyInheritanceExt >> (propertyIndex - 128);
        }

        // return
        return (nBit & 0x01) == 0x01 ? true : false;
        //return false;
    }

    /**
     * Set inheritance mode of a property
     * @param propertyIndex Index of the property to check (see NodeProperties constants)
     * @param isInherited true if this property have to be inherited
     */
    public void setInheritance(char propertyIndex, boolean isInherited)
    {
        int nBit = 0x01;

        if (propertyIndex < 32)
        {
            nBit = nBit << propertyIndex;
        }
        else if (propertyIndex < 64)
        {
            nBit = nBit << (propertyIndex - 32);
        }
        else if (propertyIndex < 96)
        {
            nBit = nBit << (propertyIndex - 64);
        }
        else if (propertyIndex < 128)
        {
            nBit = nBit << (propertyIndex - 96);
        } 
        else
        {
            nBit = nBit << (propertyIndex - 128);
        } 

        if (isInherited)
        {
            // Set bit
            if (propertyIndex < 32)
            {
                m_propertyInheritanceL0 = m_propertyInheritanceL0 | nBit;
            }
            else if (propertyIndex < 64)
            {
                m_propertyInheritanceL1 = m_propertyInheritanceL1 | nBit;
            }
            else if (propertyIndex < 96)
            {
                m_propertyInheritanceH0 = m_propertyInheritanceH0 | nBit;
            }
            else if (propertyIndex < 128)
            {
                m_propertyInheritanceH1 = m_propertyInheritanceH1 | nBit;
            }
            else
            {
                m_propertyInheritanceExt = m_propertyInheritanceExt | nBit;
            }
        }
        else
        {
            // Unset bit
            nBit = ~nBit;
            if (propertyIndex < 32)
            {
                m_propertyInheritanceL0 = m_propertyInheritanceL0 & nBit;
            }
            else if (propertyIndex < 64)
            {
                m_propertyInheritanceL1 = m_propertyInheritanceL1 & nBit;
            }
            else if (propertyIndex < 96)
            {
                m_propertyInheritanceH0 = m_propertyInheritanceH0 & nBit;
            }
            else if (propertyIndex < 128)
            {
                m_propertyInheritanceH1 = m_propertyInheritanceH1 & nBit;
            }
            else
            {
                m_propertyInheritanceExt = m_propertyInheritanceExt & nBit;
            }
        }
    }

}
