/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */

package iso.mpeg.streamezzo.player.util;

import iso.mpeg.streamezzo.player.library.Rect2D;

import java.util.Vector;

/**
 * This class manages a sensitive zone for a textual element. It provides a set of method to check mouse
 * events whitin the zone. A sensitive zone is a normalized rectangle.
 */
public class SensitiveZoneText extends SensitiveZone
{
    /** The list of listeners listening to this zone */
    protected Vector m_listeners = new Vector();

////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////// Setter/Getter, construction ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Provides the involved listeners for this zone
     * @param listeners
     */
    public void setListeners(Vector listeners)
    {
        m_listeners.ensureCapacity(listeners.size());
        for (int i = 0; i < listeners.size(); i++)
        {
            m_listeners.addElement(listeners.elementAt(i));
        }
    }

    /**
     * Returns a reference to the listeners list involved in this zone
     * @return a reference to the listeners list involved in this zone
     */
    public Vector getListeners()
    {
        return m_listeners;
    }

    /**
     * Makes the zone grow to include the provided rectangle
     * @param extension the rectangle with whitch the sensitive zone will be extended
     */
    public void addZone(Rect2D extension)
    {
        if ( (-1 == m_rect.width_r) && (-1 == m_rect.height_r) )
        {
            m_rect.x_r = extension.x_r;
            m_rect.y_r = extension.y_r;
            m_rect.width_r = extension.width_r;
            m_rect.height_r = extension.height_r;
        }
        else
        {
            if (m_rect.x_r > extension.x_r) m_rect.x_r = extension.x_r;
            if (m_rect.y_r > extension.y_r) m_rect.y_r = extension.y_r;
            if ((m_rect.x_r + m_rect.width_r) < (extension.x_r + extension.width_r)) m_rect.width_r = extension.x_r + extension.width_r - m_rect.x_r;
            if ((m_rect.y_r + m_rect.height_r) < (extension.y_r + extension.height_r)) m_rect.height_r = extension.y_r + extension.height_r - m_rect.y_r;
        }
    }
}
