/*
 * This software was originally developed by Arnaud Caigniet, Gregory Donias, Jean-Claude Dufourd,
 * Cdric Ggout, Erwann Gouesbet, Elouan LeCoq, Gaetan Lefevre, Nicolas Pierre, Cristian Spiescu
 * and Julien Tranchant (Streamezzo), in the course of development of the ISO/IEC 14496-20
 * (Lightweight Application Scene Representation -LASeR- and Simple Aggregation Format
 * -SAF-) standard for reference purposes and its performance may not have been optimized.
 * This software module is an implementation of one or more tools as specified by the
 * LASeR and SAF standard.
 * ISO/IEC gives users free license to this software module or modifications thereof
 * for use in products claiming conformance to audiovisual and image-coding related
 * ITU Recommendations and/or ISO/IEC International Standards.
 * ISO/IEC gives users the same free license to this software module or modifications
 * thereof for research purposes and further ISO/IEC standardisation.
 * Those intending to use this software module in products are advised that its use may
 * infringe existing patents. ISO/IEC have no liability for use of this software module
 * or modifications thereof.
 * Copyright is not released for products that do not conform to audiovisual and
 * image-coding related ITU Recommendations and/or ISO/IEC International Standards.
 * Streamezzo retains full right to modify and use the code for its own purpose,
 * assign or donate the code to a third party and to inhibit third parties from using
 * the code for products that do not conform to audiovisual and image-coding related ITU
 * Recommendations and/or ISO/IEC International Standards.
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2004-2007.
 */
package iso.mpeg.streamezzo.io;

import iso.mpeg.streamezzo.laser.*;
import iso.mpeg.streamezzo.sdl.SDLClasses;
import iso.mpeg.streamezzo.sdl.Event;

import java.io.PrintWriter;

public class XMLWriter extends PreXMLWriter {
    {
        classToWriter.put(XLASeRHeader.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XLASeRHeader xo = (XLASeRHeader) o;
                pw.print("<lsr:LASeRHeader");
                if (xo.ISO_15938_1_DecoderInit != null) {
                    pw.print(" ");
                    pw.print("ISO-15938-1-DecoderInit");
                    pw.print("='");
                    print(pw, xo.ISO_15938_1_DecoderInit);
                    pw.print("'");
                }
                if (xo.ISO_15938_1_DecoderInit_Configuration != -1) {
                    pw.print(" ");
                    pw.print("ISO-15938-1-DecoderInit-Configuration");
                    pw.print("='");
                    print(pw, xo.ISO_15938_1_DecoderInit_Configuration);
                    pw.print("'");
                }
                if (xo.newSceneIndicator != false) {
                    pw.print(" ");
                    pw.print("newSceneIndicator");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.newSceneIndicator);
                    pw.print("'");
                }
                if (xo.colorComponentBits_minus_1 != 0) {
                    pw.print(" ");
                    pw.print("colorComponentBits");
                    pw.print("='");
                    print(pw, xo.colorComponentBits_minus_1+1);
                    pw.print("'");
                }
                if (xo.extensionIDBits != 4) {
                    pw.print(" ");
                    pw.print("extensionIDBits");
                    pw.print("='");
                    print(pw, xo.extensionIDBits);
                    pw.print("'");
                }
                if (xo.level != 0) {
                    pw.print(" ");
                    pw.print("level");
                    pw.print("='");
                    print(pw, xo.level);
                    pw.print("'");
                }
                if (xo.pathComponents != -1) {
                    pw.print(" ");
                    pw.print("pathComponents");
                    pw.print("='");
                    print(pw, xo.pathComponents);
                    pw.print("'");
                }
                if (xo.pointsCodec != Integer.MIN_VALUE && xo.pointsCodec != 0) {
                    pw.print(" ");
                    pw.print("pointsCodec");
                    pw.print("='");
                    XMLWriterUtil.printtyp_pointsCodecType(pw, xo.pointsCodec);
                    pw.print("'");
                }
                if (xo.profile != Integer.MIN_VALUE && xo.profile != 0) {
                    pw.print(" ");
                    pw.print("profile");
                    pw.print("='");
                    XMLWriterUtil.printtyp_profileType(pw, xo.profile);
                    pw.print("'");
                }
                if (xo.timeResolution != -1) {
                    pw.print(" ");
                    pw.print("timeResolution");
                    pw.print("='");
                    print(pw, xo.timeResolution);
                    pw.print("'");
                    SDLClasses.timeResolution = xo.timeResolution;
                }
                if (xo.resolution != 0) {
                    pw.print(" ");
                    pw.print("resolution");
                    pw.print("='");
                    print(pw, xo.resolution);
                    pw.print("'");
                }
                if (xo.scaleBits_minus_coordBits != 0) {
                    pw.print(" ");
                    pw.print("scaleBits_minus_coordBits");
                    pw.print("='");
                    print(pw, xo.scaleBits_minus_coordBits);
                    pw.print("'");
                }
                if (xo.coordBits != 0) {
                    pw.print(" ");
                    pw.print("coordBits");
                    pw.print("='");
                    print(pw, xo.coordBits);
                    pw.print("'");
                }
                if (xo.useFullRequestHost != false) {
                    pw.print(" ");
                    pw.print("useFullRequestHost");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.useFullRequestHost);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }

    {
        classToWriter.put(Xa.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xa xo = (Xa) o;
                pw.print("<a");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xa.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xa.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.target != null) {
                    pw.print(" ");
                    pw.print("target");
                    pw.print("='");
                    print(pw, xo.target);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</a>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XcursorManager.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XcursorManager xo = (XcursorManager) o;
                pw.print("<cursor");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.x != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x");
                    pw.print("='");
                    printFloat(pw, xo.x);
                    pw.print("'");
                }
                if (xo.y != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y");
                    pw.print("='");
                    printFloat(pw, xo.y);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</cursor>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    //
    // anticipated implementation of StreamSource
    //
    {
        classToWriter.put(XstreamSource.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XstreamSource xo = (XstreamSource) o;
                pw.print("<lsr:streamSource");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.width != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("width");
                    pw.print("='");
                    printFloat(pw, xo.width);
                    pw.print("'");
                }
                if (xo.height != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("height");
                    pw.print("='");
                    printFloat(pw, xo.height);
                    pw.print("'");
                }
                if (xo.source != null) {
                    pw.print(" ");
                    pw.print("source");
                    pw.print("='");
                    printURIs(pw, xo.source);
                    pw.print("'");
                }
                if (xo.sourceIndex != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("sourceIndex");
                    pw.print("='");
                    print(pw, xo.sourceIndex);
                    pw.print("'");
                }
                if (xo.mode != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("mode");
                    pw.print("='");
                    XMLWriterUtil.printModeType(pw, xo.mode);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</lsr:streamSource>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xanimate.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xanimate xo = (Xanimate) o;
                pw.print("<animate");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.accumulate != false) {
                    pw.print(" ");
                    pw.print("accumulate");
                    pw.print("='");
                    print(pw, (xo.accumulate ? "sum" : "none"));
                    pw.print("'");
                }
                if (xo.additive != false) {
                    pw.print(" ");
                    pw.print("additive");
                    pw.print("='");
                    print(pw, (xo.additive ? "sum" : "replace"));
                    pw.print("'");
                }
                if (xo.by != null) {
                    pw.print(" ");
                    pw.print("by");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.by);
                    pw.print("'");
                }
                if (xo.calcMode != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("calcMode");
                    pw.print("='");
                    XMLWriterUtil.printtyp_calcModeType(pw, xo.calcMode);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.from != null) {
                    pw.print(" ");
                    pw.print("from");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.from);
                    pw.print("'");
                }
                if (xo.max != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("max");
                    pw.print("='");
                    printFloat(pw, xo.max);
                    pw.print("'");
                }
                if (xo.min != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("min");
                    pw.print("='");
                    printFloat(pw, xo.min);
                    pw.print("'");
                }
                if (xo.keySplines != null) {
                    pw.print(" ");
                    pw.print("keySplines");
                    pw.print("='");
                    print(pw, xo.keySplines);
                    pw.print("'");
                }
                if (xo.keyTimes != null) {
                    pw.print(" ");
                    pw.print("keyTimes");
                    pw.print("='");
                    print(pw, xo.keyTimes);
                    pw.print("'");
                }
                if (xo.values != null) {
                    pw.print(" ");
                    pw.print("values");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.values);
                    pw.print("'");
                }
                if (xo.attributeName != null) {
                    pw.print(" ");
                    pw.print("attributeName");
                    pw.print("='");
                    XMLWriterUtil.printtyp_animatable(pw, xo.attributeName);
                    pw.print("'");
                }
                if (xo.begin != null) {
                    pw.print(" ");
                    pw.print("begin");
                    pw.print("='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.dur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dur");
                    pw.print("='");
                    printDur(pw, xo.dur);
                    pw.print("'");
                }
                if (xo.enabled != false) {
                    pw.print(" ");
                    pw.print("enabled");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.enabled);
                    pw.print("'");
                }
                if (xo.end != null) {
                    pw.print(" ");
                    pw.print("end");
                    pw.print("='");
                    printTimes(pw, xo.end);
                    pw.print("'");
                }
                if (xo.fill != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("fill");
                    pw.print("='");
                    XMLWriterUtil.printtyp_animFillType(pw, xo.fill);
                    pw.print("'");
                }
                if (xo.repeatCount != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatCount");
                    pw.print("='");
                    printRepeatCount(pw, xo.repeatCount);
                    pw.print("'");
                }
                if (xo.repeatDur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatDur");
                    pw.print("='");
                    printRepeat(pw, xo.repeatDur);
                    pw.print("'");
                }
                if (xo.restart != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("restart");
                    pw.print("='");
                    XMLWriterUtil.printtyp_restartType(pw, xo.restart);
                    pw.print("'");
                }
                if (xo.to != null) {
                    pw.print(" ");
                    pw.print("to");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.to);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</animate>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XanimateColor.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XanimateColor xo = (XanimateColor) o;
                pw.print("<animateColor");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.accumulate != false) {
                    pw.print(" ");
                    pw.print("accumulate");
                    pw.print("='");
                    print(pw, (xo.accumulate ? "sum" : "none"));
                    pw.print("'");
                }
                if (xo.additive != false) {
                    pw.print(" ");
                    pw.print("additive");
                    pw.print("='");
                    print(pw, (xo.additive ? "sum" : "replace"));
                    pw.print("'");
                }
                if (xo.by != null) {
                    pw.print(" ");
                    pw.print("by");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.by);
                    pw.print("'");
                }
                if (xo.calcMode != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("calcMode");
                    pw.print("='");
                    XMLWriterUtil.printtyp_calcModeType(pw, xo.calcMode);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.from != null) {
                    pw.print(" ");
                    pw.print("from");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.from);
                    pw.print("'");
                }
                if (xo.max != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("max");
                    pw.print("='");
                    printFloat(pw, xo.max);
                    pw.print("'");
                }
                if (xo.min != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("min");
                    pw.print("='");
                    printFloat(pw, xo.min);
                    pw.print("'");
                }
                if (xo.keySplines != null) {
                    pw.print(" ");
                    pw.print("keySplines");
                    pw.print("='");
                    print(pw, xo.keySplines);
                    pw.print("'");
                }
                if (xo.keyTimes != null) {
                    pw.print(" ");
                    pw.print("keyTimes");
                    pw.print("='");
                    print(pw, xo.keyTimes);
                    pw.print("'");
                }
                if (xo.values != null) {
                    pw.print(" ");
                    pw.print("values");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.values);
                    pw.print("'");
                }
                if (xo.attributeName != null) {
                    pw.print(" ");
                    pw.print("attributeName");
                    pw.print("='");
                    XMLWriterUtil.printtyp_animatable(pw, xo.attributeName);
                    pw.print("'");
                }
                if (xo.begin != null) {
                    pw.print(" ");
                    pw.print("begin");
                    pw.print("='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.dur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dur");
                    pw.print("='");
                    printDur(pw, xo.dur);
                    pw.print("'");
                }
                if (xo.enabled != false) {
                    pw.print(" ");
                    pw.print("enabled");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.enabled);
                    pw.print("'");
                }
                if (xo.end != null) {
                    pw.print(" ");
                    pw.print("end");
                    pw.print("='");
                    printTimes(pw, xo.end);
                    pw.print("'");
                }
                if (xo.fill != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("fill");
                    pw.print("='");
                    XMLWriterUtil.printtyp_animFillType(pw, xo.fill);
                    pw.print("'");
                }
                if (xo.repeatCount != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatCount");
                    pw.print("='");
                    printRepeatCount(pw, xo.repeatCount);
                    pw.print("'");
                }
                if (xo.repeatDur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatDur");
                    pw.print("='");
                    printRepeat(pw, xo.repeatDur);
                    pw.print("'");
                }
                if (xo.restart != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("restart");
                    pw.print("='");
                    XMLWriterUtil.printtyp_restartType(pw, xo.restart);
                    pw.print("'");
                }
                if (xo.to != null) {
                    pw.print(" ");
                    pw.print("to");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.to);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</animateColor>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XanimateMotion.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XanimateMotion xo = (XanimateMotion) o;
                pw.print("<animateMotion");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.accumulate != false) {
                    pw.print(" ");
                    pw.print("accumulate");
                    pw.print("='");
                    print(pw, (xo.accumulate ? "sum" : "none"));
                    pw.print("'");
                }
                if (xo.additive != false) {
                    pw.print(" ");
                    pw.print("additive");
                    pw.print("='");
                    print(pw, (xo.additive ? "sum" : "replace"));
                    pw.print("'");
                }
                if (xo.by != null) {
                    pw.print(" ");
                    pw.print("by");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.by);
                    pw.print("'");
                }
                if (xo.calcMode != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("calcMode");
                    pw.print("='");
                    XMLWriterUtil.printtyp_calcModeType(pw, xo.calcMode);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.from != null) {
                    pw.print(" ");
                    pw.print("from");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.from);
                    pw.print("'");
                }
                if (xo.max != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("max");
                    pw.print("='");
                    printFloat(pw, xo.max);
                    pw.print("'");
                }
                if (xo.min != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("min");
                    pw.print("='");
                    printFloat(pw, xo.min);
                    pw.print("'");
                }
                if (xo.keySplines != null) {
                    pw.print(" ");
                    pw.print("keySplines");
                    pw.print("='");
                    print(pw, xo.keySplines);
                    pw.print("'");
                }
                if (xo.keyTimes != null) {
                    pw.print(" ");
                    pw.print("keyTimes");
                    pw.print("='");
                    print(pw, xo.keyTimes);
                    pw.print("'");
                }
                if (xo.values != null) {
                    pw.print(" ");
                    pw.print("values");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.values);
                    pw.print("'");
                }
                if (xo.attributeName != null) {
                    pw.print(" ");
                    pw.print("attributeName");
                    pw.print("='");
                    XMLWriterUtil.printtyp_animatable(pw, xo.attributeName);
                    pw.print("'");
                }
                if (xo.begin != null) {
                    pw.print(" ");
                    pw.print("begin");
                    pw.print("='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.dur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dur");
                    pw.print("='");
                    printDur(pw, xo.dur);
                    pw.print("'");
                }
                if (xo.enabled != false) {
                    pw.print(" ");
                    pw.print("enabled");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.enabled);
                    pw.print("'");
                }
                if (xo.end != null) {
                    pw.print(" ");
                    pw.print("end");
                    pw.print("='");
                    printTimes(pw, xo.end);
                    pw.print("'");
                }
                if (xo.fill != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("fill");
                    pw.print("='");
                    XMLWriterUtil.printtyp_animFillType(pw, xo.fill);
                    pw.print("'");
                }
                if (xo.repeatCount != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatCount");
                    pw.print("='");
                    printRepeatCount(pw, xo.repeatCount);
                    pw.print("'");
                }
                if (xo.repeatDur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatDur");
                    pw.print("='");
                    printRepeat(pw, xo.repeatDur);
                    pw.print("'");
                }
                if (xo.restart != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("restart");
                    pw.print("='");
                    XMLWriterUtil.printtyp_restartType(pw, xo.restart);
                    pw.print("'");
                }
                if (xo.to != null) {
                    pw.print(" ");
                    pw.print("to");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.to);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.keyPoints != null) {
                    pw.print(" ");
                    pw.print("keyPoints");
                    pw.print("='");
                    print(pw, xo.keyPoints);
                    pw.print("'");
                }
                if (xo.path != null) {
                    pw.print(" ");
                    pw.print("path");
                    pw.print("='");
                    print(pw, xo.path);
                    pw.print("'");
                }
                if (xo.rotate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("rotate");
                    pw.print("='");
                    if (xo.rotate == Integer.MAX_VALUE) pw.print("auto");
                    else if (xo.rotate == Integer.MAX_VALUE-1) pw.print("auto-reverse");
                    else print(pw, xo.rotate);  
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</animateMotion>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XanimateTransform.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XanimateTransform xo = (XanimateTransform) o;
                pw.print("<animateTransform");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.accumulate != false) {
                    pw.print(" ");
                    pw.print("accumulate");
                    pw.print("='");
                    print(pw, (xo.accumulate ? "sum" : "none"));
                    pw.print("'");
                }
                if (xo.additive != false) {
                    pw.print(" ");
                    pw.print("additive");
                    pw.print("='");
                    print(pw, (xo.additive ? "sum" : "replace"));
                    pw.print("'");
                }
                if (xo.by != null) {
                    pw.print(" ");
                    pw.print("by");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.by);
                    pw.print("'");
                }
                if (xo.calcMode != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("calcMode");
                    pw.print("='");
                    XMLWriterUtil.printtyp_calcModeType(pw, xo.calcMode);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.from != null) {
                    pw.print(" ");
                    pw.print("from");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.from);
                    pw.print("'");
                }
                if (xo.max != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("max");
                    pw.print("='");
                    printFloat(pw, xo.max);
                    pw.print("'");
                }
                if (xo.min != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("min");
                    pw.print("='");
                    printFloat(pw, xo.min);
                    pw.print("'");
                }
                if (xo.keySplines != null) {
                    pw.print(" ");
                    pw.print("keySplines");
                    pw.print("='");
                    print(pw, xo.keySplines);
                    pw.print("'");
                }
                if (xo.keyTimes != null) {
                    pw.print(" ");
                    pw.print("keyTimes");
                    pw.print("='");
                    print(pw, xo.keyTimes);
                    pw.print("'");
                }
                if (xo.values != null) {
                    pw.print(" ");
                    pw.print("values");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.values);
                    pw.print("'");
                }
                if (xo.attributeName != null) {
                    pw.print(" ");
                    pw.print("attributeName");
                    pw.print("='");
                    XMLWriterUtil.printtyp_animatable(pw, xo.attributeName);
                    pw.print("'");
                }
                if (xo.begin != null) {
                    pw.print(" ");
                    pw.print("begin");
                    pw.print("='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.dur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dur");
                    pw.print("='");
                    printDur(pw, xo.dur);
                    pw.print("'");
                }
                if (xo.enabled != false) {
                    pw.print(" ");
                    pw.print("enabled");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.enabled);
                    pw.print("'");
                }
                if (xo.end != null) {
                    pw.print(" ");
                    pw.print("end");
                    pw.print("='");
                    printTimes(pw, xo.end);
                    pw.print("'");
                }
                if (xo.fill != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("fill");
                    pw.print("='");
                    XMLWriterUtil.printtyp_animFillType(pw, xo.fill);
                    pw.print("'");
                }
                if (xo.repeatCount != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatCount");
                    pw.print("='");
                    printRepeatCount(pw, xo.repeatCount);
                    pw.print("'");
                }
                if (xo.repeatDur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatDur");
                    pw.print("='");
                    printRepeat(pw, xo.repeatDur);
                    pw.print("'");
                }
                if (xo.restart != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("restart");
                    pw.print("='");
                    XMLWriterUtil.printtyp_restartType(pw, xo.restart);
                    pw.print("'");
                }
                if (xo.to != null) {
                    pw.print(" ");
                    pw.print("to");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.to);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("type");
                    pw.print("='");
                    XMLWriterUtil.printtyp_rotscatra(pw, xo.type);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</animateTransform>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xaudio.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xaudio xo = (Xaudio) o;
                pw.print("<audio");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.audio_level != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("audio-level");
                    pw.print("='");
                    printFloat(pw, xo.audio_level);
                    pw.print("'");
                }
                if (xo.begin != null) {
                    pw.print(" ");
                    pw.print("begin");
                    pw.print("='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.dur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dur");
                    pw.print("='");
                    printDur(pw, xo.dur);
                    pw.print("'");
                }
                if (xo.end != null) {
                    pw.print(" ");
                    pw.print("end");
                    pw.print("='");
                    printTimes(pw, xo.end);
                    pw.print("'");
                }
                if (xo.max != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("max");
                    pw.print("='");
                    printFloat(pw, xo.max);
                    pw.print("'");
                }
                if (xo.min != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("min");
                    pw.print("='");
                    printFloat(pw, xo.min);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.repeatCount != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatCount");
                    pw.print("='");
                    printRepeatCount(pw, xo.repeatCount);
                    pw.print("'");
                }
                if (xo.repeatDur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatDur");
                    pw.print("='");
                    printRepeat(pw, xo.repeatDur);
                    pw.print("'");
                }
                if (xo.syncBehavior != Integer.MIN_VALUE) {
                    pw.print(" syncBehavior='");
                    XMLWriterUtil.printtyp_syncBehaviorType(pw, xo.syncBehavior);
                    pw.print("'");
                }
                if (xo.syncMaster != Integer.MIN_VALUE) {
                    if (xo.syncMaster == 1) pw.print(" syncMaster='true'");
                    else pw.print(" syncMaster='false'");;
                }
                if (xo.syncReference != null) {
                    pw.print(" ");
                    pw.print("lsr:syncReference");
                    pw.print("='");
                    print(pw, xo.syncReference);
                    pw.print("'");
                }
                if (xo.syncTolerance != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("syncTolerance");
                    pw.print("='");
                    printFloat(pw, xo.syncTolerance);
                    pw.print("'");
                }
                if (xo.type != null) {
                    pw.print(" ");
                    pw.print("type");
                    pw.print("='");
                    print(pw, xo.type);
                    pw.print("'");
                }
                if (xo.clipBegin != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("smil:clipBegin");
                    pw.print("='");
                    printDur(pw, xo.clipBegin);
                    pw.print("'");
                }
                if (xo.clipEnd != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("smil:clipEnd");
                    pw.print("='");
                    printDur(pw, xo.clipEnd);
                    pw.print("'");
                }
                if (xo.restart != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("restart");
                    pw.print("='");
                    XMLWriterUtil.printtyp_restartType(pw, xo.restart);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</audio>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xcircle.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xcircle xo = (Xcircle) o;
                pw.print("<circle");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xcircle.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xcircle.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.cx != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("cx");
                    pw.print("='");
                    printFloat(pw, xo.cx);
                    pw.print("'");
                }
                if (xo.cy != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("cy");
                    pw.print("='");
                    printFloat(pw, xo.cy);
                    pw.print("'");
                }
                if (xo.r != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("r");
                    pw.print("='");
                    printFloat(pw, xo.r);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</circle>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xdefs.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xdefs xo = (Xdefs) o;
                pw.print("<defs");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</defs>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xellipse.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xellipse xo = (Xellipse) o;
                pw.print("<ellipse");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xellipse.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xellipse.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.cx != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("cx");
                    pw.print("='");
                    printFloat(pw, xo.cx);
                    pw.print("'");
                }
                if (xo.cy != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("cy");
                    pw.print("='");
                    printFloat(pw, xo.cy);
                    pw.print("'");
                }
                if (xo.rx != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("rx");
                    pw.print("='");
                    printFloat(pw, xo.rx);
                    pw.print("'");
                }
                if (xo.ry != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("ry");
                    pw.print("='");
                    printFloat(pw, xo.ry);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</ellipse>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }


    {
        classToWriter.put(XforeignObject.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XforeignObject xo = (XforeignObject) o;
                pw.print("<foreignObject");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.height != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("height");
                    pw.print("='");
                    printFloat(pw, xo.height);
                    pw.print("'");
                }
                if (xo.width != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("width");
                    pw.print("='");
                    printFloat(pw, xo.width);
                    pw.print("'");
                }
                if (xo.x != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x");
                    pw.print("='");
                    printFloat(pw, xo.x);
                    pw.print("'");
                }
                if (xo.y != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y");
                    pw.print("='");
                    printFloat(pw, xo.y);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</foreignObject>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xg.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xg xo = (Xg) o;
                pw.print("<g");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xg.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xg.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</g>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XrectClip.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XrectClip xo = (XrectClip) o;
                pw.print("<lsr:rectClip");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("scale");
                    pw.print("='");
                    print(pw, XrectClip.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("translation");
                    pw.print("='");
                    print(pw, XrectClip.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.size != null) {
                    pw.print(" ");
                    pw.print("size");
                    pw.print("='");
                    print(pw, XrectClip.attributeName, xo.size);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</lsr:rectClip>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xselector.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xselector xo = (Xselector) o;
                pw.print("<lsr:selector");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("scale");
                    pw.print("='");
                    print(pw, Xselector.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("translation");
                    pw.print("='");
                    print(pw, Xselector.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.choice != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("choice");
                    pw.print("='");
                    printtyp_choiceType(pw, xo.choice);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</lsr:selector>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XsimpleLayout.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XsimpleLayout xo = (XsimpleLayout) o;
                pw.print("<lsr:simpleLayout");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("scale");
                    pw.print("='");
                    print(pw, XsimpleLayout.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("translation");
                    pw.print("='");
                    print(pw, XsimpleLayout.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.delta != null) {
                    pw.print(" ");
                    pw.print("delta");
                    pw.print("='");
                    print(pw, XsimpleLayout.attributeName, xo.delta);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</lsr:simpleLayout>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Ximage.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Ximage xo = (Ximage) o;
                pw.print("<image");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.preserveAspectRatio != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("preserveAspectRatio");
                    pw.print("='");
                    XMLWriterUtil.printtyp_preserveAspectRatioType(pw, xo.preserveAspectRatio);
                    pw.print("'");
                }
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Ximage.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Ximage.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.display != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("display");
                    pw.print("='");
                    XMLWriterUtil.printtyp_displayType(pw, xo.display);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.height != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("height");
                    pw.print("='");
                    printFloat(pw, xo.height);
                    pw.print("'");
                }
                if (xo.opacity != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("opacity");
                    pw.print("='");
                    printFloat(pw, xo.opacity);
                    pw.print("'");
                }
                if (xo.pointer_events != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("pointer-events");
                    pw.print("='");
                    XMLWriterUtil.printtyp_pointer_eventsType(pw, xo.pointer_events);
                    pw.print("'");
                }
                if (xo.transformBehavior != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:transformBehavior");
                    pw.print("='");
                    XMLWriterUtil.printtyp_transformBehaviorType(pw, xo.transformBehavior);
                    pw.print("'");
                }
                if (xo.type != null) {
                    pw.print(" ");
                    pw.print("type");
                    pw.print("='");
                    print(pw, xo.type);
                    pw.print("'");
                }
                if (xo.viewport_fill != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("viewport-fill");
                    pw.print("='");
                    printColor(pw, xo.viewport_fill);
                    pw.print("'");
                }
                if (xo.viewport_fill_opacity != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("viewport-fill-opacity");
                    pw.print("='");
                    printFloat(pw, xo.viewport_fill_opacity);
                    pw.print("'");
                }
                if (xo.visibility != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("visibility");
                    pw.print("='");
                    XMLWriterUtil.printtyp_visibilityType(pw, xo.visibility);
                    pw.print("'");
                }
                if (xo.width != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("width");
                    pw.print("='");
                    printFloat(pw, xo.width);
                    pw.print("'");
                }
                if (xo.x != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x");
                    pw.print("='");
                    printFloat(pw, xo.x);
                    pw.print("'");
                }
                if (xo.y != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y");
                    pw.print("='");
                    printFloat(pw, xo.y);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</image>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xline.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xline xo = (Xline) o;
                pw.print("<line");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xline.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xline.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.x1 != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x1");
                    pw.print("='");
                    printFloat(pw, xo.x1);
                    pw.print("'");
                }
                if (xo.x2 != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x2");
                    pw.print("='");
                    printFloat(pw, xo.x2);
                    pw.print("'");
                }
                if (xo.y1 != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y1");
                    pw.print("='");
                    printFloat(pw, xo.y1);
                    pw.print("'");
                }
                if (xo.y2 != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y2");
                    pw.print("='");
                    printFloat(pw, xo.y2);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</line>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XlinearGradient.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XlinearGradient xo = (XlinearGradient) o;
                pw.print("<linearGradient");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.gradientUnits != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("gradientUnits");
                    pw.print("='");
                    XMLWriterUtil.printtyp_gradientUnits(pw, xo.gradientUnits);
                    pw.print("'");
                }
                if (xo.x1 != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x1");
                    pw.print("='");
                    printFloat(pw, xo.x1);
                    pw.print("'");
                }
                if (xo.x2 != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x2");
                    pw.print("='");
                    printFloat(pw, xo.x2);
                    pw.print("'");
                }
                if (xo.y1 != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y1");
                    pw.print("='");
                    printFloat(pw, xo.y1);
                    pw.print("'");
                }
                if (xo.y2 != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y2");
                    pw.print("='");
                    printFloat(pw, xo.y2);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</linearGradient>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xmpath.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xmpath xo = (Xmpath) o;
                pw.print("<mpath");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</mpath>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xpath.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xpath xo = (Xpath) o;
                pw.print("<path");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xpath.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xpath.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.d != null) {
                    pw.print(" ");
                    pw.print("d");
                    pw.print("='");
                    print(pw, xo.d);
                    pw.print("'");
                }
                if (xo.pathLength != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("pathLength");
                    pw.print("='");
                    printFloat(pw, xo.pathLength);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</path>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xpolygon.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xpolygon xo = (Xpolygon) o;
                pw.print("<polygon");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xpolygon.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xpolygon.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.points != null) {
                    pw.print(" ");
                    pw.print("points");
                    pw.print("='");
                    print(pw, xo.points);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</polygon>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xpolyline.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xpolyline xo = (Xpolyline) o;
                pw.print("<polyline");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xpolyline.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xpolyline.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.points != null) {
                    pw.print(" ");
                    pw.print("points");
                    pw.print("='");
                    print(pw, xo.points);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</polyline>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XradialGradient.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XradialGradient xo = (XradialGradient) o;
                pw.print("<radialGradient");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.cx != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("cx");
                    pw.print("='");
                    printFloat(pw, xo.cx);
                    pw.print("'");
                }
                if (xo.cy != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("cy");
                    pw.print("='");
                    printFloat(pw, xo.cy);
                    pw.print("'");
                }
                if (xo.gradientUnits != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("gradientUnits");
                    pw.print("='");
                    XMLWriterUtil.printtyp_gradientUnits(pw, xo.gradientUnits);
                    pw.print("'");
                }
                if (xo.r != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("r");
                    pw.print("='");
                    printFloat(pw, xo.r);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</radialGradient>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xrect.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xrect xo = (Xrect) o;
                pw.print("<rect");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xrect.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xrect.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.height != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("height");
                    pw.print("='");
                    printFloat(pw, xo.height);
                    pw.print("'");
                }
                if (xo.rx != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("rx");
                    pw.print("='");
                    printFloat(pw, xo.rx);
                    pw.print("'");
                }
                if (xo.ry != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("ry");
                    pw.print("='");
                    printFloat(pw, xo.ry);
                    pw.print("'");
                }
                if (xo.width != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("width");
                    pw.print("='");
                    printFloat(pw, xo.width);
                    pw.print("'");
                }
                if (xo.x != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x");
                    pw.print("='");
                    printFloat(pw, xo.x);
                    pw.print("'");
                }
                if (xo.y != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y");
                    pw.print("='");
                    printFloat(pw, xo.y);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</rect>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xscript.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xscript xo = (Xscript) o;
                pw.print("<script");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.type != null) {
                    pw.print(" ");
                    pw.print("type");
                    pw.print("='");
                    print(pw, xo.type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</script>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xconditional.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xconditional xo = (Xconditional) o;
                pw.print("<lsr:conditional");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.begin != null) {
                    pw.print(" ");
                    pw.print("begin");
                    pw.print("='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.enabled != false) {
                    pw.print(" ");
                    pw.print("enabled");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.enabled);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.type != null) {
                    pw.print(" ");
                    pw.print("type");
                    pw.print("='");
                    print(pw, xo.type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</lsr:conditional>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XAdd.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XAdd xo = (XAdd) o;
                pw.print("<lsr:Add");
                if (xo.attributeName != null) {
                    pw.print(" ");
                    pw.print("attributeName");
                    pw.print("='");
                    XMLWriterUtil.printAttributeName(pw, xo.attributeName);
                    pw.print("'");
                }
                if (xo.operandAttribute != null) {
                    pw.print(" ");
                    pw.print("operandAttribute");
                    pw.print("='");
                    XMLWriterUtil.printAttributeName(pw, xo.operandAttribute);
                    pw.print("'");
                }
                if (xo.operandElementId != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("operandId");
                    pw.print("='");
                    printID(pw, xo.operandElementId);
                    pw.print("'");
                }
                if (xo.ref != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("ref");
                    pw.print("='");
                    printID(pw, xo.ref);
                    pw.print("'");
                }
                if (xo.value != null) {
                    pw.print(" ");
                    pw.print("value");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.value);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }

    {
        classToWriter.put(XClean.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XClean xo = (XClean) o;
                pw.print("<lsr:Clean");
                if (xo.groupID != null) {
                    pw.print(" ");
                    pw.print("groupID");
                    pw.print("='");
                    pw.print(xo.groupID);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }

    {
        classToWriter.put(XDelete.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XDelete xo = (XDelete) o;
                pw.print("<lsr:Delete");
                if (xo.index != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("index");
                    pw.print("='");
                    print(pw, xo.index);
                    pw.print("'");
                }
                if (xo.ref != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("ref");
                    pw.print("='");
                    printID(pw, xo.ref);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }

    {
        classToWriter.put(XActivate.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XActivate xo = (XActivate) o;
                pw.print("<lsr:Activate");
                if (xo.ref != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("ref");
                    pw.print("='");
                    printID(pw, xo.ref);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }

    {
        classToWriter.put(XDeactivate.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XDeactivate xo = (XDeactivate) o;
                pw.print("<lsr:Deactivate");
                if (xo.ref != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("ref");
                    pw.print("='");
                    printID(pw, xo.ref);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }

    {
        classToWriter.put(XInsert.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XInsert xo = (XInsert) o;
                pw.print("<lsr:Insert");
                if (xo.index != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("index");
                    pw.print("='");
                    print(pw, xo.index);
                    pw.print("'");
                }
                if (xo.ref != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("ref");
                    pw.print("='");
                    printID(pw, xo.ref);
                    pw.print("'");
                }
                if (xo.value != null) {
                    pw.print(" ");
                    pw.print("value");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.value);
                    pw.print("'");
                }
                if (xo.child != null) {
                    if (xo.attributeName != null /*&& xo.attributeName != BinOut.attr_children*/) {
                        pw.print(" ");
                        pw.print("attributeName");
                        pw.print("='");
                        XMLWriterUtil.printAttributeName(pw, xo.attributeName);
                        pw.print("'");
                    }
                    pw.println(">");
                    writeAny(pw, xo.child);
                    pw.println("</lsr:Insert>");
                } else {
                    if (xo.attributeName != null) {
                        pw.print(" ");
                        pw.print("attributeName");
                        pw.print("='");
                        XMLWriterUtil.printAttributeName(pw, xo.attributeName);
                        pw.print("'");
                    }
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XNewScene.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XNewScene xo = (XNewScene) o;
                pw.print("<lsr:NewScene");
                if (xo.child != null) {
                    pw.println(">");
                    writeAny(pw, xo.child);
                    pw.println("</lsr:NewScene>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xsvg.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xsvg xo = (Xsvg) o;
                pw.print("<svg");
                if (xo.contentScriptType != null) {
                    pw.print(" ");
                    pw.print("contentScriptType");
                    pw.print("='");
                    pw.print(xo.contentScriptType);
                    pw.print("'");
                }
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.height != null) {
                    pw.print(" ");
                    pw.print("height");
                    pw.print("='");
                    printUnits(pw, xo.height);
                    pw.print("'");
                }
                if (xo.preserveAspectRatio != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("preserveAspectRatio");
                    pw.print("='");
                    XMLWriterUtil.printtyp_preserveAspectRatioType(pw, xo.preserveAspectRatio);
                    pw.print("'");
                }
                if (xo.syncBehaviorDefault != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("syncBehaviorDefault");
                    pw.print("='");
                    XMLWriterUtil.printtyp_syncBehaviorDefaultType(pw, xo.syncBehaviorDefault);
                    pw.print("'");
                }
                if (xo.syncToleranceDefault != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("syncToleranceDefault");
                    pw.print("='");
                    printFloat(pw, xo.syncToleranceDefault);
                    pw.print("'");
                }
                if (xo.viewBox != null) {
                    pw.print(" ");
                    pw.print("viewBox");
                    pw.print("='");
                    print(pw, xo.viewBox);
                    pw.print("'");
                }
                if (xo.width != null) {
                    pw.print(" ");
                    pw.print("width");
                    pw.print("='");
                    printUnits(pw, xo.width);
                    pw.print("'");
                }
                if (xo.zoomAndPan == 0) {
                    pw.print(" ");
                    pw.print("zoomAndPan='disable'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</svg>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XReplace.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XReplace xo = (XReplace) o;
                pw.print("<lsr:Replace");
                if (xo.attributeName != null) {
                    pw.print(" ");
                    pw.print("attributeName");
                    pw.print("='");
                    XMLWriterUtil.printAttributeName(pw, xo.attributeName);
                    pw.print("'");
                }
                if (xo.index != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("index");
                    pw.print("='");
                    print(pw, xo.index);
                    pw.print("'");
                }
                if (xo.ref != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("ref");
                    pw.print("='");
                    printID(pw, xo.ref);
                    pw.print("'");
                }
                if (xo.value != null) {
                    pw.print(" ");
                    pw.print("value");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.value);
                    pw.print("'");
                }
                if (xo.child != null) {
                    if (xo.child.elementAt(0) instanceof XtextContent) pw.print(">");
                    else pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</lsr:Replace>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XRestore.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XRestore xo = (XRestore) o;
                pw.print("<lsr:Restore");
                if (xo.groupID != null) {
                    pw.print(" ");
                    pw.print("groupID");
                    pw.print("='");
                    pw.print(xo.groupID);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }

    {
        classToWriter.put(XSave.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XSave xo = (XSave) o;
                pw.print("<lsr:Save");
                if (xo.attributes != null) {
                    pw.print(" ");
                    pw.print("attributes");
                    pw.print("='");
                    printAttributeNameList(pw, xo.attributes);
                    pw.print("'");
                }
                if (xo.elements != null) {
                    pw.print(" ");
                    pw.print("elements");
                    pw.print("='");
                    printIDs(pw, xo.elements);
                    pw.print("'");
                }
                if (xo.groupID != null) {
                    pw.print(" ");
                    pw.print("groupID");
                    pw.print("='");
                    pw.print(xo.groupID);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }

    {
        classToWriter.put(XSendEvent.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XSendEvent xo = (XSendEvent) o;
                pw.print("<lsr:SendEvent");
                if (xo.event != null) {
                    pw.print(" ");
                    pw.print("event");
                    pw.print("='");
                    xo.event.print(pw);
                    pw.print("'");
                }
                if (xo.intvalue != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("intvalue");
                    pw.print("='");
                    print(pw, xo.intvalue);
                    pw.print("'");
                }
                if (xo.pointvalue != null) {
                    pw.print(" ");
                    pw.print("pointvalue");
                    pw.print("='");
                    print(pw, XSendEvent.attributeName, xo.pointvalue);
                    pw.print("'");
                }
                if (xo.ref != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("ref");
                    pw.print("='");
                    printID(pw, xo.ref);
                    pw.print("'");
                }
                if (xo.stringvalue != null) {
                    pw.print(" ");
                    pw.print("stringvalue");
                    pw.print("='");
                    print(pw, xo.stringvalue);
                    pw.print("'");
                }
                pw.println("/>");
            }
        });
    }


    {
        classToWriter.put(Xset.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xset xo = (Xset) o;
                pw.print("<set");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.attributeName != null) {
                    pw.print(" ");
                    pw.print("attributeName");
                    pw.print("='");
                    XMLWriterUtil.printtyp_animatable(pw, xo.attributeName);
                    pw.print("'");
                }
                if (xo.begin != null) {
                    pw.print(" ");
                    pw.print("begin");
                    pw.print("='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.dur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dur");
                    pw.print("='");
                    printDur(pw, xo.dur);
                    pw.print("'");
                }
                if (xo.enabled != false) {
                    pw.print(" ");
                    pw.print("enabled");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.enabled);
                    pw.print("'");
                }
                if (xo.end != null) {
                    pw.print(" ");
                    pw.print("end");
                    pw.print("='");
                    printTimes(pw, xo.end);
                    pw.print("'");
                }
                if (xo.fill != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("fill");
                    pw.print("='");
                    XMLWriterUtil.printtyp_animFillType(pw, xo.fill);
                    pw.print("'");
                }
                if (xo.repeatCount != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatCount");
                    pw.print("='");
                    printRepeatCount(pw, xo.repeatCount);
                    pw.print("'");
                }
                if (xo.repeatDur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatDur");
                    pw.print("='");
                    printRepeat(pw, xo.repeatDur);
                    pw.print("'");
                }
                if (xo.restart != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("restart");
                    pw.print("='");
                    XMLWriterUtil.printtyp_restartType(pw, xo.restart);
                    pw.print("'");
                }
                if (xo.to != null) {
                    pw.print(" ");
                    pw.print("to");
                    pw.print("='");
                    print(pw, xo.attributeName, xo.to);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</set>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xstop.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xstop xo = (Xstop) o;
                pw.print("<stop");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.offset != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("offset");
                    pw.print("='");
                    printFloat(pw, xo.offset);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</stop>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xswitch.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xswitch xo = (Xswitch) o;
                pw.print("<switch");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</switch>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xtext.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xtext xo = (Xtext) o;
                pw.print("<text");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xtext.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xtext.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.editable != false) {
                    pw.print(" ");
                    pw.print("editable");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.editable);
                    pw.print("'");
                }
                if (xo.rotate != null) {
                    pw.print(" ");
                    pw.print("rotate");
                    pw.print("='");
                    print(pw, xo.rotate);
                    pw.print("'");
                }
                if (xo.x != null) {
                    pw.print(" ");
                    pw.print("x");
                    pw.print("='");
                    print(pw, xo.x);
                    pw.print("'");
                }
                if (xo.y != null) {
                    pw.print(" ");
                    pw.print("y");
                    pw.print("='");
                    print(pw, xo.y);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    if (xo.child.elementAt(0) instanceof XtextContent) pw.print(">");
                    else pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</text>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xlistener.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xlistener xo = (Xlistener) o;
                pw.print("<ev:listener");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.handler != null) {
                    pw.print(" ");
                    pw.print("handler");
                    pw.print("='");
                    print(pw, xo.handler);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                /*if (xo.delay != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:delay");
                    pw.print("='");
                    printFloat(pw, xo.delay);
                    pw.print("'");
                }*/
                if (xo.enabled != false) {
                    pw.print(" ");
                    pw.print("enabled");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.enabled);
                    pw.print("'");
                }
                if (xo.event != null) {
                    pw.print(" ");
                    pw.print("event");
                    pw.print("='");
                    xo.event.print(pw);
                    if (xo.keyCode != -1) {
                        pw.print("(");
                        Event.printKeyCode(pw, xo.keyCode);
                        pw.print(")");
                    }
                    pw.print("'");
                }
                if (xo.observer != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("observer");
                    pw.print("='");
                    printID(pw, xo.observer);
                    pw.print("'");
                }
                if (xo.propagate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("propagate");
                    pw.print("='");
                    pw.print((xo.propagate == 0?"continue":"stop"));
                    pw.print("'");
                }
                if (xo.defaultAction != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("defaultAction");
                    pw.print("='");
                    pw.print((xo.defaultAction == 0?"cancel":"perform"));
                    pw.print("'");
                }
                if (xo.target != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("target");
                    pw.print("='#");
                    printID(pw, xo.target);
                    pw.print("'");
                }
                /*if (xo.timeAttribute != Integer.MIN_VALUE && xo.timeAttribute != 0) {
                    pw.print(" lsr:timeAttribute='end'");
                }*/
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</ev:listener>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xtspan.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xtspan xo = (Xtspan) o;
                pw.print("<tspan");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    if (xo.child.elementAt(0) instanceof XtextContent) pw.print(">");
                    else pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.print("</tspan>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xuse.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xuse xo = (Xuse) o;
                pw.print("<use");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xuse.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xuse.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.overflow != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("overflow");
                    pw.print("='");
                    XMLWriterUtil.printtyp_overflowType(pw, xo.overflow);
                    pw.print("'");
                }
                if (xo.x != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x");
                    pw.print("='");
                    printFloat(pw, xo.x);
                    pw.print("'");
                }
                if (xo.y != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y");
                    pw.print("='");
                    printFloat(pw, xo.y);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</use>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xvideo.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xvideo xo = (Xvideo) o;
                pw.print("<video");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.preserveAspectRatio != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("preserveAspectRatio");
                    pw.print("='");
                    XMLWriterUtil.printtyp_preserveAspectRatioType(pw, xo.preserveAspectRatio);
                    pw.print("'");
                }
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("lsr:scale");
                    pw.print("='");
                    print(pw, Xvideo.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("lsr:translation");
                    pw.print("='");
                    print(pw, Xvideo.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.audio_level != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("audio-level");
                    pw.print("='");
                    printFloat(pw, xo.audio_level);
                    pw.print("'");
                }
                if (xo.begin != null) {
                    pw.print(" ");
                    pw.print("begin");
                    pw.print("='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.dur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dur");
                    pw.print("='");
                    printDur(pw, xo.dur);
                    pw.print("'");
                }
                if (xo.display != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("display");
                    pw.print("='");
                    XMLWriterUtil.printtyp_displayType(pw, xo.display);
                    pw.print("'");
                }
                if (xo.end != null) {
                    pw.print(" ");
                    pw.print("end");
                    pw.print("='");
                    printTimes(pw, xo.end);
                    pw.print("'");
                }
                if (xo.max != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("max");
                    pw.print("='");
                    printFloat(pw, xo.max);
                    pw.print("'");
                }
                if (xo.min != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("min");
                    pw.print("='");
                    printFloat(pw, xo.min);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.height != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("height");
                    pw.print("='");
                    printFloat(pw, xo.height);
                    pw.print("'");
                }
                if (xo.overlay != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("overlay");
                    pw.print("='");
                    print(pw, xo.overlay);
                    pw.print("'");
                }
                if (xo.pointer_events != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("pointer-events");
                    pw.print("='");
                    XMLWriterUtil.printtyp_pointer_eventsType(pw, xo.pointer_events);
                    pw.print("'");
                }
                if (xo.repeatCount != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatCount");
                    pw.print("='");
                    printRepeatCount(pw, xo.repeatCount);
                    pw.print("'");
                }
                if (xo.repeatDur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatDur");
                    pw.print("='");
                    printRepeat(pw, xo.repeatDur);
                    pw.print("'");
                }
                if (xo.syncBehavior != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("syncBehavior");
                    pw.print("='");
                    XMLWriterUtil.printtyp_syncBehaviorType(pw, xo.syncBehavior);
                    pw.print("'");
                }
                if (xo.syncMaster != Integer.MIN_VALUE) {
                    if (xo.syncMaster == 1) pw.print(" syncMaster='true'");
                    else pw.print(" syncMaster='false'");;
                }
                if (xo.syncReference != null) {
                    pw.print(" ");
                    pw.print("lsr:syncReference");
                    pw.print("='");
                    print(pw, xo.syncReference);
                    pw.print("'");
                }
                if (xo.syncTolerance != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("syncTolerance");
                    pw.print("='");
                    printFloat(pw, xo.syncTolerance);
                    pw.print("'");
                }
                if (xo.transformBehavior != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("transformBehavior");
                    pw.print("='");
                    XMLWriterUtil.printtyp_transformBehaviorType(pw, xo.transformBehavior);
                    pw.print("'");
                }
                if (xo.type != null) {
                    pw.print(" ");
                    pw.print("type");
                    pw.print("='");
                    print(pw, xo.type);
                    pw.print("'");
                }
                if (xo.visibility != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("visibility");
                    pw.print("='");
                    XMLWriterUtil.printtyp_visibilityType(pw, xo.visibility);
                    pw.print("'");
                }
                if (xo.width != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("width");
                    pw.print("='");
                    printFloat(pw, xo.width);
                    pw.print("'");
                }
                if (xo.x != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x");
                    pw.print("='");
                    printFloat(pw, xo.x);
                    pw.print("'");
                }
                if (xo.y != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y");
                    pw.print("='");
                    printFloat(pw, xo.y);
                    pw.print("'");
                }
                if (xo.clipBegin != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("smil:clipBegin");
                    pw.print("='");
                    printDur(pw, xo.clipBegin);
                    pw.print("'");
                }
                if (xo.clipEnd != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("smil:clipEnd");
                    pw.print("='");
                    printDur(pw, xo.clipEnd);
                    pw.print("'");
                }
                if (xo.fullscreen != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("lsr:fullscreen");
                    pw.print("='");
                    XMLWriterUtil.printfullscreen(pw, xo.fullscreen);
                    pw.print("'");
                }
                if (xo.restart != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("restart");
                    pw.print("='");
                    XMLWriterUtil.printtyp_restartType(pw, xo.restart);
                    pw.print("'");
                }
                if (xo.initialVisibility != Integer.MIN_VALUE) {
                    pw.print(" initialVisibility='");
                    if (xo.initialVisibility == 0) pw.print("always");
                    else pw.print("whenStarted");
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</video>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    /*{
        classToWriter.put(XupdateList.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XupdateList xo = (XupdateList) o;
                pw.print("<lsr:updateList");
                if (xo.child != null && xo.child.delta() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.delta(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</lsr:updateList>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }*/

    {
        classToWriter.put(Xtitle.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xtitle xo = (Xtitle) o;
                pw.print("<title");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</title>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xdesc.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xdesc xo = (Xdesc) o;
                pw.print("<desc");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</desc>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xmetadata.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xmetadata xo = (Xmetadata) o;
                pw.print("<metadata");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</metadata>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xanimation.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xanimation xo = (Xanimation) o;
                pw.print("<animation");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.begin != null) {
                    pw.print(" ");
                    pw.print("begin");
                    pw.print("='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.dur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dur");
                    pw.print("='");
                    printDur(pw, xo.dur);
                    pw.print("'");
                }
                if (xo.end != null) {
                    pw.print(" ");
                    pw.print("end");
                    pw.print("='");
                    printTimes(pw, xo.end);
                    pw.print("'");
                }
                if (xo.fill != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("fill");
                    pw.print("='");
                    XMLWriterUtil.printtyp_animFillType(pw, xo.fill);
                    pw.print("'");
                }
                if (xo.repeatCount != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatCount");
                    pw.print("='");
                    printRepeatCount(pw, xo.repeatCount);
                    pw.print("'");
                }
                if (xo.repeatDur != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("repeatDur");
                    pw.print("='");
                    printRepeat(pw, xo.repeatDur);
                    pw.print("'");
                }
                if (xo.restart != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("restart");
                    pw.print("='");
                    XMLWriterUtil.printtyp_restartType(pw, xo.restart);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.min != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("min");
                    pw.print("='");
                    printDur(pw, xo.min);
                    pw.print("'");
                }
                if (xo.max != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("max");
                    pw.print("='");
                    printDur(pw, xo.max);
                    pw.print("'");
                }
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("scale");
                    pw.print("='");
                    print(pw, XrectClip.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("translation");
                    pw.print("='");
                    print(pw, XrectClip.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.width != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("width");
                    pw.print("='");
                    printFloat(pw, xo.width);
                    pw.print("'");
                }
                if (xo.height != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("height");
                    pw.print("='");
                    printFloat(pw, xo.height);
                    pw.print("'");
                }
                if (xo.syncBehavior != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("syncBehavior");
                    pw.print("='");
                    XMLWriterUtil.printtyp_syncBehaviorType(pw, xo.syncBehavior);
                    pw.print("'");
                }
                if (xo.syncMaster != Integer.MIN_VALUE) {
                    if (xo.syncMaster == 1) pw.print(" syncMaster='true'");
                    else pw.print(" syncMaster='false'");;
                }
                if (xo.syncReference != null) {
                    pw.print(" ");
                    pw.print("lsr:syncReference");
                    pw.print("='");
                    print(pw, xo.syncReference);
                    pw.print("'");
                }
                if (xo.syncTolerance != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("syncTolerance");
                    pw.print("='");
                    printFloat(pw, xo.syncTolerance);
                    pw.print("'");
                }
                if (xo.type != null) {
                    pw.print(" ");
                    pw.print("type");
                    pw.print("='");
                    print(pw, xo.type);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.x != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x");
                    pw.print("='");
                    printFloat(pw, xo.x);
                    pw.print("'");
                }
                if (xo.y != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y");
                    pw.print("='");
                    printFloat(pw, xo.y);
                    pw.print("'");
                }
                if (xo.clipBegin != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("smil:clipBegin");
                    pw.print("='");
                    printDur(pw, xo.clipBegin);
                    pw.print("'");
                }
                if (xo.clipEnd != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("smil:clipEnd");
                    pw.print("='");
                    printDur(pw, xo.clipEnd);
                    pw.print("'");
                }
                if (xo.preserveAspectRatio != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("preserveAspectRatio");
                    pw.print("='");
                    XMLWriterUtil.printtyp_preserveAspectRatioType(pw, xo.preserveAspectRatio);
                    pw.print("'");
                }
                if (xo.display != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("display");
                    pw.print("='");
                    XMLWriterUtil.printtyp_displayType(pw, xo.display);
                    pw.print("'");
                }
                if (xo.visibility != Integer.MIN_VALUE) {
                    pw.print(" visibility='");
                    XMLWriterUtil.printtyp_visibilityType(pw, xo.visibility);
                    pw.print("'");
                }
                if (xo.initialVisibility != Integer.MIN_VALUE) {
                    pw.print(" initialVisibility='");
                    if (xo.initialVisibility == 0) pw.print("always");
                    else pw.print("whenStarted");
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</animation>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xdiscard.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xdiscard xo = (Xdiscard) o;
                pw.print("<discard");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.begin != null) {
                    pw.print(" ");
                    pw.print("begin");
                    pw.print("='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</discard>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xfont.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xfont xo = (Xfont) o;
                pw.print("<font");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.horizAdvX != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("horiz-adv-x");
                    pw.print("='");
                    printFloat(pw, xo.horizAdvX);
                    pw.print("'");
                }
                if (xo.horizOriginX != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("horiz-origin-x");
                    pw.print("='");
                    printFloat(pw, xo.horizOriginX);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</font>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XfontFace.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XfontFace xo = (XfontFace) o;
                pw.print("<font-face");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.accentHeight != Integer.MIN_VALUE) {
                    pw.print(" accent-height='");
                    printFloat(pw, xo.accentHeight);
                    pw.print("'");
                }
                if (xo.alphabetic != Integer.MIN_VALUE) {
                    pw.print(" alphabetic='");
                    printFloat(pw, xo.alphabetic);
                    pw.print("'");
                }
                if (xo.ascent != Integer.MIN_VALUE) {
                    pw.print(" ascent='");
                    printFloat(pw, xo.ascent);
                    pw.print("'");
                }
                if (xo.bbox != null) {
                    pw.print(" bbox='");
                    print(pw, xo.bbox);
                    pw.print("'");
                }
                if (xo.capHeight != Integer.MIN_VALUE) {
                    pw.print(" cap-height='");
                    printFloat(pw, xo.capHeight);
                    pw.print("'");
                }
                if (xo.descent != Integer.MIN_VALUE) {
                    pw.print(" descent='");
                    printFloat(pw, xo.descent);
                    pw.print("'");
                }
                if (xo.fontFamily != null) {
                    pw.print(" font-family='");
                    print(pw, xo.fontFamily);
                    pw.print("'");
                }
                if (xo.fontStretch != null) {
                    pw.print(" font-stretch='");
                    print(pw, xo.fontStretch);
                    pw.print("'");
                }
                if (xo.fontStyle != null) {
                    pw.print(" font-style='");
                    print(pw, xo.fontStyle);
                    pw.print("'");
                }
                if (xo.fontVariant != null) {
                    pw.print(" font-variant='");
                    print(pw, xo.fontVariant);
                    pw.print("'");
                }
                if (xo.fontWeight != null) {
                    pw.print(" font-weight='");
                    print(pw, xo.fontWeight);
                    pw.print("'");
                }
                if (xo.hanging != Integer.MIN_VALUE) {
                    pw.print(" hanging='");
                    printFloat(pw, xo.hanging);
                    pw.print("'");
                }
                if (xo.ideographic != Integer.MIN_VALUE) {
                    pw.print(" ideographic='");
                    printFloat(pw, xo.ideographic);
                    pw.print("'");
                }
                if (xo.mathematical != Integer.MIN_VALUE) {
                    pw.print(" mathematical='");
                    printFloat(pw, xo.mathematical);
                    pw.print("'");
                }
                if (xo.overlinePosition != Integer.MIN_VALUE) {
                    pw.print(" overline-position='");
                    printFloat(pw, xo.overlinePosition);
                    pw.print("'");
                }
                if (xo.overlineThickness != Integer.MIN_VALUE) {
                    pw.print(" overline-thickness='");
                    printFloat(pw, xo.overlineThickness);
                    pw.print("'");
                }
                if (xo.panose1 != null) {
                    pw.print(" panose-1='");
                    print(pw, xo.panose1);
                    pw.print("'");
                }
                if (xo.slope != Integer.MIN_VALUE) {
                    pw.print(" slope='");
                    printFloat(pw, xo.slope);
                    pw.print("'");
                }
                if (xo.stemh != Integer.MIN_VALUE) {
                    pw.print(" stemh='");
                    printFloat(pw, xo.stemh);
                    pw.print("'");
                }
                if (xo.stemv != Integer.MIN_VALUE) {
                    pw.print(" stemv='");
                    printFloat(pw, xo.stemv);
                    pw.print("'");
                }
                if (xo.strikethroughPosition != Integer.MIN_VALUE) {
                    pw.print(" strikethrough-position='");
                    printFloat(pw, xo.strikethroughPosition);
                    pw.print("'");
                }
                if (xo.strikethroughThickness != Integer.MIN_VALUE) {
                    pw.print(" strikethrough-thickness='");
                    printFloat(pw, xo.strikethroughThickness);
                    pw.print("'");
                }
                if (xo.underlinePosition != Integer.MIN_VALUE) {
                    pw.print(" underline-position='");
                    printFloat(pw, xo.underlinePosition);
                    pw.print("'");
                }
                if (xo.underlineThickness != Integer.MIN_VALUE) {
                    pw.print(" underline-thickness='");
                    printFloat(pw, xo.underlineThickness);
                    pw.print("'");
                }
                if (xo.unicodeRange != null) {
                    pw.print(" unicode-range='");
                    print(pw, xo.unicodeRange);
                    pw.print("'");
                }
                if (xo.widths != null) {
                    pw.print(" widths='");
                    print(pw, xo.widths);
                    pw.print("'");
                }
                if (xo.xHeight != Integer.MIN_VALUE) {
                    pw.print(" x-height='");
                    printFloat(pw, xo.xHeight);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</font-face>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XfontFaceSrc.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XfontFaceSrc xo = (XfontFaceSrc) o;
                pw.print("<font-face-src");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</font-face-src>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XfontFaceUri.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XfontFaceUri xo = (XfontFaceUri) o;
                pw.print("<font-face-uri");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</font-face-uri>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xglyph.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xglyph xo = (Xglyph) o;
                pw.print("<glyph");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.arabicForm != null) {
                    pw.print(" arabic-form='");
                    print(pw, xo.arabicForm);
                    pw.print("'");
                }
                if (xo.d != null) {
                    pw.print(" d='");
                    print(pw, xo.d);
                    pw.print("'");
                }
                if (xo.glyphName != null) {
                    pw.print(" glyph-name='");
                    print(pw, xo.glyphName);
                    pw.print("'");
                }
                if (xo.horizAdvX != Integer.MIN_VALUE) {
                    pw.print(" horiz-adv-x='");
                    printFloat(pw, xo.horizAdvX);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" lang='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.unicode != null) {
                    pw.print(" unicode='");
                    print(pw, xo.unicode);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</glyph>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xhandler.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xhandler xo = (Xhandler) o;
                pw.print("<handler");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" externalResourcesRequired='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:show");
                    pw.print("='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" ");
                    pw.print("xlink:title");
                    pw.print("='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:type");
                    pw.print("='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.event != null) {
                    pw.print(" ev:event='");
                    xo.event.print(pw);
                    pw.print("'");
                }
                if (xo.type != null) {
                    pw.print(" type='");
                    print(pw, xo.type);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</handler>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xhkern.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xhkern xo = (Xhkern) o;
                pw.print("<hkern");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.g1 != null) {
                    pw.print(" g1='");
                    print(pw, xo.g1);
                    pw.print("'");
                }
                if (xo.g2 != null) {
                    pw.print(" g2='");
                    print(pw, xo.g2);
                    pw.print("'");
                }
                if (xo.k != Integer.MIN_VALUE) {
                    pw.print(" k='");
                    printFloat(pw, xo.k);
                    pw.print("'");
                }
                if (xo.u1 != null) {
                    pw.print(" u1='");
                    print(pw, xo.u1);
                    pw.print("'");
                }
                if (xo.u2 != null) {
                    pw.print(" u2='");
                    print(pw, xo.u2);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</hkern>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XmissingGlyph.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XmissingGlyph xo = (XmissingGlyph) o;
                pw.print("<missing-glyph");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.d != null) {
                    pw.print(" d='");
                    print(pw, xo.d);
                    pw.print("'");
                }
                if (xo.horizAdvX != Integer.MIN_VALUE) {
                    pw.print(" horiz-adv-x='");
                    print(pw, xo.horizAdvX);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</missing-glyph>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(Xprefetch.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                Xprefetch xo = (Xprefetch) o;
                pw.print("<prefetch");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.externalResourcesRequired != false) {
                    pw.print(" ");
                    pw.print("externalResourcesRequired");
                    pw.print("='");
                    XMLWriterUtil.print(pw, xo.externalResourcesRequired);
                    pw.print("'");
                }
                if (xo.bandwidth != Integer.MIN_VALUE) {
                    pw.print(" bandwidth='");
                    printFloat(pw, xo.bandwidth);
                    pw.print("'");
                }
                if (xo.mediaCharacterEncoding != null) {
                    pw.print(" mediaCharacterEncoding='");
                    print(pw, xo.mediaCharacterEncoding);
                    pw.print("'");
                }
                if (xo.mediaContentEncodings != null) {
                    pw.print(" mediaContentEncodings='");
                    print(pw, xo.mediaContentEncodings);
                    pw.print("'");
                }
                if (xo.mediaSize != Integer.MIN_VALUE) {
                    pw.print(" mediaSize='");
                    print(pw, xo.mediaSize);
                    pw.print("'");
                }
                if (xo.mediaTime != Integer.MIN_VALUE) {
                    pw.print(" mediaTime='");
                    print(pw, xo.mediaTime);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" xlink:show='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" xlink:title='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" xlink:type='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</prefetch>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XsolidColor.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XsolidColor xo = (XsolidColor) o;
                pw.print("<solidColor");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</solidColor>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XtBreak.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XtBreak xo = (XtBreak) o;
                pw.print("<tBreak");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</tBreak>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XtextArea.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XtextArea xo = (XtextArea) o;
                pw.print("<textArea");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                printProperties(xo, pw);
                if (xo.requiredExtensions != null) {
                    pw.print(" ");
                    pw.print("requiredExtensions");
                    pw.print("='");
                    print(pw, xo.requiredExtensions);
                    pw.print("'");
                }
                if (xo.requiredFeatures != null) {
                    pw.print(" ");
                    pw.print("requiredFeatures");
                    pw.print("='");
                    print(pw, xo.requiredFeatures);
                    pw.print("'");
                }
                if (xo.requiredFormats != null) {
                    pw.print(" ");
                    pw.print("requiredFormats");
                    pw.print("='");
                    print(pw, xo.requiredFormats);
                    pw.print("'");
                }
                if (xo.systemLanguage != null) {
                    pw.print(" ");
                    pw.print("systemLanguage");
                    pw.print("='");
                    print(pw, xo.systemLanguage);
                    pw.print("'");
                }
                if (xo.rotation != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("rotation");
                    pw.print("='");
                    printFloat(pw, xo.rotation);
                    pw.print("'");
                }
                if (xo.scale != null) {
                    pw.print(" ");
                    pw.print("scale");
                    pw.print("='");
                    print(pw, XrectClip.attributeName, xo.scale);
                    pw.print("'");
                }
                if (xo.transform != null) {
                    pw.print(" ");
                    pw.print("transform");
                    pw.print("='");
                    print(pw, xo.transform);
                    pw.print("'");
                }
                if (xo.translation != null) {
                    pw.print(" ");
                    pw.print("translation");
                    pw.print("='");
                    print(pw, XrectClip.attributeName, xo.translation);
                    pw.print("'");
                }
                if (xo.width != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("width");
                    pw.print("='");
                    printFloat(pw, xo.width);
                    pw.print("'");
                }
                if (xo.height != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("height");
                    pw.print("='");
                    printFloat(pw, xo.height);
                    pw.print("'");
                }
                if (xo.x != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("x");
                    pw.print("='");
                    printFloat(pw, xo.x);
                    pw.print("'");
                }
                if (xo.y != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("y");
                    pw.print("='");
                    printFloat(pw, xo.y);
                    pw.print("'");
                }
                if (xo.editable == 1) {
                    pw.print(" editable='simple'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</textArea>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XanimateScroll.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XanimateScroll xo = (XanimateScroll) o;
                pw.print("<lsr:animateScroll");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" xlink:show='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" xlink:title='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" xlink:type='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.by != null) {
                    pw.print(" by='");
                    print(pw, null, xo.by);
                    pw.print("'");
                }
                if (xo.from != null) {
                    pw.print(" from='");
                    print(pw, xo.from);
                    pw.print("'");
                }
                if (xo.to != null) {
                    pw.print(" to='");
                    print(pw, xo.to);
                    pw.print("'");
                }
                if (xo.begin != null) {
                    pw.print(" begin='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.dur != Integer.MIN_VALUE) {
                    pw.print(" dur='");
                    printDur(pw, xo.dur);
                    pw.print("'");
                }
                if (xo.end != null) {
                    pw.print(" end='");
                    printTimes(pw, xo.end);
                    pw.print("'");
                }
                if (xo.fill != Integer.MIN_VALUE) {
                    pw.print(" fill='");
                    XMLWriterUtil.printtyp_animFillType(pw, xo.fill);
                    pw.print("'");
                }
                if (xo.repeatCount != Integer.MIN_VALUE) {
                    pw.print(" repeatCount='");
                    printRepeatCount(pw, xo.repeatCount);
                    pw.print("'");
                }
                if (xo.repeatDur != Integer.MIN_VALUE) {
                    pw.print(" repeatDur='");
                    printRepeat(pw, xo.repeatDur);
                    pw.print("'");
                }
                if (xo.restart != Integer.MIN_VALUE) {
                    pw.print(" restart='");
                    XMLWriterUtil.printtyp_restartType(pw, xo.restart);
                    pw.print("'");
                }
                if (xo.speed != Integer.MIN_VALUE) {
                    pw.print(" speed='");
                    printFloat(pw, xo.speed);
                    pw.print("'");
                }
                if (xo.delayAtStart != Integer.MIN_VALUE) {
                    pw.print(" delayAtStart='");
                    printFloat(pw, xo.delayAtStart);
                    pw.print("'");
                }
                if (xo.delayAtEnd != Integer.MIN_VALUE) {
                    pw.print(" delayAtEnd='");
                    printFloat(pw, xo.delayAtEnd);
                    pw.print("'");
                }
                if (xo.direction != Integer.MIN_VALUE) {
                    switch(xo.direction) {
                        case 0:
                            pw.print(" direction='down'");
                            break;
                        case 1:
                            pw.print(" direction='left'");
                            break;
                        case 2:
                            pw.print(" direction='right'");
                            break;
                        case 3:
                            pw.print(" direction='up'");
                            break;
                    }
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</lsr:animateScroll>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XsetScroll.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XsetScroll xo = (XsetScroll) o;
                pw.print("<lsr:setScroll");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.xlink_actuate != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xlink:actuate");
                    pw.print("='");
                    XMLWriterUtil.printactuate_anonymousType_1(pw, xo.xlink_actuate);
                    pw.print("'");
                }
                if (xo.xlink_arcrole != null) {
                    pw.print(" ");
                    pw.print("xlink:arcrole");
                    pw.print("='");
                    print(pw, xo.xlink_arcrole);
                    pw.print("'");
                }
                if (xo.getHref() != null) {
                    pw.print(" ");
                    pw.print("xlink:href");
                    pw.print("='");
                    print(pw, xo.getHref());
                    pw.print("'");
                }
                if (xo.xlink_role != null) {
                    pw.print(" ");
                    pw.print("xlink:role");
                    pw.print("='");
                    print(pw, xo.xlink_role);
                    pw.print("'");
                }
                if (xo.xlink_show != Integer.MIN_VALUE) {
                    pw.print(" xlink:show='");
                    XMLWriterUtil.printshow_anonymousType_3(pw, xo.xlink_show);
                    pw.print("'");
                }
                if (xo.xlink_title != null) {
                    pw.print(" xlink:title='");
                    print(pw, xo.xlink_title);
                    pw.print("'");
                }
                if (xo.xlink_type != Integer.MIN_VALUE) {
                    pw.print(" xlink:type='");
                    XMLWriterUtil.printtype_anonymousType_2(pw, xo.xlink_type);
                    pw.print("'");
                }
                if (xo.increment != null) {
                    pw.print(" increment='");
                    print(pw, xo.increment);
                    pw.print("'");
                }
                if (xo.to != null) {
                    pw.print(" to='");
                    print(pw, xo.to);
                    pw.print("'");
                }
                if (xo.begin != null) {
                    pw.print(" begin='");
                    printTimes(pw, xo.begin);
                    pw.print("'");
                }
                if (xo.direction != Integer.MIN_VALUE) {
                    switch(xo.direction) {
                        case 0:
                            pw.print(" direction='down'");
                            break;
                        case 1:
                            pw.print(" direction='left'");
                            break;
                        case 2:
                            pw.print(" direction='right'");
                            break;
                        case 3:
                            pw.print(" direction='up'");
                            break;
                    }
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</lsr:setScroll>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }

    {
        classToWriter.put(XstreamSource.class, new LASeRXMLWriter() {
            public void write(PrintWriter pw, Object o) {
                XstreamSource xo = (XstreamSource) o;
                pw.print("<lsr:streamSource");
                if (xo.active != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("dims:active='");
                    pw.print((xo.active==1?"true":"false"));
                    pw.print("'");
                }
                if (xo.id != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("id");
                    pw.print("='");
                    printID(pw, xo.id);
                    pw.print("'");
                }
                if (xo.classe != null) {
                    pw.print(" ");
                    pw.print("class");
                    pw.print("='");
                    print(pw, xo.classe);
                    pw.print("'");
                }
                if (xo.base != null) {
                    pw.print(" ");
                    pw.print("xml:base");
                    pw.print("='");
                    print(pw, xo.base);
                    pw.print("'");
                }
                if (xo.lang != null) {
                    pw.print(" ");
                    pw.print("xml:lang");
                    pw.print("='");
                    print(pw, xo.lang);
                    pw.print("'");
                }
                if (xo.space != Integer.MIN_VALUE) {
                    pw.print(" ");
                    pw.print("xml:space");
                    pw.print("='");
                    XMLWriterUtil.printspace_anonymousType_4(pw, xo.space);
                    pw.print("'");
                }
                if (xo.source != null) {
                    pw.print(" sources='");
                    for (int i = 0; i < xo.source.length; i++) {
                        if (i != 0) pw.print(" ");
                        print(pw, xo.source[i]);
                    }
                    pw.print("'");
                }
                if (xo.sourceIndex != Integer.MIN_VALUE) {
                    pw.print(" sourceIndex='");
                    print(pw, xo.sourceIndex);
                    pw.print("'");
                }
                if (xo.mode != Integer.MIN_VALUE) {
                    switch(xo.mode) {
                        case 0:
                            pw.print(" mode='keepOld'");
                            break;
                        case 1:
                            pw.print(" mode='playList'");
                            break;
                        case 2:
                            pw.print(" mode='replace'");
                            break;
                        case 3:
                            pw.print(" mode='useOld'");
                            break;
                    }
                }
                if (xo.width != Integer.MIN_VALUE) {
                    pw.print(" width='");
                    printFloat(pw, xo.width);
                    pw.print("'");
                }
                if (xo.height != Integer.MIN_VALUE) {
                    pw.print(" height='");
                    printFloat(pw, xo.height);
                    pw.print("'");
                }
                if (xo.privateAttributes != null) {
                    for (int i = 0; i < xo.privateAttributes.length; i += 2) {
                        pw.print(" ");
                        pw.print(xo.privateAttributes[i]);
                        pw.print(" = \"");
                        pw.print(xo.privateAttributes[i]);
                        pw.print("\"");
                    }
                }
                if (xo.child != null && xo.child.size() > 0) {
                    pw.println(">");
                    for (int i = 0; i < xo.child.size(); i++) {
                        writeAny(pw, xo.child.elementAt(i));
                    }
                    pw.println("</lsr:streamSource>");
                } else {
                    pw.println("/>");
                }
            }
        });
    }
}
